@@ -1,9 +1,9 @@
 <?php
 /**
- * $Horde: horde/admin/setup/config.php,v 1.24.4.3 2005/01/31 09:28:27 jan Exp $
+ * $Horde: horde/admin/setup/config.php,v 1.24.4.9 2006/02/13 16:53:38 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -22,7 +22,7 @@ if (!Auth::isAdmin()) {
 }
 
 if (!Util::extensionExists('domxml') && !Util::extensionExists('dom')) {
-    Horde::fatal('You need the domxml PHP extension to use the configuration tool.', __FILE__, __LINE__);
+    Horde::fatal('You need the domxml or dom PHP extension to use the configuration tool.', __FILE__, __LINE__);
 }
 
 $app = Util::getFormData('app');
@@ -59,19 +59,22 @@ if (Util::getFormData('submitbutton') == _("Revert Configuration")) {
     $path = $registry->get('fileroot', $app) . '/config';
     if (file_exists($path . '/conf.php')) {
         if (@copy($path . '/conf.php', $path . '/conf.php.bak')) {
-            $notification->push(sprintf(_("Successfully saved the backup configuration file %s."), $path .'/conf.php.bak'), 'horde.success');
+            $notification->push(sprintf(_("Successfully saved the backup configuration file %s."), Util::realPath($path . '/conf.php.bak')), 'horde.success');
         } else {
-            $notification->push(sprintf(_("Could not save the backup configuration file %s."), $path .'/conf.php.bak'), 'horde.warning');
+            $notification->push(sprintf(_("Could not save the backup configuration file %s."), Util::realPath($path . '/conf.php.bak')), 'horde.warning');
         }
     }
     if ($fp = @fopen($path . '/conf.php', 'w')) {
         /* Can write, so output to file. */
         fwrite($fp, String::convertCharset($php, NLS::getCharset(), 'iso-8859-1'));
         fclose($fp);
-        $notification->push(sprintf(_("Successfully wrote %s"), $path . '/conf.php'), 'horde.success');
+        $notification->push(sprintf(_("Successfully wrote %s"), Util::realPath($path . '/conf.php')), 'horde.success');
+        $registry->clearCache();
+        header('Location: ' . Horde::applicationUrl('admin/setup/index.php', true));
+        exit;
     } else {
         /* Can not write. */
-        $notification->push(sprintf(_("Could not save the configuration file %s. You can either use one of the options to save the code back on %s or copy manually the code below to %s."), $path . '/conf.php', Horde::link(Horde::url('index.php') . '#update', _("Setup")) . _("Setup") . '</a>', $path . '/conf.php'), 'horde.warning', array('content.raw'));
+        $notification->push(sprintf(_("Could not save the configuration file %s. You can either use one of the options to save the code back on %s or copy manually the code below to %s."), Util::realPath($path . '/conf.php'), Horde::link(Horde::url('index.php') . '#update', _("Setup")) . _("Setup") . '</a>', Util::realPath($path . '/conf.php')), 'horde.warning', array('content.raw'));
         /* Save to session. */
         $_SESSION['_config'][$app] = $php;
     }
@@ -95,7 +98,7 @@ if (!empty($_SESSION['_config'][$app])) {
     Horde::addScriptFile('popup.js', 'horde', true);
     $url = Horde::applicationUrl('admin/setup/diff.php');
     $url = Util::addParameter($url, 'app', $app);
-    $link = Horde::link('', _("show differences"), '', '', 'popup(\'' . $url . '\',640, 480); return false;') . _("show differences") . '</a>';
+    $link = Horde::link('#', '', '', '', 'popup(\'' . $url . '\', 640, 480); return false;') . _("show differences") . '</a>';
     $template->set('diff_popup', $link);
 }
 $template->set('form', $form);
