@@ -3,9 +3,9 @@
  * Script to show the differences between the currently saved and the newly
  * generated configuration.
  *
- * $Horde: horde/admin/setup/diff.php,v 1.4.10.1 2005/01/03 12:25:29 jan Exp $
+ * $Horde: horde/admin/setup/diff.php,v 1.4.10.4 2006/01/01 21:28:05 jan Exp $
  *
- * Copyright 2004-2005 The Horde Project (http://www.horde.org/)
+ * Copyright 2004-2006 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -45,15 +45,14 @@ function _getDiff($app)
     }
 
     /* Calculate the differences. */
-    $diff = &new Text_Diff(explode("\n", htmlentities($current_config)),
-                           explode("\n", htmlentities($_SESSION['_config'][$app])));
-    //return nl2br($renderer->render($diff)); 
+    $diff = &new Text_Diff(explode("\n", $current_config),
+                           explode("\n", $_SESSION['_config'][$app]));
     $diff = $renderer->render($diff);
     if (!empty($diff)) {
         return $diff;
     } else {
         return _("No change.");
-    } 
+    }
 }
 
 $diffs = array();
@@ -66,16 +65,24 @@ if (!empty($_SESSION['_config'])) {
     if ($app = Util::getFormData('app')) {
         /* Handle a single app request. */
         $toggle_renderer = Horde::link($url . '#' . $app) . (($render_type == 'inline') ? _("unified") : _("inline")) . '</a>';
+        $diff = _getDiff($app);
+        if ($render_type != 'inline') {
+            $diff = htmlspecialchars($diff);
+        }
         $diffs[] = array('app'  => $app,
-                         'diff' => _getDiff($app),
+                         'diff' => $diff,
                          'toggle_renderer' => $toggle_renderer);
     } else {
         /* List all the apps with generated configuration. */
         ksort($_SESSION['_config']);
         foreach ($_SESSION['_config'] as $app => $config) {
             $toggle_renderer = Horde::link($url . '#' . $app) . (($render_type == 'inline') ? _("unified") : _("inline")) . '</a>';
+            $diff = _getDiff($app);
+            if ($render_type != 'inline') {
+                $diff = htmlspecialchars($diff);
+            }
             $diffs[] = array('app'  => $app,
-                             'diff' => _getDiff($app),
+                             'diff' => $diff,
                              'toggle_renderer' => $toggle_renderer);
         }
     }
