@@ -3,7 +3,7 @@
 $block_name = _("View an external web page");
 
 /**
- * $Horde: horde/lib/Block/iframe.php,v 1.18.8.6 2007/12/20 15:01:36 jan Exp $
+ * $Horde: horde/lib/Block/iframe.php,v 1.18.8.4 2006/02/21 10:53:45 jan Exp $
  *
  * @package Horde_Block
  */
@@ -18,13 +18,8 @@ class Horde_Block_Horde_iframe extends Horde_Block {
                                        'default' => ''),
                      'title'  => array('type' => 'text',
                                        'name' => _("Title")),
-                     'height' => array('type' => 'enum',
-                                       'name' => _("Height"),
-                                       'default' => '600',
-                                       'values' => array('480' => _("Small"),
-                                                         '600' => _("Medium"),
-                                                         '768' => _("Large"),
-                                                         '1024' => _("Extra Large"))));
+                     'height' => array('type' => 'int',
+                                       'name' => _("Height")));
     }
 
     /**
@@ -36,10 +31,17 @@ class Horde_Block_Horde_iframe extends Horde_Block {
     {
         global $registry;
 
-        $title = !empty($this->_params['title']) ? $this->_params['title'] : $this->_params['iframe'];
-        return htmlspecialchars($title) .
-            Horde::link($this->_params['iframe'], _("Open in a new window"), '', '_blank') .
-            Horde::img('external.png', '', array('style' => 'vertical-align:middle;padding-left:.3em')) . '</a>';
+        $title = isset($this->_params['title'])
+            ? $this->_params['title']
+            : $this->_params['iframe'];
+
+        $html = Horde::link($this->_params['iframe']) . $title .
+            '</a> :: <small>' .
+            Horde::link($this->_params['iframe'], '', '', '_blank') .
+            Horde::img('website.png', _("Open in a new window")) . ' ' .
+            _("Open in a new window") . '</a></small>';
+
+        return $html;
     }
 
     /**
