@@ -1,27 +1,15 @@
 <?php
-/**
- * @package File_CSV
- */
 
-/** PEAR */
 require_once 'PEAR.php';
-
-/** Mode to use for reading from files */
-define('HORDE_FILE_CSV_MODE_READ', 'rb');
-
-/** Mode to use for truncating files, then writing */
-define('HORDE_FILE_CSV_MODE_WRITE', 'wb');
-
-/** Mode to use for appending to files */
-define('HORDE_FILE_CSV_MODE_APPEND', 'ab');
+require_once 'File.php';
 
 /**
  * The File_CSV package allows reading and creating of CSV data and files.
  *
- * $Horde: framework/File_CSV/CSV.php,v 1.4.2.14 2008/04/22 03:58:10 chuck Exp $
+ * $Horde: framework/File_CSV/CSV.php,v 1.4.2.6 2006/01/01 21:28:15 jan Exp $
  *
  * Copyright 2002-2003 Tomas Von Veschler Cox <cox@idecnet.com>
- * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2005-2006 Jan Schneider <jan@horde.org>
  *
  * This source file is subject to version 2.0 of the PHP license, that is
  * bundled with this package in the file LICENSE, and is available at through
@@ -57,7 +45,7 @@ class File_CSV {
             return PEAR::raiseError('Could not open file: ' . $file);
         }
 
-        $seps = array("\t", ';', ':', ',', '~');
+        $seps = array("\t", ';', ':', ',');
         $seps = array_merge($seps, $extraSeps);
         $matches = array();
         $crlf = null;
@@ -94,7 +82,7 @@ class File_CSV {
             $conf['crlf'] = $crlf;
         }
 
-        /* Group the results by amount of equal occurrences. */
+        /* Group the results by amount of equal ocurrences. */
         $fields = array();
         $amount = array();
         foreach ($matches as $sep => $lines) {
@@ -115,12 +103,12 @@ class File_CSV {
         $conf['fields'] = $fields[$sep] + 1;
         $conf['sep']    = $sep;
 
-        /* Test if there are fields with quotes around in the first 10
+        /* Test if there are fields with quotes arround in the first 5
          * lines. */
         $quotes = '"\'';
         $quote  = '';
         rewind($fp);
-        for ($i = 0; ($i < 10) && ($line = fgets($fp)); $i++) {
+        for ($i = 0; ($i < 5) && ($line = fgets($fp)); $i++) {
             if (preg_match("|$sep([$quotes]).*([$quotes])$sep|U", $line, $match)) {
                 if ($match[1] == $match[2]) {
                     $quote = $match[1];
@@ -128,7 +116,8 @@ class File_CSV {
                 }
             }
             if (preg_match("|^([$quotes]).*([$quotes])$sep|", $line, $match) ||
-                preg_match("|([$quotes]).*([$quotes])$sep\s$|Us", $line, $match)) {
+                preg_match("|([$quotes]).*([$quotes])$sep\s$|Us", $line, $match))
+            {
                 if ($match[1] == $match[2]) {
                     $quote = $match[1];
                     break;
@@ -146,8 +135,6 @@ class File_CSV {
     /**
      * Reads a row from a CSV file and returns it as an array.
      *
-     * This method normalizes linebreaks to single newline characters (0x0a).
-     *
      * @param string $file  The name of the CSV file.
      * @param array $conf   The configuration for the CSV file.
      *
@@ -155,26 +142,22 @@ class File_CSV {
      */
     function read($file, &$conf)
     {
-        $fp = File_CSV::getPointer($file, $conf, HORDE_FILE_CSV_MODE_READ);
+        $fp = File_CSV::getPointer($file, $conf, FILE_MODE_READ);
         if (is_a($fp, 'PEAR_Error')) {
             return $fp;
         }
 
         $line = fgets($fp);
-        $line_length = strlen($line);
+        if (!$line) {
+            return false;
+        }
 
         /* Use readQuoted() if we have Mac line endings. */
         if (preg_match('/\r(?!\n)/', $line)) {
-            fseek($fp, -$line_length, SEEK_CUR);
+            fseek($fp, -strlen($line), SEEK_CUR);
             return File_CSV::readQuoted($file, $conf);
         }
 
-        /* Normalize line endings. */
-        $line = str_replace("\r\n", "\n", $line);
-        if (!strlen(trim($line))) {
-            return false;
-        }
-
         File_CSV::_line(File_CSV::_line() + 1);
 
         if ($conf['fields'] == 1) {
@@ -183,29 +166,27 @@ class File_CSV {
 
         $fields = explode($conf['sep'], $line);
         if ($conf['quote']) {
-            $last = ltrim($fields[count($fields) - 1]);
-            /* Fallback to read the line with readQuoted() if we assume that
-             * the simple explode won't work right. */
-            $last_len = strlen($last);
-            if (($last_len &&
-                 $last[$last_len - 1] == "\n" &&
-                 $last[0] == $conf['quote'] &&
-                 $last[strlen(rtrim($last)) - 1] != $conf['quote']) ||
+            $last = &$fields[count($fields) - 1];
+            /* Fallback to read the line with readQuoted() when guess that the
+             * simple explode won't work right. */
+            if ((substr($last, -strlen($conf['crlf'])) == $conf['crlf'] &&
+                 $last{0} == $conf['quote'] &&
+                 $last{strlen(rtrim($last)) - 1} != $conf['quote']) ||
                 (count($fields) != $conf['fields'])
-                // XXX perhaps there is a separator inside a quoted field
-                // preg_match("|{$conf['quote']}.*{$conf['sep']}.*{$conf['quote']}|U", $line)
-                ) {
-                fseek($fp, -$line_length, SEEK_CUR);
+                    // XXX perhaps there is a separator inside a quoted field
+                    //preg_match("|{$conf['quote']}.*{$conf['sep']}.*{$conf['quote']}|U", $line)
+                )
+            {
+                fseek($fp, -strlen($line), SEEK_CUR);
                 return File_CSV::readQuoted($file, $conf);
             } else {
+                $last = rtrim($last);
                 foreach ($fields as $k => $v) {
-                    $fields[$k] = File_CSV::unquote(trim($v), $conf['quote']);
+                    $fields[$k] = File_CSV::unquote($v, $conf['quote'], $conf['crlf']);
                 }
             }
         } else {
-            foreach ($fields as $k => $v) {
-                $fields[$k] = trim($v);
-            }
+            $fields[count($fields) -1] = rtrim($fields[count($fields) -1]);
         }
 
         if (count($fields) < $conf['fields']) {
@@ -221,7 +202,6 @@ class File_CSV {
 
     /**
      * Reads a row from a CSV file and returns it as an array.
-     *
      * This method is able to read fields with multiline data and normalizes
      * linebreaks to single newline characters (0x0a).
      *
@@ -232,7 +212,7 @@ class File_CSV {
      */
     function readQuoted($file, &$conf)
     {
-        $fp = File_CSV::getPointer($file, $conf, HORDE_FILE_CSV_MODE_READ);
+        $fp = File_CSV::getPointer($file, $conf, FILE_MODE_READ);
         if (is_a($fp, 'PEAR_Error')) {
             return $fp;
         }
@@ -242,30 +222,13 @@ class File_CSV {
         /* The current character. */
         $c = null;
         /* The read fields. */
-        $ret = false;
+        $ret = array();
         /* The number of the current field. */
-        $i = 0;
+        $i = 1;
         /* Are we inside a quoted field? */
         $in_quote = false;
-        /* Did we just process an escaped quote? */
-        $quote_escaped = false;
-        /* Is the last processed quote the first of a field? */
-        $first_quote = false;
 
         while (($ch = fgetc($fp)) !== false) {
-            /* Normalize line breaks. */
-            if ($ch == $conf['crlf']) {
-                $ch = "\n";
-            } elseif (strlen($conf['crlf']) == 2 && $ch == $conf['crlf'][0]) {
-                $next = fgetc($fp);
-                if (!$next) {
-                    break;
-                }
-                if ($next == $conf['crlf'][1]) {
-                    $ch = "\n";
-                }
-            }
-
             /* Previous character. */
             $prev = $c;
             /* Current character. */
@@ -274,99 +237,68 @@ class File_CSV {
             /* Simple character. */
             if ($c != $conf['quote'] &&
                 $c != $conf['sep'] &&
-                $c != "\n") {
+                strpos($conf['crlf'], $c) === false) {
                 $buff .= $c;
-                if (!$i) {
-                    $i = 1;
-                }
-                $quote_escaped = false;
-                $first_quote = false;
                 continue;
             }
 
-            if ($c == $conf['quote'] && !$in_quote) {
-                /* Quoted field begins. */
+            if ($conf['quote'] && $c == $conf['quote'] &&
+                ($prev == $conf['sep'] ||
+                 $prev == substr($conf['crlf'], -1) ||
+                 $prev === null))
+            {
+                /* Quote begin. */
                 $in_quote = true;
-                $buff = '';
-                if (!$i) {
-                    $i = 1;
-                }
             } elseif ($in_quote) {
-                /* We do NOT check for the closing quote immediately, but when
-                 * we got the character AFTER the closing quote. */
-                if ($c == $conf['quote'] && $prev == $conf['quote'] &&
-                    !$quote_escaped) {
-                    /* Escaped (double) quotes. */
-                    $quote_escaped = true;
-                    $first_quote = true;
-                    $prev = null;
-                    /* Simply skip the second quote. */
-                    continue;
-                } elseif ($c == $conf['sep'] && $prev == $conf['quote']) {
-                    /* Quoted field ends with a delimiter. */
+                /* Quote end? */
+                if ($c == $conf['sep'] && $prev == $conf['quote']) {
                     $in_quote = false;
-                    $quote_escaped = false;
-                    $first_quote = true;
-                } elseif ($c == "\n") {
-                    /* We have a linebreak inside the quotes. */
-                    if (strlen($buff) == 1 &&
-                        $buff[0] == $conf['quote'] &&
-                        $quote_escaped && $first_quote) {
-                        /* A line break after a closing quote of an empty
-                         * field, field and row end here. */
-                        $in_quote = false;
-                    } elseif (strlen($buff) >= 1 &&
-                        $buff[strlen($buff) - 1] == $conf['quote'] &&
-                        !$quote_escaped && !$first_quote) {
-                        /* A line break after a closing quote, field and row
-                         * end here. This is NOT the closing quote if we
-                         * either process an escaped (double) quote, or if the
-                         * quote before the line break was the opening
-                         * quote. */
+                } elseif ($c == substr($conf['crlf'], -1)) {
+                    $sub = (strlen($conf['crlf']) > 1 &&
+                            $prev == substr($conf['crlf'], -2, 1))
+                        ? 2
+                        : 1;
+                    if ((strlen($buff) >= $sub) &&
+                        ($buff{strlen($buff) - $sub} == $conf['quote']))
+                    {
                         $in_quote = false;
                     } else {
-                        /* Only increment the line number. Line breaks inside
-                         * quoted fields are part of the field content. */
                         File_CSV::_line(File_CSV::_line() + 1);
                     }
-                    $quote_escaped = false;
-                    $first_quote = true;
                 }
             }
 
+            /* End of line or end of field. */
             if (!$in_quote &&
-                ($c == $conf['sep'] || $c == "\n")) {
-                /* End of line or end of field. */
+                ($c == $conf['sep'] || $c == substr($conf['crlf'], -1))) {
+                /* More fields than expected. */
                 if ($c == $conf['sep'] &&
                     (count($ret) + 1) == $conf['fields']) {
-                    /* More fields than expected. Forward the line pointer to
-                     * the EOL and drop the remainder. */
-                    while ($c !== false && $c != "\n") {
+                    while ($c !== false && $c != substr($conf['crlf'], -1)) {
                         $c = fgetc($fp);
                     }
                     File_CSV::warning(sprintf(_("More fields found in line %d than the expected %d."), File_CSV::_line(), $conf['fields']));
                 }
 
-                if ($c == "\n" &&
-                    $i != $conf['fields']) {
-                    /* Less fields than expected. */
-                    if ($i == 0) {
+                if ($prev == substr($conf['crlf'], -2, 1)) {
+                    $buff = substr($buff, 0, -1);
+                }
+
+                /* Less fields than expected. */
+                if (($c == substr($conf['crlf'], -1)) &&
+                    ($i != $conf['fields'])) {
+                    if ($i == 1) {
                         /* Skip empty lines. */
                         return $ret;
                     }
                     File_CSV::warning(sprintf(_("Wrong number of fields in line %d. Expected %d, found %d."), File_CSV::_line(), $conf['fields'], $i));
 
-                    $ret[] = File_CSV::unquote($buff, $conf['quote']);
+                    $ret[] = File_CSV::unquote($buff, $conf['quote'], $conf['crlf']);
                     $ret = array_merge($ret, array_fill(0, $conf['fields'] - $i, ''));
                     return $ret;
                 }
 
-                /* Remove surrounding quotes from quoted fields. */
-                if ($buff == '"') {
-                    $ret[] = '';
-                } else {
-                    $ret[] = File_CSV::unquote($buff, $conf['quote']);
-                }
+                $ret[] = File_CSV::unquote($buff, $conf['quote'], $conf['crlf']);
                 if (count($ret) == $conf['fields']) {
                     return $ret;
                 }
@@ -378,7 +310,7 @@ class File_CSV {
             $buff .= $c;
         }
 
-        return $ret;
+        return feof($fp) ? false : $ret;
     }
 
     /**
@@ -392,7 +324,7 @@ class File_CSV {
      */
     function write($file, $fields, &$conf)
     {
-        if (is_a($fp = File_CSV::getPointer($file, $conf, HORDE_FILE_CSV_MODE_WRITE), 'PEAR_Error')) {
+        if (is_a($fp = File_CSV::getPointer($file, $conf, FILE_MODE_WRITE), 'PEAR_Error')) {
             return $fp;
         }
 
@@ -415,7 +347,7 @@ class File_CSV {
         }
 
         if (!fwrite($fp, $write)) {
-            return PEAR::raiseError(sprintf(_("Cannot write to file \"%s\""), $file));
+            return PEAR::raiseError(sprintf(_("Can not write to file \"%s\""), $file));
         }
 
         return true;
@@ -430,19 +362,14 @@ class File_CSV {
      *
      * @return string  The unquoted data.
      */
-    function unquote($field, $quote, $crlf = null)
+    function unquote($field, $quote, $crlf)
     {
         /* Skip empty fields (form: ;;) */
         if (!strlen($field)) {
             return $field;
         }
-        if ($quote && $field[0] == $quote &&
-            $field[strlen($field) - 1] == $quote) {
-            /* Normalize only for BC. */
-            if ($crlf) {
-                $field = str_replace($crlf, "\n", $field);
-            }
-            return substr($field, 1, -1);
+        if ($quote && $field{0} == $quote && $field{strlen($field)-1} == $quote) {
+            return str_replace($crlf, "\n", substr($field, 1, -1));
         }
         return $field;
     }
@@ -495,33 +422,33 @@ class File_CSV {
      *
      * @param string $file  The name of the file
      * @param array $conf   The configuration
-     * @param string $mode  The open mode. HORDE_FILE_CSV_MODE_READ or
-     *                      HORDE_FILE_CSV_MODE_WRITE.
+     * @param string $mode  The open mode. FILE_MODE_READ or FILE_MODE_WRITE.
      *
      * @return resource  The file resource or PEAR_Error on error.
      */
-    function getPointer($file, &$conf, $mode = HORDE_FILE_CSV_MODE_READ)
+    function getPointer($file, &$conf, $mode = FILE_MODE_READ)
     {
         static $resources = array();
         static $config = array();
 
-        if (isset($resources[$file])) {
-            $conf = $config[$file];
-            return $resources[$file];
+        $signature = $file . serialize($conf);
+        if (isset($resources[$signature])) {
+            $conf = $config[$signature];
+            return $resources[$signature];
         }
         if (is_a($error = File_CSV::_checkConfig($conf), 'PEAR_Error')) {
             return $error;
         }
-        $config[$file] = $conf;
+        $config[$signature] = $conf;
 
         $fp = @fopen($file, $mode);
         if (!is_resource($fp)) {
-            return PEAR::raiseError(sprintf(_("Cannot open file \"%s\"."), $file));
+            return PEAR::raiseError(sprintf(_("Can't open file \"%s\"."), $file));
         }
-        $resources[$file] = $fp;
+        $resources[$signature] = $fp;
         File_CSV::_line(0);
 
-        if ($mode == HORDE_FILE_CSV_MODE_READ && !empty($conf['header'])) {
+        if ($mode == FILE_MODE_READ && !empty($conf['header'])) {
             if (is_a($header = File_CSV::read($file, $conf), 'PEAR_Error')) {
                 return $header;
             }
