@@ -7,10 +7,10 @@ include_once 'Horde/Util.php';
  * The Horde:: class provides the functionality shared by all Horde
  * applications.
  *
- * $Horde: framework/Horde/Horde.php,v 1.489.2.11 2005/03/16 11:59:42 jan Exp $
+ * $Horde: framework/Horde/Horde.php,v 1.489.2.44 2006/03/24 00:40:38 chuck Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -25,8 +25,6 @@ class Horde {
     /**
      * Logs a message to the global Horde log backend.
      *
-     * @access public
-     *
      * @param mixed $message     Either a string or a PEAR_Error object.
      * @param string $file       What file was the log function called from
      *                           (e.g. __FILE__)?
@@ -66,7 +64,7 @@ class Horde {
             $message = $message->getMessage();
             if (!empty($userinfo)) {
                 if (is_array($userinfo)) {
-                    $userinfo = implode(', ', $userinfo);
+                    $userinfo = @implode(', ', $userinfo);
                 }
                 $message .= ': ' . $userinfo;
             }
@@ -89,6 +87,8 @@ class Horde {
         return true;
     }
 
+    /**
+     */
     function &getLogger()
     {
         global $conf;
@@ -118,8 +118,6 @@ class Horde {
      * Destroys any existing session on login and make sure to use a new
      * session ID, to avoid session fixation issues. Should be called before
      * checking a login.
-     *
-     * @access public
      */
     function getCleanSession()
     {
@@ -142,18 +140,25 @@ class Horde {
             Horde::setupSessionHandler();
             @session_start();
         }
+
+        /* Reset cookie timeouts, if necessary. */
+        if (!empty($GLOBALS['conf']['session']['timeout'])) {
+            $app = $GLOBALS['registry']->getApp();
+            if (Secret::clearKey($app)) {
+                Secret::setKey($app);
+            }
+            Secret::setKey('auth');
+        }
     }
 
     /**
      * Aborts with a fatal error, displaying debug information to the user.
      *
-     * @access public
-     *
      * @param mixed $error   A PEAR_Error object with debug information or an
      *                       error message.
      * @param integer $file  The file in which the error occured.
      * @param integer $line  The line on which the error occured.
-     * @param boolean $log   Log this message via Horde::logMesage()?
+     * @param boolean $log   Log this message via Horde::logMessage()?
      */
     function fatal($error, $file, $line, $log = true)
     {
@@ -177,7 +182,7 @@ class Horde {
                 break;
 
             case HORDE_ERROR_DRIVER_CONFIG:
-                $message = sprintf(_("Required '%s' not specified in %s configuration."), $info['field'], $info['name']) . '<br />' .
+                $message = sprintf(_("Required \"%s\" not specified in %s configuration."), $info['field'], $info['name']) . '<br />' .
                     sprintf(_("The file %s should contain a %s setting."),
                             $GLOBALS['registry']->get('fileroot') . '/config/' . $info['file'],
                             sprintf("%s['%s']['params']['%s']", $info['variable'], $info['driver'], $info['field']));
@@ -192,7 +197,7 @@ class Horde {
         } elseif (is_object($error) && method_exists($error, 'getMessage')) {
             $errortext .= '<h3>' . htmlspecialchars($error->getMessage()) . '</h3>';
         } elseif (is_string($error)) {
-            $errortext .= '<h3>' . $error . '</h3>';
+            $errortext .= '<h3>' . htmlspecialchars($error) . '</h3>';
         }
 
         if ($admin) {
@@ -216,7 +221,7 @@ class Horde {
             echo <<< HTML
 <html>
 <head><title>Horde :: Fatal Error</title></head>
-<body style="background-color: white; color: black;">$errortext</body>
+<body style="background:#fff; color:#000">$errortext</body>
 </html>
 HTML;
         }
@@ -227,8 +232,6 @@ HTML;
      * Adds the javascript code to the output (if output has already started)
      * or to the list of script files to include via includeScriptFiles().
      *
-     * @access public
-     *
      * @param string $file     The full javascript file name.
      * @param string $app      The application name. Defaults to the current
      *                         application.
@@ -244,12 +247,25 @@ HTML;
             $app = $registry->getApp();
         }
 
+        // Skip any js files that have since been deprecated.
+        $ignored_files = array('horde' => array('tooltip.js' => 1));
+        if (!empty($ignored_files[$app][$file])) {
+            return;
+        }
+
         // Don't include scripts multiple times.
         if (!empty($included[$app][$file])) {
             return;
         }
         $included[$app][$file] = true;
 
+        // Explicitly check for a non-PHP version of the script.
+        if (!$direct && file_exists($file{0} == '/' ? $registry->get('fileroot', $app) . $file : $registry->get('jsfs', $app) . '/' . $file)) {
+            $direct = true;
+        }
+
+        // If headers have already been sent, we need to output a
+        // <script> tag directly.
         if (ob_get_length() || headers_sent()) {
             if ($direct) {
                 $url = Horde::url($file{0} == '/' ? $registry->get('webroot', $app) . $file : $registry->get('jsuri', $app) . '/' . $file);
@@ -258,7 +274,7 @@ HTML;
                 $url = Util::addParameter($url, array('file' => $file,
                                                       'app'  => $app));
             }
-            echo '<script language="JavaScript" type="text/javascript" src="' . $url . '"></script>';
+            echo '<script type="text/javascript" src="' . $url . '"></script>' . "\n";
         } else {
             global $_horde_script_files;
             $_horde_script_files[$app][] = array($file, $direct);
@@ -267,24 +283,42 @@ HTML;
 
     /**
      * Includes javascript files that were needed before any headers were sent.
-     *
-     * @access public
      */
     function includeScriptFiles()
     {
         global $_horde_script_files, $registry;
 
+        /* If it has been requested in the session to disable javascript,
+         * insert the script to test for it being enabled again. */
+        if (isset($_SESSION['callbacks']['browser.disableJavascript'])) {
+            Horde::addScriptFile('test_javascript.js', 'horde');
+        }
+
+        /* If there is no javascript available, there's no point in including
+         * the rest of the files. */
+        if (!$GLOBALS['browser']->hasFeature('javascript')) {
+            return;
+        }
+
+        /* Add general UI js library. */
+        Horde::addScriptFile('horde.js', 'horde', true);
+        if ($GLOBALS['browser']->hasQuirk('windowed_controls')) {
+            /* Fixes for IE that can't easily be done without browser
+             * detection. */
+            Horde::addScriptFile('horde.ie.js', 'horde', true);
+        }
+
         if (!empty($_horde_script_files)) {
             $base_url = Horde::url($registry->get('webroot', 'horde') . '/services/javascript.php');
             foreach ($_horde_script_files as $app => $files) {
                 foreach ($files as $file) {
                     if (!empty($file[1])) {
                         $url = $file[0]{0} == '/' ? $registry->get('webroot', $app) . $file[0] : $registry->get('jsuri', $app) . '/' . $file[0];
-                        echo '<script language="JavaScript" type="text/javascript" src="' . Horde::url($url) . "\"></script>\n";
+                        echo '<script type="text/javascript" src="' . Horde::url($url) . "\"></script>\n";
                     } else {
                         $url = Util::addParameter($base_url, array('file' => $file[0],
                                                                    'app'  => $app));
-                        echo '<script language="JavaScript" type="text/javascript" src="' . $url . "\"></script>\n";
+                        echo '<script type="text/javascript" src="' . $url . "\"></script>\n";
                     }
                 }
             }
@@ -293,8 +327,6 @@ HTML;
 
     /**
      * Includes javascript files that were needed before any headers were sent.
-     *
-     * @access public
      */
     function inlineScriptFiles()
     {
@@ -318,8 +350,6 @@ HTML;
     /**
      * Checks if link should be shown and return the nescessary code.
      *
-     * @access public
-     *
      * @param string  $type      Type of link to display
      * @param string  $app       The name of the current Horde application.
      * @param boolean $override  Override Horde settings?
@@ -336,20 +366,16 @@ HTML;
         switch ($type) {
         case 'help':
             if ($GLOBALS['browser']->hasFeature('javascript')) {
-                Horde::addScriptFile('open_help_win.js', 'horde');
-                return "javascript:open_help_win('$app');";
-            } else {
-                $url = Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/help/', true);
-                return Util::addParameter($url, array('module' => $app,
-                                                      'show' => 'topics'));
+                Horde::addScriptFile('popup.js', 'horde', true);
             }
-            break;
+            $url = Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/help/', true);
+            return Util::addParameter($url, 'module', $app);
 
         case 'problem':
-            return Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/problem.php?return_url=' . urlencode(Horde::selfUrl(true)));
+            return Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/problem.php?return_url=' . urlencode(Horde::selfUrl(true, true, true)));
 
         case 'logout':
-            return Auth::addLogoutParameters(Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/login.php'), AUTH_REASON_LOGOUT);
+            return Horde::url(Auth::addLogoutParameters($GLOBALS['registry']->get('webroot', 'horde') . '/login.php', AUTH_REASON_LOGOUT));
 
         case 'login':
             return Auth::getLoginScreen('', $referrer ? Horde::selfUrl(true) : null);
@@ -366,8 +392,6 @@ HTML;
     }
 
     /**
-     * @access public
-     *
      * @param string $type  The type of link.
      *
      * @return boolean  True if the link is to be shown.
@@ -500,7 +524,7 @@ HTML;
         foreach ($fields as $field) {
             if (!isset($params[$field])) {
                 Horde::fatal(PEAR::raiseError(
-                    sprintf(_("Required '%s' not specified in %s configuration."), $field, $name) . "\n\n" .
+                    sprintf(_("Required \"%s\" not specified in %s configuration."), $field, $name) . "\n\n" .
                     sprintf(_("The file %s should contain a %s setting."),
                         $fileroot . '/config/' . $file,
                         sprintf("%s['%s']['params']['%s']", $variable, $driver, $field))),
@@ -514,11 +538,11 @@ HTML;
      * If a full URL is requested, all parameter separators get converted to
      * "&", otherwise to "&amp;".
      *
-     * @access public
-     *
-     * @param string $uri          The URI to be modified.
-     * @param bool $full           Generate a full (http://server/path/) URL.
-     * @param int $append_session  0 = only if needed, 1 = always, -1 = never.
+     * @param string $uri              The URI to be modified.
+     * @param boolean $full            Generate a full (http://server/path/)
+     *                                 URL.
+     * @param integer $append_session  0 = only if needed, 1 = always, -1 =
+     *                                 never.
      *
      * @return string  The URL with the session id appended (if needed).
      */
@@ -595,11 +619,11 @@ HTML;
      * Returns a session-id-ified version of $uri, using the current
      * application's webroot setting.
      *
-     * @access public
-     *
-     * @param string $uri          The URI to be modified.
-     * @param bool $full           Generate a full (http://server/path/) URL.
-     * @param int $append_session  0 = only if needed, 1 = always, -1 = never.
+     * @param string $uri              The URI to be modified.
+     * @param boolean $full            Generate a full (http://server/path/)
+     *                                 URL.
+     * @param integer $append_session  0 = only if needed, 1 = always, -1 =
+     *                                 never.
      *
      * @return string  The url with the session id appended.
      */
@@ -647,8 +671,6 @@ HTML;
      * Returns a URL to be used for downloading, that takes into account any
      * special browser quirks (i.e. IE's broken filename handling).
      *
-     * @access public
-     *
      * @param string $filename  The filename of the download data.
      * @param array $params     Any additional parameters needed.
      * @param string $url       The URL to alter. If none passed in, will use
@@ -671,9 +693,7 @@ HTML;
 
         /* Add parameters. */
         if (!is_null($params)) {
-            foreach ($params as $key => $val) {
-                $url = Util::addParameter($url, $key, $val);
-            }
+            $url = Util::addParameter($url, $params);
         }
 
         /* If we are using the default Horde download link, add the
@@ -681,7 +701,7 @@ HTML;
          * many browsers, this should allow every browser to download
          * correctly. */
         if ($horde_url) {
-            $url = Util::addParameter($url, 'fn=/' . rawurlencode($filename));
+            $url = Util::addParameter($url, 'fn', '/' . rawurlencode($filename));
         } elseif ($browser->hasQuirk('break_disposition_filename')) {
             /* Some browsers will only obtain the filename correctly
              * if the extension is the last argument in the query
@@ -715,7 +735,7 @@ HTML;
 
             /* Append the extension, if it exists. */
             if (($server == 'apache2') || !empty($ext)) {
-                $url = Util::addParameter($url, 'fn_ext=/' . $filename);
+                $url = Util::addParameter($url, 'fn_ext', '/' . $filename);
             }
         }
 
@@ -725,30 +745,38 @@ HTML;
     /**
      * Returns an anchor tag with the relevant parameters
      *
-     * @access public
-     *
-     * @param string $url        The full URL to be linked to
-     * @param string $status     The JavaScript mouse-over string
-     * @param string $class      The CSS class of the link
+     * @param string $url        The full URL to be linked to.
+     * @param string $title      The link title/description.
+     * @param string $class      The CSS class of the link.
      * @param string $target     The window target to point to.
      * @param string $onclick    JavaScript action for the 'onclick' event.
-     * @param string $title      The link title (tooltip).
+     * @param string $title2     The link title (tooltip) (deprecated - just
+     *                           use $title).
      * @param string $accesskey  The access key to use.
      * @param array $attributes  Any other name/value pairs to add to the <a>
      *                           tag.
+     * @param boolean $escape    Whether to escape special characters in the
+     *                           title attribute.
      *
      * @return string  The full <a href> tag.
      */
-    function link($url, $status = '', $class = '', $target = '', $onclick = '',
-                  $title = '', $accesskey = '', $attributes = array())
+    function link($url, $title = '', $class = '', $target = '', $onclick = '',
+                  $title2 = '', $accesskey = '', $attributes = array(),
+                  $escape = true)
     {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
+        if (!empty($title2)) {
+            $title = $title2;
+        }
+
         $ret = "<a href=\"$url\"";
         if (!empty($onclick)) {
             $ret .= " onclick=\"$onclick\"";
         }
-        if (!empty($status)) {
-            $ret .= ' onmouseout="window.status=\'\';" onmouseover="window.status=\'' . @htmlspecialchars(strtr(addslashes($status), array("\r" => '', "\n" => '')), ENT_QUOTES, NLS::getCharset()) . '\'; return true;"';
-        }
         if (!empty($class)) {
             $ret .= " class=\"$class\"";
         }
@@ -756,7 +784,10 @@ HTML;
             $ret .= " target=\"$target\"";
         }
         if (!empty($title)) {
-            $ret .= ' title="' . @htmlspecialchars($title, ENT_QUOTES, NLS::getCharset()) . '"';
+            if ($escape) {
+                $title = nl2br(@htmlspecialchars(@htmlspecialchars($title, ENT_QUOTES, $charset), ENT_QUOTES, $charset));
+            }
+            $ret .= ' title="' . $title . '"';
         }
         if (!empty($accesskey)) {
             $ret .= ' accesskey="' . htmlspecialchars($accesskey) . '"';
@@ -769,36 +800,69 @@ HTML;
         return "$ret>";
     }
 
+    /**
+     * Uses DOM Tooltips to display the 'title' attribute for
+     * Horde::link() calls.
+     *
+     * @param string $url        The full URL to be linked to
+     * @param string $status     The JavaScript mouse-over string
+     * @param string $class      The CSS class of the link
+     * @param string $target     The window target to point to.
+     * @param string $onclick    JavaScript action for the 'onclick' event.
+     * @param string $title      The link title (tooltip).
+     * @param string $accesskey  The access key to use.
+     * @param array  $attributes Any other name/value pairs to add to the <a>
+     *                           tag.
+     *
+     * @return string  The full <a href> tag.
+     */
+    function linkTooltip($url, $status = '', $class = '', $target = '',
+                         $onclick = '', $title = '', $accesskey = '',
+                         $attributes = array())
+    {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
+        if (!empty($title)) {
+            $title = '&lt;pre&gt;' . preg_replace(array('/\n/', '/((?<!<br)\s{1,}(?<!\/>))/em', '/<br \/><br \/>/', '/<br \/>/'), array('', 'str_repeat("&nbsp;", strlen("$1"))', '&lt;br /&gt; &lt;br /&gt;', '&lt;br /&gt;'), nl2br(@htmlspecialchars(@htmlspecialchars($title, ENT_QUOTES, $charset), ENT_QUOTES, $charset))) . '&lt;/pre&gt;';
+        }
+        return Horde::link($url, $title, $class, $target, $onclick, null, $accesskey, $attributes, false);
+    }
+
     /**
      * Returns an anchor sequence with the relevant parameters for a widget
      * with accesskey and text.
      *
      * @access public
      *
-     * @param string $url       The full URL to be linked to
-     * @param string $status    The JavaScript mouse-over string
-     * @param string $class     The CSS class of the link
-     * @param string $target    The window target to point to.
-     * @param string $onclick   JavaScript action for the 'onclick' event.
-     * @param string $title     The link title (tooltip).
+     * @param string  $url      The full URL to be linked to.
+     * @param string  $title    The link title/description.
+     * @param string  $class    The CSS class of the link
+     * @param string  $target   The window target to point to.
+     * @param string  $onclick  JavaScript action for the 'onclick' event.
+     * @param string  $title2   The link title (tooltip) (deprecated - just use
+     *                          $title).
      * @param boolean $nocheck  Don't check if the access key already has been
-     *                          used?
+     *                          used. Defaults to false (= check).
      *
      * @return string  The full <a href>Title</a> sequence.
      */
-    function widget($url, $status = '', $class = 'widget', $target = '',
-                    $onclick = '', $title = '', $nocheck = false)
+    function widget($url, $title = '', $class = 'widget', $target = '',
+                    $onclick = '', $title2 = '', $nocheck = false)
     {
-        $ak = Horde::getAccessKey($title, $nocheck);
-        $plaintitle = Horde::stripAccessKey($title);
+        if (!empty($title2)) {
+            $title = $title2;
+        }
 
-        return Horde::link($url, $status, $class, $target, $onclick, $plaintitle, $ak) . Horde::highlightAccessKey($title, $ak) . '</a>';
+        $ak = Horde::getAccessKey($title, $nocheck);
+
+        return Horde::link($url, '', $class, $target, $onclick, '', $ak) . Horde::highlightAccessKey($title, $ak) . '</a>';
     }
 
     /**
-     * Returns a session-id-ified version of $PHP_SELF.
-     *
-     * @access public
+     * Returns a session-id-ified version of $SCRIPT_NAME resp. $PHP_SELF.
      *
      * @param boolean $query_string  Include any QUERY_STRING?
      * @param boolean $nocache       Include a nocache parameter in the URL?
@@ -809,7 +873,16 @@ HTML;
     function selfUrl($query_string = false, $nocache = true, $full = false,
                      $force_ssl = false)
     {
-        $url = $_SERVER['PHP_SELF'];
+        if (substr(php_sapi_name(), 0, 3) == 'cgi') {
+            // When using CGI PHP, SCRIPT_NAME may contain the path to
+            // the PHP binary instead of the script being run; use
+            // PHP_SELF instead.
+            $url = $_SERVER['PHP_SELF'];
+        } else {
+            $url = isset($_SERVER['SCRIPT_NAME']) ?
+                $_SERVER['SCRIPT_NAME'] :
+                $_SERVER['PHP_SELF'];
+        }
 
         if ($query_string && !empty($_SERVER['QUERY_STRING'])) {
             $url .= '?' . $_SERVER['QUERY_STRING'];
@@ -827,22 +900,25 @@ HTML;
     /**
      * Constructs a correctly-pathed link to an image.
      *
-     * @access public
-     *
-     * @param          string $src   The image file.
-     * @param optional string $alt   Text describing the image.
-     * @param optional mixed  $attr  Any additional attributes for the image tag.
-     *                               Can be a pre-built string or an array of key/value
-     *                               pairs that will be assembled and html-encoded.
-     * @param optional string $dir   The root graphics directory.
+     * @param string $src   The image file.
+     * @param string $alt   Text describing the image.
+     * @param mixed  $attr  Any additional attributes for the image tag. Can be
+     *                      a pre-built string or an array of key/value pairs
+     *                      that will be assembled and html-encoded.
+     * @param string $dir   The root graphics directory.
      *
      * @return string  The full image tag.
      */
     function img($src, $alt = '', $attr = '', $dir = null)
     {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
         /* If browser does not support images, simply return the ALT text. */
         if (!$GLOBALS['browser']->hasFeature('images')) {
-            return @htmlspecialchars($alt, ENT_COMPAT, NLS::getCharset());
+            return @htmlspecialchars($alt, ENT_COMPAT, $charset);
         }
 
         /* If no directory has been specified, get it from the registry. */
@@ -858,15 +934,14 @@ HTML;
 
         /* Build all of the tag attributes. */
         $attributes = array('src' => $src,
-                            'alt' => $alt,
-                            'title' => $alt);
+                            'alt' => $alt);
         if (is_array($attr)) {
             $attributes = array_merge($attributes, $attr);
         }
 
         $img = '<img';
         foreach ($attributes as $attribute => $value) {
-            $img .= ' ' . $attribute . '="' . ($attribute == 'src' ? $value : @htmlspecialchars($value, ENT_COMPAT, NLS::getCharset())) . '"';
+            $img .= ' ' . $attribute . '="' . ($attribute == 'src' ? $value : @htmlspecialchars($value, ENT_COMPAT, $charset)) . '"';
         }
 
         /* If the user supplied a pre-built string of attributes, add that. */
@@ -882,9 +957,7 @@ HTML;
      * Determines the location of the system temporary directory. If a specific
      * setting cannot be found, it defaults to /tmp.
      *
-     * @access public
-     *
-     * @return string  A directory name which can be used for temp files.
+     * @return string  A directory name that can be used for temp files.
      *                 Returns false if one could not be found.
      */
     function getTempDir()
@@ -910,8 +983,6 @@ HTML;
      * Creates a temporary filename for the lifetime of the script, and
      * (optionally) registers it to be deleted at request shutdown.
      *
-     * @access public
-     *
      * @param string $prefix   Prefix to make the temporary name more
      *                         recognizable.
      * @param boolean $delete  Delete the file at the end of the request?
@@ -934,10 +1005,6 @@ HTML;
 
     /**
      * Starts output compression, if requested.
-     *
-     * @access public
-     *
-     * @since Horde 2.2
      */
     function compressOutput()
     {
@@ -951,6 +1018,7 @@ HTML;
         if ($GLOBALS['conf']['compress_pages'] &&
             !$GLOBALS['browser']->hasQuirk('buggy_compression') &&
             ini_get('zlib.output_compression') == '' &&
+            ini_get('zend_accelerator.compress_all') == '' &&
             ini_get('output_handler') != 'ob_gzhandler') {
             if (ob_get_level()) {
                 ob_end_clean();
@@ -964,8 +1032,6 @@ HTML;
     /**
      * Determines if output compression can be used.
      *
-     * @access public
-     *
      * @return boolean  True if output compression can be used, false if not.
      */
     function allowOutputCompression()
@@ -979,6 +1045,7 @@ HTML;
         }
 
         return (ini_get('zlib.output_compression') == '' &&
+                ini_get('zend_accelerator.compress_all') == '' &&
                 ini_get('output_handler') != 'ob_gzhandler');
     }
 
@@ -986,8 +1053,6 @@ HTML;
      * Returns the Web server being used.
      * PHP string list built from the PHP 'configure' script.
      *
-     * @access public
-     *
      * @return string  A web server identification string.
      * <pre>
      * 'aolserver' = AOL Server
@@ -1026,8 +1091,6 @@ HTML;
     /**
      * Returns the <link> tags for the CSS stylesheets.
      *
-     * @access public
-     *
      * @param string|array $app  The Horde application(s).
      * @param mixed $theme       The theme to use; specify an empty value to
      *                           retrieve the theme from user preferences, and
@@ -1042,7 +1105,9 @@ HTML;
             $theme = $GLOBALS['prefs']->getValue('theme');
         }
 
+        $rtl = isset($GLOBALS['nls']['rtl'][$GLOBALS['language']]);
         $css = array();
+
         $themes_fs = $GLOBALS['registry']->get('themesfs', 'horde');
         if ($inherit) {
             $themes_uri = Horde::url($GLOBALS['registry']->get('themesuri', 'horde'), false, -1);
@@ -1051,26 +1116,45 @@ HTML;
                 file_exists($themes_fs . '/' . $theme . '/screen.css')) {
                 $css[] = $themes_uri . '/' . $theme . '/screen.css';
             }
+
+            if ($rtl) {
+                $css[] = $themes_uri . '/rtl.css';
+                if (!empty($theme) &&
+                    file_exists($themes_fs . '/' . $theme . '/rtl.css')) {
+                    $css[] = $themes_uri . '/' . $theme . '/rtl.css';
+                }
+            }
         }
 
         if (!empty($apps)) {
             if (!is_array($apps)) {
                 $apps = array($apps);
             }
+
             foreach ($apps as $app) {
                 if ($inherit && $app == 'horde') {
                     continue;
                 }
 
                 $themes_fs = $GLOBALS['registry']->get('themesfs', $app);
+                $themes_uri = Horde::url($GLOBALS['registry']->get('themesuri', $app), false, -1);
                 if (file_exists($themes_fs . '/screen.css')) {
-                    $themes_uri = Horde::url($GLOBALS['registry']->get('themesuri', $app), false, -1);
                     $css[] = $themes_uri . '/screen.css';
                 }
                 if (!empty($theme) &&
                     file_exists($themes_fs . '/' . $theme . '/screen.css')) {
                     $css[] = $themes_uri . '/' . $theme . '/screen.css';
                 }
+
+                if ($rtl) {
+                    if (file_exists($themes_fs . '/rtl.css')) {
+                        $css[] = $themes_uri . '/rtl.css';
+                    }
+                    if (!empty($theme) &&
+                        file_exists($themes_fs . '/' . $theme . '/rtl.css')) {
+                        $css[] = $themes_uri . '/' . $theme . '/rtl.css';
+                    }
+                }
             }
         }
 
@@ -1087,8 +1171,8 @@ HTML;
         }
 
         /* Load browser specific stylesheets if needed. */
-        if ($GLOBALS['browser']->isBrowser('msie')) {
-            $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/ie.css" rel="stylesheet" type="text/css" />' . "\n";
+        if ($GLOBALS['browser']->isBrowser('msie') && $GLOBALS['browser']->getMajor() < 7) {
+            $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/ie6_or_less.css" rel="stylesheet" type="text/css" />' . "\n";
             if ($GLOBALS['browser']->getPlatform() == 'mac') {
                 $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/ie5mac.css" rel="stylesheet" type="text/css" />' . "\n";
             }
@@ -1111,8 +1195,6 @@ HTML;
 
     /**
      * Sets a custom session handler up, if there is one.
-     *
-     * @access public
      */
     function setupSessionHandler()
     {
@@ -1154,9 +1236,31 @@ HTML;
     }
 
     /**
-     * Returns an un-used access key from the label given.
+     * This function is triggered when the session is loaded. It checks for
+     * any callbacks stored in the session that need to be run immediately
+     * the session is loaded.
+     * The way to store the callbacks is:
+     * <pre>
+     * $_SESSION['callbacks']['<class object>.<function>'] = true;
+     * </pre>
      *
-     * @access public
+     * Any objects that are not available in $GLOBALS will be silently
+     * ignored.
+     */
+    function sessionCallbacks()
+    {
+        if (isset($_SESSION['callbacks'])) {
+            foreach (array_keys($_SESSION['callbacks']) as $callback) {
+                list($class, $function) = explode('.', $callback);
+                if (isset($GLOBALS[$class])) {
+                    call_user_func(array(&$GLOBALS[$class], $function));
+                }
+            }
+        }
+    }
+
+    /**
+     * Returns an un-used access key from the label given.
      *
      * @param string $label     The label to choose an access key from.
      * @param boolean $nocheck  Don't check if the access key already has been
@@ -1221,8 +1325,6 @@ HTML;
      * For multibyte charset strings the access key gets removed completely,
      * otherwise only the underscore gets removed.
      *
-     * @access public
-     *
      * @param string $label  The label containing an access key.
      *
      * @return string  The label with the access key being stripped.
@@ -1240,8 +1342,6 @@ HTML;
     /**
      * Highlights an access key in a label.
      *
-     * @access public
-     *
      * @param string $label      The label to to highlight the access key in.
      * @param string $accessKey  The access key to highlight.
      *
@@ -1315,8 +1415,6 @@ HTML;
 
     /**
      * Redirects to the main Horde login page on authentication failure.
-     *
-     * @access public
      */
     function authenticationFailureRedirect()
     {
@@ -1327,63 +1425,18 @@ HTML;
         }
 
         $url = $GLOBALS['registry']->get('webroot', 'horde') . '/login.php';
-        $url = Util::addParameter($url, 'url', Horde::selfUrl(true));
+        $url = Util::addParameter($url, array('url' => Horde::selfUrl(true), 'nosidebar' => 1), null, false);
         $url = Auth::addLogoutParameters($url);
         header('Location: ' . Horde::url($url, true));
         exit;
     }
 
-    /**
-     * Uses DOM Tooltips (via javascript) to display the 'title'
-     * attribute for Horde::link() calls.
-     *
-     * If using this function, the following function must be called:
-     *   Horde::addScriptFile('tooltip.js', 'horde', true);
-     *
-     * @access public
-     *
-     * @param string $url        The full URL to be linked to
-     * @param string $status     The JavaScript mouse-over string
-     * @param string $class      The CSS class of the link
-     * @param string $target     The window target to point to.
-     * @param string $onclick    JavaScript action for the 'onclick' event.
-     * @param string $title      The link title (tooltip).
-     * @param string $accesskey  The access key to use.
-     * @param array $attributes  Any other name/value pairs to add to the <a>
-     *                           tag.
-     *
-     * @return string  The full <a href> tag.
-     */
-    function linkTooltip($url, $status = '', $class = '', $target = '',
-                         $onclick = '', $title = '', $accesskey = '',
-                         $attributes = array())
-    {
-        $url = substr(Horde::link($url, null, $class, $target, $onclick, null, $accesskey, $attributes), 0, -1);
-
-        if (strlen($status)) {
-            $status = @htmlspecialchars(addslashes($status), ENT_QUOTES, NLS::getCharset());
-        }
-
-        $title = trim($title);
-        if (strlen($title)) {
-            require_once 'Horde/Text/Filter.php';
-            $title = Text_Filter::filter($title, 'text2html', array('parselevel' => TEXT_HTML_NOHTML, 'charset' => '', 'class' => ''));
-            $title = '<pre style="margin:0px;">' . strtr(addslashes($title), array("\r" => '', "\n" => '')) . '</pre>';
-            $url .= ' onmouseover="tooltipLink(\'' . @htmlspecialchars($title, ENT_QUOTES, NLS::getCharset()) . '\', \'' . $status . '\'); return true;" onmouseout="tooltipClose();"';
-        }
-        $url .= '>';
-
-        return $url;
-    }
-
     /**
      * Provides a standardised function to call a Horde hook, checking whether
      * a hook config file exists and whether the function itself exists. If
      * these two conditions are not satisfied it will return the specified
      * value (by default a PEAR error).
      *
-     * @access public
-     *
      * @param string $hook  The function to call.
      * @param array  $args  An array of any arguments to pass to the hook
      *                      function.
@@ -1442,6 +1495,35 @@ HTML;
         }
     }
 
+    /**
+     * Returns the specified permission for the current user.
+     *
+     * @since Horde 3.1
+     *
+     * @param string $permission  A permission, currently only 'max_blocks'.
+     *
+     * @return mixed  The value of the specified permission.
+     */
+    function hasPermission($permission)
+    {
+        global $perms;
+
+        if (!$perms->exists('horde:' . $permission)) {
+            return true;
+        }
+
+        $allowed = $perms->getPermissions('horde:' . $permission);
+        if (is_array($allowed)) {
+            switch ($permission) {
+            case 'max_blocks':
+                $allowed = array_reduce($allowed, create_function('$a, $b', 'return max($a, $b);'), 0);
+                break;
+            }
+        }
+
+        return $allowed;
+    }
+
     /**
      * Returns an array of available menu items when in a Horde script.
      *
