@@ -36,6 +36,11 @@ define('AUTH_REASON_BROWSER', 'browser');
  */
 define('AUTH_REASON_FAILED', 'failed');
 
+/**
+ * The 'expired' reason.
+ */
+define('AUTH_REASON_EXPIRED', 'expired');
+
 /**
  * The 'logout' reason.
  */
@@ -60,15 +65,14 @@ define('AUTH_REASON_SESSIONIP', 'sessionip');
  * The Auth:: class provides a common abstracted interface into the various
  * backends for the Horde authentication system.
  *
- * $Horde: framework/Auth/Auth.php,v 1.142.10.4 2005/03/04 23:51:11 jan Exp $
+ * $Horde: framework/Auth/Auth.php,v 1.142.10.21 2006/01/18 12:20:02 jan Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.142.10.4 $
  * @since   Horde 1.3
  * @package Horde_Auth
  */
@@ -78,7 +82,7 @@ class Auth {
      * An array of capabilities, so that the driver can report which
      * operations it supports and which it doesn't.
      *
-     * @var array $capabilities
+     * @var array
      */
     var $capabilities = array('add'           => false,
                               'update'        => false,
@@ -91,7 +95,7 @@ class Auth {
    /**
     * Hash containing parameters.
     *
-    * @var array $_params
+    * @var array
     */
     var $_params = array();
 
@@ -100,7 +104,7 @@ class Auth {
      *
      * @access protected
      *
-     * @var array $_authCredentials
+     * @var array
      */
     var $_authCredentials = array();
 
@@ -118,8 +122,6 @@ class Auth {
      * Finds out if a set of login credentials are valid, and if requested,
      * mark the user as logged in in the current session.
      *
-     * @access public
-     *
      * @param string $userId      The userId to check.
      * @param array $credentials  The credentials to check.
      * @param boolean $login      Whether to log the user in. If false, we'll
@@ -140,26 +142,34 @@ class Auth {
             include_once HORDE_BASE . '/config/hooks.php';
             if (function_exists('_horde_hook_preauthenticate')) {
                 if (!call_user_func('_horde_hook_preauthenticate', $userId, $credentials, $realm)) {
-                    $this->_setAuthError(AUTH_REASON_FAILED);
+                    if ($this->_getAuthError() != AUTH_REASON_MESSAGE) {
+                        $this->_setAuthError(AUTH_REASON_FAILED);
+                    }
                     return false;
                 }
             }
         }
 
-        /* Store the credentials being checked so that subclasses can
-         * modify them if necessary (like transparent auth does). */
+        /* Store the credentials being checked so that subclasses can modify
+         * them if necessary (like transparent auth does). */
         $this->_authCredentials = array(
             'userId' => $userId,
             'credentials' => $credentials,
-            'realm' => $realm
+            'realm' => $realm,
+            'changeRequested' => false
         );
 
-        if ($this->_authenticate($userId, $credentials)) {
+        if ($authenticated = $this->_authenticate($userId, $credentials)) {
+            if (is_a($authenticated, 'PEAR_Error')) {
+                return false;
+            }
             if (!empty($conf['hooks']['postauthenticate'])) {
                 include_once HORDE_BASE . '/config/hooks.php';
                 if (function_exists('_horde_hook_postauthenticate')) {
                     if (!call_user_func('_horde_hook_postauthenticate', $userId, $credentials, $realm)) {
-                        $this->_setAuthError(AUTH_REASON_FAILED);
+                        if ($this->_getAuthError() != AUTH_REASON_MESSAGE) {
+                            $this->_setAuthError(AUTH_REASON_FAILED);
+                        }
                         return false;
                     }
                 }
@@ -168,7 +178,8 @@ class Auth {
             if ($login) {
                 $this->setAuth($this->_authCredentials['userId'],
                                $this->_authCredentials['credentials'],
-                               $this->_authCredentials['realm']);
+                               $this->_authCredentials['realm'],
+                               $this->_authCredentials['changeRequested']);
                 $auth = true;
             } else {
                 if (!$this->_checkSessionIP()) {
@@ -188,8 +199,6 @@ class Auth {
     /**
      * Formats a password using the current encryption.
      *
-     * @access public
-     *
      * @param string $plaintext      The plaintext password to encrypt.
      * @param string $salt           The salt to use to encrypt the password.
      *                               If not present, a new salt will be
@@ -224,7 +233,7 @@ class Auth {
         case 'crypt-des':
         case 'crypt-md5':
         case 'crypt-blowfish':
-            return (($show_encrypt) ? '{crypt}' : '') . crypt($plaintext, $salt);
+            return ($show_encrypt ? '{crypt}' : '') . crypt($plaintext, $salt);
 
         case 'md5-base64':
             $encrypted = base64_encode(mhash(MHASH_MD5, $plaintext));
@@ -277,7 +286,7 @@ class Auth {
                                        5);
             }
 
-            return '$apr1$' . $salt . '$' . implode($p) . Auth::_toAPRMD5(ord($binary[11]), 3);
+            return '$apr1$' . $salt . '$' . implode('', $p) . Auth::_toAPRMD5(ord($binary[11]), 3);
 
         case 'md5-hex':
         default:
@@ -286,12 +295,10 @@ class Auth {
     }
 
     /**
-     * Returns a salt for the appropriate kind of password
-     * encryption. Optionally takes a seed and a plaintext password, to
-     * extract the seed of an existing password, or for encryption types that
-     * use the plaintext in the generation of the salt.
-     *
-     * @access public
+     * Returns a salt for the appropriate kind of password encryption.
+     * Optionally takes a seed and a plaintext password, to extract the seed
+     * of an existing password, or for encryption types that use the plaintext
+     * in the generation of the salt.
      *
      * @param string $encryption  The kind of pasword encryption to use.
      *                            Defaults to md5-hex.
@@ -310,21 +317,21 @@ class Auth {
         case 'crypt':
         case 'crypt-des':
             if ($seed) {
-                return substr(preg_replace('|^{crypt}|', '', $seed), 0, 2);
+                return substr(preg_replace('|^{crypt}|i', '', $seed), 0, 2);
             } else {
                 return substr(md5(mt_rand()), 0, 2);
             }
 
         case 'crypt-md5':
             if ($seed) {
-                return substr(preg_replace('|^{crypt}|', '', $seed), 0, 12);
+                return substr(preg_replace('|^{crypt}|i', '', $seed), 0, 12);
             } else {
                 return '$1$' . substr(md5(mt_rand()), 0, 8) . '$';
             }
 
         case 'crypt-blowfish':
             if ($seed) {
-                return substr(preg_replace('|^{crypt}|', '', $seed), 0, 16);
+                return substr(preg_replace('|^{crypt}|i', '', $seed), 0, 16);
             } else {
                 return '$2$' . substr(md5(mt_rand()), 0, 12) . '$';
             }
@@ -344,10 +351,7 @@ class Auth {
             }
 
         case 'aprmd5':
-            /**
-             * 64 characters that are valid for APRMD5 passwords.
-             * @var string $_APRMD5
-             */
+            /* 64 characters that are valid for APRMD5 passwords. */
             $APRMD5 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
 
             if ($seed) {
@@ -371,9 +375,9 @@ class Auth {
      */
     function genRandomPassword()
     {
-        $vowels     ='aeiouy';
-        $constants  ='bcdfghjklmnpqrstvwxz';
-        $numbers    ='0123456789';
+        $vowels    = 'aeiouy';
+        $constants = 'bcdfghjklmnpqrstvwxz';
+        $numbers   = '0123456789';
 
         /* Alternate constant and vowel random chars with two random numbers
          * at the end. This should produce a fairly pronounceable password. */
@@ -391,7 +395,6 @@ class Auth {
     /**
      * Adds a set of authentication credentials.
      *
-     * @access public
      * @abstract
      *
      * @param string $userId      The userId to add.
@@ -407,7 +410,6 @@ class Auth {
     /**
      * Updates a set of authentication credentials.
      *
-     * @access public
      * @abstract
      *
      * @param string $oldID        The old userId.
@@ -424,7 +426,6 @@ class Auth {
     /**
      * Deletes a set of authentication credentials.
      *
-     * @access public
      * @abstract
      *
      * @param string $userId  The userId to delete.
@@ -461,7 +462,6 @@ class Auth {
     /**
      * Lists all users in the system.
      *
-     * @access public
      * @abstract
      *
      * @return mixed  The array of userIds, or a PEAR_Error object on failure.
@@ -474,7 +474,6 @@ class Auth {
     /**
      * Checks if $userId exists in the system.
      *
-     * @access public
      * @abstract
      *
      * @return boolean  Whether or not $userId already exists.
@@ -488,7 +487,6 @@ class Auth {
      * Automatic authentication: Finds out if the client matches an allowed IP
      * block.
      *
-     * @access public
      * @abstract
      *
      * @return boolean  Whether or not the client is allowed.
@@ -503,8 +501,6 @@ class Auth {
      * specified user. If there isn't, but the configured Auth driver supports
      * transparent authentication, then we try that.
      *
-     * @access public
-     *
      * @param string $realm  The authentication realm to check.
      *
      * @return boolean  Whether or not the user is authenticated.
@@ -540,8 +536,6 @@ class Auth {
     /**
      * Returns the currently logged in user, if there is one.
      *
-     * @access public
-     *
      * @return mixed  The userId of the current user, or false if no user is
      *                logged in.
      */
@@ -557,12 +551,26 @@ class Auth {
         return false;
     }
 
+    /**
+     * Return whether the authentication backend requested a password change.
+     *
+     * @return boolean Whether the backend requested a password change.
+     */
+    function isPasswordChangeRequested()
+    {
+        if (isset($_SESSION['__auth']) &&
+            !empty($_SESSION['__auth']['authenticated']) &&
+            !empty($_SESSION['__auth']['changeRequested'])) {
+            return true;
+        }
+
+        return false;
+    }
+
     /**
      * Returns the curently logged-in user without any domain information
      * (e.g., bob@example.com would be returned as 'bob').
      *
-     * @access public
-     *
      * @return mixed  The user ID of the current user, or false if no user
      *                is logged in.
      */
@@ -579,12 +587,30 @@ class Auth {
         return $user;
     }
 
+    /**
+     * Returns the domain of currently logged-in user (e.g., bob@example.com
+     * would be returned as 'example.com').
+     *
+     * @since Horde 3.0.6
+     *
+     * @return mixed  The domain suffix of the current user, or false.
+     */
+    function getAuthDomain()
+    {
+        if ($user = Auth::getAuth()) {
+            $pos = strpos($user, '@');
+            if ($pos !== false) {
+                return substr($user, $pos + 1);
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Returns the requested credential for the currently logged in user, if
      * present.
      *
-     * @access public
-     *
      * @param string $credential  The credential to retrieve.
      *
      * @return mixed  The requested credential, or false if no user is
@@ -611,8 +637,6 @@ class Auth {
     /**
      * Sets the requested credential for the currently logged in user.
      *
-     * @access public
-     *
      * @param string $credential  The credential to set.
      * @param string $value       The value to set the credential to.
      */
@@ -638,13 +662,13 @@ class Auth {
      * If a user name hook was defined in the configuration, it gets applied
      * to the userId at this point.
      *
-     * @access public
-     *
-     * @param string $userId      The userId who has been authorized.
-     * @param array $credentials  The credentials of the user.
-     * @param string $realm       The authentication realm to use.
+     * @param string $userId            The userId who has been authorized.
+     * @param array $credentials        The credentials of the user.
+     * @param string $realm             The authentication realm to use.
+     * @param boolean $changeRequested  Whether to request that the user change
+     *                                  their password.
      */
-    function setAuth($userId, $credentials, $realm = null)
+    function setAuth($userId, $credentials, $realm = null, $changeRequested = false)
     {
         $userId = trim($userId);
         $userId = Auth::addHook($userId);
@@ -673,10 +697,12 @@ class Auth {
             'timestamp' => time(),
             'remote_addr' => isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : null,
             'browser' => $GLOBALS['browser']->getAgentString(),
+            'changeRequested' => $changeRequested
         );
 
         /* Reload preferences for the new user. */
         $GLOBALS['registry']->loadPrefs();
+        NLS::setLang($GLOBALS['prefs']->getValue('language'));
 
         /* Fetch the user's last login time. */
         $old_login = @unserialize($GLOBALS['prefs']->getValue('last_login'));
@@ -696,16 +722,49 @@ class Auth {
         }
 
         // Set the user's last_login information.
+        $host = empty($_SERVER['HTTP_X_FORWARDED_FOR'])
+            ? $_SERVER['REMOTE_ADDR']
+            : $_SERVER['HTTP_X_FORWARDED_FOR'];
         $last_login = array('time' => time(),
-                            'host' => @gethostbyaddr($_SERVER['REMOTE_ADDR']));
+                            'host' => @gethostbyaddr($host));
         $GLOBALS['prefs']->setValue('last_login', serialize($last_login));
+
+        if ($changeRequested) {
+            $GLOBALS['notification']->push(_("Your password has expired."),
+                                           'horde.message');
+            if ($this->hasCapability('update')) {
+                /* A bit of a kludge.  URL is set from the login screen, but
+                 * we aren't completely certain we got here from the login
+                 * screen.  So any screen which calls setAuth() which has a
+                 * url will end up going there.  Should be OK. */
+                $url_param = Util::getFormData('url');
+
+                if ($url_param) {
+                    $url = Horde::url(Util::removeParameter($url_param,
+                                                            session_name()),
+                                      true);
+                    $return_to = $GLOBALS['registry']->get('webroot', 'horde') .
+                                 '/index.php';
+                    $return_to = Util::addParameter($return_to, 'url', $url);
+                } else {
+                    $return_to = Horde::url($GLOBALS['registry']->get('webroot', 'horde')
+                                 . '/index.php');
+                }
+
+                $url = Horde::applicationUrl('services/changepassword.php');
+                $url = Util::addParameter($url,
+                                          array('return_to' => $return_to),
+                                          null, false);
+
+                header('Location: ' . $url);
+                exit;
+            }
+        }
     }
 
     /**
      * Clears any authentication tokens in the current session.
      *
-     * @access public
-     *
      * @param string $realm  The authentication realm to clear.
      */
     function clearAuth($realm = null)
@@ -727,8 +786,6 @@ class Auth {
     /**
      * Is the current user an administrator?
      *
-     * @access public
-     *
      * @param string $permission  Allow users with this permission admin access
      *                            in the current context.
      * @param integer $permlevel  The level of permissions to check for
@@ -737,8 +794,6 @@ class Auth {
      * @param string $user        The user to check. Defaults to Auth::getAuth().
      *
      * @return boolean  Whether or not this is an admin user.
-     *
-     * @since Horde 2.2
      */
     function isAdmin($permission = null, $permlevel = null, $user = null)
     {
@@ -818,8 +873,6 @@ class Auth {
      * Queries the current Auth object to find out if it supports the given
      * capability.
      *
-     * @access public
-     *
      * @param string $capability  The capability to test for.
      *
      * @return boolean  Whether or not the capability is supported.
@@ -833,8 +886,6 @@ class Auth {
      * Returns the URI of the login screen for the current authentication
      * method.
      *
-     * @access public
-     *
      * @param string $app  The application to use.
      * @param string $url  The URL to redirect to after login.
      *
@@ -850,8 +901,6 @@ class Auth {
     /**
      * Returns the named parameter for the current auth driver.
      *
-     * @access public
-     *
      * @param string $param  The parameter to fetch.
      *
      * @return string  The parameter's value.
@@ -864,8 +913,6 @@ class Auth {
     /**
      * Returns the name of the authentication provider.
      *
-     * @access public
-     *
      * @param string $driver  Used by recursive calls when untangling composite
      *                        auth.
      * @param array  $params  Used by recursive calls when untangling composite
@@ -903,8 +950,6 @@ class Auth {
     /**
      * Returns the logout reason.
      *
-     * @access public
-     *
      * @return string One of the logout reasons (see the AUTH_LOGOUT_*
      *                constants for the valid reasons).  Returns null if there
      *                is no logout reason present.
@@ -921,8 +966,6 @@ class Auth {
     /**
      * Returns the status string to use for logout messages.
      *
-     * @access public
-     *
      * @return string  The logout reason string.
      */
     function getLogoutReasonString()
@@ -941,7 +984,7 @@ class Auth {
             break;
 
         case AUTH_REASON_LOGOUT:
-            $text = _("You have been logged out.") . '<br />' . _("Thank you for using the system.");
+            $text = _("You have been logged out.");
             break;
 
         case AUTH_REASON_FAILED:
@@ -952,6 +995,10 @@ class Auth {
             $text = _("Login failed because your username or password was entered incorrectly.");
             break;
 
+        case AUTH_REASON_EXPIRED:
+            $text = _("Your login has expired.");
+            break;
+
         case AUTH_REASON_MESSAGE:
             if (isset($GLOBALS['__autherror']['msg'])) {
                 $text = $GLOBALS['__autherror']['msg'];
@@ -974,8 +1021,6 @@ class Auth {
      * If no reason/msg is passed in, use the current global authentication
      * error message.
      *
-     * @access public
-     *
      * @param string $url     The URL to redirect to.
      * @param string $reason  The reason for logout.
      * @param string $msg     If reason is AUTH_REASON_MESSAGE, the message to
@@ -1022,7 +1067,6 @@ class Auth {
     /**
      * Authentication stub.
      *
-     * @access protected
      * @abstract
      *
      * @return boolean  False.
@@ -1048,6 +1092,18 @@ class Auth {
         $GLOBALS['__autherror']['msg'] = $msg;
     }
 
+    /**
+     * Returns the error type for an invalid authentication or false on error.
+     */
+    function _getAuthError()
+    {
+        if (isset($GLOBALS['__autherror']['type'])) {
+            return $GLOBALS['__autherror']['type'];
+        }
+
+        return false;
+    }
+
     /**
      * Returns the appropriate authentication driver, if any, selecting by the
      * specified parameter.
@@ -1094,7 +1150,8 @@ class Auth {
      */
     function _checkBrowserString()
     {
-        return ($_SESSION['__auth']['browser'] == $GLOBALS['browser']->getAgentString());
+        return (empty($GLOBALS['conf']['auth']['checkbrowser']) ||
+                $_SESSION['__auth']['browser'] == $GLOBALS['browser']->getAgentString());
     }
 
     /**
@@ -1109,10 +1166,7 @@ class Auth {
      */
     function _toAPRMD5($value, $count)
     {
-        /**
-         * 64 characters that are valid for APRMD5 passwords.
-         * @var string $_APRMD5
-         */
+        /* 64 characters that are valid for APRMD5 passwords. */
         $APRMD5 = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
 
         $aprmd5 = '';
@@ -1138,7 +1192,8 @@ class Auth {
         $bin = '';
         $length = strlen($hex);
         for ($i = 0; $i < $length; $i += 2) {
-            $bin .= chr(array_shift(sscanf(substr($hex, $i, 2), '%x')));
+            $tmp = sscanf(substr($hex, $i, 2), '%x');
+            $bin .= chr(array_shift($tmp));
         }
         return $bin;
     }
@@ -1146,8 +1201,6 @@ class Auth {
     /**
      * Attempts to return a concrete Auth instance based on $driver.
      *
-     * @access public
-     *
      * @param mixed $driver  The type of concrete Auth subclass to return. This
      *                       is based on the storage driver ($driver). The code
      *                       is dynamically included. If $driver is an array,
@@ -1169,7 +1222,8 @@ class Auth {
         $driver = basename($driver);
 
         if (empty($driver) || ($driver == 'none')) {
-            return $auth = &new Auth();
+            $auth = &new Auth();
+            return $auth;
         }
 
         if (is_null($params)) {
@@ -1190,10 +1244,12 @@ class Auth {
 
         $class = 'Auth_' . $driver;
         if (class_exists($class)) {
-            return $auth = &new $class($params);
+            $auth = &new $class($params);
         } else {
-            return PEAR::raiseError('Auth Driver (' . $class . ') not found' . ($include_error ? ': ' . $include_error : '') . '.');
+            $auth = PEAR::raiseError('Auth Driver (' . $class . ') not found' . ($include_error ? ': ' . $include_error : '') . '.');
         }
+
+        return $auth;
     }
 
     /**
@@ -1206,8 +1262,6 @@ class Auth {
      *
      * This method must be invoked as: $var = &Auth::singleton()
      *
-     * @access public
-     *
      * @param string $driver  The type of concrete Auth subclass to return.
      *                        This is based on the storage driver ($driver).
      *                        The code is dynamically included.
