@@ -9,15 +9,14 @@ require_once 'Horde/Form/Renderer.php';
  * new users sign themselves up into the horde installation, depending
  * on how the admin has configured Horde.
  *
- * $Horde: framework/Auth/Auth/Signup.php,v 1.38.2.2 2005/01/19 15:05:03 jan Exp $
+ * $Horde: framework/Auth/Auth/Signup.php,v 1.38.2.10 2006/01/01 21:28:07 jan Exp $
  *
- * Copyright 2002-2005 Marko Djukic <marko@oblo.com>
+ * Copyright 2002-2006 Marko Djukic <marko@oblo.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Marko Djukic <marko@oblo.com>
- * @version $Revision: 1.38.2.2 $
  * @since   Horde 3.0
  * @package Horde_Auth
  */
@@ -26,7 +25,7 @@ class Auth_Signup {
     /**
      * Pointer to a DataTree instance to manage/store signups
      *
-     * @var object DataTree $_datatree
+     * @var DataTree
      */
     var $_datatree;
 
@@ -50,8 +49,7 @@ class Auth_Signup {
      *
      * This method must be invoked as: $var = &Auth_Signup::singleton()
      *
-     * @return object Signup  The concrete Signup reference, or false on an
-     *                        error.
+     * @return Signup  The concrete Signup reference, or false on error.
      */
     function &singleton()
     {
@@ -65,10 +63,8 @@ class Auth_Signup {
     }
 
     /**
-     * Adds a new user to the system and handles any extra fields
-     * that may have been compiled, relying on the hooks.php file.
-     *
-     * @access public
+     * Adds a new user to the system and handles any extra fields that may have
+     * been compiled, relying on the hooks.php file.
      *
      * @return mixed  PEAR_Error if any errors, otherwise true.
      */
@@ -105,10 +101,7 @@ class Auth_Signup {
     }
 
     /**
-     * Queues the user's submitted registration info for later admin
-     * approval.
-     *
-     * @access public
+     * Queues the user's submitted registration info for later admin approval.
      *
      * @return mixed  PEAR_Error if any errors, otherwise true.
      */
@@ -128,7 +121,7 @@ class Auth_Signup {
         // Check to see if the username already exists.
         if ($auth->exists($info['user_name']) ||
             $this->_datatree->exists($info['user_name'])) {
-            return PEAR::raiseError(sprintf(_("Username '%s' already exists."), $info['user_name']));
+            return PEAR::raiseError(sprintf(_("Username \"%s\" already exists."), $info['user_name']));
         }
 
         // If it's a unique username, go ahead and queue the request.
@@ -156,11 +149,10 @@ class Auth_Signup {
     /**
      * Get a user's queued signup information.
      *
-     * @access public
-     *
      * @param string $username  The username to retrieve the queued info for.
-     * @return object DataTreeObject_Signup  The DataTreeObject for the
-     *                                       requested signup.
+     *
+     * @return DataTreeObject_Signup  The DataTreeObject for the requested
+     *                                signup.
      */
     function getQueuedSignup($username)
     {
@@ -170,8 +162,6 @@ class Auth_Signup {
     /**
      * Get the queued information for all pending signups.
      *
-     * @access public
-     *
      * @return array  An array of DataTreeObject_Signup objects, one for
      *                each signup in the queue.
      */
@@ -189,8 +179,6 @@ class Auth_Signup {
     /**
      * Remove a queued signup.
      *
-     * @access public
-     *
      * @param string $username  The user to remove from the signup queue.
      */
     function removeQueuedSignup($username)
@@ -201,16 +189,17 @@ class Auth_Signup {
     /**
      * Return a new signup object.
      *
-     * @param string $name The signups's name.
+     * @param string $name  The signups's name.
      *
-     * @return object DataTreeObject_Signup A new signup object.
+     * @return DataTreeObject_Signup  A new signup object.
      */
     function &newSignup($name)
     {
         if (empty($name)) {
             return PEAR::raiseError('Signup names must be non-empty');
         }
-        return $ret = &new DataTreeObject_Signup($name);
+        $signup = &new DataTreeObject_Signup($name);
+        return $signup;
     }
 
 }
@@ -222,7 +211,6 @@ class Auth_Signup {
  * instead of extending DataTreeObject directly.
  *
  * @author  Marko Djukic <marko@oblo.com>
- * @version $Revision: 1.38.2.2 $
  * @since   Horde 3.0
  * @package Horde_Auth
  */
@@ -231,7 +219,8 @@ class DataTreeObject_Signup extends DataTreeObject {
     /**
      * We want to see queued signups in descending order of receipt.
      * Insert new signups at position 0 and push the rest down.
-     * @var integer $order
+     *
+     * @var integer
      */
     var $order = 0;
 
@@ -240,7 +229,7 @@ class DataTreeObject_Signup extends DataTreeObject {
      * the parent constructor so that the signup's is is set
      * properly.
      *
-     * @param string $id The id of the signup.
+     * @param string $id  The id of the signup.
      */
     function DataTreeObject_Signup($id)
     {
@@ -255,13 +244,12 @@ class DataTreeObject_Signup extends DataTreeObject {
 /**
  * Horde Signup Form, extending of Horde_Form::
  *
- * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2006 Marko Djukic <marko@oblo.com>
  *
  * See the enclosed file COPYING for license information (GPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Marko Djukic <marko@oblo.com>
- * @version $Revision: 1.38.2.2 $
  * @since   Horde 3.0
  * @package Horde_Auth
  */
