@@ -1,57 +1,56 @@
 <?php
 
 require_once 'Horde/Auth/sql.php';
+require_once 'Horde/IMAP/Admin.php';
 
 /**
  * The Auth_cyrsql class provides a SQL implementation of the Horde
- * authentication system for the Cyrus IMAP server. Most of the
- * functionality is the same as for the SQL class; only what is
- * different overrides the parent class implementations.
+ * authentication system for the Cyrus IMAP server. Most of the functionality
+ * is the same as for the SQL class; only what is different overrides the
+ * parent class implementations.
  *
- * Required parameters:
- * ====================
- *   'cyradmin'  --  The username of the cyrus administrator.
- *   'cyrpass'   --  The password for the cyrus administrator.
- *   'database'  --  The name of the database.
- *   'hostspec'  --  The hostname of the database server.
- *   'imap_dsn'  --  The full IMAP DSN
- *                   (i.e. {localhost:993/imap/ssl/novalidate-cert}).
- *   'password'  --  The password associated with 'username'.
- *   'phptype'   --  The database type (ie. 'pgsql', 'mysql, etc.).
- *   'protocol'  --  The communication protocol ('tcp', 'unix', etc.).
- *   'username'  --  The username with which to connect to the database.
+ * Required parameters:<pre>
+ *   'cyradmin'  The username of the cyrus administrator.
+ *   'cyrpass'   The password for the cyrus administrator.
+ *   'imap_dsn'  The full IMAP DSN
+ *               (i.e. {localhost:993/imap/ssl/novalidate-cert}).
+ *   'phptype'   The database type (ie. 'pgsql', 'mysql', etc.).</pre>
  *
- * Optional parameters:
- * ====================
- *   'domain_field'    --  If set to anything other than 'none' this is used as
- *                         field name where domain is stored.
- *                         DEFAULT: 'domain_name'
- *   'encryption'      --  The encryption to use to store the password in the
- *                         table (e.g. plain, crypt, md5-hex, md5-base64, smd5,
- *                         sha, ssha).
- *                         DEFAULT: 'md5-hex'
- *   'folders'         --  An array of folders to create under username.
- *                         DEFAULT: NONE
- *   'password_field'  --  The name of the password field in the auth table.
- *                         DEFAULT: 'password'
- *   'quota'           --  The quota (in kilobytes) to grant on the mailbox.
- *                         DEFAULT: NONE
- *   'table'           --  The name of the auth table in 'database'.
- *                         DEFAULT: 'accountuser'
- *   'unixhier'        --  The value of imapd.conf's unixhierarchysep setting.
- *                         Set this to true if the value is true in imapd.conf.
- *   'username_field'  --  The name of the username field in the auth table.
- *                         DEFAULT: 'username'
+ * Optional parameters:<pre>
+ *   'domain_field'    If set to anything other than 'none' this is used as
+ *                     field name where domain is stored.
+ *                     DEFAULT: 'domain_name'
+ *   'encryption'      The encryption to use to store the password in the
+ *                     table (e.g. plain, crypt, md5-hex, md5-base64, smd5,
+ *                     sha, ssha).
+ *                     DEFAULT: 'md5-hex'
+ *   'folders'         An array of folders to create under username.
+ *                     DEFAULT: NONE
+ *   'password_field'  The name of the password field in the auth table.
+ *                     DEFAULT: 'password'
+ *   'quota'           The quota (in kilobytes) to grant on the mailbox.
+ *                     DEFAULT: NONE
+ *   'table'           The name of the auth table in 'database'.
+ *                     DEFAULT: 'accountuser'
+ *   'unixhier'        The value of imapd.conf's unixhierarchysep setting.
+ *                     Set this to true if the value is true in imapd.conf.
+ *   'username_field'  The name of the username field in the auth table.
+ *                     DEFAULT: 'username'</pre>
  *
- * Required by some database implementations:
- * ==========================================
- *   'options'  --  Additional options to pass to the database.
- *   'port'     --  The port on which to connect to the database.
- *   'tty'      --  The TTY on which to connect to the database.
+ * Required by some database implementations:<pre>
+ *   'database'  The name of the database.
+ *   'hostspec'  The hostname of the database server.
+ *   'protocol'  The communication protocol ('tcp', 'unix', etc.).
+ *   'username'  The username with which to connect to the database.
+ *   'password'  The password associated with 'username'.
+ *   'options'   Additional options to pass to the database.
+ *   'port'      The port on which to connect to the database.
+ *   'tty'       The TTY on which to connect to the database.</pre>
  *
  *
  * The table structure for the auth system is as follows:
  *
+ * <pre>
  * CREATE TABLE accountuser (
  *     username    VARCHAR(255) BINARY NOT NULL DEFAULT '',
  *     password    VARCHAR(32) BINARY NOT NULL DEFAULT '',
@@ -120,17 +119,18 @@ require_once 'Horde/Auth/sql.php';
  *     pid         VARCHAR(255) NOT NULL DEFAULT '',
  *     PRIMARY KEY (id)
  * );
+ * </pre>
  *
+ * $Horde: framework/Auth/Auth/cyrsql.php,v 1.33.10.13 2006/01/01 21:28:07 jan Exp $
  *
- * $Horde: framework/Auth/Auth/cyrsql.php,v 1.33.10.2 2005/01/19 15:05:03 jan Exp $
- *
- * Copyright 2002-2005 Ilya <mail@krel.org>
+ * Copyright 2002-2006 Ilya <mail@krel.org>
+ * Copyright 2005-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Ilya <mail@krel.org>
- * @version $Revision: 1.33.10.2 $
+ * @author  Jan Schneider <jan@horde.org>
  * @since   Horde 3.0
  * @package Horde_Auth
  */
@@ -139,23 +139,28 @@ class Auth_cyrsql extends Auth_sql {
     /**
      * Handle for the current IMAP connection.
      *
-     * @var resource $_imapStream
+     * @var resource
      */
     var $_imapStream;
 
+    /**
+     * Handle for an IMAP_Admin object.
+     *
+     * @var IMAP_Admin
+     */
+    var $_admin;
+
     /**
      * Hierarchy separator to use (e.g., is it user/mailbox or user.mailbox)
      *
-     * @var string $_separator
+     * @var string
      */
     var $_separator = '.';
 
     /**
      * Constructor.
      *
-     * @access public
-     *
-     * @param optional array $params  A hash containing connection parameters.
+     * @param array $params  A hash containing connection parameters.
      */
     function Auth_cyrsql($params = array())
     {
@@ -163,13 +168,19 @@ class Auth_cyrsql extends Auth_sql {
             Horde::fatal(_("Auth_cyrsql: Required imap extension not found."), __FILE__, __LINE__);
         }
         parent::Auth_sql($params);
+
+        $admin_params = array('admin_user' => $params['cyradmin'],
+                              'admin_password' => $params['cyrpass'],
+                              'dsn' => $params['imap_dsn']);
+        if (!empty($this->_params['unixhier'])) {
+            $admin_params['userhierarchy'] = 'user/';
+        }
+        $this->_admin = &new IMAP_Admin($admin_params);
     }
 
     /**
      * Add a set of authentication credentials.
      *
-     * @access public
-     *
      * @param string $userId       The userId to add.
      * @param array  $credentials  The credentials to add.
      *
@@ -183,23 +194,27 @@ class Auth_cyrsql extends Auth_sql {
             ($this->_params['domain_field'] != 'none')) {
             list($name, $domain) = explode('@', $userId);
             /* Build the SQL query. */
-            $query = sprintf('INSERT INTO %s (%s, %s, %s) VALUES (%s, %s, %s)',
+            $query = sprintf('INSERT INTO %s (%s, %s, %s) VALUES (?, ?, ?)',
                              $this->_params['table'],
                              $this->_params['username_field'],
                              $this->_params['domain_field'],
-                             $this->_params['password_field'],
-                             $this->_db->quote($name),
-                             $this->_db->quote($domain),
-                             $this->_db->quote($this->getCryptedPassword($credentials['password'],
-                                                                         '',
-                                                                         $this->_params['encryption'],
-                                                                         $this->_params['show_encryption'])));
-            $dbresult = $this->_db->query($query);
-            $query = sprintf('INSERT INTO virtual (alias, dest, username, status) VALUES (%s, %s, %s, 1)',
-                             $this->_db->quote($userId),
-                             $this->_db->quote($name),
-                             $this->_db->quote($name));
-            $dbresult2 = $this->_db->query($query);
+                             $this->_params['password_field']);
+            $values = array($name,
+                            $domain,
+                            $this->getCryptedPassword($credentials['password'],
+                                                      '',
+                                                      $this->_params['encryption'],
+                                                      $this->_params['show_encryption']));
+
+            Horde::logMessage('SQL Query by Auth_cyrsql::addUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+            $dbresult = $this->_db->query($query, $values);
+            $query = 'INSERT INTO virtual (alias, dest, username, status) VALUES (?, ?, ?, 1)';
+            $values = array($userId, $name, $name);
+
+            Horde::logMessage('SQL Query by Auth_cyrsql::addUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+            $dbresult2 = $this->_db->query($query, $values);
             if (is_a($dbresult2, 'PEAR_Error')) {
                 return $dbresult2;
             }
@@ -211,16 +226,13 @@ class Auth_cyrsql extends Auth_sql {
             return $dbresult;
         }
 
-        $name = imap_utf7_encode($name);
-        if (@imap_createmailbox($this->_imapStream,
-                                imap_utf7_encode($this->_params['imap_dsn'] .
-                                                 'user' . $this->_separator . $name))) {
+        if (!is_a($result = $this->_admin->addMailbox($name), 'PEAR_Error')) {
             @array_walk($this->_params['folders'],
                         array($this, '_createSubFolders'), $name);
         } else {
-            Horde::logMessage('IMAP mailbox creation for ' . $name . ' failed ',
+            Horde::logMessage('IMAP mailbox creation for ' . $name . ' failed: ' . $result->getMessage(),
                               __FILE__, __LINE__, PEAR_LOG_ERR);
-            return PEAR::raiseError(sprintf(_("IMAP mailbox creation failed: %s"), imap_last_error()));
+            return PEAR::raiseError(sprintf(_("IMAP mailbox creation failed: %s"), $result->getMessage()));
         }
 
         if (isset($this->_params['quota']) && $this->_params['quota'] >= 0) {
@@ -237,8 +249,6 @@ class Auth_cyrsql extends Auth_sql {
     /**
      * Delete a set of authentication credentials.
      *
-     * @access public
-     *
      * @param string $userId  The userId to delete.
      *
      * @return boolean        Success or failure.
@@ -251,16 +261,21 @@ class Auth_cyrsql extends Auth_sql {
             ($this->_params['domain_field'] != 'none')) {
             list($name, $domain) = explode('@', $userId);
             /* Build the SQL query. */
-            $query = sprintf('DELETE FROM %s WHERE %s = %s and %s = %s',
+            $query = sprintf('DELETE FROM %s WHERE %s = ? and %s = ?',
                              $this->_params['table'],
                              $this->_params['username_field'],
-                             $this->_db->quote($name),
-                             $this->_params['domain_field'],
-                             $this->_db->quote($domain));
-            $dbresult = $this->_db->query($query);
-            $query = sprintf('DELETE FROM virtual WHERE dest = %s',
-                             $this->_db->quote($name));
-            $dbresult2 = $this->_db->query($query);
+                             $this->_params['domain_field']);
+            $values = array($name, $domain);
+
+            Horde::logMessage('SQL Query by Auth_cyrsql::removeUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+            $dbresult = $this->_db->query($query, $values);
+            $query = 'DELETE FROM virtual WHERE dest = ?';
+            $values = array($name);
+
+            Horde::logMessage('SQL Query by Auth_cyrsql::removeUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+            $dbresult2 = $this->_db->query($query, $values);
             if (is_a($dbresult2, 'PEAR_Error')) {
                 return $dbresult2;
             }
@@ -273,19 +288,11 @@ class Auth_cyrsql extends Auth_sql {
             return $dbresult;
         }
 
-        /* Set ACL for mailbox deletion. */
-        list($admin) = explode('@', $this->_params['cyradmin']);
-        @imap_setacl($this->_imapStream,
-                     'user' . $this->_separator . $name,
-                     $admin, 'lrswipcda');
-
         /* Delete IMAP mailbox. */
-        $imapresult = @imap_deletemailbox($this->_imapStream,
-                                          $this->_params['imap_dsn'] .
-                                          'user' . $this->_separator . $name);
+        $imapresult = $this->_admin->removeMailbox($name);
 
-        if (!$imapresult) {
-            return PEAR::raiseError(sprintf(_("IMAP mailbox deletion failed: %s"), imap_last_error()));
+        if (is_a($imapresult, 'PEAR_Error')) {
+            return PEAR::raiseError(sprintf(_("IMAP mailbox deletion failed: %s"), $imapresult->getMessage()));
         }
 
         return $this->removeUserData($userId);
@@ -303,8 +310,8 @@ class Auth_cyrsql extends Auth_sql {
         if (!$this->_connected) {
             parent::_connect();
 
-            // Reset the $_connected flag; we haven't yet successfully
-            // opened everything.
+            // Reset the $_connected flag; we haven't yet successfully opened
+            // everything.
             $this->_connected = false;
 
             $this->_imapStream = @imap_open($this->_params['imap_dsn'], $this->_params['cyradmin'], $this->_params['cyrpass'], OP_HALFOPEN);
@@ -322,8 +329,7 @@ class Auth_cyrsql extends Auth_sql {
     }
 
     /**
-     * Disconnect from the SQL and IMAP servers and clean up the
-     * connections.
+     * Disconnect from the SQL and IMAP servers and clean up the connections.
      *
      * @access private
      *
@@ -340,32 +346,22 @@ class Auth_cyrsql extends Auth_sql {
     }
 
     /**
-     * Creates all mailboxes supllied in configuration
+     * Creates all mailboxes supplied in configuration
      *
      * @access private
      */
     function _createSubFolders($value, $key, $userName)
     {
         if (!empty($this->_params['domain_field']) &&
-            ($this->_params['domain_field'] != 'none')){
-            list($name, $domain) = explode('@', $userName);
-            @imap_createmailbox($this->_imapStream,
-                           	    imap_utf7_encode($this->_params['imap_dsn'] .
-                                                 'user' . $this->_separator . $name .
-                                                 $this->_separator . $value . '@' . $domain));
-        } else {
-            @imap_createmailbox($this->_imapStream,
-                                imap_utf7_encode($this->_params['imap_dsn'] .
-                                                 'user' . $this->_separator . $userName .
-                                                 $this->_separator . $value));
+            ($this->_params['domain_field'] != 'none')) {
+            list($userName) = explode('@', $userName);
         }
+        $this->_admin->addMailbox($userName . $this->_separator . $value);
     }
 
     /**
      * List all users in the system.
      *
-     * @access public
-     *
      * @return mixed  The array of userIds, or false on failure/unsupported.
      */
     function listUsers()
@@ -374,9 +370,9 @@ class Auth_cyrsql extends Auth_sql {
         $this->_connect();
 
         if (!empty($this->_params['domain_field']) &&
-            ($this->_params['domain_field'] != 'none')){
+            ($this->_params['domain_field'] != 'none')) {
             /* Build the SQL query with domain. */
-            $query = sprintf('SELECT %s , %s FROM %s ORDER BY %s',
+            $query = sprintf('SELECT %s, %s FROM %s ORDER BY %s',
                              $this->_params['username_field'],
                              $this->_params['domain_field'],
                              $this->_params['table'],
@@ -389,6 +385,8 @@ class Auth_cyrsql extends Auth_sql {
                              $this->_params['username_field']);
         }
 
+        Horde::logMessage('SQL Query by Auth_cyrsql::listUsers(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
         $result = $this->_db->getAll($query, null, DB_FETCHMODE_ORDERED);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
@@ -397,7 +395,7 @@ class Auth_cyrsql extends Auth_sql {
         /* Loop through and build return array. */
         $users = array();
         if (!empty($this->_params['domain_field']) &&
-            ($this->_params['domain_field'] != 'none')){
+            ($this->_params['domain_field'] != 'none')) {
             foreach ($result as $ar) {
                 $users[] = $ar[0] . '@' . $ar[1];
             }
@@ -413,8 +411,6 @@ class Auth_cyrsql extends Auth_sql {
     /**
      * Update a set of authentication credentials.
      *
-     * @access public
-     *
      * @param string $oldID       The old userId.
      * @param string $newID       The new userId.
      * @param array $credentials  The new credentials
@@ -430,31 +426,32 @@ class Auth_cyrsql extends Auth_sql {
             ($this->_params['domain_field'] != 'none')) {
             list($name, $domain) = explode('@', $oldID);
             /* Build the SQL query with domain. */
-            $query = sprintf('UPDATE %s SET %s = %s WHERE %s = %s and %s = %s',
+            $query = sprintf('UPDATE %s SET %s = ? WHERE %s = ? and %s = ?',
                              $this->_params['table'],
                              $this->_params['password_field'],
-                             $this->_db->quote($this->getCryptedPassword($credentials['password'],
-                                                                         '',
-                                                                         $this->_params['encryption'],
-                                                                         $this->_params['show_encryption'])),
                              $this->_params['username_field'],
-                             $this->_db->quote($name),
-                             $this->_params['domain_field'],
-                             $this->_db->quote($domain));
+                             $this->_params['domain_field']);
+            $values = array($this->getCryptedPassword($credentials['password'],
+                                                      '',
+                                                      $this->_params['encryption'],
+                                                      $this->_params['show_encryption']),
+                            $name, $domain);
         } else {
             /* Build the SQL query. */
-            $query = sprintf('UPDATE %s SET %s = %s WHERE %s = %s',
+            $query = sprintf('UPDATE %s SET %s = ? WHERE %s = ?',
                              $this->_params['table'],
                              $this->_params['password_field'],
-                             $this->_db->quote($this->getCryptedPassword($credentials['password'],
-                                                                         '',
-                                                                         $this->_params['encryption'],
-                                                                         $this->_params['show_encryption'])),
-                             $this->_params['username_field'],
-                             $this->_db->quote($oldID));
+                             $this->_params['username_field']);
+            $values = array($this->getCryptedPassword($credentials['password'],
+                                                      '',
+                                                      $this->_params['encryption'],
+                                                      $this->_params['show_encryption']),
+                            $oldID);
         }
 
-        return $this->_db->query($query);
+        Horde::logMessage('SQL Query by Auth_cyrsql::updateUser(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+        return $this->_db->query($query, $values);
     }
 
 }
