@@ -1,19 +1,21 @@
 <?php
+
+require_once 'Horde/Block.php';
+
 /**
  * The Horde_Block_Collection:: class provides an API to the blocks
  * (applets) framework.
  *
- * $Horde: framework/Block/Block/Collection.php,v 1.36.4.2 2005/01/03 12:18:53 jan Exp $
+ * $Horde: framework/Block/Block/Collection.php,v 1.36.4.17 2006/03/08 17:53:52 chuck Exp $
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2003-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
  * @author  Jan Schneider <jan@horde.org>
- * @version $Revision: 1.36.4.2 $
  * @since   Horde 3.0
  * @package Horde_Block
  */
@@ -22,7 +24,7 @@ class Horde_Block_Collection {
     /**
      * What kind of blocks are we collecting? Defaults to any.
      *
-     * @var string $_type
+     * @var string
      */
     var $_type = 'portal';
 
@@ -30,23 +32,25 @@ class Horde_Block_Collection {
      * A hash storing the information about all available blocks from
      * all applications.
      *
-     * @var array $_blocks
+     * @var array
      */
     var $_blocks = array();
 
     /**
      * Constructor.
      *
-     * @param string $type  (optional) The kind of blocks to list.
+     * @param string $type  The kind of blocks to list.
+     * @param array $apps   The applications whose blocks to list.
      */
-    function Horde_Block_Collection($type = null)
+    function Horde_Block_Collection($type = null, $apps = array())
     {
         if (!is_null($type)) {
             $this->_type = $type;
         }
 
-        if (isset($_SESSION['horde']['blocks'])) {
-            $this->_blocks = &$_SESSION['horde']['blocks'];
+        $signature = serialize($apps);
+        if (isset($_SESSION['horde']['blocks'][$signature])) {
+            $this->_blocks = &$_SESSION['horde']['blocks'][$signature];
             return;
         }
 
@@ -54,6 +58,9 @@ class Horde_Block_Collection {
         require_once 'Horde/Block.php';
 
         foreach ($registry->listApps() as $app) {
+            if (count($apps) && !in_array($app, $apps)) {
+                continue;
+            }
             if (is_a($result = $registry->pushApp($app), 'PEAR_Error')) {
                 continue;
             }
@@ -67,7 +74,8 @@ class Horde_Block_Collection {
                         if (is_readable($blockdir . '/' . $file)) {
                             include_once $blockdir . '/' . $file;
                         }
-                        if (!is_null($block_type) && !is_null($this->_type) && $block_type != $this->_type) {
+                        if (!is_null($block_type) && !is_null($this->_type) &&
+                            $block_type != $this->_type) {
                             continue;
                         }
                         if (!empty($block_name)) {
@@ -86,7 +94,7 @@ class Horde_Block_Collection {
                                 return strcasecmp($registry->get("name", $a), $registry->get("name", $b));')
                );
 
-        $_SESSION['horde']['blocks'] = &$this->_blocks;
+        $_SESSION['horde']['blocks'][$signature] = &$this->_blocks;
     }
 
     /**
@@ -94,20 +102,21 @@ class Horde_Block_Collection {
      *
      * @static
      *
-     * @param string $type  (optional) The kind of blocks to list.
+     * @param string $type  The kind of blocks to list.
+     * @param array $apps   The applications whose blocks to list.
      *
-     * @return object Horde_Blocks  The Horde_Blocks intance.
+     * @return Horde_Block_Collection  The Horde_Block_Collection instance.
      */
-    function &singleton($type = null)
+    function &singleton($type = null, $apps = array())
     {
         static $instances = array();
 
-        // Use array_key_exists since the default key is NULL.
-        if (!array_key_exists($type, $instances)) {
-            $instances[$type] = new Horde_Block_Collection($type);
+        $signature = serialize(array($type, $apps));
+        if (!isset($instances[$signature])) {
+            $instances[$signature] = new Horde_Block_Collection($type, $apps);
         }
 
-        return $instances[$type];
+        return $instances[$signature];
     }
 
     function &getBlock($app, $name, $params = null)
@@ -120,10 +129,12 @@ class Horde_Block_Collection {
             include_once $path;
         }
         if (!class_exists($class)) {
-            return PEAR::raiseError(sprintf(_("%s not found."), $class));
+            $error = PEAR::raiseError(sprintf(_("%s not found."), $class));
+            return $error;
         }
 
-        return $block = &new $class($params);
+        $block = &new $class($params);
+        return $block;
     }
 
     /**
@@ -151,6 +162,31 @@ class Horde_Block_Collection {
         return $blocks;
     }
 
+    /**
+     * Returns a layout hash with all fixed blocks as per configuration.
+     *
+     * @return array  A hash representing a default layout with all fixed
+     *                blocks.
+     */
+    function getFixedBlocks()
+    {
+        global $conf;
+
+        $layout = array();
+        if (isset($conf['portal']['fixed_blocks'])) {
+            foreach ($conf['portal']['fixed_blocks'] as $block) {
+                list($app, $type) = explode(':', $block, 2);
+                $layout[] = array(array('app' => $app,
+                                        'params' => array('type' => $type,
+                                                          'params' => false),
+                                        'height' => 1,
+                                        'width' => 1));
+            }
+        }
+
+        return $layout;
+    }
+
     /**
      * Returns a select widget with all available blocks.
      *
@@ -220,11 +256,15 @@ class Horde_Block_Collection {
 
         $this->getParams($app, $block);
         $param = $this->_blocks[$app][$block]['params'][$param_id];
+        if (!isset($param['default'])) {
+            $param['default'] = '';
+        }
+
         switch ($param['type']) {
         case 'boolean':
         case 'checkbox':
             $checked = !empty($val[$param_id]) ? ' checked="checked"' : '';
-            $widget = sprintf('<input type="checkbox" name="params[%s]"%s>', $param_id, $checked);
+            $widget = sprintf('<input type="checkbox" name="params[%s]"%s />', $param_id, $checked);
             break;
 
         case 'enum':
@@ -262,7 +302,8 @@ class Horde_Block_Collection {
             if (is_array($val) && isset($val['__' . $param_id])) {
                 $firstval = $val['__' . $param_id];
             } else {
-                $firstval = current(array_keys($param['values']));
+                $tmp = array_keys($param['values']);
+                $firstval = current($tmp);
             }
             $blockvalues = $param['values'][$firstval];
             asort($blockvalues);
@@ -275,7 +316,7 @@ class Horde_Block_Collection {
                                    $key == $firstval ? ' selected="selected"' : '',
                                    htmlspecialchars($name));
             }
-            $widget .= "</select><br/>\n";
+            $widget .= "</select><br />\n";
 
             $widget .= sprintf("<select name=\"params[%s]\">\n", $param_id);
             foreach ($blockvalues as $key => $name) {
@@ -285,7 +326,7 @@ class Horde_Block_Collection {
                                    $val[$param_id] == $key ? ' selected="selected"' : '',
                                    htmlspecialchars($name));
             }
-            $widget .= "</select><br/>\n";
+            $widget .= "</select><br />\n";
             break;
 
         case 'color':
@@ -301,11 +342,11 @@ class Horde_Block_Collection {
 
         case 'int':
         case 'text':
-            $widget = sprintf('<input type="text" name="params[%s]" value="%s" />', $param_id, empty($val) ? $param['default'] : $val[$param_id]);
+            $widget = sprintf('<input type="text" name="params[%s]" value="%s" />', $param_id, !isset($val[$param_id]) ? $param['default'] : $val[$param_id]);
             break;
 
         case 'error':
-            $widget = '<span class="form-error">' . $val[$param_id] . '</span>';
+            $widget = '<span class="form-error">' . $param['default'] . '</span>';
             break;
         }
 
