@@ -6,10 +6,10 @@ require_once 'Horde/Block/Collection.php';
 /**
  * The Horde_Block_Layout_View class represents the user defined portal layout.
  *
- * $Horde: framework/Block/Block/Layout/View.php,v 1.4.2.5 2007/04/23 15:39:22 jan Exp $
+ * $Horde: framework/Block/Block/Layout/View.php,v 1.4.2.2 2006/03/08 17:54:08 chuck Exp $
  *
- * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2003-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -64,12 +64,14 @@ class Horde_Block_Layout_View {
         $tplDir = $GLOBALS['registry']->get('templates', 'horde');
         $interval = $GLOBALS['prefs']->getValue('summary_refresh_time');
 
-        $html = '<table class="nopadding" cellspacing="6" width="100%">';
+        ob_start();
+
+        echo '<table class="nopadding" cellspacing="6" width="100%">';
 
         $covered = array();
         foreach ($this->_layout as $row_num => $row) {
             $width = floor(100 / count($row));
-            $html .= '<tr>';
+            echo '<tr>';
             foreach ($row as $col_num => $item) {
                 if (isset($covered[$row_num]) && isset($covered[$row_num][$col_num])) {
                     continue;
@@ -79,6 +81,7 @@ class Horde_Block_Layout_View {
                     $block = &Horde_Block_Collection::getBlock($item['app'], $item['params']['type'], $item['params']['params']);
                     $rowspan = $item['height'];
                     $colspan = $item['width'];
+                    $width *= $colspan;
                     for ($i = 0; $i < $item['height']; $i++) {
                         if (!isset($covered[$row_num + $i])) {
                             $covered[$row_num + $i] = array();
@@ -90,25 +93,23 @@ class Horde_Block_Layout_View {
                     if (is_a($block, 'PEAR_Error')) {
                         $header = _("Error");
                         $content = $block->getMessage();
-                        ob_start();
                         include $tplDir . '/portal/block.inc';
-                        $html .= ob_get_clean();
                     } elseif (is_a($block, 'Horde_Block')) {
                         $header = $block->getTitle();
                         $content = $block->getContent();
-                        ob_start();
                         include $tplDir . '/portal/block.inc';
-                        $html .= ob_get_clean();
                     } else {
-                        $html .= '<td width="' . ($width * $colspan) . '%">&nbsp;</td>';
+                        echo '<td width="' . $width . '%">&nbsp;</td>';
                     }
                 } else {
-                    $html .= '<td width="' . ($width * $colspan) . '%">&nbsp;</td>';
+                    echo '<td width="' . $width . '%">&nbsp;</td>';
                 }
             }
-            $html .= '</tr>';
+            echo '</tr>';
         }
-        $html .= '</table>';
+        echo '</table>';
+
+        $html = ob_get_clean();
 
         // Strip any CSS <link> tags out of the returned content so
         // they can be handled seperately.
