@@ -1,27 +1,30 @@
 <?php
 /**
- * The Cipher_rc2:: class implements the Cipher interface enryption data
+ * The Cipher_rc2:: class implements the Cipher interface encryption data
  * using the RC2 algorithm as described in RFC2268.
  *
  * Based on the notes by Peter Gutmann <pgut01@cs.auckland.ac.nz>
  * http://www.mirrors.wiretapped.net/security/cryptography/
  *   algorithms/rc2/comments/gutman-960211
  *
- * $Horde: framework/Cipher/Cipher/rc2.php,v 1.9.12.1 2005/01/03 12:18:55 jan Exp $
+ * $Horde: framework/Cipher/Cipher/rc2.php,v 1.9.12.7 2006/01/01 21:28:10 jan Exp $
  *
- * Copyright 2002-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2002-2006 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.9.12.1 $
  * @since   Horde 2.2
  * @package Horde_Cipher
  */
 class Horde_Cipher_rc2 extends Horde_Cipher {
 
-    /* Permutations array */
+    /**
+     * Permutations array
+     *
+     * @var array
+     */
     var $_perm = array(
         0xD9, 0x78, 0xF9, 0xC4, 0x19, 0xDD, 0xB5, 0xED,
         0x28, 0xE9, 0xFD, 0x79, 0x4A, 0xA0, 0xD8, 0x9D,
@@ -57,18 +60,17 @@ class Horde_Cipher_rc2 extends Horde_Cipher {
         0x0A, 0xA6, 0x20, 0x68, 0xFE, 0x7F, 0xC1, 0xAD
     );
 
-    /* Array to hold the key shecdule */
+    /**
+     * Array to hold the key schedule
+     *
+     * @var array
+     */
     var $_keySchedule = array();
 
-    /* Constructor */
-    function Horde_Cipher_rc2($params = null)
-    {
-    }
-
     /**
-     * Set the key to be used for en/decryption
+     * Set the key to be used for en/decryption.
      *
-     * @param String $key   The key to use
+     * @param string $key  The key to use.
      */
     function setKey($key)
     {
@@ -96,13 +98,12 @@ class Horde_Cipher_rc2 extends Horde_Cipher {
         for ($i = 63; $i >= 0; $i--) {
             $this->_keySchedule[$i] = ($key[$i * 2 + 1] << 8 | $key[$i * 2]) & 0xFFFF;
         }
-
     }
 
     /**
-     * Return the size of the blocks that this cipher needs
+     * Return the size of the blocks that this cipher needs.
      *
-     * @return Integer  The number of characters per block
+     * @return integer  The number of characters per block.
      */
     function getBlockSize()
     {
@@ -110,12 +111,12 @@ class Horde_Cipher_rc2 extends Horde_Cipher {
     }
 
     /**
-     * Encrypt a block on data.
+     * Encrypt a block of data.
      *
-     * @param String $block         The data to encrypt
-     * @param optional String $key  The key to use
+     * @param string $block  The data to encrypt.
+     * @param string $key    The key to use.
      *
-     * @return String the encrypted output
+     * @return string  The encrypted output.
      */
     function encryptBlock($block, $key = null)
     {
@@ -156,12 +157,12 @@ class Horde_Cipher_rc2 extends Horde_Cipher {
     }
 
     /**
-     * Decrypt a block on data.
+     * Decrypt a block of data.
      *
-     * @param String $block         The data to decrypt
-     * @param optional String $key  The key to use
+     * @param string $block  The data to decrypt.
+     * @param string $key    The key to use.
      *
-     * @return String the decrypted output
+     * @return string  The decrypted output.
      */
     function decryptBlock($block, $key = null)
     {
@@ -196,7 +197,7 @@ class Horde_Cipher_rc2 extends Horde_Cipher {
             }
         }
 
-        $decrypted = pack("v*", $cipher[1], $cipher[2], $cipher[3], $cipher[4]);
-        return $decrypted;
+        return pack("v*", $cipher[1], $cipher[2], $cipher[3], $cipher[4]);
     }
+
 }
