@@ -2,10 +2,10 @@
 /**
  * The Horde_Compress_gzip class allows gzip files to be read.
  *
- * $Horde: framework/Compress/Compress/gzip.php,v 1.7.12.10 2007/01/02 13:54:11 jan Exp $
+ * $Horde: framework/Compress/Compress/gzip.php,v 1.7.12.8 2006/01/29 10:14:36 chuck Exp $
  *
- * Copyright 2002-2007 Michael Cochrane <mike@graftonhall.co.nz>
- * Copyright 2003-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2006 Michael Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -47,10 +47,7 @@ class Horde_Compress_gzip extends Horde_Compress {
 
         /* Gzipped File - decompress it first. */
         $position = 0;
-        $info = @unpack('CCM/CFLG/VTime/CXFL/COS', substr($data, $position + 2));
-        if (!$info) {
-            return PEAR::raiseError(_("Unable to decompress data."));
-        }
+        $info = unpack('CCM/CFLG/VTime/CXFL/COS', substr($data, $position + 2));
         $position += 10;
 
         if ($info['FLG'] & $this->_flags['FEXTRA']) {
