@@ -1,14 +1,12 @@
 <?php
 
 /**
- * For decompress(), return a list of files/information about the
- * zipfile.
+ * For decompress(), return a list of files/information about the zipfile.
  */
 define('HORDE_COMPRESS_ZIP_LIST', 1);
 
 /**
- * For decompress(), return the data for an individual file in the
- * zipfile.
+ * For decompress(), return the data for an individual file in the zipfile.
  */
 define('HORDE_COMPRESS_ZIP_DATA', 2);
 
@@ -16,7 +14,7 @@ define('HORDE_COMPRESS_ZIP_DATA', 2);
  * The Horde_Compress_zip class allows ZIP files to be created and
  * read.
  *
- * $Horde: framework/Compress/Compress/zip.php,v 1.11.12.1 2005/01/03 12:18:56 jan Exp $
+ * $Horde: framework/Compress/Compress/zip.php,v 1.11.12.14 2006/01/02 18:34:04 jan Exp $
  *
  * The ZIP compression code is partially based on code from:
  *   Eric Mueller <eric@themepark.com>
@@ -28,9 +26,9 @@ define('HORDE_COMPRESS_ZIP_DATA', 2);
  * The ZIP compression date code is partially based on code from
  *   Peter Listiak <mlady@users.sourceforge.net>
  *
- * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2002-2005 Michael Cochrane <mike@graftonhall.co.nz>
- * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2000-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Michael Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -38,7 +36,6 @@ define('HORDE_COMPRESS_ZIP_DATA', 2);
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Michael Cochrane <mike@graftonhall.co.nz>
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.11.12.1 $
  * @since   Horde 3.0
  * @package Horde_Compress
  */
@@ -47,7 +44,7 @@ class Horde_Compress_zip extends Horde_Compress {
     /**
      * ZIP compression methods.
      *
-     * @var array $_methods
+     * @var array
      */
     var $_methods = array(
         0x0 => 'None',
@@ -63,30 +60,28 @@ class Horde_Compress_zip extends Horde_Compress {
     /**
      * Beginning of central directory record.
      *
-     * @var string $_ctrlDirHeader
+     * @var string
      */
     var $_ctrlDirHeader = "\x50\x4b\x01\x02";
 
     /**
      * End of central directory record.
      *
-     * @var string $_ctrlDirEnd
+     * @var string
      */
     var $_ctrlDirEnd = "\x50\x4b\x05\x06\x00\x00\x00\x00";
 
     /**
      * Beginning of file contents.
      *
-     * @var string $_fileHeader
+     * @var string
      */
     var $_fileHeader = "\x50\x4b\x03\x04";
 
     /**
      * Create a ZIP compressed file from an array of file data.
      *
-     * @access public
-     *
-     * @param array $data             The data to compress.
+     * @param array $data    The data to compress.
      * <pre>
      * Requires an array of arrays - each subarray should contain the
      * following fields:
@@ -94,11 +89,11 @@ class Horde_Compress_zip extends Horde_Compress {
      * 'name' (string)   --  The pathname to the file.
      * 'time' (integer)  --  [optional] The timestamp to use for the file.
      * </pre>
-     * @param optional array $params  The parameter array (Unused).
+     * @param array $params  The parameter array (Unused).
      *
      * @return string  The ZIP file.
      */
-    function &compress($data, $params = array())
+    function compress($data, $params = array())
     {
         $contents = $ctrldir = array();
 
@@ -112,8 +107,6 @@ class Horde_Compress_zip extends Horde_Compress {
     /**
      * Decompress a ZIP file and get information from it.
      *
-     * @access public
-     *
      * @param string $data   The zipfile data.
      * @param array $params  The parameter array.
      * <pre>
@@ -129,9 +122,9 @@ class Horde_Compress_zip extends Horde_Compress {
      *
      * @return mixed  The requested data.
      */
-    function &decompress($data, $params)
+    function decompress($data, $params)
     {
-        if (array_key_exists('action', $params)) {
+        if (isset($params['action'])) {
             if ($params['action'] == HORDE_COMPRESS_ZIP_LIST) {
                 return $this->_getZipInfo($data);
             } elseif ($params['action'] == HORDE_COMPRESS_ZIP_DATA) {
@@ -140,9 +133,9 @@ class Horde_Compress_zip extends Horde_Compress {
             } else {
                 return PEAR::raiseError(_("Incorrect action code given."), 'horde.error');
             }
-        } else {
-            return PEAR::raiseError(_("You must specify what action to perform."), 'horde.error');
         }
+
+        return PEAR::raiseError(_("You must specify what action to perform."), 'horde.error');
     }
 
     /**
@@ -154,6 +147,7 @@ class Horde_Compress_zip extends Horde_Compress {
      *
      * @return array  KEY: Position in zipfile
      *                VALUES: 'attr'    --  File attributes
+     *                        'crc'     --  CRC checksum
      *                        'csize'   --  Compressed file size
      *                        'date'    --  File modification time
      *                        'name'    --  Filename
@@ -161,63 +155,68 @@ class Horde_Compress_zip extends Horde_Compress {
      *                        'size'    --  Original file size
      *                        'type'    --  File type
      */
-    function &_getZipInfo(&$data)
+    function _getZipInfo(&$data)
     {
         $entries = array();
 
-        /* Get details from local file header. */
-        $fhStart = strpos($data, $this->_fileHeader);
-
-        do {
-            $file = array();
-
-            $info = unpack('vMethod/VTime/VCRC32/VCompressed/VUncompressed/vLength/vExtraLength', substr($data, $fhStart + 8, 25));
-
-            $file['name'] = substr($data, $fhStart + 30, $info['Length']);
-            $file['csize'] = $info['Compressed'];
-            $file['size'] = $info['Uncompressed'];
-            $file['method'] = $this->_methods[$info['Method']];
-            $file['crc'] = dechex($info['CRC32']);
-
-            $modDate = $info['Time'];
-            $file['date'] = mktime((($modDate >> 11) & 0x1f),
-                                   (($modDate >> 5) & 0x3f),
-                                   (($modDate << 1) & 0x3e),
-                                   (($modDate >> 21) & 0x07),
-                                   (($modDate >> 16) & 0x1f),
-                                   ((($modDate >> 25) & 0x7f) + 80));
-
-            $file['_dataStart'] = $fhStart + 30 + $info['Length'] + $info['ExtraLength'];
-            $file['_method'] = $info['Method'];
-
-            /* Filenames are unique in zips so this can be done. */
-            $entries[$file['name']] = $file;
-        } while (($fhStart = strpos($data, $this->_fileHeader, $fhStart + 30 + $info['Length'])) !== false);
-
         /* Get details from Central directory structure. */
         $fhStart = strpos($data, $this->_ctrlDirHeader);
 
         do {
+            if (strlen($data) < $fhStart + 31) {
+                return PEAR::raiseError(_("Invalid ZIP data"));
+            }
             $info = unpack('vMethod/VTime/VCRC32/VCompressed/VUncompressed/vLength', substr($data, $fhStart + 10, 20));
             $name = substr($data, $fhStart + 46, $info['Length']);
-            $entries[$name]['csize'] = $info['Compressed'];
-            $entries[$name]['size'] = $info['Uncompressed'];
-            $entries[$name]['crc'] = dechex($info['CRC32']);
 
+            $entries[$name] = array(
+                'attr' => null,
+                'crc' => sprintf("%08s", dechex($info['CRC32'])),
+                'csize' => $info['Compressed'],
+                'date' => null,
+                '_dataStart' => null,
+                'name' => $name,
+                'method' => $this->_methods[$info['Method']],
+                '_method' => $info['Method'],
+                'size' => $info['Uncompressed'],
+                'type' => null
+            );
+
+            $entries[$name]['date'] =
+                mktime((($info['Time'] >> 11) & 0x1f),
+                       (($info['Time'] >> 5) & 0x3f),
+                       (($info['Time'] << 1) & 0x3e),
+                       (($info['Time'] >> 21) & 0x07),
+                       (($info['Time'] >> 16) & 0x1f),
+                       ((($info['Time'] >> 25) & 0x7f) + 1980));
+
+            if (strlen($data) < $fhStart + 43) {
+                return PEAR::raiseError(_("Invalid ZIP data"));
+            }
             $info = unpack('vInternal/VExternal', substr($data, $fhStart + 36, 6));
 
-            if (array_key_exists($name, $entries)) {
-                $entries[$name]['type']  = ($info['Internal'] & 0x01) ? 'text' : 'binary';
-                $entries[$name]['attr']  = '';
-                $entries[$name]['attr'] .= ($info['External'] & 0x10) ? 'D' : '-';
-                $entries[$name]['attr'] .= ($info['External'] & 0x20) ? 'A' : '-';
-                $entries[$name]['attr'] .= ($info['External'] & 0x03) ? 'S' : '-';
-                $entries[$name]['attr'] .= ($info['External'] & 0x02) ? 'H' : '-';
-                $entries[$name]['attr'] .= ($info['External'] & 0x01) ? 'R' : '-';
-            }
-
+            $entries[$name]['type'] = ($info['Internal'] & 0x01) ? 'text' : 'binary';
+            $entries[$name]['attr'] =
+                (($info['External'] & 0x10) ? 'D' : '-') .
+                (($info['External'] & 0x20) ? 'A' : '-') .
+                (($info['External'] & 0x03) ? 'S' : '-') .
+                (($info['External'] & 0x02) ? 'H' : '-') .
+                (($info['External'] & 0x01) ? 'R' : '-');
         } while (($fhStart = strpos($data, $this->_ctrlDirHeader, $fhStart + 46)) !== false);
 
+        /* Get details from local file header. */
+        $fhStart = strpos($data, $this->_fileHeader);
+
+        do {
+            if (strlen($data) < $fhStart + 34) {
+                return PEAR::raiseError(_("Invalid ZIP data"));
+            }
+            $info = unpack('vMethod/VTime/VCRC32/VCompressed/VUncompressed/vLength/vExtraLength', substr($data, $fhStart + 8, 25));
+            $name = substr($data, $fhStart + 30, $info['Length']);
+            $entries[$name]['_dataStart'] = $fhStart + 30 + $info['Length'] + $info['ExtraLength'];
+        } while (strlen($data) > $fhStart + 30 + $info['Length'] &&
+                 ($fhStart = strpos($data, $this->_fileHeader, $fhStart + 30 + $info['Length'])) !== false);
+
         return array_values($entries);
     }
 
@@ -232,7 +231,7 @@ class Horde_Compress_zip extends Horde_Compress {
      *
      * @return string  The file data.
      */
-    function &_getZipData(&$data, &$info, $key)
+    function _getZipData(&$data, &$info, $key)
     {
         if (($info[$key]['_method'] == 0x8) && Util::extensionExists('zlib')) {
             /* If the file has been deflated, and zlib is installed,
@@ -241,16 +240,14 @@ class Horde_Compress_zip extends Horde_Compress {
         } elseif ($info[$key]['_method'] == 0x0) {
             /* Files that aren't compressed. */
             return substr($data, $info[$key]['_dataStart'], $info[$key]['csize']);
-        } else {
-            return '';
         }
+
+        return '';
     }
 
     /**
      * Checks to see if the data is a valid ZIP file.
      *
-     * @access public
-     *
      * @param string &$data  The ZIP file data.
      *
      * @return boolean  True if valid, false if invalid.
@@ -271,7 +268,7 @@ class Horde_Compress_zip extends Horde_Compress {
      *
      * @access private
      *
-     * @param optional integer $unixtime  The current UNIX timestamp.
+     * @param integer $unixtime  The current UNIX timestamp.
      *
      * @return integer  The current date in a 4-byte DOS format.
      */
@@ -312,7 +309,7 @@ class Horde_Compress_zip extends Horde_Compress {
 
         /* See if time/date information has been provided. */
         $ftime = null;
-        if (array_key_exists('time', $file)) {
+        if (isset($file['time'])) {
             $ftime = $file['time'];
         }
 
@@ -390,24 +387,22 @@ class Horde_Compress_zip extends Horde_Compress {
      *
      * @return string  The ZIP file.
      */
-    function &_createZIPFile(&$contents, &$ctrlDir)
+    function _createZIPFile(&$contents, &$ctrlDir)
     {
         $data = implode('', $contents);
-        $dir  = implode('', $ctrlDir);
+        $dir = implode('', $ctrlDir);
 
-        return $data .
-               $dir .
-               $this->_ctrlDirEnd .
-               /* Total # of entries "on this disk". */
-               pack('v', count($ctrlDir)) .
-               /* Total # of entries overall. */
-               pack('v', count($ctrlDir)) .
-               /* Size of central directory. */
-               pack('V', strlen($dir)) .
-               /* Offset to start of central dir. */
-               pack('V', strlen($data)) .
-               /* ZIP file comment length. */
-               "\x00\x00";
+        return $data . $dir . $this->_ctrlDirEnd .
+            /* Total # of entries "on this disk". */
+            pack('v', count($ctrlDir)) .
+            /* Total # of entries overall. */
+            pack('v', count($ctrlDir)) .
+            /* Size of central directory. */
+            pack('V', strlen($dir)) .
+            /* Offset to start of central dir. */
+            pack('V', strlen($data)) .
+            /* ZIP file comment length. */
+            "\x00\x00";
     }
 
 }
