@@ -2,16 +2,15 @@
 /**
  * Horde_Data implementation for tab-separated data (TSV).
  *
- * $Horde: framework/Data/Data/tsv.php,v 1.23.10.1 2005/01/03 12:18:57 jan Exp $
+ * $Horde: framework/Data/Data/tsv.php,v 1.23.10.8 2006/01/01 21:28:13 jan Exp $
  *
- * Copyright 1999-2005 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jan Schneider <jan@horde.org>
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.23.10.1 $
  * @since   Horde 1.3
  * @package Horde_Data
  */
@@ -49,12 +48,11 @@ class Horde_Data_tsv extends Horde_Data {
      * Builds a TSV file from a given data structure and returns it as a
      * string.
      *
-     * @access public
-     * @param array $data        A two-dimensional array containing the data
-     *                           set.
-     * @param bool $header       If true, the rows of $data are associative
-     *                           arrays with field names as their keys.
-     * @return string            The TSV data.
+     * @param array $data      A two-dimensional array containing the data set.
+     * @param boolean $header  If true, the rows of $data are associative
+     *                         arrays with field names as their keys.
+     *
+     * @return string  The TSV data.
      */
     function exportData($data, $header = false)
     {
@@ -84,11 +82,11 @@ class Horde_Data_tsv extends Horde_Data {
      * It DOES NOT exit the current script but only outputs the correct headers
      * and data.
      *
-     * @access public
-     * @param string $filename   The name of the file to be downloaded.
-     * @param array $data        A two-dimensional array containing the data set.
-     * @param bool $header       If true, the rows of $data are associative arrays
-     *                           with field names as their keys.
+     * @param string $filename  The name of the file to be downloaded.
+     * @param array $data       A two-dimensional array containing the data
+     *                          set.
+     * @param boolean $header   If true, the rows of $data are associative
+     *                          arrays with field names as their keys.
      */
     function exportFile($filename, $data, $header = false)
     {
@@ -101,12 +99,9 @@ class Horde_Data_tsv extends Horde_Data {
      * Takes all necessary actions for the given import step, parameters and
      * form values and returns the next necessary step.
      *
-     * @access public
-     *
-     * @param integer $action        The current step. One of the IMPORT_*
-     *                               constants.
-     * @param optional array $param  An associative array containing needed
-     *                               parameters for the current step.
+     * @param integer $action  The current step. One of the IMPORT_* constants.
+     * @param array $param     An associative array containing needed
+     *                         parameters for the current step.
      *
      * @return mixed  Either the next step as an integer constant or imported
      *                data set after the final step.
@@ -147,7 +142,7 @@ class Horde_Data_tsv extends Horde_Data {
                                 $hash[$key] = stripslashes(preg_replace('/\\\\r/', "\n", $row[$key]));
                             }
                         }
-                    } elseif ($format = 'pine') {
+                    } elseif ($format == 'pine') {
                         if (count($row) < 3 || preg_match("/^#DELETED/", $row[0]) || preg_match("/[()]/", $row[2])) {
                             continue;
                         }
