@@ -16,28 +16,36 @@ define('DATATREE_FORMAT_FLAT', 2);
 define('DATATREE_ROOT', -1);
 
 /**
- * The DataTree:: class provides a common abstracted interface into
- * the various backends for the Horde DataTree system.
+ * Build a normal select query.
+ */
+define('DATATREE_BUILD_SELECT', 0);
+
+/**
+ * Build a count only query.
+ */
+define('DATATREE_BUILD_COUNT', 1);
+
+/**
+ * The DataTree:: class provides a common abstracted interface into the
+ * various backends for the Horde DataTree system.
  *
- * A piece of data is just a title that is saved in the page for the
- * null driver or can be saved in a database to be accessed from
- * everywhere. Every stored object must have a different name (inside
- * each groupid).
+ * A piece of data is just a title that is saved in the page for the null
+ * driver or can be saved in a database to be accessed from everywhere. Every
+ * stored object must have a different name (inside each groupid).
  *
- * Required values for $params:
- * group: define each group of objects we want to build.
+ * Required values for $params:<pre>
+ *   'group' -- Define each group of objects we want to build.</pre>
  *
- * $Horde: framework/DataTree/DataTree.php,v 1.151 2004/12/07 16:09:12 chuck Exp $
+ * $Horde: framework/DataTree/DataTree.php,v 1.151.2.16 2006/01/01 21:28:13 jan Exp $
  *
  * Copyright 1999-2003 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2004 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2001-2006 Chuck Hagenbuch <chuck@horde.org>
  *
- * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you did
+ * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Stephane Huther <shuther@bigfoot.com>
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.151 $
  * @since   Horde 2.1
  * @package Horde_DataTree
  */
@@ -47,7 +55,7 @@ class DataTree {
      * Array of all data: indexed by id. The format is: array(id =>
      * 'name' => name, 'parent' => parent).
      *
-     * @var array $_data
+     * @var array
      */
     var $_data = array();
 
@@ -55,14 +63,21 @@ class DataTree {
      * A hash that can be used to map a full object name
      * (parent:child:object) to that object's unique ID.
      *
-     * @var array $_nameMap
+     * @var array
      */
     var $_nameMap = array();
 
+     /**
+     * Actual attribute sorting hash.
+     *
+     * @var array
+     */
+    var $_sortHash = null;
+
     /**
      * Hash containing connection parameters.
      *
-     * @var array $_params
+     * @var array
      */
     var $_params = array();
 
@@ -125,6 +140,16 @@ class DataTree {
         return $id;
     }
 
+    /**
+     * Removes all DataTree objects owned by a certain user.
+     *
+     * @param string $user  A user name.
+     */
+    function removeUserData($user)
+    {
+        return PEAR::raiseError('not supported');
+    }
+
     /**
      * Move an object to a new parent.
      *
@@ -295,12 +320,14 @@ class DataTree {
     function &getObject($object, $class = 'DataTreeObject')
     {
         if (empty($object)) {
-            return PEAR::raiseError('No object requested.');
+            $error = PEAR::raiseError('No object requested.');
+            return $error;
         }
 
         $this->_load($object);
         if (!$this->exists($object)) {
-            return PEAR::raiseError($object . ' not found.');
+            $error = PEAR::raiseError($object . ' not found.');
+            return $error;
         }
 
         $use_attributes = is_null($class) || is_callable(array($class, '_fromAttributes'));
@@ -316,7 +343,8 @@ class DataTree {
         }
 
         if (!class_exists($class)) {
-            return PEAR::raiseError($class . ' not found.');
+            $error = PEAR::raiseError($class . ' not found.');
+            return $error;
         }
 
         $dataOb = &new $class($object);
@@ -348,7 +376,8 @@ class DataTree {
     function &getObjectById($id, $class = 'DataTreeObject')
     {
         if (empty($id)) {
-            return PEAR::raiseError('No id requested.');
+            $object = PEAR::raiseError('No id requested.');
+            return $object;
         }
 
         $result = $this->_loadById($id);
@@ -445,26 +474,38 @@ class DataTree {
     /**
      * Export a list of objects.
      *
-     * @param constant $format   Format of the export
-     * @param string $startleaf  The name of the leaf from which we start the
-     *                           export tree.
-     * @param boolean $reload    Re-load the requested chunk? Defaults to
-     *                           false (only what is currently loaded).
-     * @param string $rootname   The label to use for the root element.
-     *                           Defaults to DATATREE_ROOT.
-     * @param integer $maxdepth  The maximum number of levels to return.
-     *                           Defaults to DATATREE_ROOT, which is no limit.
-     * @param boolean $loadTree  Load a tree starting at $root, or just the
-     *                           requested level and direct parents?
-     *                           Defaults to single level.
+     * @param constant $format      Format of the export
+     * @param string   $startleaf   The name of the leaf from which we start
+     *                              the export tree.
+     * @param boolean  $reload      Re-load the requested chunk? Defaults to
+     *                              false (only what is currently loaded).
+     * @param string   $rootname    The label to use for the root element.
+     *                              Defaults to DATATREE_ROOT.
+     * @param integer  $maxdepth    The maximum number of levels to return.
+     *                              Defaults to DATATREE_ROOT, which is no
+     *                              limit.
+     * @param boolean  $loadTree    Load a tree starting at $root, or just the
+     *                              requested level and direct parents?
+     *                              Defaults to single level.
+     * @param string   $sortby_name Attribute name to use for sorting.
+     * @param string   $sortby_key  Attribute key to use for sorting.
+     * @param integer  $direction   Sort direction:
+     *                               0 - ascending
+     *                               1 - descending
      *
      * @return mixed  The tree representation of the objects, or a PEAR_Error
      *                on failure.
      */
     function get($format, $startleaf = DATATREE_ROOT, $reload = false,
-                 $rootname = DATATREE_ROOT, $maxdepth = -1, $loadTree = false)
+                 $rootname = DATATREE_ROOT, $maxdepth = -1, $loadTree = false,
+                 $sortby_name = null, $sortby_key = null, $direction = 0)
     {
-        $this->_load($startleaf, $loadTree, $reload);
+        /* Set sorting hash */
+        if (!is_null($sortby_name) || !is_null($sortby_name)) {
+            $this->_sortHash = DataTree::sortHash($startleaf, $sortby_name, $sortby_key, $direction);
+        }
+
+        $this->_load($startleaf, $loadTree, $reload, $sortby_name, $sortby_key, $direction);
         $out = array();
 
         switch ($format) {
@@ -491,6 +532,14 @@ class DataTree {
             return PEAR::raiseError('Not supported');
         }
 
+        if (!is_null($this->_sortHash)) {
+            /* Reset sorting hash. */
+            $this->_sortHash = null;
+            /* Reverse since the attribute sorting combined with tree up-ward
+             * sorting produces a reversed object order. */
+            $out = array_reverse($out, true);
+        }
+
         return $out;
     }
 
@@ -507,6 +556,27 @@ class DataTree {
         return $this->_count($startleaf);
     }
 
+    /**
+     * Create attribute sort hash
+     *
+     * @since Horde 3.1
+     *
+     * @param string  $root         The name of the leaf from which we start
+     *                              the export tree.
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                              0 - ascending
+     *                              1 - descending
+     *
+     * @return string The sort hash
+     */
+    function sortHash($root, $sortby_name = null, $sortby_key = null,
+                      $direction = 0)
+    {
+        return sprintf('%s-%s-%s-%s', $root, $sortby_name, $sortby_key, $direction);
+    }
+
     /**
      * Export a list of objects just like get() above, but uses an
      * object id to fetch the list of objects.
@@ -572,10 +642,8 @@ class DataTree {
     }
 
     /**
-     * Import a list of objects. Used by drivers to populate the
-     * internal $_data array.
-     *
-     * @access private
+     * Import a list of objects. Used by drivers to populate the internal
+     * $_data array.
      *
      * @param array   $data     The data to import.
      * @param string  $charset  The charset to convert the object name from.
@@ -584,18 +652,22 @@ class DataTree {
     {
         $cats = array();
         $cids = array();
-        foreach ($data as $cat) {
+        foreach ($data as $id => $cat) {
             if (!is_null($charset)) {
                 $cat[1] = String::convertCharset($cat[1], $charset);
             }
             $cids[$cat[0]] = $cat[1];
             $cparents[$cat[0]] = $cat[2];
             $corders[$cat[0]] = $cat[3];
+            $sorders[$cat[0]] = $id;
         }
 
         foreach ($cids as $id => $name) {
             $this->_data[$id]['name'] = $name;
             $this->_data[$id]['order'] = $corders[$id];
+            if (!is_null($this->_sortHash)) {
+                $this->_data[$id]['sorter'][$this->_sortHash] = $sorders[$id];
+            }
             if (!empty($cparents[$id])) {
                 $parents = explode(':', substr($cparents[$id], 1));
                 $par = $parents[count($parents) - 1];
@@ -630,7 +702,7 @@ class DataTree {
      * their order and then by name. This function is a way to get a
      * collection of $leaf's children.
      *
-     * @param optional string $leaf  Name of the parent from which to start.
+     * @param string $leaf  Name of the parent from which to start.
      *
      * @return array
      */
@@ -643,7 +715,12 @@ class DataTree {
             }
         }
 
-        uasort($out, array($this, '_cmp'));
+        if (is_null($this->_sortHash)) {
+            uasort($out, array($this, '_cmp'));
+        } else {
+            uasort($out, array($this, '_cmpSorted'));
+        }
+
         return $out;
     }
 
@@ -651,12 +728,10 @@ class DataTree {
      * Extract all levels of data, starting from a given parent
      * leaf in the datatree.
      *
-     * @param array $out                  This is an iterating function, so $out
-     *                                    is passed by reference to contain the
-     *                                    result.
-     * @param optional string  $parent    The name of the parent from which to
-     *                                    begin.
-     * @param optional integer $maxdepth  Max of levels of depth to check.
+     * @param array $out         This is an iterating function, so $out is
+     *                           passed by reference to contain the result.
+     * @param string  $parent    The name of the parent from which to begin.
+     * @param integer $maxdepth  Max of levels of depth to check.
      *
      * @note If nothing is returned that means there is no child, but
      * don't forget to add the parent if any subsequent operations are
@@ -686,12 +761,10 @@ class DataTree {
      *
      * Returned array format: array(parent => array(child => true))
      *
-     * @param array $out                  This is an iterating function, so $out
-     *                                    is passed by reference to contain the
-     *                                    result.
-     * @param optional string  $parent    The name of the parent from which to
-     *                                    begin.
-     * @param optional integer $maxdepth  Max number of levels of depth to check.
+     * @param array $out         This is an iterating function, so $out is
+     *                           passed by reference to contain the result.
+     * @param string  $parent    The name of the parent from which to begin.
+     * @param integer $maxdepth  Max number of levels of depth to check.
      */
     function _extractAllLevelList(&$out, $parent = DATATREE_ROOT, $maxdepth = -1)
     {
@@ -788,7 +861,7 @@ class DataTree {
      * $child.
      *
      * @param integer $childId  The id of the child.
-     * @param array   $parents  (optional) The array, as we build it up.
+     * @param array $parents    The array, as we build it up.
      *
      * @return array  A flat list of all of the parents of $child,
      *                hashed in $id => $name format.
@@ -836,9 +909,9 @@ class DataTree {
      * Get the number of children an object has, only counting immediate
      * children, not grandchildren, etc.
      *
-     * @param optional mixed $parent  Either the object or the
-     *                                name for which to count the
-     *                                children, defaults to the root (DATATREE_ROOT).
+     * @param mixed $parent  Either the object or the name for which to count
+     *                       the children, defaults to the root
+     *                       (DATATREE_ROOT).
      *
      * @return integer
      */
@@ -979,8 +1052,6 @@ class DataTree {
     /**
      * Replace all occurences of ':' in an object name with '.'.
      *
-     * @access public
-     *
      * @param string $name  The name of the object.
      *
      * @return string  The encoded name.
@@ -991,10 +1062,9 @@ class DataTree {
     }
 
     /**
-     * Get the short name of an object, returns only the last portion
-     * of the full name. For display purposes only.
+     * Get the short name of an object, returns only the last portion of the
+     * full name. For display purposes only.
      *
-     * @access public
      * @static
      *
      * @param string $name  The name of the object.
@@ -1003,13 +1073,37 @@ class DataTree {
      */
     function getShortName($name)
     {
-        /* If there are several components to the name, explode and
-         * get the last one, otherwise just return the name. */
+        /* If there are several components to the name, explode and get the
+         * last one, otherwise just return the name. */
         if (strpos($name, ':') !== false) {
-            return array_pop(explode(':', $name));
-        } else {
-            return $name;
+            $name = explode(':', $name);
+            $name = array_pop($name);
         }
+        return $name;
+    }
+
+    /**
+     * Returns a tree sorted by the specified attribute name and/or key.
+     *
+     * @abstract
+     *
+     * @since Horde 3.1
+     *
+     * @param string $root         Which portion of the tree to sort.
+     *                             Defaults to all of it.
+     * @param boolean $loadTree    Sort the tree starting at $root, or just the
+     *                             requested level and direct parents?
+     *                             Defaults to single level.
+     * @param string $sortby_name  Attribute name to use for sorting.
+     * @param string $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction   Sort direction:
+     *                             0 - ascending
+     *                             1 - descending
+     */
+    function getSortedTree($root, $loadTree = false, $sortby_name = null,
+                           $sortby_key = null, $direction = 0)
+    {
+        return PEAR::raiseError('not supported');
     }
 
     /**
@@ -1017,11 +1111,12 @@ class DataTree {
      *
      * @abstract
      *
-     * @param mixed $object     The object to add (string or DataTreeObject).
-     * @param bool $id_as_name  True or false to indicate if object ID is to be
-     *                          used as object name. Used in situations where
-     *                          there is no available unique input for object
-     *                          name. Defaults to false.
+     * @param mixed $object        The object to add (string or
+     *                             DataTreeObject).
+     * @param boolean $id_as_name  True or false to indicate if object ID is to
+     *                             be used as object name. Used in situations
+     *                             where there is no available unique input for
+     *                             object name.
      */
     function add($object, $id_as_name = false)
     {
@@ -1068,37 +1163,72 @@ class DataTree {
         return PEAR::raiseError('not supported');
     }
 
+    /**
+     * Returns the number of objects matching a set of attribute criteria.
+     *
+     * @abstract
+     *
+     * @see buildAttributeQuery()
+     *
+     * @param array   $criteria   The array of criteria.
+     * @param string  $parent     The parent node to start searching from.
+     * @param boolean $allLevels  Return all levels, or just the direct
+     *                            children of $parent? Defaults to all levels.
+     * @param string  $restrict   Only return attributes with the same
+     *                            attribute_name or attribute_id.
+     */
+    function countByAttributes($criteria, $parent = DATATREE_ROOT,
+                               $allLevels = true, $restrict = 'name')
+    {
+        return PEAR::raiseError('not supported');
+    }
+
     /**
      * Returns a set of object ids based on a set of attribute criteria.
      *
      * @abstract
      *
-     * @param array $criteria      The array of criteria. Example:
-     *                             $criteria['OR'] = array(
-     *                                 array('AND' => array(
-     *                                     array('field' => 'name',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'foo'),
-     *                                     array('field' => 'key',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'abc'))),
-     *                                 array('AND' => array(
-     *                                     array('field' => 'name',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'bar'),
-     *                                     array('field' => 'key',
-     *                                           'op'    => '=',
-     *                                           'test'  => 'xyz'))));
-     *                             This would fetch all object ids where
-     *                             attribute name is "foo" AND key is "abc", OR
-     *                             "bar" AND "xyz".
-     * @param string $parent       The parent node to start searching from.
-     * @param bool $allLevels      Return all levels, or just the direct
-     *                             children of $parent? Defaults to all levels.
-     * @param bool $restrictNames  Only return attributes with the same
-     *                             attribute_name.
+     * @see buildAttributeQuery()
+     *
+     * @param array   $criteria     The array of criteria.
+     * @param string  $parent       The parent node to start searching from.
+     * @param boolean $allLevels    Return all levels, or just the direct
+     *                              children of $parent? Defaults to all levels.
+     * @param string  $restrict     Only return attributes with the same
+     *                              attribute_name or attribute_id.
+     * @param integer $from         The object to start to fetching
+     * @param integer $count        The number of objects to fetch
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
      */
-    function getByAttributes($criteria, $parent = DATATREE_ROOT, $allLevels = true, $restrictNames = true)
+    function getByAttributes($criteria, $parent = DATATREE_ROOT,
+                             $allLevels = true, $restrict = 'name', $from = 0,
+                             $count = 0, $sortby_name = null,
+                             $sortby_key = null, $direction = 0)
+    {
+        return PEAR::raiseError('not supported');
+    }
+
+    /**
+     * Sorts IDs by attribute values. IDs without attributes will be added to
+     * the end of the sorted list.
+     *
+     * @abstract
+     *
+     * @param array $unordered_ids  Array of ids to sort.
+     * @param array $sortby_name    Attribute name to use for sorting.
+     * @param array $sortby_key     Attribute key to use for sorting.
+     * @param array $direction      Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
+     *
+     * @return array  Sorted ids.
+     */
+    function sortByAttributes($unordered_ids, $sortby_name = null,
+                              $sortby_key = null, $direction = 0)
     {
         return PEAR::raiseError('not supported');
     }
@@ -1141,6 +1271,30 @@ class DataTree {
         }
     }
 
+     /**
+     * Sorts two objects by their sorter hash field.
+     *
+     * You never call this function; it's used in uasort() calls. Do NOT use
+     * usort(); you'll lose key => value associations.
+     *
+     * @since Horde 3.1
+     *
+     * @param array $a  The first object
+     * @param array $b  The second object
+     *
+     * @return integer  1 if $a should be first,
+     *                 -1 if $b should be first,
+     *                  0 if they are entirely equal.
+     */
+    function _cmpSorted($a, $b)
+    {
+        if ($a['sorter'][$this->_sortHash] < $b['sorter'][$this->_sortHash]) {
+            return 1;
+        } else {
+            return 0;
+        }
+    }
+
     /**
      * Attempts to return a concrete DataTree instance based on $driver.
      *
@@ -1172,10 +1326,12 @@ class DataTree {
         include_once 'Horde/DataTree/' . $driver . '.php';
         $class = 'DataTree_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $dt = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $dt = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+ 
+        return $dt;
     }
 
     /**
@@ -1227,7 +1383,6 @@ class DataTree {
  *
  * @author  Stephane Huther <shuther@bigfoot.com>
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.151 $
  * @since   Horde 2.1
  * @package Horde_DataTree
  */
@@ -1236,7 +1391,7 @@ class DataTreeObject {
     /**
      * This object's DataTree instance.
      *
-     * @var DataTree $datatree
+     * @var DataTree
      */
     var $datatree;
 
@@ -1244,7 +1399,7 @@ class DataTreeObject {
      * Key-value hash that will be serialized.
      *
      * @see getData()
-     * @var array $data
+     * @var array
      */
     var $data = array();
 
@@ -1253,14 +1408,14 @@ class DataTreeObject {
      * These names have the same requirements as other object names - they must
      * be unique, etc.
      *
-     * @var string $name.
+     * @var string
      */
     var $name;
 
     /**
      * If this object has ordering data, store it here.
      *
-     * @var integer $order
+     * @var integer
      */
     var $order = null;
 
@@ -1352,7 +1507,7 @@ class DataTreeObject {
     /**
      * Sets the order of this object in its object collection.
      *
-     * @var integer $order
+     * @param integer $order
      */
     function setOrder($order)
     {
@@ -1405,10 +1560,8 @@ class DataTreeObject {
     }
 
     /**
-     * Saves any changes to this object to the backend
-     * permanently. New objects are added instead.
-     *
-     * @access public
+     * Saves any changes to this object to the backend permanently. New objects
+     * are added instead.
      *
      * @return boolean|PEAR_Error  PEAR_Error on failure.
      */
@@ -1424,8 +1577,6 @@ class DataTreeObject {
     /**
      * Delete this object from the backend permanently.
      *
-     * @access public
-     *
      * @return boolean|PEAR_Error  PEAR_Error on failure.
      */
     function delete()
