@@ -31,7 +31,7 @@ define('HORDE_DATE_MASK_ALLPARTS', 63);
  * Horde Date wrapper/logic class, including some calculation
  * functions.
  *
- * $Horde: framework/Date/Date.php,v 1.8.10.1 2005/01/07 03:48:27 chuck Exp $
+ * $Horde: framework/Date/Date.php,v 1.8.10.6 2005/12/31 21:50:04 chuck Exp $
  *
  * @package Horde_Date
  */
@@ -39,37 +39,43 @@ class Horde_Date {
 
     /**
      * Year
-     * @var integer $year
+     *
+     * @var integer
      */
     var $year;
 
     /**
      * Month
-     * @var integer $month
+     *
+     * @var integer
      */
     var $month;
 
     /**
      * Day
-     * @var integer $mday
+     *
+     * @var integer
      */
     var $mday;
 
     /**
      * Hour
-     * @var integer $hour
+     *
+     * @var integer
      */
     var $hour = 0;
 
     /**
      * Minute
-     * @var integer $min
+     *
+     * @var integer
      */
     var $min = 0;
 
     /**
      * Second
-     * @var integer $sec
+     *
+     * @var integer
      */
     var $sec = 0;
 
@@ -86,14 +92,25 @@ class Horde_Date {
                 }
             }
         } elseif (!is_null($date)) {
-            $parts = @getdate($date);
-            if ($parts) {
-                $this->year = $parts['year'];
-                $this->month = $parts['mon'];
-                $this->mday = $parts['mday'];
-                $this->hour = $parts['hours'];
-                $this->min = $parts['minutes'];
-                $this->sec = $parts['seconds'];
+            // Match YYYY-MM-DD HH:MM:SS and YYYYMMDDHHMMSS.
+            if (preg_match('/(\d{4})-?(\d{2})-?(\d{2}) ?(\d{2}):?(\d{2}):?(\d{2})/', $date, $parts)) {
+                $this->year = (int)$parts[1];
+                $this->month = (int)$parts[2];
+                $this->mday = (int)$parts[3];
+                $this->hour = (int)$parts[4];
+                $this->min = (int)$parts[5];
+                $this->sec = (int)$parts[6];
+            } else {
+                // Try as a timestamp.
+                $parts = @getdate($date);
+                if ($parts) {
+                    $this->year = $parts['year'];
+                    $this->month = $parts['mon'];
+                    $this->mday = $parts['mday'];
+                    $this->hour = $parts['hours'];
+                    $this->min = $parts['minutes'];
+                    $this->sec = $parts['seconds'];
+                }
             }
         }
     }
@@ -285,7 +302,7 @@ class Horde_Date {
     /**
      * Correct any over- or underflows in any of the date's members.
      *
-     * @param optional integer $mask  We may not want to correct some overflows.
+     * @param integer $mask  We may not want to correct some overflows.
      */
     function correct($mask = HORDE_DATE_MASK_ALLPARTS)
     {
@@ -461,15 +478,25 @@ class Horde_Date {
     }
 
     /**
-     * Format time in RFC-2822 format.
+     * Format time in RFC 2822 format.
      *
-     * @return string  Date and time in RFC-2822 format.
+     * @return string  Date and time in RFC 2822 format.
      */
     function rfc2822DateTime()
     {
         return date('D, j M Y H:i:s ', $this->timestamp()) . $this->tzOffset(false);
     }
 
+    /**
+     * Format time in RFC 3339 format.
+     *
+     * @return string  Date and time in RFC 3339 format.
+     */
+    function rfc3339DateTime()
+    {
+        return date('Y-m-d\TH:i', $this->timestamp());
+    }
+
     /**
      * Format time to standard 'ctime' format.
      *
@@ -480,4 +507,14 @@ class Horde_Date {
         return date('D M j H:i:s Y', $this->timestamp());
     }
 
+    /**
+     * Format date and time using strftime() format.
+     *
+     * @return string  strftime() formatted date and time.
+     */
+    function strftime($format)
+    {
+        return strftime($format, $this->timestamp());
+    }
+
 }
