@@ -6,10 +6,10 @@ require_once 'Horde/History.php';
 /**
  * The Group:: class provides the Horde groups system.
  *
- * $Horde: framework/Group/Group.php,v 1.67.2.1 2005/01/03 12:19:01 jan Exp $
+ * $Horde: framework/Group/Group.php,v 1.67.2.10 2006/01/01 21:28:18 jan Exp $
  *
- * Copyright 1999-2005 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Stephane Huther <shuther@bigfoot.com>
+ * Copyright 2001-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -23,7 +23,8 @@ class Group {
 
     /**
      * Pointer to a DataTree instance to manage the different groups.
-     * @var object DataTree $_datatree
+     *
+     * @var DataTree
      */
     var $_datatree;
 
@@ -47,15 +48,21 @@ class Group {
     /**
      * Return a new group object.
      *
-     * @param string $name The group's name.
+     * @param string $name    The group's name.
+     * @param string $parent  The group's parent's name.
      *
-     * @return object DataTreeObject_Group A new group object.
+     * @return DataTreeObject_Group  A new group object.
      */
-    function &newGroup($name)
+    function &newGroup($name, $parent = DATATREE_ROOT)
     {
         if (empty($name)) {
             return PEAR::raiseError(_("Group names must be non-empty"));
         }
+
+        if ($parent != DATATREE_ROOT) {
+            $name = $this->getGroupName($parent) . ':' . DataTree::encodeName($name);
+        }
+
         $group = &new DataTreeObject_Group($name);
         $group->setGroupOb($this);
         return $group;
@@ -91,7 +98,7 @@ class Group {
      * unique ID, with the users and other data retrieved
      * appropriately.
      *
-     * @param string $cid  The unique ID of the group to retrieve.
+     * @param integer $cid  The unique ID of the group to retrieve.
      */
     function &getGroupById($cid)
     {
@@ -105,7 +112,7 @@ class Group {
     /**
      * Get a globally unique ID for a group.
      *
-     * @param object DataTreeObject_Group $group  The group.
+     * @param DataTreeObject_Group $group  The group.
      *
      * @return string  A GUID referring to $group.
      */
@@ -119,7 +126,7 @@ class Group {
      * created with Group::newGroup(), and have any initial users
      * added to it, before this function is called.
      *
-     * @param object DataTreeObject_Group $group The new group object.
+     * @param DataTreeObject_Group $group  The new group object.
      */
     function addGroup($group)
     {
@@ -142,10 +149,9 @@ class Group {
     }
 
     /**
-     * Store updated data - users, etc. - of a group to the backend
-     * system.
+     * Store updated data - users, etc. - of a group to the backend system.
      *
-     * @param object DataTreeObject_Group $group   The group to update.
+     * @param DataTreeObject_Group $group  The group to update.
      */
     function updateGroup($group)
     {
@@ -174,8 +180,8 @@ class Group {
      * Change the name of a group without changing its contents or
      * where it is in the groups hierarchy.
      *
-     * @param object DataTreeObject_Group $group   The group to rename.
-     * @param string                      $newName The group's new name.
+     * @param DataTreeObject_Group $group  The group to rename.
+     * @param string $newName              The group's new name.
      */
     function renameGroup($group, $newName)
     {
@@ -196,10 +202,8 @@ class Group {
     /**
      * Remove a group from the groups system permanently.
      *
-     * @param object DataTreeObject_Group $group  The group to remove.
-     *
-     * @param optional boolean force [default = false] Force to remove
-     *                         every child
+     * @param DataTreeObject_Group $group  The group to remove.
+     * @param boolean $force               Force to remove every child.
      */
     function removeGroup($group, $force = false)
     {
@@ -216,23 +220,35 @@ class Group {
     /**
      * Retrieve the name of a group.
      *
-     * @param integer $groupId  The id of the group to retrieve the name for..
+     * @param integer $gid  The id of the group to retrieve the name for.
      *
      * @return string  The group's name.
      */
-    function getGroupName($groupId)
+    function getGroupName($gid)
     {
-        if (is_a($groupId, 'DataTreeObject_Group')) {
-            return $this->_datatree->getName($groupId->getId());
+        if (is_a($gid, 'DataTreeObject_Group')) {
+            return $this->_datatree->getName($gid->getId());
         } else {
-            return $this->_datatree->getName($groupId);
+            return $this->_datatree->getName($gid);
         }
     }
 
+    /**
+     * Strips all parent references off of the given group name.
+     *
+     * @param string $group  Name of the group.
+     *
+     * @return The name of the group without parents.
+     */
+    function getGroupShortName($group)
+    {
+        return $this->_datatree->getShortName($group);
+    }
+
     /**
      * Retrieve the ID of a group.
      *
-     * @param string $group  The group to retrieve the ID for..
+     * @param string $group  The group to retrieve the ID for.
      *
      * @return integer  The group's ID.
      */
@@ -248,9 +264,9 @@ class Group {
     /**
      * Check if a group exists in the system.
      *
-     * @param string $group           The group to check.
+     * @param string $group  The group to check.
      *
-     * @return boolean true if the group exists, false otherwise.
+     * @return boolean  True if the group exists, false otherwise.
      */
     function exists($group)
     {
@@ -260,25 +276,53 @@ class Group {
     /**
      * Get a list of the parents of a child group.
      *
-     * @param string $group The name of the child group.
+     * @param integer $gid The id of the child group.
      *
      * @return array
      */
-    function getGroupParents($group)
+    function getGroupParents($gid)
     {
-        return $this->_datatree->getParents($group);
+        return $this->_datatree->getParents($gid);
+    }
+
+    /**
+     * Return the single parent of the given group.
+     *
+     * @param integer $gid  The DataTree ID of the child group.
+     *
+     * @return the parent of the given group.
+     */
+    function getGroupParent($gid)
+    {
+        return $this->_datatree->getParentById($gid);
+    }
+
+    /**
+     * Get a list of parents all the way up to the root object for $group.
+     *
+     * @param integer $gid  The id of the group.
+     *
+     * @return array  A flat list of all of the parents of $group,
+     *                hashed in $id => $name format.
+     */
+    function getGroupParentList($gid)
+    {
+        return $this->_datatree->getParentList($gid);
     }
 
     /**
      * Get a list of every group, in the format cid => groupname.
      *
+     * @param boolean $refresh  If true, the cached value is ignored and the group
+     *                          list is refreshed from the group backend.
+     *
      * @return array  CID => groupname hash.
      */
-    function listGroups()
+    function listGroups($refresh = false)
     {
         static $groups;
 
-        if (is_null($groups)) {
+        if ($refresh || is_null($groups)) {
             $groups = $this->_datatree->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
             unset($groups[DATATREE_ROOT]);
         }
@@ -289,14 +333,13 @@ class Group {
     /**
      * Get a list of every user that is a part of this group ONLY.
      *
-     * @param string $group  The name of the group.
+     * @param integer $gid  The ID of the group.
      *
      * @return array  The user list.
-     * @access public
      */
-    function listUsers($group)
+    function listUsers($gid)
     {
-        $groupOb = &$this->getGroup($group);
+        $groupOb = &$this->getGroupById($gid);
         if (is_a($groupOb, 'PEAR_Error')) {
             return $groupOb;
         }
@@ -313,24 +356,22 @@ class Group {
      * Get a list of every user that is part of the specified group
      * and any of its subgroups.
      *
-     * @access public
-     *
-     * @param string $group  The name of the parent group.
+     * @param integer $group  The ID of the parent group.
      *
      * @return array  The complete user list.
      */
-    function listAllUsers($group)
+    function listAllUsers($gid)
     {
         // Get a list of every group that is a sub-group of $group.
-        $groups = $this->_datatree->get(DATATREE_FORMAT_FLAT, $group, true);
+        $groups = $this->_datatree->getById(DATATREE_FORMAT_FLAT, $gid, true);
         if (is_a($groups, 'PEAR_Error')) {
             return $groups;
         }
 
-        $groups = array($group) + $groups;
+        $groups = array_keys($groups);
         $users = array();
-        foreach ($groups as $group) {
-            $users = array_merge($users, $this->listUsers($group));
+        foreach ($groups as $groupId) {
+            $users = array_merge($users, $this->listUsers($groupId));
         }
         return array_values(array_flip(array_flip($users)));
     }
@@ -377,17 +418,16 @@ class Group {
     /**
      * Say if a user is a member of a group or not.
      *
-     * @param          string  $user       The name of the user.
-     * @param          string  $group      The name of the group.
-     * @param optional boolean $subgroups  Return true if the user is in any subgroups
-     *                                     of $group, also.
+     * @param string $user        The name of the user.
+     * @param integer $gid        The ID of the group.
+     * @param boolean $subgroups  Return true if the user is in any subgroups
+     *                            of group with ID $gid, also.
      *
      * @return boolean
-     * @access public
      */
-    function userIsInGroup($user, $group, $subgroups = true)
+    function userIsInGroup($user, $gid, $subgroups = true)
     {
-        if (!$this->exists($group)) {
+        if (!$this->exists($this->getGroupName($gid))) {
             return false;
         } elseif ($subgroups) {
             $groups = $this->getGroupMemberships($user, true);
@@ -395,9 +435,9 @@ class Group {
                 return $groups;
             }
 
-            return !empty($groups[$this->getGroupId($group)]);
+            return !empty($groups[$gid]);
         } else {
-            $users = $this->listUsers($group);
+            $users = $this->listUsers($gid);
             if (is_a($users, 'PEAR_Error')) {
                 return $users;
             }
@@ -405,20 +445,29 @@ class Group {
         }
     }
 
+    /**
+     * Returns the DataTree level of the given group. 0 is returned for
+     * and object directly below DATATREE_ROOT.
+     *
+     * @param integer $gid  The DataTree ID of the group.
+     *
+     * @return The DataTree level of the group.
+     */
+    function getLevel($gid)
+    {
+        $name = $this->_datatree->getName($gid);
+        return substr_count($name, ':');
+    }
+
     /**
      * Attempts to return a concrete Group instance based on $driver.
      *
-     * @access public
+     * @param mixed $driver  The type of concrete Group subclass to return.
+     * @param array $params  A hash containing any additional configuration or
+     *                       connection parameters a subclass might need.
      *
-     * @param mixed $driver           The type of concrete Group subclass to
-     *                                return. The code is dynamically
-     *                                included.
-     * @param optional array $params  A hash containing any additional
-     *                                configuration or connection parameters a
-     *                                subclass might need.
-     *
-     * @return object Group   The newly created concrete Group instance, or a
-     *                        PEAR_Error object on an error.
+     * @return Group  The newly created concrete Group instance, or a
+     *                PEAR_Error object on an error.
      */
     function &factory($driver = '', $params = null)
     {
@@ -431,10 +480,12 @@ class Group {
         }
         $class = 'Group_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $group = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $group = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $group;
     }
 
     /**
@@ -444,8 +495,7 @@ class Group {
      *
      * This method must be invoked as: $var = &Group::singleton()
      *
-     * @return object Group  The concrete Group reference, or false on an
-     *                       error.
+     * @return Group  The concrete Group reference, or false on an error.
      */
     function &singleton()
     {
@@ -486,7 +536,7 @@ class DataTreeObject_Group extends DataTreeObject {
      * The Group object which this group is associated with - needed
      * for updating data in the backend to make changes stick, etc.
      *
-     * @var object Group $_groupOb
+     * @var Group
      */
     var $_groupOb;
 
@@ -494,7 +544,7 @@ class DataTreeObject_Group extends DataTreeObject {
      * This variable caches the users added or removed from the group
      * for History logging of user-groups relationship.
      *
-     * @var array $_auditLog
+     * @var array
      */
     var $_auditLog = array();
 
@@ -503,7 +553,7 @@ class DataTreeObject_Group extends DataTreeObject {
      * the parent constructor so that the group's name is set
      * properly.
      *
-     * @param string $name The name of the group.
+     * @param string $name  The name of the group.
      */
     function DataTreeObject_Group($name)
     {
@@ -513,7 +563,7 @@ class DataTreeObject_Group extends DataTreeObject {
     /**
      * Associates a Group object with this group.
      *
-     * @param object Group $groupOb The Group object.
+     * @param Group $groupOb  The Group object.
      */
     function setGroupOb(&$groupOb)
     {
@@ -535,7 +585,8 @@ class DataTreeObject_Group extends DataTreeObject {
      */
     function save()
     {
-        $this->_groupOb->updateGroup($this);
+        return $this->_groupOb->updateGroup($this);
+
     }
 
     /**
@@ -548,8 +599,8 @@ class DataTreeObject_Group extends DataTreeObject {
     {
         $this->data['users'][$username] = 1;
         $this->_auditLog[$username] = 'addUser';
-        if ($update && $this->_groupOb->_datatree->exists($this->getName())) {
-            $this->save();
+        if ($update && $this->_groupOb->exists($this->getName())) {
+            $result = $this->save();
         }
     }
 
@@ -572,24 +623,22 @@ class DataTreeObject_Group extends DataTreeObject {
      * Get a list of every user that is a part of this group
      * (and only this group)
      *
-     * @return array The user list
-     * @access public
+     * @return array  The user list
      */
     function listUsers()
     {
-        return $this->_groupOb->listUsers($this->name);
+        return $this->_groupOb->listUsers($this->getId());
     }
 
     /**
      * Get a list of every user that is a part of this group and
      * any of it's subgroups
      *
-     * @return array The complete user list
-     * @access public
+     * @return array  The complete user list
      */
     function listAllUsers()
     {
-        return $this->_groupOb->listAllUsers($this->name);
+        return $this->_groupOb->listAllUsers($this->getId());
     }
 
     /**
