@@ -2,41 +2,36 @@
 /**
  * The IMAP_Admin:: class allow managing of mailboxes on IMAP servers.
  *
- * <pre>
- * Required parameters:
- * ====================
- *  'admin_user'      --  The name of a user with admin privileges.
- *  'admin_password'  --  The password of the adminstrator.
+ * Required parameters:<pre>
+ *   'admin_user'      The name of a user with admin privileges.
+ *   'admin_password'  The password of the adminstrator.</pre>
  *
- * Optional parameters:
- * ====================
- *  'hostspec'       --  The hostname or IP address of the server.
- *                       DEFAULT: 'localhost'
- *  'port'           --  The server port to which we will connect.
- *                       IMAP is generally 143, while IMAP-SSL is generally 993.
- *                       DEFAULT: 143
- *  'protocol'       --  The connection protocol (e.g. 'imap', 'pop3', 'nntp').
- *                       Protocol is one of 'imap/notls' (or only 'imap' if you
- *                       have a c-client version 2000c or older), 'imap/ssl',
- *                       or 'imap/ssl/novalidate-cert' (for a self-signed
- *                       certificate).
- *                       DEFAULT: 'imap'
- *  'userhierarchy'  --  The hierarchy where user mailboxes are stored.
- *                       DEFAULT: 'user.'
- *  'dsn'            --  The full IMAP connection string.
- *                       If not present, this is built from 'hostspec', 'port'
- *                       and 'protocol' parameters.
- * </pre>
+ * Optional parameters:<pre>
+ *   'hostspec'       The hostname or IP address of the server.
+ *                    DEFAULT: 'localhost'
+ *   'port'           The server port to which we will connect.
+ *                    IMAP is generally 143, while IMAP-SSL is generally 993.
+ *                    DEFAULT: 143
+ *   'protocol'       The connection protocol (e.g. 'imap', 'pop3', 'nntp').
+ *                    Protocol is one of 'imap/notls' (or only 'imap' if you
+ *                    have a c-client version 2000c or older), 'imap/ssl',
+ *                    or 'imap/ssl/novalidate-cert' (for a self-signed
+ *                    certificate).
+ *                    DEFAULT: 'imap'
+ *   'userhierarchy'  The hierarchy where user mailboxes are stored.
+ *                    DEFAULT: 'user.'
+ *   'dsn'            The full IMAP connection string.
+ *                    If not present, this is built from 'hostspec', 'port'
+ *                    and 'protocol' parameters.</pre>
  *
- * $Horde: framework/IMAP/IMAP/Admin.php,v 1.5.6.1 2005/01/03 12:19:02 jan Exp $
+ * $Horde: framework/IMAP/IMAP/Admin.php,v 1.5.6.8 2006/01/01 21:28:21 jan Exp $
  *
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jan Schneider <jan@horde.org>
- * @version $Revision: 1.5.6.1 $
  * @since   Horde 3.0
  * @package Horde_IMAP
  */
@@ -45,14 +40,14 @@ class IMAP_Admin {
     /**
      * Parameter hash.
      *
-     * @var array $_params
+     * @var array
      */
     var $_params;
 
     /**
      * IMAP resource.
      *
-     * @var resource $_imap
+     * @var resource
      */
     var $_imap;
 
@@ -82,7 +77,15 @@ class IMAP_Admin {
                 $this->_params['dsn'] .= $this->_params['folder'];
             }
         } else {
-            $this->_ref = substr($this->_params['dsn'], 0, strpos($this->_params['dsn'], '}') + 1);
+            if (preg_match('/^{([^:]+):(\d+)\/([^}]+)}/', $this->_params['dsn'], $matches)) {
+                $this->_params['hostspec'] = $matches[1];
+                $this->_params['port'] = $matches[2];
+                $this->_params['protocol'] = $matches[3];
+                $this->_ref = sprintf('{%s:%d/%s}',
+                                      $this->_params['hostspec'],
+                                      $this->_params['port'],
+                                      $this->_params['protocol']);
+            }
         }
     }
 
@@ -100,7 +103,7 @@ class IMAP_Admin {
                                       $this->_params['admin_password'],
                                       OP_HALFOPEN);
             if (!$this->_imap) {
-                $this->_imap = PEAR::raiseError(imap_last_error(), 'horde.error');
+                $this->_imap = PEAR::raiseError(_("Authentication at IMAP server failed."), 'horde.error');
             }
         }
         return $this->_imap;
