@@ -1,30 +1,40 @@
 <?php
 
 require_once dirname(__FILE__) . '/Message.php';
-require_once dirname(__FILE__) . '/Part.php';
 require_once HORDE_BASE . '/config/mime_drivers.php';
 
-/** @constant MIME_CONTENTS_CACHE  The name of the URL parameter that holds the MIME_Contents cache identifier.  */
+/**
+ * The name of the URL parameter that holds the MIME_Contents cache
+ * identifier.
+ */
 define('MIME_CONTENTS_CACHE', 'mimecache');
 
-/* Attachment display constants. */
-/** @constant MIME_CONTENTS_DISPLAY_TYPE_LIST  Display attachment information in list format. */
+/**
+ * Display attachment information in list format.
+ */
 define('MIME_CONTENTS_DISPLAY_TYPE_LIST', 0);
-/** @constant MIME_CONTENTS_DISPLAY_TYPE_INLINE  Display attachment information inline with attachment. */
+
+/**
+ * Display attachment information inline with attachment.
+ */
 define('MIME_CONTENTS_DISPLAY_TYPE_INLINE', 1);
-/** @constant MIME_CONTENTS_DISPLAY_TYPE_BOTH  Display attachment information both in list format and inline with attachment. */
+
+/**
+ * Display attachment information both in list format and inline with
+ * attachment.
+ */
 define('MIME_CONTENTS_DISPLAY_TYPE_BOTH', 2);
 
 /**
  * The MIME_Contents:: class contains functions related to handling the output
  * of MIME content.
  *
- * $Horde: framework/MIME/MIME/Contents.php,v 1.129.4.9 2005/03/22 11:55:09 jan Exp $
+ * $Horde: framework/MIME/MIME/Contents.php,v 1.129.4.33 2006/03/08 16:40:39 slusarz Exp $
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ * See the enclosed file COPYING for license information (GPL). If you did not
+ * receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
  * @since   Horde 3.0
@@ -35,63 +45,70 @@ class MIME_Contents {
     /**
      * The MIME_Message object for the message.
      *
-     * @var object MIME_Message $_message
+     * @var MIME_Message
      */
     var $_message;
 
+    /**
+     * The MIME_Message object we use when caching.
+     *
+     * @var MIME_Message
+     */
+    var $_cachemessage;
+
     /**
      * The attachments list.
      *
-     * @var array $_atc
+     * @var array
      */
     var $_atc = array();
 
     /**
      * The message parts list.
      *
-     * @var array $_parts
+     * @var array
      */
     var $_parts = array();
 
     /**
      * The summary parts list.
      *
-     * @var array $_summary
+     * @var array
      */
     var $_summary = array();
 
     /**
      * The summary type.
      *
-     * @var string $_summaryType
+     * @var string
      */
     var $_summaryType = null;
 
     /**
      * The Cache_session identifier.
      *
-     * @var string $_sessionCacheID
+     * @var string
      */
     var $_sessionCacheID = null;
 
     /**
      * The MIME_Viewer object cache.
      *
-     * @var array $_viewerCache
+     * @var array
      */
     var $_viewerCache = array();
 
     /**
      * The attachment display type to use.
      *
-     * @var integer $_displayType
+     * @var integer
      */
     var $_displayType = MIME_CONTENTS_DISPLAY_TYPE_BOTH;
 
     /**
      * The MIME index key to use.
      *
-     * @var string $_mimekey
+     * @var string
      */
     var $_mimekey = null;
 
@@ -100,21 +117,21 @@ class MIME_Contents {
      * 'download'  --  Downloading a part/attachment.
      * 'view'      --  Viewing a part/attachment.
      *
-     * @var array $_viewID
+     * @var array
      */
     var $_viewID = array();
 
     /**
      * Show the links in the summaries?
      *
-     * @var boolean $_links
+     * @var boolean
      */
     var $_links = true;
 
     /**
      * The base MIME_Contents object.
      *
-     * @var object MIME_Contents &$_base
+     * @var MIME_Contents
      */
     var $_base = null;
 
@@ -122,15 +139,13 @@ class MIME_Contents {
      * The number of message/rfc822 levels labeled as 'attachments' of the
      * current part.
      *
-     * @var integer $_attach822
+     * @var integer
      */
     var $_attach822 = 0;
 
     /**
      * Constructor.
      *
-     * @access public
-     *
      * @param MIME_Message $messageOb  The object to work with.
      * @param array $viewID            The actionID values for viewing
      *                                 parts/attachments.
@@ -142,6 +157,7 @@ class MIME_Contents {
     function MIME_Contents($messageOb, $viewID = array(), $contents = array())
     {
         $this->_message = $messageOb;
+        $this->_cachemessage = Util::cloneObject($messageOb);
         $this->_viewID = $viewID;
 
         /* Create the pointer to the base object. */
@@ -156,8 +172,6 @@ class MIME_Contents {
      * Returns the entire body of the message.
      * You probably want to override this function in any subclass.
      *
-     * @access public
-     *
      * @return string  The text of the body of the message.
      */
     function getBody()
@@ -169,8 +183,6 @@ class MIME_Contents {
      * Returns the raw text for one section of the message.
      * You probably want to override this function in any subclass.
      *
-     * @access public
-     *
      * @param string $id  The ID of the MIME_Part.
      *
      * @return string  The text of the part.
@@ -187,9 +199,7 @@ class MIME_Contents {
     /**
      * Returns the MIME_Message object for the mail message.
      *
-     * @access public
-     *
-     * @return object MIME_Message  A MIME_Message object.
+     * @return MIME_Message  A MIME_Message object.
      */
     function getMIMEMessage()
     {
@@ -199,25 +209,22 @@ class MIME_Contents {
     /**
      * Fetch a part of a MIME message.
      *
-     * @access public
-     *
      * @param integer $id  The MIME index of the part requested.
      *
-     * @return object MIME_Part  The raw MIME part asked for.
+     * @return MIME_Part  The raw MIME part asked for.
      */
     function &getMIMEPart($id)
     {
-        return $part = &$this->_message->getPart($id);
+        $part = $this->_message->getPart($id);
+        return $part;
     }
 
     /**
      * Rebuild the MIME_Part structure of a message.
      * You probably want to override this function in any subclass.
      *
-     * @access public
-     *
-     * @return object MIME_Message  A MIME_Message object with all of the body
-     *                              text stored in the individual MIME_Parts.
+     * @return MIME_Message  A MIME_Message object with all of the body text
+     *                       stored in the individual MIME_Parts.
      */
     function rebuildMessage()
     {
@@ -227,19 +234,18 @@ class MIME_Contents {
     /**
      * Fetch part of a MIME message.
      *
-     * @access public
+     * @param integer $id   The MIME ID of the part requested.
+     * @param boolean $all  If this is a header part, should we return all text
+     *                      in the body?
      *
-     * @param integer $id            The MIME ID of the part requested.
-     * @param optional boolean $all  If this is a header part, should we
-     *                               return all text in the body?
-     *
-     * @return object MIME_Part  The MIME_Part.
+     * @return MIME_Part  The MIME_Part.
      */
     function &getRawMIMEPart($id, $all = false)
     {
         $mime_part = &$this->getMIMEPart($id);
         if (!is_a($mime_part, 'MIME_Part')) {
-            return;
+            $mime_part = null;
+            return $mime_part;
         }
 
         /* If all text is requested, change the ID now. */
@@ -259,36 +265,35 @@ class MIME_Contents {
      * Fetch part of a MIME message and decode it, if it is base_64 or
      * qprint encoded.
      *
-     * @access public
+     * @param integer $id   The MIME ID of the part requested.
+     * @param boolean $all  If this is a header part, should we return all text
+     *                      in the body?
      *
-     * @param integer $id            The MIME ID of the part requested.
-     * @param optional boolean $all  If this is a header part, should we
-     *                               return all text in the body?
-     *
-     * @return object MIME_Part  The MIME_Part with its contents decoded.
+     * @return MIME_Part  The MIME_Part with its contents decoded.
      */
     function &getDecodedMIMEPart($id, $all = false)
     {
         if (($mime_part = &$this->getRawMIMEPart($id, $all))) {
             $mime_part->transferDecodeContents();
-            return $mime_part;
+            // Noop.
         } else {
-            return;
+            $mime_part = null;
         }
+        return $mime_part;
     }
 
     /**
      * Return the attachment list (HTML table format).
      *
-     * @access public
-     *
      * @return string  The list of attachments formatted into HTML.
      */
     function getAttachments()
     {
         $msg = '';
 
-        foreach ($this->_atc as $key => $val) {
+        $akeys = array_keys($this->_atc);
+        natsort($akeys);
+        foreach ($akeys as $key) {
             $msg .= $this->_arrayToTableRow($this->_atc[$key]);
         }
 
@@ -298,10 +303,8 @@ class MIME_Contents {
     /**
      * Return the message list (HTML table format).
      *
-     * @access public
-     *
-     * @param optional boolean $oneframe  Should the output be designed
-     *                                    for display in a single frame?
+     * @param boolean $oneframe  Should the output be designed for display in a
+     *                           single frame?
      *
      * @return string  The message formatted into HTML.
      */
@@ -312,13 +315,20 @@ class MIME_Contents {
         $partDisplayed = false;
 
         if ($oneframe) {
-            $msg .= '<style type="text/css">.onefr {padding: 1px; border: 1px dashed #00c; background-color: #eef; font-color: #006}</style>' . "\n";
+            $msg .= '<style type="text/css">.onefr {padding:1px; margin-bottom:1px; border:1px dashed #00c; background:#eef; color:#006}</style>' . "\n";
         }
 
+        // TODO: Temporary hack to display header info for a message with one
+        // MIME part that cannot be displayed inline.
+        if (!$msgCount || ($msgCount == 1 && !reset($this->_parts))) {
+            $this->setSummary($this->_message, 'part');
+            $msgCount = 1;
+        }
+            
         foreach ($this->_parts as $key => $value) {
             if (isset($this->_summary[$key]) &&
                 ((($msgCount == 1) && empty($value)) || ($msgCount > 1))) {
-                $msg .= '<tr><td align="left">';
+                $msg .= '<tr><td>';
                 if ($oneframe) {
                     $msg .= '<table class="onefr">';
                     $summary = $this->_summary[$key];
@@ -328,16 +338,16 @@ class MIME_Contents {
                     $msg .= '<table>';
                     $msg .= $this->_arrayToTableRow($this->_summary[$key]);
                 }
-                $msg .= '</table><div style="font-size: 1px;">&nbsp;<br /></div></td></tr>';
+                $msg .= '</table></td></tr>';
             }
             if (!empty($value)) {
-                $msg .= '<tr><td class="text" align="left">' . $value . '</td></tr>';
+                $msg .= '<tr><td class="text">' . $value . '</td></tr>';
                 $partDisplayed = true;
             }
         }
 
         if (!$partDisplayed) {
-            $msg .= '<tr><td class="text" align="left"><i>' . _("There are no parts that can be displayed inline.") . '</i></td></tr>';
+            $msg .= '<tr><td class="text" align="left"><em>' . _("There are no parts that can be displayed inline.") . '</em></td></tr>';
         }
 
         return $msg;
@@ -368,8 +378,6 @@ class MIME_Contents {
     /**
      * Returns the data for a specific MIME index.
      *
-     * @access public
-     *
      * @param string $id     The MIME index.
      * @param string $field  The field to return (message, atc, summary)
      *
@@ -389,8 +397,6 @@ class MIME_Contents {
     /**
      * Removes the message text and summary for a specific MIME index.
      *
-     * @access public
-     *
      * @param string $id  The MIME index.
      */
     function removeIndex($id)
@@ -403,36 +409,35 @@ class MIME_Contents {
     /**
      * Determine if we can (and know how to) inline a MIME Part.
      *
-     * @param object MIME_Part &$mime_part  A MIME_Part object.
+     * @param MIME_Part &$mime_part  A MIME_Part object.
      *
      * @return boolean  True if part can be inlined.
      *                  False if it cannot.
      */
     function canDisplayInline(&$mime_part)
     {
+        $viewer = &$this->getMIMEViewer($mime_part);
+
         /* First check: The MIME headers allow the part to be inlined.
          * However, if we are already in view mode, then we can skip this
          * check. */
         if (!$this->viewAsAttachment() &&
-            ($mime_part->getDisposition() != 'inline')) {
+            ($mime_part->getDisposition() != 'inline') &&
+            !$viewer->forceInlineView()) {
             return false;
         }
 
         /* Second check (save the most expensive for last):
          * Check to see if the driver is set to inline. */
-        $viewer = &$this->getMIMEViewer($mime_part);
         return (is_a($viewer, 'MIME_Viewer') && $viewer->canDisplayInline());
     }
 
     /**
      * Get MIME_Viewer object.
      *
-     * @access public
+     * @param MIME_Part &$mime_part  A MIME_Part object.
      *
-     * @param object MIME_Part &$mime_part  A MIME_Part object.
-     *
-     * @return object MIME_Viewer  The MIME_Viewer object.
-     *                             Return false on error.
+     * @return MIME_Viewer  The MIME_Viewer object, or false on error.
      */
     function &getMIMEViewer(&$mime_part)
     {
@@ -455,12 +460,10 @@ class MIME_Contents {
      * Get the MIME Content-Type output by a MIME_Viewer for a particular
      * MIME_Part.
      *
-     * @access public
+     * @param MIME_Part &$mime_part  A MIME_Part object.
      *
-     * @param object MIME_Part &$mime_part  A MIME_Part object.
-     *
-     * @return string  The MIME type output by the MIME_Viewer.
-     *                 Return false on error.
+     * @return string  The MIME type output by the MIME_Viewer, or false on
+     *                 error.
      */
     function getMIMEViewerType(&$mime_part)
     {
@@ -476,9 +479,8 @@ class MIME_Contents {
      *
      * @access private
      *
-     * @param object MIME_Part &$mime_part  A MIME_Part object.
-     * @param optional boolean $override    Respect the MIME key override
-     *                                      value?
+     * @param MIME_Part &$mime_part  A MIME_Part object.
+     * @param boolean $override      Respect the MIME key override value?
      *
      * @return string  The unique identifier of the MIME_Part.
      *                 Returns false if no key found.
@@ -502,8 +504,6 @@ class MIME_Contents {
     /**
      * Gets the MIME key override.
      *
-     * @access public
-     *
      * @return string  The MIME key override - null if no override.
      */
     function getMIMEKeyOverride()
@@ -514,9 +514,7 @@ class MIME_Contents {
     /**
      * Sets an override for the MIME key.
      *
-     * @access public
-     *
-     * @param optional string $mimekey
+     * @param string $mimekey
      */
     function setMIMEKeyOverride($mimekey = null)
     {
@@ -526,9 +524,7 @@ class MIME_Contents {
     /**
      * Should we display links for the summaries?
      *
-     * @access public
-     *
-     * @param optional boolean $show  Show the summary links?
+     * @param boolean $show  Show the summary links?
      */
     function showSummaryLinks($show = null)
     {
@@ -542,9 +538,7 @@ class MIME_Contents {
     /**
      * Render a MIME Part.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  A MIME_Part object.
+     * @param MIME_Part &$mime_part  A MIME_Part object.
      *
      * @return string  The rendered data.
      */
@@ -556,9 +550,7 @@ class MIME_Contents {
     /**
      * Render MIME Part attachment info.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  A MIME_Part object.
+     * @param MIME_Part &$mime_part  A MIME_Part object.
      *
      * @return string  The rendered data.
      */
@@ -572,13 +564,13 @@ class MIME_Contents {
      *
      * @access private
      *
-     * @param object MIME_Part &$mime_part  A MIME_Part object.
-     * @param optional boolean $attachment  Render MIME Part attachment info?
+     * @param MIME_Part &$mime_part  A MIME_Part object.
+     * @param boolean $attachment    Render MIME Part attachment info?
      *
      * @return string  The rendered data.
      */
-     function _renderMIMEPart(&$mime_part, $attachment = false)
-     {
+    function _renderMIMEPart(&$mime_part, $attachment = false)
+    {
         /* Get the MIME_Viewer object for this MIME part */
         $viewer = &$this->getMIMEViewer($mime_part);
         if (!is_a($viewer, 'MIME_Viewer')) {
@@ -607,7 +599,7 @@ class MIME_Contents {
                             sprintf(_("This message was written in a character set (%s) other than your own."), htmlspecialchars($charset)),
                             sprintf(_("If it is not displayed correctly, %s to open it in a new window."), $this->linkViewJS($mime_part, 'view_attach', _("click here")))
                         );
-                        $msg = $this->formatStatusMsg($status, null, false) . '<p />' . $msg;
+                        $msg = $this->formatStatusMsg($status, null, false) . $msg;
                     }
                 }
             }
@@ -627,8 +619,6 @@ class MIME_Contents {
     /**
      * Build the message deciding what MIME Parts to show.
      *
-     * @access public
-     *
      * @return boolean  False on error.
      */
     function buildMessage()
@@ -642,7 +632,7 @@ class MIME_Contents {
         }
 
         /* Now display the parts. */
-        $mime_part = &$this->_message->getBasePart();
+        $mime_part = $this->_message->getBasePart();
         $this->buildMessagePart($mime_part);
 
         return true;
@@ -652,9 +642,7 @@ class MIME_Contents {
      * Processes a MIME_Part and stores the display information in the internal
      * class variables.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part object to process.
+     * @param MIME_Part &$mime_part  The MIME_Part object to process.
      *
      * @return string  The rendered text.
      */
@@ -687,21 +675,22 @@ class MIME_Contents {
             }
         } else {
             $msg = $this->renderMIMEPart($mime_part);
-            if (!empty($msg)) {
-                $key = $this->_getMIMEKey($mime_part);
-                if (!$this->_attach822) {
-                    $this->_parts[$key] = $msg;
-                }
-                /* Some MIME_Viewers set the summary by themelves, so only
-                 * add to attachment/inline lists if nothing has been set
-                 * as of yet. */
-                if (!empty($key) && !$this->getIndex($key, 'summary')) {
-                    $this->setSummary($mime_part, 'part');
-                    if ($this->_attach822 &&
-                        (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
-                        ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH))) {
-                        $this->setSummary($mime_part, 'attachment');
-                    }
+            $key = $this->_getMIMEKey($mime_part);
+            if (!$this->_attach822) {
+                $this->_parts[$key] = $msg;
+            }
+            /* Some MIME_Viewers set the summary by themelves, so only
+             * add to attachment/inline lists if nothing has been set
+             * as of yet. */
+            if ((($mime_part->getType() != 'multipart/mixed') ||
+                 !empty($msg)) &&
+                !empty($key) &&
+                !$this->getIndex($key, 'summary')) {
+                $this->setSummary($mime_part, 'part');
+                if ($this->_attach822 &&
+                    (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_LIST) ||
+                    ($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH))) {
+                    $this->setSummary($mime_part, 'attachment');
                 }
             }
         }
@@ -734,10 +723,8 @@ class MIME_Contents {
      * Are we viewing this page as an attachment through view.php?
      * This method can also be called via MIME_Contents::viewAsAttachment().
      *
-     * @access public
-     *
-     * @param optional boolean $popup  If true, also check if we are viewing
-     *                                 attachment in popup view window.
+     * @param boolean $popup  If true, also check if we are viewing attachment
+     *                        in popup view window.
      *
      * @return boolean  True if we are viewing this part as an attachment
      *                  through view.php.
@@ -751,10 +738,8 @@ class MIME_Contents {
     /**
      * Sets a summary entry.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part object.
-     * @param string $type                  The summary cache to use.
+     * @param MIME_Part &$mime_part  The MIME_Part object.
+     * @param string $type           The summary cache to use.
      */
     function setSummary(&$mime_part, $type)
     {
@@ -783,11 +768,8 @@ class MIME_Contents {
     /**
      * Returns an array summarizing a part of a MIME message.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part to summarize.
-     * @param optional boolean $guess       Is this a temporary guessed-type
-     *                                      part?
+     * @param MIME_Part &$mime_part  The MIME_Part to summarize.
+     * @param boolean $guess         Is this a temporary guessed-type part?
      *
      * @return array  The summary of the part.
      *                [0] = Icon
@@ -805,6 +787,9 @@ class MIME_Contents {
         $summary = array();
         $type = htmlspecialchars($mime_part->getType());
         $viewer = &$this->getMIMEViewer($mime_part);
+        if (!$viewer) {
+            return $summary;
+        }
 
         /* Return if we still didn't find a native mime viewer
          * for a guessed mime type. */
@@ -821,7 +806,7 @@ class MIME_Contents {
         /* Number column. */
         if (($this->_displayType == MIME_CONTENTS_DISPLAY_TYPE_BOTH) &&
             !is_null($this->_summaryType)) {
-            $summary[] = '<a name="mime_contents_' . $this->_summaryType . '_' . $id . '" href="#mime_contents_' . (($this->_summaryType == 'attachment') ?'part' : 'attachment') . '_' . $id . '">' . $id . '</a>';
+            $summary[] = '<a id="mime_contents_' . $this->_summaryType . '_' . $id . '" href="#mime_contents_' . (($this->_summaryType == 'attachment') ?'part' : 'attachment') . '_' . $id . '">' . $id . '</a>';
         } else {
             $summary[] = $id;
         }
@@ -842,7 +827,7 @@ class MIME_Contents {
 
         /* MIME type column. */
         if ($guess) {
-            $summary[] = '<i>' . sprintf(_("View as %s"), $type) . '</i>';
+            $summary[] = '<em>' . sprintf(_("View as %s"), $type) . '</em>';
         } else {
             $summary[] = '[' . $type . ']';
         }
@@ -893,15 +878,12 @@ class MIME_Contents {
     /**
      * Return the URL to the view.php page.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part object to view.
-     * @param integer $actionID             The ActionID to perform.
-     * @param optional array $params        A list of any additional
-     *                                      parameters that need to be passed
-     *                                      to view.php. (key = name)
-     * @param optional boolean $dload       Should we generate a download
-     *                                      link?
+     * @param MIME_Part &$mime_part  The MIME_Part object to view.
+     * @param integer $actionID      The ActionID to perform.
+     * @param array $params          A list of any additional parameters that
+     *                               need to be passed to view.php. (key =
+     *                               name)
+     * @param boolean $dload         Should we generate a download link?
      *
      * @return string  The URL to view.php.
      */
@@ -927,18 +909,15 @@ class MIME_Contents {
     /**
      * Generate a link to the view.php page.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part object to view.
-     * @param integer $actionID             The actionID value.
-     * @param string $text                  The link text.
-     * @param optional array $params        A list of additional parameters.
+     * @param MIME_Part &$mime_part  The MIME_Part object to view.
+     * @param integer $actionID      The actionID value.
+     * @param string $text           The link text.
+     * @param array $params          A list of additional parameters.
      *   'class'       -  The CSS class to use.
      *   'jstext'      -  The JS text to use.
      *   'viewparams'  -  A list of any additional parameters that need to be
      *                    passed to view.php.
-     * @param optional boolean $dload       Should we generate a download
-     *                                      link?
+     * @param boolean $dload         Should we generate a download link?
      *
      * @return string  A HTML href link to view.php.
      */
@@ -967,18 +946,16 @@ class MIME_Contents {
     /**
      * Generate a javascript link to the view.php page.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part object to view.
-     * @param integer $actionID             The ActionID to perform.
-     * @param string $text                  The link text.
-     * @param optional string $jstext       The Javascript link text.
-     * @param optional string $css          The CSS class to use.
-     * @param optional array $params        A list of any additional
-     *                                      parameters that need to be passed
-     *                                      to view.php. (key = name)
-     * @param optional bool $widget         If true use Horde::widget() to
-     *                                      generate, Horde::link() otherwise.
+     * @param MIME_Part &$mime_part  The MIME_Part object to view.
+     * @param integer $actionID      The ActionID to perform.
+     * @param string $text           The link text.
+     * @param string $jstext         The Javascript link text.
+     * @param string $css            The CSS class to use.
+     * @param array $params          A list of any additional parameters that
+     *                               need to be passed to view.php. (key =
+     *                               name)
+     * @param boolean $widget        If true use Horde::widget() to generate,
+     *                               Horde::link() otherwise.
      *
      * @return string  A HTML href link to view.php.
      */
@@ -1002,17 +979,15 @@ class MIME_Contents {
         }
 
         if ($widget) {
-            return Horde::widget('', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;", $text);
+            return Horde::widget('#', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;", $text);
         } else {
-            return Horde::link('#', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;", $text) . $text . '</a>';
+            return Horde::link('#', $jstext, $css, null, "view('" . $url . "', '" . $id . "'); return false;") . $text . '</a>';
         }
     }
 
     /**
      * Prints out the status message for a given MIME Part.
      *
-     * @access public
-     *
      * @param string $msg     The message to output.
      * @param string $img     An image link to add to the beginning of the
      *                        message.
@@ -1059,22 +1034,22 @@ class MIME_Contents {
             $text .= '</td></tr>' . "\n";
         }
 
-        $text .= '</table><div style="font-size: 1px;">&nbsp;<br /></div>';
-
-        return $text;
+        return $text . '</table>';
     }
 
     /**
      * Return a pointer to the base object.
      *
-     * @access public
-     *
      * @return mixed  Returns a pointer to the base object.
      *                Returns false if there is no base object.
      */
     function &getBaseObjectPtr()
     {
-        return (is_null($this->_base)) ? $this : $this->_base;
+        if (is_null($this->_base)) {
+            return $this;
+        } else {
+            return $this->_base;
+        }
     }
 
     /**
@@ -1120,6 +1095,9 @@ class MIME_Contents {
            way to regenerate them on cache reload. */
         $this->_viewerCache = array();
 
+        /* Copy the MIME_Message cache object to the _message variable. */
+        $this->_message = $this->_cachemessage;
+
         $cache->store($this->_sessionCacheID, $this);
     }
 
@@ -1128,14 +1106,12 @@ class MIME_Contents {
      * This function should be called statically e.g.:
      * $ptr = &MIME_Contents::getCache().
      *
-     * @access public
+     * @param string $cacheid  The cache ID to use.  If empty, will use the
+     *                         cache ID located in the URL parameter named
+     *                         MIME_CONTENTS_CACHE.
      *
-     * @param optional string $cacheid  The cache ID to use.  If empty, will
-     *                                  use the cache ID located in the URL
-     *                                  parameter named MIME_CONTENTS_CACHE.
-     *
-     * @return object MIME_Contents  Returns the MIME_Contents object, or
-     *                               null if it does not exist.
+     * @return MIME_Contents  The MIME_Contents object, or null if it does not
+     *                        exist.
      */
     function &getCache($cacheid = null)
     {
@@ -1145,15 +1121,14 @@ class MIME_Contents {
 
         require_once 'Horde/SessionObjects.php';
         $cache = &Horde_SessionObjects::singleton();
-        return $ret = &$cache->query($cacheid);
+        $contents = &$cache->query($cacheid);
+        return $contents;
     }
 
     /**
      * Add the current object to the cache, and return the cache identifier
      * to be used in URLs.
      *
-     * @access public
-     *
      * @return array  The parameter key/value set to use in URLs.
      */
     function cacheIDURLParam()
