@@ -5,7 +5,7 @@ require_once dirname(__FILE__) . '/Part.php';
 require_once dirname(__FILE__) . '/../MIME.php';
 
 /**
- * $Horde: framework/MIME/MIME/Structure.php,v 1.87.10.29 2008/05/27 21:48:48 slusarz Exp $
+ * $Horde: framework/MIME/MIME/Structure.php,v 1.87.10.18 2006/03/24 05:39:36 slusarz Exp $
  *
  * The MIME_Structure:: class provides methods for dealing with MIME mail.
  *
@@ -15,13 +15,15 @@ require_once dirname(__FILE__) . '/../MIME.php';
  *
  * TODO: Convert to OO
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @author  Michael Slusarz <slusarz@horde.org>
+ * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * @since   Horde 1.3
  * @package Horde_MIME
  */
 class MIME_Structure {
@@ -41,7 +43,7 @@ class MIME_Structure {
         $msgOb->buildMessage();
 
         $ptr = array(&$msgOb);
-        MIME_Structure::addMultipartInfo($ptr);
+        MIME_Structure::_addMultipartInfo($ptr);
 
         return $msgOb;
     }
@@ -86,8 +88,7 @@ class MIME_Structure {
         /* Deal with multipart data. */
         if (isset($body->parts)) {
             $sub_id = 1;
-            reset($body->parts);
-            while (list(,$sub_part) = each($body->parts)) {
+            foreach ($body->parts as $sub_part) {
                 /* Are we dealing with a multipart message? */
                 if (isset($body->type) && ($body->type == $message) &&
                     isset($sub_part->type) && ($sub_part->type == $multipart)) {
@@ -119,8 +120,8 @@ class MIME_Structure {
     function _setInfo($part, &$ob, $ref)
     {
         /* Store Content-type information. */
-        $primary_type = (isset($part->type)) ? MIME::type($part->type, MIME_STRING) : 'text';
-        $sec_type = ($part->ifsubtype && $part->subtype) ? $part->subtype : 'x-unknown';
+        $primary_type = (isset($part->type)) ? String::lower($part->type) : MIME::type('text');
+        $sec_type = ($part->ifsubtype && $part->subtype) ? String::lower($part->subtype) : 'x-unknown';
         $ob->setType($primary_type . '/' . $sec_type);
 
         /* Set transfer encoding. */
@@ -132,7 +133,7 @@ class MIME_Structure {
         }
 
         /* Set transfer disposition. */
-        $ob->setDisposition(($part->ifdisposition) ? $part->disposition : MIME_DEFAULT_DISPOSITION);
+        $ob->setDisposition(($part->ifdisposition) ? String::lower($part->disposition) : MIME_DEFAULT_DISPOSITION);
 
         /* If 'body' is set, set as the contents of the part. */
         if (isset($part->body)) {
@@ -149,6 +150,10 @@ class MIME_Structure {
             $ob->setContentID($part->id);
         }
 
+        /* Set the default character set. */
+        $ob->setCharset(isset($GLOBALS['mime_structure']['default_charset']) ? $GLOBALS['mime_structure']['default_charset'] : MIME_DEFAULT_CHARSET);
+
+
         /* Go through the content-type parameters, if any. */
         foreach (MIME_Structure::_getParameters($part, 1) as $key => $val) {
             if ($key == 'charset') {
@@ -158,28 +163,21 @@ class MIME_Structure {
             }
         }
 
-        /* Set the default character set. */
-        if (($ob->getPrimaryType() == 'text') &&
-            (String::lower($ob->getCharset()) == 'us-ascii') &&
-            isset($GLOBALS['mime_structure']['default_charset'])) {
-            $ob->setCharset($GLOBALS['mime_structure']['default_charset']);
-        }
-
         /* Go through the disposition parameters, if any. */
         foreach (MIME_Structure::_getParameters($part, 2) as $key => $val) {
             $ob->setDispositionParameter($key, $val);
         }
 
         /* Set the name. */
-        if (($fname = $ob->getContentTypeParameter('filename'))) {
-            $ob->setName($fname);
-        } elseif (($fname = $ob->getDispositionParameter('filename'))) {
-            $ob->setName($fname);
+        if ($ob->getContentTypeParameter('filename')) {
+            $ob->setName($ob->getContentTypeParameter('filename'));
+        } elseif ($ob->getDispositionParameter('filename')) {
+            $ob->setName($ob->getDispositionParameter('filename'));
         }
 
         /* Set the description. */
         if (isset($part->description)) {
-            $ob->setDescription(preg_replace('/\s+/', ' ', $part->description));
+            $ob->setDescription($part->description);
         }
     }
 
@@ -199,39 +197,29 @@ class MIME_Structure {
     {
         $param_list = array();
 
-        $ptype = ($type == 1) ? 'parameters' : 'dparameters';
+        if ($type == 1) {
+            $ptype = 'parameters';
+        } elseif ($type == 2) {
+            $ptype = 'dparameters';
+        }
         $pexists = 'if' . $ptype;
 
         if ($part->$pexists) {
-            $attr_list = $rfc2231_list = array();
             foreach ($part->$ptype as $param) {
                 $param->value = str_replace(array("\t", '\"'), array(' ', '"'), $param->value);
-                /* Look for an asterisk in the attribute name.  If found we
-                 * know we have RFC 2231 information. */
-                $pos = strpos($param->attribute, '*');
-                if ($pos) {
-                    $attr = substr($param->attribute, 0, $pos);
-                    $rfc2231_list[$attr][] = $param->attribute . '=' . $param->value;
-                } else {
-                    $attr_list[$param->attribute] = $param->value;
-                }
-            }
 
-            foreach ($rfc2231_list as $val) {
-                $res = MIME::decodeRFC2231(implode(' ', $val));
+                $res = MIME::decodeRFC2231($param->attribute . '=' . $param->value);
                 if ($res) {
-                    $attr_list[$res['attribute']] = $res['value'];
+                    $param->attribute = $res['attribute'];
+                    $param->value = $res['value'];
                 }
-            }
-
-            foreach ($attr_list as $attr => $val) {
-                $field = String::lower($attr);
+                $field = String::lower($param->attribute);
                 if ($field == 'type') {
-                    if (($type = MIME::type($val))) {
+                    if (($type = MIME::type($param->value))) {
                         $param_list['type'] = $type;
                     }
                 } else {
-                    $param_list[$field] = $val;
+                    $param_list[$field] = $param->value;
                 }
             }
         }
@@ -242,20 +230,19 @@ class MIME_Structure {
     /**
      * Set the special information for certain MIME types.
      *
-     * @since Horde 3.2
+     * @access private
      *
      * @param array &$parts  The list of parts contained within the multipart
      *                       object.
      * @param array $info    Information about the multipart structure.
      */
-    function addMultipartInfo(&$parts, $info = array())
+    function _addMultipartInfo(&$parts, $info = array())
     {
         if (empty($parts)) {
             return;
         }
 
-        reset($parts);
-        while (list($key,) = each($parts)) {
+        foreach (array_keys($parts) as $key) {
             $ptr = &$parts[$key];
             $new_info = $info;
 
@@ -287,10 +274,11 @@ class MIME_Structure {
                 break;
             }
 
-            MIME_Structure::addMultipartInfo($ptr->_parts, $new_info);
+            MIME_Structure::_addMultipartInfo($ptr->_parts, $new_info);
         }
     }
 
+
     /**
      * Attempts to build a MIME_Message object from a text message.
      *
@@ -300,20 +288,21 @@ class MIME_Structure {
      */
     function &parseTextMIMEMessage($text)
     {
-        /* Set up the options for the mimeDecode class. */
-        $decode_args = array(
-            'include_bodies' => true,
-            'decode_bodies' => false,
-            'decode_headers' => false
-        );
-
         require_once 'Mail/mimeDecode.php';
+
+        /* Set up the options for the mimeDecode class. */
+        $decode_args = array();
+        $decode_args['include_bodies'] = true;
+        $decode_args['decode_bodies'] = false;
+        $decode_args['decode_headers'] = false;
+
         $mimeDecode = &new Mail_mimeDecode($text, MIME_PART_EOL);
         if (!($structure = $mimeDecode->decode($decode_args))) {
             $message = false;
         } else {
             /* Put the object into imap_parsestructure() form. */
             MIME_Structure::_convertMimeDecodeData($structure);
+
             $message = MIME_Structure::parse($structure);
         }
 
@@ -331,10 +320,6 @@ class MIME_Structure {
     function _convertMimeDecodeData(&$ob)
     {
         /* Primary content-type. */
-        if (!isset($ob->ctype_primary)) {
-            $ob->ctype_primary = 'application';
-            $ob->ctype_secondary = 'octet-stream';
-        }
         $ob->type = intval(MIME::type($ob->ctype_primary));
 
         /* Secondary content-type. */
@@ -351,8 +336,8 @@ class MIME_Structure {
         }
 
         /* Content-type and Disposition parameters. */
-        $param_types = array('ctype_parameters' => 'parameters',
-                             'd_parameters' => 'dparameters');
+        $param_types = array ('ctype_parameters' => 'parameters',
+                              'd_parameters' => 'dparameters');
         foreach ($param_types as $param_key => $param_value) {
             $if_var = 'if' . $param_value;
             if (isset($ob->$param_key)) {
@@ -397,8 +382,7 @@ class MIME_Structure {
 
         /* Process parts also. */
         if (isset($ob->parts)) {
-            reset($ob->parts);
-            while (list($key,) = each($ob->parts)) {
+            foreach (array_keys($ob->parts) as $key) {
                 MIME_Structure::_convertMimeDecodeData($ob->parts[$key]);
             }
         }
@@ -419,45 +403,40 @@ class MIME_Structure {
      */
     function parseMIMEHeaders($headers, $decode = true, $lowercase = false)
     {
-        $header = $headval = '';
-        $ob = $toprocess = array();
+        $header = '';
+        $ob = array();
 
         foreach (explode("\n", $headers) as $val) {
-            $val = rtrim($val);
-            if (preg_match("/^([^\s]+)\:\s*(.*)/", $val, $matches)) {
-                if (!empty($header)) {
-                    $toprocess[] = array($header, $headval);
-                }
-                $header = $matches[1];
-                $headval = $matches[2];
-            } else {
-                $headval .= ' ' . ltrim($val);
-            }
-        }
-
-        if (!empty($header)) {
-            $toprocess[] = array($header, $headval);
-        }
-
-        foreach ($toprocess as $val) {
             if ($decode) {
-                // Fields defined in RFC 2822 that contain address information
-                if (in_array(String::lower($val[0]), array('from', 'to', 'cc', 'bcc', 'reply-to', 'resent-to', 'resent-cc', 'resent-bcc', 'resent-from', 'sender'))) {
-                    $val[1] = MIME::decodeAddrString($val[1]);
-                } else {
-                    $val[1] = MIME::decode($val[1]);
+                $val = MIME::decode($val);
+            }
+            if (preg_match("/^([^\s]+)\:(.*)/", $val, $matches)) {
+                $val = trim($matches[2]);
+                $header = $matches[1];
+                if (isset($ob[$header])) {
+                    if (!is_array($ob[$header])) {
+                        $temp = $ob[$header];
+                        $ob[$header] = array();
+                        $ob[$header][] = $temp;
+                    }
+                    $ob[$header][] = $val;
+                    continue;
                 }
+            } else {
+                $val = ' ' . trim($val);
             }
 
-            if (isset($ob[$val[0]])) {
-                if (!is_array($ob[$val[0]])) {
-                    $temp = $ob[$val[0]];
-                    $ob[$val[0]] = array();
-                    $ob[$val[0]][] = $temp;
+            if (!empty($header)) {
+                if (isset($ob[$header])) {
+                    if (is_array($ob[$header])) {
+                        end($ob[$header]);
+                        $ob[$header][key($ob[$header])] .= $val;
+                    } else {
+                        $ob[$header] .= $val;
+                    }
+                } else {
+                    $ob[$header] = $val;
                 }
-                $ob[$val[0]][] = $val[1];
-            } else {
-                $ob[$val[0]] = $val[1];
             }
         }
 
