@@ -5,15 +5,14 @@
  * set of MIME_Viewer_* drivers which handle the actual rendering,
  * and also a configuration file to map MIME types to drivers.
  *
- * $Horde: framework/MIME/MIME/Viewer.php,v 1.64.10.1 2005/01/03 12:19:05 jan Exp $
+ * $Horde: framework/MIME/MIME/Viewer.php,v 1.64.10.9 2006/01/01 21:28:24 jan Exp $
  *
- * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
+ * Copyright 1999-2006 Anil Madhavapeddy <anil@recoil.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Anil Madhavapeddy <anil@recoil.org>
- * @version $Revision: 1.64.10.1 $
  * @since   Horde 1.3
  * @package Horde_MIME_Viewer
  */
@@ -22,36 +21,42 @@ class MIME_Viewer {
     /**
      * The MIME_Part object to render.
      *
-     * @var object MIME_Part $mime_part
+     * @var MIME_Part
      */
     var $mime_part;
 
     /**
      * Configuration parameters.
      *
-     * @var array $_conf
+     * @var array
      */
     var $_conf = array();
 
     /**
      * getDriver cache.
      *
-     * @var array $_driverCache
+     * @var array
      */
     var $_driverCache = array();
 
+    /**
+     * Force viewing of a part inline, regardless of the Content-Disposition
+     * of the MIME Part.
+     *
+     * @var boolean
+     */
+    var $_forceinline = false;
+
     /**
      * Attempts to return a concrete MIME_Viewer_* object based on the
      * type of MIME_Part passed onto it.
      *
-     * @param object MIME_Part &$mime_part  Reference to a MIME_Part object
-     *                                      with the information to be
-     *                                      rendered.
-     * @param optional string $mime_type    Use this MIME type instead of the
-     *                                      type stored in the $mime_part.
+     * @param MIME_Part &$mime_part  Reference to a MIME_Part object with the
+     *                               information to be rendered.
+     * @param string $mime_type      Use this MIME type instead of the type
+     *                               stored in the $mime_part.
      *
-     * @return object MIME_Viewer  The MIME_Viewer object.
-     *                             Returns false on error.
+     * @return MIME_Viewer  The MIME_Viewer object, or false on error.
      */
     function &factory(&$mime_part, $mime_type = null)
     {
@@ -68,25 +73,25 @@ class MIME_Viewer {
             }
         }
 
+        $viewer = false;
+
         /* Spawn the relevant driver, and return it (or false on failure) */
         if (($ob = MIME_Viewer::includeDriver($mime_type))) {
             $class = (($ob->module == 'horde') ? '' : $ob->module . '_') . 'MIME_Viewer_' . $ob->driver;
             if (class_exists($class)) {
-               return $ret = &new $class($mime_part, $GLOBALS['mime_drivers'][$ob->module][$ob->driver]);
+                $viewer = &new $class($mime_part, $GLOBALS['mime_drivers'][$ob->module][$ob->driver]);
             }
         }
 
-        return false;
+        return $viewer;
     }
 
     /**
      * Include the code for the relevant driver.
      *
-     * @access public
-     *
      * @param string $mime_type  The Content-type of the part to be rendered.
      *
-     * @return object stdClass  See MIME_Driver::getDriver().
+     * @return stdClass  See MIME_Driver::getDriver().
      */
     function includeDriver($mime_type)
     {
@@ -106,10 +111,8 @@ class MIME_Viewer {
     /**
      * Constructor for MIME_Viewer
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  Reference to a MIME_Part object
-     *                                      with the information to be rendered
+     * @param MIME_Part &$mime_part  Reference to a MIME_Part object with the
+     *                               information to be rendered.
      */
     function MIME_Viewer(&$mime_part, $conf = array())
     {
@@ -120,10 +123,8 @@ class MIME_Viewer {
     /**
      * Sets the MIME_Part object for the class.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  Reference to a MIME_Part object
-     *                                      with the information to be rendered
+     * @param MIME_Part &$mime_part  Reference to a MIME_Part object with the
+     *                               information to be rendered.
      */
     function setMIMEPart(&$mime_part)
     {
@@ -137,8 +138,6 @@ class MIME_Viewer {
      * the object, or replaces custom extension types with
      * 'text/plain' to let the browser do a best-guess render.
      *
-     * @access public
-     *
      * @return string  MIME-type of the output content.
      */
     function getType()
@@ -152,12 +151,11 @@ class MIME_Viewer {
 
     /**
      * Return the rendered version of the object.
+     *
      * Should be overridden by individual drivers to perform custom tasks.
      * The $mime_part class variable has the information to render,
      * encapsulated in a MIME_Part object.
      *
-     * @access public
-     *
      * @param mixed $params  Any optional parameters this driver needs at
      *                       runtime.
      *
@@ -170,16 +168,14 @@ class MIME_Viewer {
 
     /**
      * Return text/html output used as alternative output when the fully
-     * rendered object can not (or should not) be displayed.  For example,
-     * this function should be used for MIME attachments that can not be
+     * rendered object cannot (or should not) be displayed.  For example,
+     * this function should be used for MIME attachments that cannot be
      * viewed inline, where the user may be given options on how to view
      * the attachment.
      * Should be overridden by individual drivers to perform custom tasks.
      * The $mime_part class variable has the information to render,
      * encapsulated in a MIME_Part object.
      *
-     * @access public
-     *
      * @param mixed $params  Any optional parameters this driver needs at
      *                       runtime.
      *
@@ -192,8 +188,6 @@ class MIME_Viewer {
     /**
      * Can this driver render the the data inline?
      *
-     * @access public
-     *
      * @return boolean  True if the driver can display inline.
      */
     function canDisplayInline()
@@ -209,10 +203,8 @@ class MIME_Viewer {
      * Given a driver and an application, this returns the fully
      * qualified filesystem path to the driver source file.
      *
-     * @access public
-     *
-     * @param optional string $driver  Driver name.
-     * @param optional string $app     Application name.
+     * @param string $driver  Driver name.
+     * @param string $app     Application name.
      *
      * @return string  Filesystem path of the driver/application queried.
      */
@@ -232,7 +224,7 @@ class MIME_Viewer {
      * @param string $mimeType  MIME type to resolve.
      * @param string $module    Module in which to search for the driver.
      *
-     * @return object stdClass  Object with the following items:
+     * @return stdClass  Object with the following items:
      * <pre>
      * 'driver'  --  Name of driver (e.g. 'enscript')
      * 'exact'   --  Was the driver and exact match? (true/false)
@@ -326,8 +318,6 @@ class MIME_Viewer {
      * Given a MIME type, this function will return an appropriate
      * icon.
      *
-     * @access public
-     *
      * @param string $mimeType  The MIME type that we need an icon for.
      *
      * @return string  The URL to the appropriate icon.
@@ -337,23 +327,23 @@ class MIME_Viewer {
         $app = $GLOBALS['registry']->getApp();
         $ob = MIME_Viewer::_getIcon($mimeType, $app);
 
-        if (!isset($ob) && ($app != 'horde')) {
-            $obHorde = MIME_Viewer::_getIcon($mimeType, 'horde');
-            $icon = $obHorde->url;
-        } elseif (!isset($ob)) {
-            return null;
+        if ($ob === null) {
+            if ($app != 'horde') {
+                $obHorde = MIME_Viewer::_getIcon($mimeType, 'horde');
+                return ($obHorde === null) ? null : $obHorde->url;
+            } else {
+                return null;
+            }
         } elseif (($ob->match !== 0) && ($app != 'horde')) {
             $obHorde = MIME_Viewer::_getIcon($mimeType, 'horde');
-            if (!is_null($ob->match) && ($ob->match <= $obHorde->match)) {
-                $icon = $ob->url;
+            if ($ob->match !== null && $ob->match <= $obHorde->match) {
+                return $ob->url;
             } else {
-                $icon = $obHorde->url;
+                return $obHorde->url;
             }
         } else {
-            $icon = $ob->url;
+            return $ob->url;
         }
-
-        return $icon;
     }
 
     /**
@@ -364,12 +354,12 @@ class MIME_Viewer {
      *
      * @param string $mimeType  MIME type to get the icon for.
      *
-     * @return object stdClass  url:   URL to an icon, or null if none
-     *                                 could be found.
-     *                          exact: How exact the match is.
-     *                                 0 => 'exact', 1 => 'primary',
-     *                                 2 => 'driver', 3 => 'default'
-     *                                 or null.
+     * @return stdClass  url:   URL to an icon, or null if none
+     *                          could be found.
+     *                   exact: How exact the match is.
+     *                          0 => 'exact', 1 => 'primary',
+     *                          2 => 'driver', 3 => 'default'
+     *                          or null.
      */
     function _getIcon($mimeType, $module = 'horde')
     {
@@ -427,8 +417,6 @@ class MIME_Viewer {
      * Returns the character set used for the Viewer.
      * Should be overridden by individual drivers to perform custom tasks.
      *
-     * @access public
-     *
      * @return string  The character set used by this Viewer.
      */
     function getCharset()
@@ -439,8 +427,6 @@ class MIME_Viewer {
     /**
      * Return a configuration parameter for the current viewer.
      *
-     * @access public
-     *
      * @param string $param  The parameter name.
      *
      * @return mixed  The value of the parameter; returns null if the parameter
@@ -451,4 +437,15 @@ class MIME_Viewer {
         return (isset($this->_conf[$param])) ? $this->_conf[$param] : null;
     }
 
+    /**
+     * Should we force viewing of this MIME Part inline, regardless of the
+     * Content-Disposition of the MIME Part?
+     *
+     * @return boolean  Force viewing inline?
+     */
+    function forceInlineView()
+    {
+        return $this->_forceinline;
+    }
+
 }
