@@ -1,17 +1,16 @@
 <?php
 /**
- * The MIME_Viewer_deb class renders out lists of files in Debian
- * packages by using the dpkg tool to query the package.
+ * The MIME_Viewer_deb class renders out lists of files in Debian packages by
+ * using the dpkg tool to query the package.
  *
- * $Horde: framework/MIME/MIME/Viewer/deb.php,v 1.11.10.1 2005/01/03 12:19:05 jan Exp $
+ * $Horde: framework/MIME/MIME/Viewer/deb.php,v 1.11.10.8 2006/01/01 21:28:25 jan Exp $
  *
- * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
+ * Copyright 1999-2006 Anil Madhavapeddy <anil@recoil.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Anil Madhavapeddy <anil@recoil.org>
- * @version $Revision: 1.11.10.1 $
  * @since   Horde 1.3
  * @package Horde_MIME_Viewer
  */
@@ -20,9 +19,7 @@ class MIME_Viewer_deb extends MIME_Viewer {
     /**
      * Render the data.
      *
-     * @access public
-     *
-     * @param optional array $params  Any parameters the viewer may need.
+     * @param array $params  Any parameters the viewer may need.
      *
      * @return string  The rendered data.
      */
@@ -47,19 +44,17 @@ class MIME_Viewer_deb extends MIME_Viewer {
         }
         pclose($fh);
 
-        return '<pre>' . htmlentities($data) . '</pre>';
+        return '<pre>' . htmlspecialchars($data) . '</pre>';
     }
 
     /**
      * Return the MIME content type of the rendered content.
      *
-     * @access public
-     *
      * @return string  The content type of the output.
      */
     function getType()
     {
-        return 'text/html';
+        return 'text/html; charset=' . NLS::getCharset();
     }
 
 }
