@@ -1,33 +1,54 @@
 <?php
 
-/** @constant MAINTENANCE_YEARLY Do task yearly (First login after/on January 1). */
+/**
+ * Do task yearly (First login after/on January 1).
+ */
 define('MAINTENANCE_YEARLY', 1);
 
-/** @constant MAINTENANCE_MONTHLY Do task monthly (First login after/on first of month). */
+/**
+ * Do task monthly (First login after/on first of month).
+ */
 define('MAINTENANCE_MONTHLY', 2);
 
-/** @constant MAINTENANCE_WEEKLY Do task weekly (First login after/on a Sunday). */
+/**
+ * Do task weekly (First login after/on a Sunday).
+ */
 define('MAINTENANCE_WEEKLY', 3);
 
-/** @constant MAINTENANCE_DAILY Do task daily (First login of the day). */
+/**
+ * Do task daily (First login of the day).
+ */
 define('MAINTENANCE_DAILY', 4);
 
-/** @constant MAINTENANCE_EVERY Do task every login. */
+/**
+ * Do task every login.
+ */
 define('MAINTENANCE_EVERY', 5);
 
-/** @constant MAINTENANCE_FIRST_LOGIN Do task on first login only. */
+/**
+ * Do task on first login only.
+ */
 define('MAINTENANCE_FIRST_LOGIN', 6);
 
-/** @constant MAINTENANCE_OUTPUT_CONFIRM Confirmation-style output for maintenance page. */
+/**
+ * Confirmation-style output for maintenance page.
+ */
 define('MAINTENANCE_OUTPUT_CONFIRM', 7);
 
-/** @constant MAINTENANCE_OUTPUT_AGREE Agreement-style output for maintenance page. */
+/**
+ * Agreement-style output for maintenance page.
+ */
 define('MAINTENANCE_OUTPUT_AGREE', 8);
 
-/** @constant MAINTENANCE_OUTPUT_NOTICE Notice-style output for maintenance page. */
+/**
+ * Notice-style output for maintenance page.
+ */
 define('MAINTENANCE_OUTPUT_NOTICE', 9);
 
-/** @constant MAINTENANCE_DONE_PARAM The name of the URL parameter that indicates that the maintenance tasks are completed. */
+/**
+ * The name of the URL parameter that indicates that the maintenance tasks are
+ * completed.
+ */
 define('MAINTENANCE_DONE_PARAM', 'maintenance_done');
 
 /* Intervals hash - used to build select tables in preferences menu. */
@@ -43,15 +64,14 @@ $GLOBALS['intervals'] = &$intervals;
  * The Maintenance:: class provides a set of methods for dealing with
  * maintenance operations run upon login to Horde applications.
  *
- * $Horde: framework/Maintenance/Maintenance.php,v 1.52.10.2 2005/01/03 12:19:06 jan Exp $
+ * $Horde: framework/Maintenance/Maintenance.php,v 1.52.10.13 2006/01/01 21:28:26 jan Exp $
  *
- * Copyright 2001-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2001-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.52.10.2 $
  * @since   Horde 1.3.5
  * @package Horde_Maintenance
  */
@@ -72,43 +92,40 @@ class Maintenance {
      * This array should be filled in for each Horde module that extends
      *   the Maintenance class.
      *
-     * @var array $maint_tasks
+     * @var array
      */
     var $maint_tasks = array();
 
     /**
      * UNIX timestamp of the last maintenance run for user.
      *
-     * @var integer $_lastRun
+     * @var integer
      */
     var $_lastRun = 0;
 
     /**
      * The Maintenance_Tasklist object for this login.
      *
-     * @var object Maintenance_Tasklist $_tasklist
+     * @var Maintenance_Tasklist
      */
     var $_tasklist;
 
     /**
      * Array to store Maintenance_Task objects.
      *
-     * @var array $_modulesCache
+     * @var array
      */
     var $_taskCache = array();
 
     /**
-     * Attempts to return a concrete Maintenance_* object based on the
-     * module name passed into it.
+     * Attempts to return a concrete Maintenance_* object based on the module
+     * name passed into it.
      *
-     * @access public
+     * @param string $module  The name of the Horde module.
+     * @param array $params   A hash containing additional data needed by the
+     *                        constructor.
      *
-     * @param string $module           The name of the Horde module.
-     * @param optional array $params   A hash containing additional data
-     *                                 needed by the constructor.
-     *
-     * @return object Maintenance  The Maintenance object.
-     *                             Returns false on error.
+     * @return Maintenance  The Maintenance object, or false on error.
      */
     function &factory($module, $params = array())
     {
@@ -118,17 +135,17 @@ class Maintenance {
         include_once $registry->get('fileroot', $module) . '/lib/Maintenance/' . $module . '.php';
         $class = 'Maintenance_' . $module;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $maintenance = &new $class($params);
         } else {
-            return false;
+            $maintenance = false;
         }
+
+        return $maintenance;
     }
 
     /**
      * Constructor.
      *
-     * @access public
-     *
      * @param array $params  A hash containing the following entries:
      *                       'last_maintenance' => The last time maintenance was run (unix timestamp).
      */
@@ -149,8 +166,6 @@ class Maintenance {
      * and will redirect to the maintenance page if necessary.  This
      * is the function that should be called from the application upon
      * login.
-     *
-     * @access public
      */
     function runMaintenance()
     {
@@ -169,11 +184,9 @@ class Maintenance {
     }
 
     /**
-     * Do the necessary maintenance tasks for this loading of the
-     * maintenance page.  This is the function that is called from the
-     * maintenance page every time it is loaded.
-     *
-     * @access public
+     * Do the necessary maintenance tasks for this loading of the maintenance
+     * page.  This is the function that is called from the maintenance page
+     * every time it is loaded.
      *
      * @return integer  The display required for the maintenance page.
      */
@@ -190,7 +203,7 @@ class Maintenance {
         if (!$this->_tasklist->processed(true)) {
             if (count($tasks)) {
                 reset($tasks);
-                $this->_tasklist->unsetNewPage(key($tasks));
+                $this->_tasklist->setNewPage(key($tasks), false);
             }
         }
 
@@ -214,8 +227,6 @@ class Maintenance {
      * by default or not. Operations that have been locked by the
      * admin will return null.
      *
-     * @access public
-     *
      * @param string $pref  Name of the operation to get information for.
      *
      * @return array  1st element - Description of what the operation is about
@@ -240,8 +251,6 @@ class Maintenance {
      * Export variable names to use for creating select tables in the
      * preferences menu.
      *
-     * @access public
-     *
      * @return array  An array of variable names to be imported into the
      *                prefs.php namespace.
      */
@@ -261,11 +270,9 @@ class Maintenance {
     }
 
     /**
-     * Output hidden for elements for the POST form to ensure the
-     * calling script has the same POST elements as when the
-     * maintenance operations first run.
-     *
-     * @access public
+     * Output hidden for elements for the POST form to ensure the calling
+     * script has the same POST elements as when the maintenance operations
+     * first run.
      *
      * @return string  The form data.
      */
@@ -285,8 +292,6 @@ class Maintenance {
     /**
      * Return the URL needed for the maintenance form.
      *
-     * @access public
-     *
      * @return string  The URL to redirect to.
      */
     function getMaintenanceFormURL()
@@ -400,7 +405,7 @@ class Maintenance {
      *
      * @param string $modname  Name of the module to load.
      *
-     * @return object Maintenance_Task  A reference to the requested module.
+     * @return Maintenance_Task  A reference to the requested module.
      */
     function &_loadModule($modname)
     {
@@ -428,7 +433,7 @@ class Maintenance {
      */
     function _getInitialPageURL()
     {
-        return Util::addParameter($this->_tasklist->getTarget(), MAINTENANCE_DONE_PARAM, 1);
+        return Util::addParameter($this->_tasklist->getTarget(), array(MAINTENANCE_DONE_PARAM => 1, 'domaintenance' => 1), null, false);
     }
 
     /**
@@ -444,7 +449,7 @@ class Maintenance {
 
         /* We need to pass the cache ID and 'module' parameters. */
         $url = Horde::url($registry->get('webroot', 'horde') . '/services/maintenance.php', true);
-        $url = Util::addParameter($url, 'module', $this->_tasklist->getModule(), false);
+        $url = Util::addParameter($url, array('module' => $this->_tasklist->getModule(), 'domaintenance' => 1), null, false);
 
         return $url;
     }
@@ -497,7 +502,7 @@ class Maintenance {
         foreach ($tasks as $key => $val) {
             if ($val['newpage']) {
                 if ($this->_tasklist->processed()) {
-                    $this->_tasklist->unsetNewPage($key);
+                    $this->_tasklist->setNewPage($key, false);
                 }
                 break;
             } elseif ($val['confirmed'] ||
@@ -529,10 +534,15 @@ class Maintenance {
     function _needMaintenancePage()
     {
         $i = 0;
-        foreach ($this->_tasklist->getList() as $val) {
+        foreach ($this->_tasklist->getList() as $key => $val) {
             if ($val['newpage']) {
                 return $i;
             }
+            if (!empty($val['process']) &&
+                !Util::getFormData('domaintenance')) {
+                $this->_tasklist->setNewPage($key, true);
+                return $i;
+            }
             $i++;
         }
 
@@ -545,13 +555,12 @@ class Maintenance {
  * The Maintenance_Tasklist:: class is used to store the list of maintenance
  * tasks that need to be run during this login.
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.52.10.2 $
  * @since   Horde 3.0
  * @package Horde_Maintenance
  */
@@ -560,21 +569,21 @@ class Maintenance_Tasklist {
     /**
      * The Horde module running the maintenance tasks.
      *
-     * @var string $_module
+     * @var string
      */
     var $_module;
 
     /**
      * The URL of the web page to load after maintenance is complete.
      *
-     * @var string $_target
+     * @var string
      */
     var $_target;
 
     /**
      * POST data for the calling script.
      *
-     * @var array $_postdata
+     * @var array
      */
     var $_postdata;
 
@@ -585,45 +594,42 @@ class Maintenance_Tasklist {
      * VALUE:  Array => (
      *           'confirmed'  =>  boolean,
      *           'display'    =>  integer,
-     *           'newpage'    =>  boolean
+     *           'newpage'    =>  boolean,
+     *           'process'    =>  boolean
      *         )
      *
-     * @var array $_tasks
+     * @var array
      */
     var $_tasks = array();
 
     /**
      * Internal flag for addTask().
      *
-     * @var boolean $_addFlag
+     * @var boolean
      */
     var $_addFlag = false;
 
     /**
      * Has the tasklist been processed yet?
      *
-     * @var boolean $_processed
+     * @var boolean
      */
     var $_processed = false;
 
     /**
      * Constructor.
-     *
-     * @access public
      */
     function Maintenance_Tasklist()
     {
         global $registry;
 
         $this->_module = $registry->getApp();
-        $this->_target = Horde::selfURL(true);
+        $this->_target = Horde::selfUrl(true);
     }
 
     /**
      * Adds a task to the tasklist.
      *
-     * @access public
-     *
      * @param string $key         The name of the task to perform.
      * @param boolean $confirmed  Has the task been confirmed?
      * @param integer $display    The display type of the task.
@@ -647,24 +653,22 @@ class Maintenance_Tasklist {
     }
 
     /**
-     * Unsets the newpage flag for a task.
-     *
-     * @access public
+     * Sets the newpage flag for a task.
      *
      * @param string $task  The name of the task to alter.
+     * @param string $flag  How to set the flag.
      */
-    function unsetNewPage($task)
+    function setNewPage($task, $flag)
     {
         if ($this->inList($task)) {
-            $this->_tasks[$task]['newpage'] = false;
+            $this->_tasks[$task]['newpage'] = $flag;
+            $this->_tasks[$task]['process'] = !$flag;
         }
     }
 
     /**
      * Removes the task from the tasklist.
      *
-     * @access public
-     *
      * @param string $task  The name of the task to alter.
      */
     function removeTask($task)
@@ -677,8 +681,6 @@ class Maintenance_Tasklist {
     /**
      * Is this task already in the tasklist?
      *
-     * @access public
-     *
      * @param string $task  The name of the task.
      *
      * @return boolean  Whether the task is already in the tasklist.
@@ -691,8 +693,6 @@ class Maintenance_Tasklist {
     /**
      * Return the list of tasks.
      *
-     * @access public
-     *
      * @return array  The list of tasks that still need to be done.
      */
     function getList()
@@ -703,8 +703,6 @@ class Maintenance_Tasklist {
     /**
      * Return the Horde module the tasks are running under.
      *
-     * @access public
-     *
      * @return string  The Horde module name.
      */
     function getModule()
@@ -715,8 +713,6 @@ class Maintenance_Tasklist {
     /**
      * Return the POST data.
      *
-     * @access public
-     *
      * @return array  The POST data from the initial URL.
      */
     function getPostData()
@@ -727,8 +723,6 @@ class Maintenance_Tasklist {
     /**
      * Return the URL of the web page to load after maintenance is complete.
      *
-     * @access public
-     *
      * @return string  The target URL.
      */
     function getTarget()
@@ -740,9 +734,7 @@ class Maintenance_Tasklist {
      * Sets/displays the flag to show that tasklist has been processed at
      * least once.
      *
-     * @access public
-     *
-     * @param optional boolean $set  Set the flag?
+     * @param boolean $set  Set the flag?
      *
      * @return boolean  Has the tasklist been processed before?
      */
@@ -813,13 +805,12 @@ class Maintenance_Tasklist {
  *     text. doMaintenance() will ALWAYS be run.
  *     * This style will be displayed on its own, separate maintenance page. *
  *
- * Copyright 2001-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2001-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.52.10.2 $
  * @since   Horde 1.3.5
  * @package Horde_Maintenance
  */
@@ -831,14 +822,12 @@ class Maintenance_Task {
      *                  MAINTENANCE_OUTPUT_AGREE,
      *                  MAINTENANCE_OUTPUT_NOTICE
      *
-     * @var integer $_display_type
+     * @var integer
      */
     var $_display_type = MAINTENANCE_OUTPUT_CONFIRM;
 
     /**
      * Constructor
-     *
-     * @access public
      */
     function Maintenance_Task()
     {
@@ -847,8 +836,6 @@ class Maintenance_Task {
     /**
      * Do maintenance operation (if it has been confirmed).
      *
-     * @access public
-     *
      * @return boolean  Whether the maintenance operation was successful or
      *                  not.
      */
@@ -860,8 +847,6 @@ class Maintenance_Task {
     /**
      * Return description information for the maintenance page.
      *
-     * @access public
-     *
      * @return string  Description that will be displayed on the maintenance
      *                 confirmation page.
      */
@@ -873,8 +858,6 @@ class Maintenance_Task {
     /**
      * Returns the desired output type for the maintenance page.
      *
-     * @access public
-     *
      * @return integer  Desired output type for the maintenance confirmation
      *                  page.
      */
