@@ -3,7 +3,7 @@
  * Horde optimized interface to the MaxMind IP Address->Country
  * listing.
  *
- * $Horde: framework/NLS/NLS/GeoIP.php,v 1.10.10.7 2006/06/27 15:19:12 slusarz Exp $
+ * $Horde: framework/NLS/NLS/GeoIP.php,v 1.10.10.6 2006/01/01 21:28:27 jan Exp $
  *
  * Based on PHP geoip.inc library by MaxMind LLC:
  *   http://www.maxmind.com/download/geoip/api/php/
@@ -181,7 +181,7 @@ class NLS_GeoIP {
     function _open()
     {
         /* Return if we already have an object. */
-        if (!empty($this->_filehandle)) {
+        if (!empty($this->_gi)) {
             return true;
         }
 
@@ -220,7 +220,7 @@ class NLS_GeoIP {
             return false;
         }
         $addr = gethostbyname($name);
-        if (!$addr || ($addr == $name)) {
+        if (!$addr || $addr == $name) {
             return false;
         }
         return $this->countryIdByAddr($addr);
