@@ -1,16 +1,25 @@
 <?php
 
 // Preference constants (bitmasks)
-/** @constant _PREF_LOCKED Preference is administratively locked. */
+
+/**
+ * Preference is administratively locked.
+ */
 define('_PREF_LOCKED', 1);
 
-/** @constant _PREF_SHARED Preference is shared amongst applications. */
+/**
+ * Preference is shared amongst applications.
+ */
 define('_PREF_SHARED', 2);
 
-/** @constant _PREF_DIRTY Preference value has been changed. */
+/**
+ * Preference value has been changed.
+ */
 define('_PREF_DIRTY', 4);
 
-/** @constant _PREF_DEFAULT Preference value is the application default. */
+/**
+ * Preference value is the application default.
+ */
 define('_PREF_DEFAULT', 8);
 
 require_once 'Horde/String.php';
@@ -43,15 +52,14 @@ require_once 'Horde/String.php';
  *     'help'   => *Name of the entry in the XML help file*
  * );
  *
- * $Horde: framework/Prefs/Prefs.php,v 1.137 2004/10/19 00:45:29 chuck Exp $
+ * $Horde: framework/Prefs/Prefs.php,v 1.137.10.20 2006/02/28 06:06:46 slusarz Exp $
  *
- * Copyright 1999-2004 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
- * @version $Revision: 1.137 $
  * @since   Horde 1.3
  * @package Horde_Prefs
  */
@@ -69,7 +77,7 @@ class Prefs {
      *
      * @access private
      *
-     * @var array $_prefs
+     * @var array
      */
     var $_prefs = array();
 
@@ -78,44 +86,42 @@ class Prefs {
      * maintain the application scope between sets of preferences. By
      * default, all preferences belong to the "global" (Horde) scope.
      *
-     * @var string $scope
+     * @var string
      */
     var $_scope = 'horde';
 
     /**
-     * String containing the current username. This indicates the
-     * owner of the preferences.
+     * String containing the current username. This indicates the owner of the
+     * preferences.
      *
-     * @var string $user
+     * @var string
      */
     var $_user = '';
 
     /**
      * Boolean indicating whether preference caching should be used.
      *
-     * @var boolean $caching
+     * @var boolean
      */
     var $_caching = false;
 
     /**
      * Hash holding preferences with hook functions defined.
      *
-     * @var array $_hooks
+     * @var array
      */
     var $_hooks = array();
 
     /**
      * Have we run hook functions yet?
      *
-     * @var boolean $_hooksCalled
+     * @var boolean
      */
     var $_hooksCalled = null;
 
     /**
-     * Default constructor (must be called from each extending class
-     * in their constructors via parent::Prefs()).
-     *
-     * @access public
+     * Default constructor (must be called from each extending class in their
+     * constructors via parent::Prefs()).
      */
     function Prefs()
     {
@@ -125,8 +131,6 @@ class Prefs {
     /**
      * Returns the charset used by the concrete preference backend.
      *
-     * @access public
-     *
      * @return string  The preference backend's charset.
      */
     function getCharset()
@@ -135,12 +139,12 @@ class Prefs {
     }
 
     /**
-     * Updates the session-based preferences cache (if available) with
-     * the current set of preferences.
+     * Updates the session-based preferences cache (if available).
      *
-     * @access public
+     * @param string $pref  The preference to update.  If empty, will update
+     *                      the entire cache with the current set of prefs.
      */
-    function cacheUpdate()
+    function cacheUpdate($pref = null)
     {
         /* Return immediately if caching is disabled. */
         if (!$this->_caching) {
@@ -148,19 +152,26 @@ class Prefs {
         }
 
         if (isset($_SESSION['prefs_cache'])) {
+            if (is_null($pref)) {
+                $prefs = $this->_prefs;
+            } else {
+                $prefs = array($pref => $this->_prefs[$pref]);
+            }
+
             /* Place each preference in the cache according to its
              * scope. */
-            foreach ($this->_prefs as $name => $pref) {
-                $_SESSION['prefs_cache'][$this->getScope($name)][$name] = $pref;
+            foreach ($prefs as $name => $pref) {
+                $scope = $this->getScope($name);
+                if (isset($_SESSION['prefs_cache'][$scope])) {
+                    $_SESSION['prefs_cache'][$scope][$name] = $pref;
+                }
             }
         }
     }
 
     /**
-     * Tries to find the requested preferences in the cache. If they
-     * exist, update the $prefs hash with the cached values.
-     *
-     * @access public
+     * Tries to find the requested preferences in the cache. If they exist,
+     * update the $prefs hash with the cached values.
      *
      * @return boolean  True on success, false on failure.
      */
@@ -172,18 +183,10 @@ class Prefs {
         }
 
         if (isset($_SESSION['prefs_cache']['horde']) &&
-            !empty($_SESSION['prefs_cache']['_filled'][$this->_scope]) &&
             isset($_SESSION['prefs_cache'][$this->_scope])) {
 
             /* Restore global preferences. */
-            foreach ($_SESSION['prefs_cache']['horde'] as $name => $pref) {
-                $this->_prefs[$name] = $pref;
-            }
-
-            /* Restore application-specific preferences. */
-            foreach ($_SESSION['prefs_cache'][$this->_scope] as $name => $pref) {
-                $this->_prefs[$name] = $pref;
-            }
+            $this->_prefs = array_merge($this->_prefs, $_SESSION['prefs_cache']['horde'], $_SESSION['prefs_cache'][$this->_scope]);
 
             return true;
         }
@@ -194,11 +197,9 @@ class Prefs {
     /**
      * Adds a new preference entry to the $prefs hash.
      *
-     * @access public
-     *
-     * @param string $pref            The name of the preference to add.
-     * @param optional string $val    The initial value of the preference.
-     * @param optional integer $mask  The initial bitmask of the preference.
+     * @param string $pref   The name of the preference to add.
+     * @param string $val    The initial value of the preference.
+     * @param integer $mask  The initial bitmask of the preference.
      */
     function add($pref, $val = '', $mask = 0)
     {
@@ -210,14 +211,14 @@ class Prefs {
     /**
      * Removes a preference entry from the $prefs hash.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to remove.
      */
     function remove($pref)
     {
         if (is_array($this->_prefs)) {
+            $scope = $this->getScope($pref);
             unset($this->_prefs[$pref]);
+            unset($_SESSION['prefs_cache'][$scope][$pref]);
         }
     }
 
@@ -225,8 +226,6 @@ class Prefs {
      * Sets the given preferences ($pref) to the specified value
      * ($val), if the preference is modifiable.
      *
-     * @access public
-     *
      * @param string $pref      The name of the preference to modify.
      * @param string $val       The new value for this preference.
      * @param boolean $convert  If true the preference value gets converted
@@ -246,6 +245,11 @@ class Prefs {
         return $this->_setValue($pref, $val, true, $convert);
     }
 
+    function __set($name, $value)
+    {
+        return $this->setValue($name, $value);
+    }
+
     /**
      * Sets the given preferences ($pref) to the specified value
      * ($val), whether or not the preference is user-modifiable, unset
@@ -305,14 +309,14 @@ class Prefs {
             $this->setDirty($pref, true);
         }
 
+        $this->cacheUpdate($pref);
+
         return true;
     }
 
     /**
      * Returns the value of the requested preference.
      *
-     * @access public
-     *
      * @param string $pref      The name of the preference to retrieve.
      * @param boolean $convert  If true the preference value gets converted
      *                          from the backend's charset to the current
@@ -322,18 +326,26 @@ class Prefs {
      */
     function getValue($pref, $convert = true)
     {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
         return (isset($this->_prefs[$pref]['v'])) ?
             ($convert ?
-             $this->convertFromDriver($this->_prefs[$pref]['v'], NLS::getCharset()) :
+             $this->convertFromDriver($this->_prefs[$pref]['v'], $charset) :
              $this->_prefs[$pref]['v']) :
             null;
     }
 
+    function __get($name)
+    {
+        return $this->getValue($name);
+    }
+
     /**
      * Modifies the "locked" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref   The name of the preference to modify.
      * @param boolean $bool  The new boolean value for the "locked" bit.
      */
@@ -345,8 +357,6 @@ class Prefs {
     /**
      * Returns the state of the "locked" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  The boolean state of $pref's "locked" bit.
@@ -359,8 +369,6 @@ class Prefs {
     /**
      * Modifies the "shared" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref   The name of the preference to modify.
      * @param boolean $bool  The new boolean value for the "shared" bit.
      */
@@ -372,8 +380,6 @@ class Prefs {
     /**
      * Returns the state of the "shared" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  The boolean state of $pref's "shared" bit.
@@ -397,8 +403,6 @@ class Prefs {
     /**
      * Returns the state of the "dirty" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  The boolean state of $pref's "dirty" bit.
@@ -411,12 +415,8 @@ class Prefs {
     /**
      * Modifies the "default" bit for the given preference.
      *
-     * @access public
-     *
      * @param string $pref   The name of the preference to modify.
      * @param boolean $bool  The new boolean value for the "default" bit.
-     *
-     * @since Horde 2.1
      */
     function setDefault($pref, $bool)
     {
@@ -426,8 +426,6 @@ class Prefs {
     /**
      * Returns the default value of the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to get the default for.
      *
      * @return string       The preference's default value.
@@ -443,14 +441,10 @@ class Prefs {
      * Determines if the current preference value is the default
      * value from prefs.php or a user defined value
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  True if the preference is the application default
      *                  value.
-     *
-     * @since Horde 2.1
      */
     function isDefault($pref)
     {
@@ -497,8 +491,6 @@ class Prefs {
     /**
      * Determines whether the current preference is empty.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to check.
      *
      * @return boolean  True if the preference is empty.
@@ -511,8 +503,6 @@ class Prefs {
     /**
      * Returns the scope of the given preference.
      *
-     * @access public
-     *
      * @param string $pref  The name of the preference to examine.
      *
      * @return string  The scope of the $pref.
@@ -582,19 +572,14 @@ class Prefs {
      * This function provides common cleanup functions for all of the driver
      * implementations.
      *
-     * @param optional boolean $all  Clean up all Horde preferences.
+     * @param boolean $all  Clean up all Horde preferences.
      */
     function cleanup($all = false)
     {
         /* Remove this scope from the preferences cache, if it
            exists. */
-        if (isset($_SESSION['prefs_cache'])) {
-            if (isset($_SESSION['prefs_cache'][$this->_scope])) {
-                unset($_SESSION['prefs_cache'][$this->_scope]);
-            }
-            if (isset($_SESSION['prefs_cache']['_filled'][$this->_scope])) {
-                unset($_SESSION['prefs_cache']['_filled'][$this->_scope]);
-            }
+        if (isset($_SESSION['prefs_cache'][$this->_scope])) {
+            unset($_SESSION['prefs_cache'][$this->_scope]);
         }
 
         /* Perform a Horde-wide cleanup? */
@@ -644,12 +629,10 @@ class Prefs {
     }
 
     /**
-     * Populates the $prefs hash with new entries and externally
-     * defined default values.
+     * Populates the $prefs hash with new entries and externally defined
+     * default values.
      *
      * @param string $app  The application to load defaults for.
-     *
-     * @access public
      */
     function _setDefaults($app)
     {
@@ -719,7 +702,7 @@ class Prefs {
      */
     function _callHooks()
     {
-        if (!is_null($this->_hooksCalled) && !Auth::getAuth()) {
+        if (!is_null($this->_hooksCalled)) {
             return;
         }
 
@@ -733,7 +716,8 @@ class Prefs {
         include $registry->get('fileroot', 'horde') . '/config/hooks.php';
         foreach ($this->_hooks as $pref) {
             if ($this->isLocked($pref) ||
-                !$this->getValue($pref)) {
+                !$this->getValue($pref) ||
+                $this->isDefault($pref)) {
                 $func = '_prefs_hook_' . $pref;
                 if (function_exists($func)) {
                     $this->_setValue($pref, $func($this->_user));
@@ -745,24 +729,20 @@ class Prefs {
     /**
      * Attempts to return a concrete Prefs instance based on $driver.
      *
-     * @param mixed $driver              The type of concrete Prefs subclass
-     *                                   to return. This is based on the
-     *                                   storage driver ($driver). The code is
-     *                                   dynamically included. If $driver is
-     *                                   an array, then we will look in
-     *                                   $driver[0]/lib/Prefs/ for the subclass
-     *                                   implementation named $driver[1].php.
-     * @param optional string $scope     The scope for this set of preferences.
-     * @param optional string $user      The name of the user who owns this
-     *                                   set of preferences.
-     * @param optional string $password  The password associated with $user.
-     * @param optional array $params     A hash containing any additional
-     *                                   configuration or connection
-     *                                   parameters a subclass might need.
-     * @param optional boolean $caching  Should caching be used?
+     * @param mixed $driver     The type of concrete Prefs subclass to return.
+     *                          If $driver is an array, then we will look in
+     *                          $driver[0]/lib/Prefs/ for the subclass
+     *                          implementation named $driver[1].php.
+     * @param string $scope     The scope for this set of preferences.
+     * @param string $user      The name of the user who owns this set of
+     *                          preferences.
+     * @param string $password  The password associated with $user.
+     * @param array $params     A hash containing any additional configuration
+     *                          or connection parameters a subclass might need.
+     * @param boolean $caching  Should caching be used?
      *
-     * @return object Prefs  The newly created concrete Prefs instance, or
-     *                       false on error.
+     * @return Prefs  The newly created concrete Prefs instance, or false on
+     *                error.
      */
     function &factory($driver, $scope = 'horde', $user = '', $password = '',
                       $params = null, $caching = true)
@@ -777,7 +757,6 @@ class Prefs {
         if ($caching) {
             if (!isset($_SESSION['prefs_cache'])) {
                 $_SESSION['prefs_cache'] = array();
-                $_SESSION['prefs_cache']['_filled'] = array();
             }
         }
 
@@ -807,12 +786,15 @@ class Prefs {
         } else {
             @include_once 'Horde/Prefs/' . $driver . '.php';
         }
+
         $class = 'Prefs_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($user, $password, $scope, $params, $caching);
+            $prefs = &new $class($user, $password, $scope, $params, $caching);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $prefs = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $prefs;
     }
 
     /**
@@ -825,33 +807,24 @@ class Prefs {
      *
      * This method must be invoked as: $var = &Prefs::singleton()
      *
-     * @param mixed $driver              The type of concrete Prefs subclass
-     *                                   to return. This is based on the
-     *                                   storage driver ($driver). The code is
-     *                                   dynamically included. If $driver is
-     *                                   an array, then we will look in
-     *                                   $driver[0]/lib/Prefs/ for the subclass
-     *                                   implementation named $driver[1].php.
-     * @param optional string $scope     The scope for this set of preferences.
-     * @param optional string $user      The name of the user who owns this
-     *                                   set of preferences.
-     * @param optional string $password  The password associated with $user.
-     * @param optional array $params     A hash containing any additional
-     *                                   configuration or connection
-     *                                   parameters a subclass might need.
-     * @param optional boolean $caching  Should caching be used?
+     * @param mixed $driver     The type of concrete Prefs subclass to return.
+     *                          If $driver is an array, then we will look in
+     *                          $driver[0]/lib/Prefs/ for the subclass
+     *                          implementation named $driver[1].php.
+     * @param string $scope     The scope for this set of preferences.
+     * @param string $user      The name of the user who owns this set of
+     *                          preferences.
+     * @param string $password  The password associated with $user.
+     * @param array $params     A hash containing any additional configuration
+     *                          or connection parameters a subclass might need.
+     * @param boolean $caching  Should caching be used?
      *
-     * @return object Prefs  The concrete Prefs reference, or false on an
-     *                       error.
+     * @return Prefs  The concrete Prefs reference, or false on an error.
      */
     function &singleton($driver, $scope = 'horde', $user = '', $password = '',
                         $params = null, $caching = true)
     {
-        static $instances;
-
-        if (!isset($instances)) {
-            $instances = array();
-        }
+        static $instances = array();
 
         if (is_null($params)) {
             $params = Horde::getDriverConfig('prefs', $driver);
