@@ -2,52 +2,53 @@
 /**
  * Preferences storage implementation for PHP's LDAP extention.
  *
- * Required parameters:
- * ====================
- *   'basedn'    --  The base DN for the LDAP server.
- *   'hostspec'  --  The hostname of the LDAP server.
- *   'uid'       --  The username search key.
+ * Required parameters:<pre>
+ *   'basedn'    The base DN for the LDAP server.
+ *   'hostspec'  The hostname of the LDAP server.
+ *   'uid'       The username search key.</pre>
  *
- * Optional parameters:
- * ====================
- *   'password'  --  'rootdn's password for bind authentication.
- *   'port'      --  The port of the LDAP server.
- *                   DEFAULT: 389
- *   'rootdn'    --  The DN of the root (administrative) account to bind for
- *                   write operations.
- *   'username'  --  TODO
- *   'version'   --  The version of the LDAP protocol to use.
- *                   DEFAULT: NONE (system default will be used)
+ * Optional parameters:<pre>
+ *   'password'  'rootdn's password for bind authentication.
+ *   'port'      The port of the LDAP server.
+ *               DEFAULT: 389
+ *   'rootdn'    The DN of the root (administrative) account to bind for
+ *               write operations.
+ *   'username'  TODO
+ *   'version'   The version of the LDAP protocol to use.
+ *               DEFAULT: NONE (system default will be used)</pre>
  *
- * NOTE: parameter 'username' for $params has been deprecated. Use 'rootdn'.
+ * NOTE: parameter 'username' has been deprecated. Use 'rootdn'.
  *
  *
  * If setting up as the Horde preference handler in conf.php, the following
  * is an example configuration.
  * The schemas needed for ldap are in horde/scripts/ldap.
  *
+ * <pre>
  *   $conf['prefs']['driver'] = 'ldap';
  *   $conf['prefs']['params']['hostspec'] = 'localhost';
  *   $conf['prefs']['params']['port'] = '389';
  *   $conf['prefs']['params']['basedn'] = 'dc=example,dc=org';
  *   $conf['prefs']['params']['uid'] = 'mail';
+ * </pre>
  *
  * The following is valid but would only be necessary if users
  * do NOT have permission to modify their own LDAP accounts.
  *
+ * <pre>
  *   $conf['prefs']['params']['rootdn'] = 'cn=Manager,dc=example,dc=org';
  *   $conf['prefs']['params']['password'] = 'password';
+ * </pre>
  *
  *
- * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.1 2005/01/03 12:19:11 jan Exp $
+ * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.18 2006/02/28 05:59:19 slusarz Exp $
  *
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
- * @version $Revision: 1.85.10.1 $
  * @since   Horde 1.3
  * @package Horde_Prefs
  */
@@ -56,49 +57,45 @@ class Prefs_ldap extends Prefs {
     /**
      * Hash containing connection parameters.
      *
-     * @var array $params
+     * @var array
      */
     var $_params = array();
 
     /**
      * Handle for the current LDAP connection.
      *
-     * @var integer $connection
+     * @var resource
      */
     var $_connection;
 
     /**
      * Boolean indicating whether or not we're connected to the LDAP server.
      *
-     * @var boolean $_connected
+     * @var boolean
      */
     var $_connected = false;
 
     /**
      * String holding the user's DN.
      *
-     * @var string $dn
+     * @var string
      */
     var $_dn = '';
 
-
     /**
      * Constructs a new LDAP preferences object.
      *
-     * @access public
-     *
-     * @param string $user               The user who owns these preferences.
-     * @param string $password           The password associated with $user.
-     * @param string $scope              The current application scope.
-     * @param array $params              A hash containing connection
-     *                                   parameters.
-     * @param optional boolean $caching  Should caching be used?
+     * @param string $user      The user who owns these preferences.
+     * @param string $password  The password associated with $user.
+     * @param string $scope     The current application scope.
+     * @param array $params     A hash containing connection parameters.
+     * @param boolean $caching  Should caching be used?
      */
     function Prefs_ldap($user, $password, $scope = '',
                         $params = array(), $caching = false)
     {
         if (!Util::extensionExists('ldap')) {
-            Horde::fatal(PEAR::raiseError(_("Prefs_ldap: Required LDAP extension not found."), __FILE__, __LINE__));
+            Horde::fatal(PEAR::raiseError(_("Prefs_ldap: Required LDAP extension not found.")), __FILE__, __LINE__);
         }
 
         $this->_user = $user;
@@ -138,9 +135,14 @@ class Prefs_ldap extends Prefs {
         }
 
         Horde::assertDriverConfig($this->_params, 'prefs',
-            array('hostspec', 'basedn', 'uid', 'rootdn', 'password'),
+            array('hostspec', 'basedn', 'uid', 'password'),
             'preferences LDAP');
 
+        /* Allow for multiple uid search fields */
+        if (!is_array($this->_params['uid'])) {
+            $this->_params['uid'] = array($this->_params['uid']);
+        }
+
         /* Connect to the LDAP server anonymously. */
         $conn = ldap_connect($this->_params['hostspec'], $this->_params['port']);
         if (!$conn) {
@@ -175,58 +177,66 @@ class Prefs_ldap extends Prefs {
             return false;
         }
 
-        /* Define the DN of the current user */
-        $this->_dn = sprintf('%s=%s,%s', $this->_params['uid'],
-                            $this->_user,
-                            $this->_params['basedn']);
-
-        /* And the DN of the authenticating user (may be the same as above) */
-        if (!empty($this->_params['rootdn'])) {
-            $bind_dn = $this->_params['rootdn'];
-        } else {
-            $bind_dn = sprintf('%s=%s,%s', $this->_params['uid'],
-                               $this->_params['username'],
-                               $this->_params['basedn']);
-        }
-
         /* Store the connection handle at the instance level. */
         $this->_connection = $conn;
         $this->_connected = true;
 
-        /* Bind to the LDAP server as the authenticating user. */
-        $bind = @ldap_bind($this->_connection, $bind_dn,
-                           $this->_params['password']);
-        if (!$bind) {
-            Horde::logMessage(
-                sprintf('Bind to server %s:%d with DN %s failed: [%d] %s',
-                        $this->_params['hostspec'],
-                        $this->_params['port'],
-                        $bind_dn,
-                        ldap_errno($this->_connection),
-                        ldap_error($this->_connection)),
-                __FILE__, __LINE__);
-            return false;
+        if (!$this->_params['fetchdn']) {
+            /* Define the DN of the current user. */
+            $this->_dn = sprintf('%s=%s,%s', $this->_params['uid'][0],
+                                 $this->_user,
+                                 $this->_params['basedn']);
+
+            /* And the DN of the authenticating user (may be the same as above). */
+            if (!empty($this->_params['rootdn'])) {
+                $bind_dn = $this->_params['rootdn'];
+            } else {
+                $bind_dn = sprintf('%s=%s,%s', $this->_params['uid'][0],
+                                   $this->_params['username'],
+                                   $this->_params['basedn']);
+            }
+
+            /* Bind to the LDAP server as the authenticating user. */
+            $bind = @ldap_bind($this->_connection, $bind_dn,
+                               $this->_params['password']);
+            if (!$bind) {
+                Horde::logMessage(
+                    sprintf('Bind to server %s:%d with DN %s failed: [%d] %s',
+                            $this->_params['hostspec'],
+                            $this->_params['port'],
+                            $bind_dn,
+                            ldap_errno($this->_connection),
+                            ldap_error($this->_connection)),
+                    __FILE__, __LINE__);
+                return false;
+            }
         }
 
         /* Search for the user's full DN. */
-        $search = ldap_search($this->_connection, $this->_params['basedn'],
-                              $this->_params['uid'] . '=' . $this->_user,
-                              array('dn'));
-        if ($search) {
-            $result = ldap_get_entries($this->_connection, $search);
-            if ($result && !empty($result[0]['dn'])) {
-                $this->_dn = $result[0]['dn'];
+        foreach ($this->_params['uid'] as $uid_field) {
+            $search = ldap_search($this->_connection, $this->_params['basedn'],
+                                  $uid_field . '=' . $this->_user, array('dn'));
+            if ($search) {
+                $result = ldap_get_entries($this->_connection, $search);
+                if ($result && $result['count'] > 0) {
+                    $this->_dn = $result[0]['dn'];
+                    if ($this->_params['fetchdn']) {
+                        // Re-bind as the user.
+                        $bind = @ldap_bind($this->_connection, $this->_dn,
+                                           $this->_params['password']);
+                    }
+                    return true;
+                }
             }
-        } else {
-            Horde::logMessage(
-                sprintf('Failed to retrieve user\'s DN: [%d] %s',
-                        ldap_errno($this->_connection),
-                        ldap_error($this->_connection)),
-                __FILE__, __LINE__);
-            return false;
         }
 
-        return true;
+        Horde::logMessage(
+            sprintf('Failed to retrieve user\'s DN: [%d] %s',
+                    ldap_errno($this->_connection),
+                    ldap_error($this->_connection)),
+            __FILE__, __LINE__);
+
+        return false;
     }
 
     /**
@@ -254,8 +264,6 @@ class Prefs_ldap extends Prefs {
      * @access private
      *
      * @return integer  1 on error, 0 on success.
-     *
-     * @since Horde 2.1
      */
     function _rebindProc($conn, $who)
     {
@@ -266,7 +274,7 @@ class Prefs_ldap extends Prefs {
         if (!empty($this->_params['rootdn'])) {
             $bind_dn = $this->_params['rootdn'];
         } else {
-            $bind_dn = sprintf('%s=%s,%s', $this->_params['uid'],
+            $bind_dn = sprintf('%s=%s,%s', $this->_params['uid'][0],
                                $this->_params['username'],
                                $this->_params['basedn']);
         }
@@ -297,10 +305,7 @@ class Prefs_ldap extends Prefs {
     }
 
     /**
-     * Retrieves the requested set of preferences from the user's LDAP
-     * entry.
-     *
-     * @access public
+     * Retrieves the requested set of preferences from the user's LDAP entry.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -320,16 +325,22 @@ class Prefs_ldap extends Prefs {
         /* Only fetch the fields for the attributes we need. */
         $attrs = array('hordePrefs');
         if (strcmp($this->_scope, 'horde') != 0) {
-            array_push($attrs, $this->_scope . 'Prefs');
+            $attrs[] = $this->_scope . 'Prefs';
         }
 
         /* Search for the multi-valued field containing the array of
            preferences. */
-        $search = ldap_search($this->_connection, $this->_params['basedn'],
-                              $this->_params['uid'] . '=' . $this->_user, $attrs);
-        if ($search) {
-            $result = ldap_get_entries($this->_connection, $search);
-        } else {
+        foreach ($this->_params['uid'] as $uid_field) {
+            $search = ldap_search($this->_connection, $this->_params['basedn'],
+                                  $uid_field . '=' . $this->_user, $attrs);
+            if ($search) {
+                $result = ldap_get_entries($this->_connection, $search);
+                if ($result && $result['count'] > 0) {
+                    break;
+                }
+            }
+        }
+        if (!isset($result)) {
             Horde::logMessage('Failed to connect to LDAP preferences server.', __FILE__, __LINE__);
         }
 
@@ -383,11 +394,6 @@ class Prefs_ldap extends Prefs {
                 }
             }
 
-            /* Make sure we know that we've loaded these
-             * preferences. */
-            $_SESSION['prefs_cache']['_filled']['horde'] = true;
-            $_SESSION['prefs_cache']['_filled'][$this->_scope] = true;
-
             /* Call hooks. */
             $this->_callHooks();
         } else {
@@ -404,8 +410,6 @@ class Prefs_ldap extends Prefs {
     /**
      * Stores preferences to the LDAP server.
      *
-     * @access public
-     *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
     function store()
@@ -441,30 +445,33 @@ class Prefs_ldap extends Prefs {
         /* Entries must have the objectclasses 'top' and 'hordeperson'
          * to successfully store LDAP prefs. Check for both of them,
          * and add them if necessary. */
-        $search = ldap_search($this->_connection, $this->_params['basedn'],
-                              $this->_params['uid'] . '=' . $this->_user,
-                              array('objectclass'));
-        if ($search) {
-            $result = ldap_get_entries($this->_connection, $search);
-            if ($result) {
-                $top = false;
-                $hordeperson = false;
+        foreach ($this->_params['uid'] as $uid_field) {
+            $search = ldap_search($this->_connection, $this->_params['basedn'],
+                                  $uid_field . '=' . $this->_user,
+                                  array('objectclass'));
+            if ($search) {
+                $result = ldap_get_entries($this->_connection, $search);
+                if ($result && $result['count'] > 0) {
+                    $top = false;
+                    $hordeperson = false;
 
-                for ($i = 0; $i < $result[0]['objectclass']['count']; $i++) {
-                    if ($result[0]['objectclass'][$i] == 'top') {
-                        $top = true;
-                    } elseif ($result[0]['objectclass'][$i] == 'hordePerson') {
-                        $hordeperson = true;
+                    for ($i = 0; $i < $result[0]['objectclass']['count']; $i++) {
+                        if ($result[0]['objectclass'][$i] == 'top') {
+                            $top = true;
+                        } elseif ($result[0]['objectclass'][$i] == 'hordePerson') {
+                            $hordeperson = true;
+                        }
                     }
-                }
 
-                /* Add any missing objectclasses. */
-                if (!$top) {
-                    ldap_mod_add($this->_connection, $this->_dn, array('objectclass' => 'top'));
-                }
+                    /* Add any missing objectclasses. */
+                    if (!$top) {
+                        ldap_mod_add($this->_connection, $this->_dn, array('objectclass' => 'top'));
+                    }
 
-                if (!$hordeperson) {
-                    ldap_mod_add($this->_connection, $this->_dn, array('objectclass' => 'hordePerson'));
+                    if (!$hordeperson) {
+                        ldap_mod_add($this->_connection, $this->_dn, array('objectclass' => 'hordePerson'));
+                    }
+                    continue;
                 }
             }
         }
@@ -489,12 +496,55 @@ class Prefs_ldap extends Prefs {
         return $updated;
     }
 
+    /**
+     * Clears all preferences from the LDAP backend.
+     */
+    function clear()
+    {
+        /* Make sure we are connected. */
+        $this->_connect();
+
+        $attrs = $GLOBALS['registry']->listApps(array('inactive', 'active', 'hidden', 'notoolbar', 'admin'));
+        foreach ($attrs as $key => $val) {
+            $attrs[$key] = $val . 'Prefs';
+        }
+
+        $search = ldap_read($this->_connection, $this->_dn,
+                            'objectClass=hordePerson', $attrs, 1);
+
+        if ($search) {
+            $result = ldap_get_entries($this->_connection, $search);
+        } else {
+            Horde::logMessage('Failed to connect to LDAP preferences server.', __FILE__, __LINE__);
+        }
+
+        $attrs = array();
+        if ($result) {
+            for ($i = 0; $i < $result[0]['count']; $i++) {
+                $attrs[$result[0][$i]] = array();
+            }
+            if (!ldap_mod_del($this->_connection, $this->_dn, $attrs)) {
+                Horde::logMessage(
+                    sprintf('Unable to clear preferences: [%d] %s',
+                            ldap_errno($this->_connection),
+                            ldap_error($this->_connection)),
+                    __FILE__, __LINE__);
+            }
+        } else {
+            Horde::logMessage(
+                sprintf('Unable to clear preferences: [%d] %s',
+                        ldap_errno($this->_connection),
+                        ldap_error($this->_connection)),
+                __FILE__, __LINE__);
+        }
+
+        $this->cleanup(true);
+    }
+
     /**
      * Perform cleanup operations.
      *
-     * @access public
-     *
-     * @param optional boolean $all  Cleanup all Horde preferences.
+     * @param boolean $all  Cleanup all Horde preferences.
      */
     function cleanup($all = false)
     {
