@@ -4,15 +4,10 @@
  */
 
 /**
- * PEAR DB layer.
+ * Required database layer.
  */
 require_once 'DB.php';
 
-/**
- * Horde_String class.
- */
-require_once 'Horde/String.php';
-
 /**
  * Preferences storage implementation for PHP's PEAR database
  * abstraction layer.
@@ -38,9 +33,9 @@ require_once 'Horde/String.php';
  * The table structure for the Prefs system is in
  * scripts/sql/horde_prefs.sql.
  *
- * $Horde: framework/Prefs/Prefs/sql.php,v 1.91.10.21 2007/01/02 13:54:35 jan Exp $
+ * $Horde: framework/Prefs/Prefs/sql.php,v 1.91.10.19 2006/02/28 05:59:19 slusarz Exp $
  *
- * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
