@@ -1,24 +1,23 @@
 <?php
 
-include_once 'Horde/RPC.php';
-include_once 'Horde/SyncML.php';
-include_once 'Horde/SyncML/State.php';
-include_once 'Horde/SyncML/Command/Status.php';
+require_once 'SyncML.php';
+require_once 'SyncML/State.php';
+require_once 'SyncML/Command/Status.php';
 
 /**
- * The Horde_RPC_syncml class provides a SyncML implementation of the
- * Horde RPC system.
+ * The Horde_RPC_syncml class provides a SyncML implementation of the Horde
+ * RPC system.
  *
- * $Horde: framework/RPC/RPC/syncml.php,v 1.18.10.1 2005/01/03 12:19:12 jan Exp $
+ * $Horde: framework/RPC/RPC/syncml.php,v 1.18.10.6 2006/01/01 21:28:33 jan Exp $
  *
- * Copyright 2003-2005 Chuck Hagenbuch <chuck@horde.org>, Anthony Mills <amills@pyramid6.com>
+ * Copyright 2003-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2003-2006 Anthony Mills <amills@pyramid6.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Anthony Mills <amills@pyramid6.com>
- * @version $Revision: 1.18.10.1 $
  * @since   Horde 3.0
  * @package Horde_RPC
  */
@@ -26,28 +25,33 @@ class Horde_RPC_syncml extends Horde_RPC {
 
     /**
      * Output ContentHandler used to output XML events.
-     * @var object $_output
+     *
+     * @var object
      */
     var $_output;
 
     /**
-     * @var integer $_xmlStack
+     * @var integer
      */
     var $_xmlStack = 0;
 
     /**
      * Debug directory, if set will store copies of all packets.
+     *
+     * @var string
      */
     var $_debugDir = '/tmp/sync';
 
     /**
      * Default character set.  Only supports UTF-8(ASCII?).
+     *
+     * @var string
      */
     var $_charset = 'UTF-8';
 
     /**
-     * SyncML handles authentication internally, so bypass the RPC
-     * framework auth check by just returning true here.
+     * SyncML handles authentication internally, so bypass the RPC framework
+     * auth check by just returning true here.
      */
     function authorize()
     {
@@ -59,17 +63,17 @@ class Horde_RPC_syncml extends Horde_RPC {
      *
      * @param string $request  The raw request string.
      *
-     * @return string   The XML encoded response from the server.
+     * @return string  The XML encoded response from the server.
      */
     function getResponse($request)
     {
-        // Catch any errors/warnings/notices that may get thrown while
-        // processing. Don't want to let anything go to the client
-        // that's not part of the valid response.
+        /* Catch any errors/warnings/notices that may get thrown while
+         * processing. Don't want to let anything go to the client that's not
+         * part of the valid response. */
         ob_start();
 
-        // Very useful for debugging. Logs XML packets to
-        // $this->_debugDir.
+        /* Very useful for debugging. Logs WBXML packets to
+         * $this->_debugDir. */
         if (!empty($this->_debugDir) && is_dir($this->_debugDir)) {
             $packetNum = @intval(file_get_contents($this->_debugDir . '/syncml.packetnum'));
             if (!isset($packetNum)) {
@@ -83,21 +87,13 @@ class Horde_RPC_syncml extends Horde_RPC {
             }
         }
 
-        // $this->_output can be already set by a subclass.
-        // The subclass can use it's own ContentHandler and bypass
-        // this classes use of the ContentHandler.  In this case
-        // no output is return from this method, instead the output
-        // comes from the subclasses ContentHandler
-        // We may need to add this code back when we get to the content
-        //if (!isset($this->_output)) {
-            include_once 'XML/WBXML/ContentHandler.php';
-            $this->_output = &new XML_WBXML_ContentHandler();
-        //}
+        require_once 'XML/WBXML/ContentHandler.php';
+        $this->_output = &new XML_WBXML_ContentHandler();
 
         $this->_parse($request);
         $response = $this->_output->getOutput();
 
-        // Very useful for debugging.
+        /* Very useful for debugging. */
         if (!empty($this->_debugDir) && is_dir($this->_debugDir)) {
             $f = @fopen($this->_debugDir . '/syncml_server_' . $packetNum . '.xml', 'wb');
             if ($f) {
@@ -112,11 +108,12 @@ class Horde_RPC_syncml extends Horde_RPC {
             }
         }
 
-        // Clear the output buffer that we started above, and log
-        // anything that came up for later debugging.
+        /* Clear the output buffer that we started above, and log anything
+         * that came up for later debugging. */
         $errorLogging = ob_get_clean();
         if (!empty($errorLogging)) {
-            Horde::logMessage($errorLogging, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            Horde::logMessage('SyncML: caught output=' .
+                              $errorLogging, __FILE__, __LINE__, PEAR_LOG_DEBUG);
         }
 
         return $response;
@@ -124,7 +121,15 @@ class Horde_RPC_syncml extends Horde_RPC {
 
     function _parse($xml)
     {
-        // Create the XML parser and set method references.
+        /* try to extract charset from XML text */
+        if(preg_match('/^\s*<\?xml[^>]*encoding\s*=\s*"([^"]*)"/i',
+                $xml, $m)) {
+            $this->_charset = $m[1];
+        }
+        NLS::setCharset($this->_charset);
+        String::setDefaultCharset($this->_charset);
+
+        /* Create the XML parser and set method references. */
         $this->_parser = xml_parser_create_ns($this->_charset);
         xml_set_object($this->_parser, $this);
         xml_parser_set_option($this->_parser, XML_OPTION_CASE_FOLDING, false);
@@ -170,7 +175,7 @@ class Horde_RPC_syncml extends Horde_RPC {
     }
 
     /**
-     * Get the Content-Type of the response.
+     * Returns the Content-Type of the response.
      *
      * @return string  The MIME Content-Type of the RPC response.
      */
@@ -194,7 +199,7 @@ class Horde_RPC_syncml extends Horde_RPC {
             // Either <SyncML><SyncHdr> or <SyncML><SyncBody>
             if (!isset($this->_contentHandler)) {
                 // If not defined then create SyncHdr.
-                $this->_contentHandler = &new Horde_SyncML_SyncmlHdr();
+                $this->_contentHandler = &new SyncML_SyncmlHdr();
                 $this->_contentHandler->setOutput($this->_output);
             }
 
@@ -221,13 +226,13 @@ class Horde_RPC_syncml extends Horde_RPC {
         case 2:
             // Either </SyncHdr></SyncML> or </SyncBody></SyncML>
             if ($element == 'SyncHdr') {
-                // Then we get the state from SyncMLHdr, and create a
-                // new SyncMLBody.
+                // Then we get the state from SyncMLHdr, and create a new
+                // SyncMLBody.
                 $this->_contentHandler->endElement($uri, $element);
 
                 unset($this->_contentHandler);
 
-                $this->_contentHandler = &new Horde_SyncML_SyncmlBody();
+                $this->_contentHandler = &new SyncML_SyncmlBody();
                 $this->_contentHandler->setOutput($this->_output);
             } else {
                 // No longer used.
