@@ -1,16 +1,17 @@
 <?php
 
-include_once 'Horde/RPC/syncml.php';
-include_once 'XML/WBXML/Decoder.php';
-include_once 'XML/WBXML/Encoder.php';
+require_once dirname(__FILE__) . '/syncml.php';
+require_once 'XML/WBXML/Decoder.php';
+require_once 'XML/WBXML/Encoder.php';
 
 /**
- * The Horde_RPC_syncml class provides a SyncML implementation of the Horde
- * RPC system.
+ * The Horde_RPC_syncml_wbxml class provides a SyncML implementation of the
+ * Horde RPC system using WBXML encoding.
  *
- * $Horde: framework/RPC/RPC/syncml_wbxml.php,v 1.11.10.3 2005/08/29 17:18:40 jan Exp $
+ * $Horde: framework/RPC/RPC/syncml_wbxml.php,v 1.11.10.6 2006/01/01 21:28:33 jan Exp $
  *
- * Copyright 2003-2005 Chuck Hagenbuch <chuck@horde.org>, Anthony Mills <amills@pyramid6.com>
+ * Copyright 2003-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2003-2006 Anthony Mills <amills@pyramid6.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -39,7 +40,7 @@ class Horde_RPC_syncml_wbxml extends Horde_RPC_syncml {
         /* Very useful for debugging. Logs WBXML packets to
          * $this->_debugDir. */
         if (!empty($this->_debugDir) && is_dir($this->_debugDir)) {
-            $packetNum = @intval(file_get_contents($this->_debugDir . '/syncml_wbxml.packetnum'));
+            $packetNum = @intval(file_get_contents($this->_debugDir . '/syncml.packetnum'));
             if (!isset($packetNum)) {
                 $packetNum = 0;
             }
@@ -49,34 +50,46 @@ class Horde_RPC_syncml_wbxml extends Horde_RPC_syncml {
             fclose($fp);
         }
 
+
         $decoder = &new XML_WBXML_Decoder();
-        $xmlinput = $decoder->decode($request);
-        if (is_a($xmlinput, 'PEAR_Error')) {
-            return '';
+        $this->_output = &new XML_WBXML_Encoder();
+
+        $decoder->setContentHandler($this);
+
+        $r = $decoder->decode($request);
+        if (is_a($r, 'PEAR_Error')) {
+            Horde::logMessage('SyncML: ' .
+                              $r->getMessage(), __FILE__, __LINE__, PEAR_LOG_ERR);
         }
 
-        $xmloutput = parent::getResponse($xmlinput);
+        $this->_output->setVersion($decoder->getVersion());
+        $this->_output->setCharset($decoder->getCharsetStr());
+        $response = $this->_output->getOutput();
 
-        $encoder = &new XML_WBXML_Encoder();
-        $encoder->setVersion($decoder->getVersion());
-        $encoder->setCharset($decoder->getCharsetStr());
-        $response = $encoder->encode($xmloutput);
+        if (is_a($response, 'PEAR_Error')) {
+            Horde::logMessage($response, __FILE__, __LINE__, PEAR_LOG_ERR);
+            $response = $response->getMessage();
+        }
 
         if (!empty($this->_debugDir) && is_dir($this->_debugDir)) {
             $fp = fopen($this->_debugDir . '/syncml_server_' . $packetNum . '.wbxml', 'wb');
             fwrite($fp, $response);
             fclose($fp);
 
-            $fp = fopen($this->_debugDir . '/syncml_wbxml.packetnum', 'w');
-            fwrite($fp, ++$packetNum);
-            fclose($fp);
+            $fp = @fopen($this->_debugDir . '/syncml.packetnum', 'w');
+            if ($fp) {
+                fwrite($fp, ++$packetNum);
+                fclose($fp);
+            }
+
         }
 
         /* Clear the output buffer that we started above, and log anything
          * that came up for later debugging. */
         $errorLogging = ob_get_clean();
         if (!empty($errorLogging)) {
-            Horde::logMessage($errorLogging, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            Horde::logMessage('SyncML: caught output=' . $errorLogging,
+            __FILE__, __LINE__, PEAR_LOG_DEBUG);
         }
 
         return $response;
