@@ -3,15 +3,14 @@
  * The Horde_RPC_xmlrpc class provides an XMLRPC implementation of the
  * Horde RPC system.
  *
- * $Horde: framework/RPC/RPC/xmlrpc.php,v 1.9.10.1 2005/01/03 12:19:12 jan Exp $
+ * $Horde: framework/RPC/RPC/xmlrpc.php,v 1.9.10.7 2006/01/01 21:28:33 jan Exp $
  *
- * Copyright 2002-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2002-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jan Schneider <jan@horde.org>
- * @version $Revision: 1.9.10.1 $
  * @since   Horde 3.0
  * @package Horde_RPC
  */
@@ -19,7 +18,8 @@ class Horde_RPC_xmlrpc extends Horde_RPC {
 
     /**
      * Resource handler for the XMLRPC server.
-     * @var object $_server
+     *
+     * @var resource
      */
     var $_server;
 
@@ -27,7 +27,6 @@ class Horde_RPC_xmlrpc extends Horde_RPC {
      * XMLRPC server constructor
      *
      * @access private
-     * @return object   A RPC server instance
      */
     function Horde_RPC_xmlrpc()
     {
@@ -51,9 +50,9 @@ class Horde_RPC_xmlrpc extends Horde_RPC {
     /**
      * Sends an RPC request to the server and returns the result.
      *
-     * @param string    The raw request string.
+     * @param string  The raw request string.
      *
-     * @return string   The XML encoded response from the server.
+     * @return string  The XML encoded response from the server.
      */
     function getResponse($request)
     {
@@ -67,11 +66,11 @@ class Horde_RPC_xmlrpc extends Horde_RPC {
      *
      * @access private
      *
-     * @param string $method    The name of the method called by the RPC request.
-     * @param array $params     The passed parameters.
-     * @param mixed $data       Unknown.
+     * @param string $method  The name of the method called by the RPC request.
+     * @param array $params   The passed parameters.
+     * @param mixed $data     Unknown.
      *
-     * @return mixed            The result of the called registry method.
+     * @return mixed  The result of the called registry method.
      */
     function _dispatcher($method, $params, $data)
     {
@@ -79,7 +78,7 @@ class Horde_RPC_xmlrpc extends Horde_RPC {
         $method = str_replace('.', '/', $method);
 
         if (!$registry->hasMethod($method)) {
-            return sprintf(_("Method '%s' is not defined"), $method);
+            return sprintf(_("Method \"%s\" is not defined"), $method);
         }
 
         return $registry->call($method, $params);
@@ -90,10 +89,10 @@ class Horde_RPC_xmlrpc extends Horde_RPC {
      *
      * This statically called method is actually the XMLRPC client.
      *
-     * @param string $url       The path to the XMLRPC server on the called host.
-     * @param string $method    The method to call.
-     * @param array $params     (optional) A hash containing any necessary
-     *                          parameters for the method call.
+     * @param string $url     The path to the XMLRPC server on the called host.
+     * @param string $method  The method to call.
+     * @param array $params   A hash containing any necessary parameters for
+     *                        the method call.
      * @param $options  Optional associative array of parameters which can be:
      *                  user           - Basic Auth username
      *                  pass           - Basic Auth password
