@@ -11,11 +11,11 @@ define('HORDE_SESSION_READONLY', 2);
  * between Horde applications and keeping track of application
  * configuration information.
  *
- * $Horde: framework/Horde/Horde/Registry.php,v 1.243.2.4 2005/02/14 18:06:12 jan Exp $
+ * $Horde: framework/Horde/Horde/Registry.php,v 1.243.2.20 2006/02/12 22:47:15 mrubinsk Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
- * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Anil Madhavapeddy <anil@recoil.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -31,14 +31,14 @@ class Registry {
     /**
      * Hash storing all of the known services and callbacks.
      *
-     * @var array $_apiCache
+     * @var array
      */
     var $_apiCache = array();
 
     /**
      * Hash storing all known data types.
      *
-     * @var array $_typeCache
+     * @var array
      */
     var $_typeCache = array();
 
@@ -46,54 +46,82 @@ class Registry {
      * Hash storing all of the registered interfaces that applications
      * provide.
      *
-     * @var array $_interfaces
+     * @var array
      */
     var $_interfaces = array();
 
     /**
      * Hash storing information on each registry-aware application.
      *
-     * @var array $applications
+     * @var array
      */
     var $applications = array();
 
     /**
      * Stack of in-use applications.
      *
-     * @var array $_appStack
+     * @var array
      */
     var $_appStack = array();
 
     /**
      * Quick pointer to the current application.
      *
-     * @var $_currentApp
+     * @var string
      */
     var $_currentApp = null;
 
     /**
      * Cache of $prefs objects
      *
-     * @var array $_prefsCache
+     * @var array
      */
     var $_prefsCache = array();
 
     /**
      * Cache of application configurations.
      *
-     * @var array $_confCache
+     * @var array
      */
     var $_confCache = array();
 
+    /**
+     * Are we using registry caching?
+     *
+     * @param boolean
+     */
+    var $_usecache = false;
+
+    /**
+     * Update these cache entries on shutdown.
+     *
+     * @param array
+     */
+    var $_updatecache = array();
+
+    /**
+     * Update these mtime entries on shutdown.
+     *
+     * @param array
+     */
+    var $_updatemtime = array();
+
+    /**
+     * Don't update cache at end of request?
+     *
+     * @param boolean
+     */
+    var $_nocache = false;
+
     /**
      * Returns a reference to the global Registry object, only
      * creating it if it doesn't already exist.
      *
      * This method must be invoked as: $registry = &Registry::singleton()
      *
-     * @param optional integer $session_flags  Any session flags.
+     * @param integer $session_flags  Any session flags.
      *
-     * @return object Registry  The Horde Registry instance.
+     * @return Registry  The Horde Registry instance.
      */
     function &singleton($session_flags = 0)
     {
@@ -110,7 +138,7 @@ class Registry {
      * Create a new registry instance. Should never be called except
      * by &Registry::singleton().
      *
-     * @param optional integer $session_flags  Any session flags.
+     * @param integer $session_flags  Any session flags.
      *
      * @access private
      */
@@ -131,56 +159,76 @@ class Registry {
                 /* Close the session immediately so no changes can be
                    made but values are still available. */
                 @session_write_close();
+            } else {
+                $this->_usecache = true;
             }
+            /* Call any function triggers which may have been set in the
+             * session. */
+            Horde::sessionCallbacks();
         }
 
-        /* Read the registry configuration file. */
-        require_once HORDE_BASE . '/config/registry.php';
-
         /* Initialize the localization routines and variables. */
         NLS::setLang();
         NLS::setTextdomain('horde', HORDE_BASE . '/locale', NLS::getCharset());
         String::setDefaultCharset(NLS::getCharset());
 
-        /* Stop system if Horde is inactive. */
-        if ($this->applications['horde']['status'] == 'inactive') {
-            Horde::fatal(_("This system is currently deactivated."), __FILE__, __LINE__);
-        }
+        $reg_mtime = filemtime(HORDE_BASE . '/config/registry.php');
 
-        /* Scan for all APIs provided by each app, and set other
-         * common defaults like templates and graphics. */
-        $appList = array_keys($this->applications);
-        foreach ($appList as $appName) {
-            $app = &$this->applications[$appName];
-            if (($app['status'] == 'heading') ||
-                ($app['status'] == 'inactive') ||
-                ($app['status'] == 'admin' && !Auth::isAdmin())) {
-                continue;
+        /* Load registry information from the session, if available. */
+        if (!empty($_SESSION['_registrycache']) &&
+            ($_SESSION['_registrymtime']['registry.php'] == $reg_mtime)) {
+            foreach ($_SESSION['_registrycache'] as $key => $val) {
+                $this->$key = $val;
             }
-            if (isset($app['provides'])) {
-                if (is_array($app['provides'])) {
-                    foreach ($app['provides'] as $interface) {
-                        $this->_interfaces[$interface] = $appName;
+        } else {
+            /* Read the registry configuration file. */
+            require_once HORDE_BASE . '/config/registry.php';
+
+            /* Stop system if Horde is inactive. */
+            if ($this->applications['horde']['status'] == 'inactive') {
+                Horde::fatal(_("This system is currently deactivated."), __FILE__, __LINE__);
+            }
+
+            /* Scan for all APIs provided by each app, and set other
+             * common defaults like templates and graphics. */
+            $appList = array_keys($this->applications);
+            foreach ($appList as $appName) {
+                $app = &$this->applications[$appName];
+                if (($app['status'] == 'heading') ||
+                    ($app['status'] == 'inactive') ||
+                    ($app['status'] == 'admin' && !Auth::isAdmin())) {
+                    continue;
+                }
+                if (isset($app['provides'])) {
+                    if (is_array($app['provides'])) {
+                        foreach ($app['provides'] as $interface) {
+                            $this->_interfaces[$interface] = $appName;
+                        }
+                    } else {
+                        $this->_interfaces[$app['provides']] = $appName;
                     }
-                } else {
-                    $this->_interfaces[$app['provides']] = $appName;
+                }
+                if (!isset($app['templates']) && isset($app['fileroot'])) {
+                    $app['templates'] = $app['fileroot'] . '/templates';
+                }
+                if (!isset($app['jsuri']) && isset($app['webroot'])) {
+                    $app['jsuri'] = $app['webroot'] . '/js';
+                }
+                if (!isset($app['jsfs']) && isset($app['fileroot'])) {
+                    $app['jsfs'] = $app['fileroot'] . '/js';
+                }
+                if (!isset($app['themesuri']) && isset($app['webroot'])) {
+                    $app['themesuri'] = $app['webroot'] . '/themes';
+                }
+                if (!isset($app['themesfs']) && isset($app['fileroot'])) {
+                    $app['themesfs'] = $app['fileroot'] . '/themes';
                 }
             }
-            if (!isset($app['templates']) && isset($app['fileroot'])) {
-                $app['templates'] = $app['fileroot'] . '/templates';
-            }
-            if (!isset($app['jsuri']) && isset($app['webroot'])) {
-                $app['jsuri'] = $app['webroot'] . '/js';
-            }
-            if (!isset($app['jsfs']) && isset($app['fileroot'])) {
-                $app['jsfs'] = $app['fileroot'] . '/js';
-            }
-            if (!isset($app['themesuri']) && isset($app['webroot'])) {
-                $app['themesuri'] = $app['webroot'] . '/themes';
-            }
-            if (!isset($app['themesfs']) && isset($app['fileroot'])) {
-                $app['themesfs'] = $app['fileroot'] . '/themes';
-            }
+
+            /* Clear out the API & Type caches, if they exists. */
+            $this->_apiCache = $this->_typeCache = array();
+
+            $this->_cacheVars(array('applications', '_interfaces', '_apiCache', '_typeCache'), array('registry.php' => $reg_mtime));
         }
 
         /* Create the global Perms object. */
@@ -196,19 +244,113 @@ class Registry {
         }
     }
 
+    /**
+     * Add a list of variable names and modification times to be cached.
+     *
+     * @access private
+     *
+     * @param array $vars    A list of variable names to be cached.
+     * @param array $mtimes  A list of file modificaion times to be cached.
+     */
+    function _cacheVars($vars = array(), $mtimes = array())
+    {
+        static $regcacheshutdown = false;
+
+        if ($this->_usecache) {
+            $this->_updatecache = array_merge($this->_updatecache, $vars);
+            $this->_updatemtime = array_merge($this->_updatemtime, $mtimes);
+            if (!$regcacheshutdown) {
+                register_shutdown_function(array(&$this, '_shutdowncache'));
+                $regcacheshutdown = true;
+            }
+        }
+    }
+
+    /**
+     * Stores cacheable member variables in the session at shutdown.
+     *
+     * @access private
+     */
+    function _shutdowncache()
+    {
+        if (isset($_SESSION) && Auth::getAuth() && !$this->_nocache) {
+            if (!isset($_SESSION['_registrycache'])) {
+                $_SESSION['_registrycache'] = array();
+            }
+
+            foreach (array_unique($this->_updatecache) as $val) {
+                $_SESSION['_registrycache'][$val] = $this->$val;
+            }
+
+            if (!isset($_SESSION['_registrymtime'])) {
+                $_SESSION['_registrymtime'] = array();
+            }
+
+            foreach (array_unique($this->_updatemtime) as $key => $val) {
+                $_SESSION['_registrymtime'][$key] = $val;
+            }
+        }
+    }
+
+    /**
+     * Clear the registry cache.
+     *
+     * @since Horde 3.1
+     */
+    function clearCache()
+    {
+        unset($_SESSION['_registrycache']);
+        $this->_nocache = true;
+    }
+
+    /**
+     * Fills the registry's API cache with the available services.
+     *
+     * @access private
+     */
+    function _fillAPICache()
+    {
+        if (!empty($this->_apiCache)) {
+            return;
+        }
+
+        $status = array('active', 'notoolbar', 'hidden');
+        if (Auth::isAdmin()) {
+            $status[] = 'admin';
+        }
+        $apps = $this->listApps($status);
+        foreach ($apps as $app) {
+            $_services = $_types = null;
+            $api = $this->get('fileroot', $app) . '/lib/api.php';
+            if (is_readable($api)) {
+                include_once $api;
+            }
+            if (!isset($_services)) {
+                $this->_apiCache[$app] = array();
+            } else {
+                $this->_apiCache[$app] = $_services;
+            }
+            if (isset($_types)) {
+                foreach ($_types as $type => $params) {
+                    /* Prefix non-Horde types with the application
+                     * name. */
+                    $prefix = $app == 'horde' ? '' : "${app}_";
+                    $this->_typeCache[$prefix . $type] = $params;
+                }
+            }
+        }
+
+        $this->_cacheVars(array('_apiCache', '_typeCache'));
+    }
+
     /**
      * Return a list of the installed and registered applications.
      *
-     * @since Horde 2.2
-     *
-     * @access public
-     *
-     * @param array   $filter      (optional) An array of the statuses that
-     *                             should be returned. Defaults to non-hidden.
-     * @param boolean $assoc       (optional) Associative array with app names
-     *                             as keys.
-     * @param integer $permission  (optional) The permission level to check
-     *                             for in the list. Defaults to PERMS_SHOW.
+     * @param array   $filter      An array of the statuses that should be
+     *                             returned. Defaults to non-hidden.
+     * @param boolean $assoc       Associative array with app names as keys.
+     * @param integer $permission  The permission level to check for in the
+     *                             list. Defaults to PERMS_SHOW.
      *
      * @return array  List of apps registered with Horde. If no
      *                applications are defined returns an empty array.
@@ -233,13 +375,14 @@ class Registry {
     /**
      * Returns all available registry APIs.
      *
-     * @access public
-     *
      * @return array  The API list.
      */
     function listAPIs()
     {
-        $apis = array();
+        static $apis = array();
+        if ($apis) {
+            return $apis;
+        }
 
         foreach (array_keys($this->_interfaces) as $interface) {
             @list($api, ) = explode('/', $interface);
@@ -253,10 +396,8 @@ class Registry {
      * Returns all of the available registry methods, or alternately
      * only those for a specified API.
      *
-     * @access public
-     *
-     * @param optional string $api  Defines the API for which the methods
-     *                              shall be returned.
+     * @param string $api  Defines the API for which the methods shall be
+     *                     returned.
      *
      * @return array  The method list.
      */
@@ -298,8 +439,6 @@ class Registry {
     /**
      * Returns all of the available registry data types.
      *
-     * @access public
-     *
      * @return array  The data type list.
      */
     function listTypes()
@@ -311,8 +450,6 @@ class Registry {
     /**
      * Returns a method's signature.
      *
-     * @access public
-     *
      * @param string $method  The full name of the method to check for.
      *
      * @return array  A two dimensional array. The first element contains an
@@ -335,10 +472,7 @@ class Registry {
     }
 
     /**
-     * Determine if an interface is implemented by an active
-     * application.
-     *
-     * @access public
+     * Determine if an interface is implemented by an active application.
      *
      * @param string $interface  The interface to check for.
      *
@@ -355,10 +489,8 @@ class Registry {
     /**
      * Determine if a method has been registered with the registry.
      *
-     * @access public
-     *
-     * @param string  $method      The full name of the method to check for.
-     * @param string  $app         (optional) Only check this application.
+     * @param string $method  The full name of the method to check for.
+     * @param string $app     Only check this application.
      *
      * @return mixed  The application implementing $method if we have it,
      *                false if the method doesn't exist.
@@ -389,10 +521,8 @@ class Registry {
      * parameter. $method is a string consisting of
      * "packagetype/methodname".
      *
-     * @access public
-     *
-     * @param string $method        The method to call.
-     * @param optional array $args  Arguments to the method.
+     * @param string $method  The method to call.
+     * @param array $args     Arguments to the method.
      *
      * @return  TODO
      *          Returns PEAR_Error on error.
@@ -415,11 +545,9 @@ class Registry {
     /**
      * Output the hook corresponding to the specific package named.
      *
-     * @access public
-     *
-     * @param string $app           The application being called.
-     * @param string $call          The method to call.
-     * @param optional array $args  Arguments to the method.
+     * @param string $app   The application being called.
+     * @param string $call  The method to call.
+     * @param array $args   Arguments to the method.
      *
      * @return  TODO
      *          Returns PEAR_Error on error.
@@ -433,6 +561,14 @@ class Registry {
             return PEAR::raiseError(sprintf('The method "%s" is not defined in the API for %s.', $call, $app));
         }
 
+        /* Load the API now. */
+        if (!empty($_SESSION['_registrycache']['_apiCache'])) {
+            $api = $this->get('fileroot', $app) . '/lib/api.php';
+            if (is_readable($api)) {
+                include_once $api;
+            }
+        }
+
         /* Make sure that the function actually exists. */
         $function = '_' . $app . '_' . $call;
         if (!function_exists($function)) {
@@ -467,12 +603,9 @@ class Registry {
      * parameter. $method is a string consisting of
      * "packagetype/methodname".
      *
-     * @access public
-     *
-     * @param string $method         The method to link to.
-     * @param optional array $args   Arguments to the method.
-     * @param optional mixed $extra  Extra, non-standard arguments to the
-     *                               method.
+     * @param string $method  The method to link to.
+     * @param array $args     Arguments to the method.
+     * @param mixed $extra    Extra, non-standard arguments to the method.
      *
      * @return  TODO
      *          Returns PEAR_Error on error.
@@ -495,13 +628,10 @@ class Registry {
     /**
      * Output the hook corresponding to the specific package named.
      *
-     * @access public
-     *
-     * @param string $app            The application being called.
-     * @param string $call           The method to link to.
-     * @param optional array $args   Arguments to the method.
-     * @param optional mixed $extra  Extra, non-standard arguments to the
-     *                               method.
+     * @param string $app   The application being called.
+     * @param string $call  The method to link to.
+     * @param array $args   Arguments to the method.
+     * @param mixed $extra  Extra, non-standard arguments to the method.
      *
      * @return  TODO
      *          Returns PEAR_Error on error.
@@ -561,13 +691,11 @@ class Registry {
     }
 
     /**
-     * Replace any %application% strings with the filesystem path to
-     * the application.
+     * Replace any %application% strings with the filesystem path to the
+     * application.
      *
-     * @access public
-     *
-     * @param string $path          The application string.
-     * @param optional string $app  The application being called.
+     * @param string $path  The application string.
+     * @param string $app   The application being called.
      *
      * @return  TODO
      *          Returns PEAR_Error on error.
@@ -579,20 +707,17 @@ class Registry {
         }
 
         if (!isset($this->applications[$app])) {
-            return PEAR::raiseError(sprintf(_("'%s' is not configured in the Horde Registry."), $app));
+            return PEAR::raiseError(sprintf(_("\"%s\" is not configured in the Horde Registry."), $app));
         }
 
         return str_replace('%application%', $this->applications[$app]['fileroot'], $path);
     }
 
     /**
-     * Replace any %application% strings with the web path to the
-     * application.
+     * Replace any %application% strings with the web path to the application.
      *
-     * @access public
-     *
-     * @param string $path          The application string.
-     * @param optional string $app  The application being called.
+     * @param string $path  The application string.
+     * @param string $app   The application being called.
      *
      * @return  TODO
      *          Returns PEAR_Error on error.
@@ -614,14 +739,12 @@ class Registry {
      * pushApp() also reads the application's configuration file and
      * sets up its global $conf hash.
      *
-     * @access public
-     *
      * @param string  $app         The name of the application to push.
-     * @param boolean $checkPerms  (optional) Make sure that the current user
-     *                             has permissions to the application being
-     *                             loaded. Defaults to true. Should ONLY
-     *                             be disabled by system scripts (cron jobs,
-     *                             etc.) and scripts that handle login.
+     * @param boolean $checkPerms  Make sure that the current user has
+     *                             permissions to the application being loaded
+     *                             Defaults to true. Should ONLY be disabled by
+     *                             system scripts (cron jobs, etc.) and scripts
+     *                             that handle login.
      *
      * @return boolean  Whether or not the _appStack was modified.
      *                  Return PEAR_Error on error.
@@ -674,18 +797,15 @@ class Registry {
 
         /* Once we know everything succeeded and is in a consistent
          * state again, push the new application onto the stack. */
-        array_push($this->_appStack, $app);
+        $this->_appStack[] = $app;
         $this->_currentApp = $app;
 
         return true;
     }
 
     /**
-     * Remove the current app from the application stack, setting the
-     * current app to whichever app was current before this one took
-     * over.
-     *
-     * @access public
+     * Remove the current app from the application stack, setting the current
+     * app to whichever app was current before this one took over.
      *
      * @return string  The name of the application that was popped.
      */
@@ -712,10 +832,8 @@ class Registry {
     }
 
     /**
-     * Return the current application - the app at the top of the
-     * application stack.
-     *
-     * @access public
+     * Return the current application - the app at the top of the application
+     * stack.
      *
      * @return string  The current application.
      */
@@ -727,22 +845,19 @@ class Registry {
     /**
      * Check permissions on an application.
      *
-     * @access public
-     *
      * @return boolean  Whether or not access is allowed.
      */
     function hasPermission($app, $permission = PERMS_READ)
     {
-        return Auth::isAdmin() || ($GLOBALS['perms']->exists($app) ?
-                                   $GLOBALS['perms']->hasPermission($app, Auth::getAuth(), $permission) :
-                                   (bool)Auth::getAuth());
+        return Auth::isAdmin() ||
+            ($GLOBALS['perms']->exists($app)
+             ? $GLOBALS['perms']->hasPermission($app, Auth::getAuth(), $permission)
+             : (bool)Auth::getAuth());
     }
 
     /**
-     * Reads the configuration values for the given application and
-     * imports them into the global $conf variable.
-     *
-     * @access public
+     * Reads the configuration values for the given application and imports
+     * them into the global $conf variable.
      *
      * @param string $app  The name of the application.
      *
@@ -763,7 +878,7 @@ class Registry {
                 $errors = ob_get_contents();
                 ob_end_clean();
                 if (!empty($errors)) {
-                    return PEAR::raiseError(sprintf('Failed to import Horde configuration: %s', strip_tags($errors)));
+                    return PEAR::raiseError('Failed to import Horde configuration: ' . strip_tags($errors));
                 }
                 if (!$success) {
                     return PEAR::raiseError('Failed to import Horde configuration.');
@@ -788,12 +903,20 @@ class Registry {
             }
 
             if ($app !== 'horde') {
-                $success = @include $this->applications[$app]['fileroot'] . '/config/conf.php';
+                ob_start();
+                $success = include $this->applications[$app]['fileroot'] . '/config/conf.php';
+                $errors = ob_get_contents();
+                ob_end_clean();
+                if (!empty($errors)) {
+                    return PEAR::raiseError('Failed to import application configuration for ' . $app . ': ' . strip_tags($errors));
+                }
                 if (!$success) {
                     return PEAR::raiseError('Failed to import application configuration for ' . $app);
                 }
             }
 
+            $this->_cacheVars(array('_confCache'));
+
             $this->_confCache[$app] = &$conf;
         }
 
@@ -802,10 +925,8 @@ class Registry {
     }
 
     /**
-     * Loads the preferences for the current user for the current
-     * application and imports them into the global $prefs variable.
-     *
-     * @access public
+     * Loads the preferences for the current user for the current application
+     * and imports them into the global $prefs variable.
      *
      * @param string $app  The name of the application.
      */
@@ -851,10 +972,8 @@ class Registry {
      * After unloading, if there is an application on the app stack to
      * load preferences from, then we reload a fresh set.
      *
-     * @access public
-     *
-     * @param string $app  (optional) The application to unload prefrences for.
-     *                     If null, ALL preferences are reset.
+     * @param string $app  The application to unload prefrences for. If null,
+     *                     ALL preferences are reset.
      */
     function unloadPrefs($app = null)
     {
@@ -879,10 +998,8 @@ class Registry {
      * Horde-wide value is used instead. If that is not present, we
      * return null.
      *
-     * @access public
-     *
-     * @param string $parameter     The configuration value to retrieve.
-     * @param optional string $app  The application to get the value for.
+     * @param string $parameter  The configuration value to retrieve.
+     * @param string $app        The application to get the value for.
      *
      * @return string  The requested parameter, or null if it is not set.
      */
@@ -910,14 +1027,12 @@ class Registry {
     }
 
     /**
-     * Function to work out an application's graphics URI, taking into
-     * account any themes directories that may be set up.
+     * Function to work out an application's graphics URI, taking into account
+     * any themes directories that may be set up.
      *
-     * @access public
-     *
-     * @param optional string $app  The application for which to get the
-     *                              image directory. If blank will default
-     *                              to current application.
+     * @param string $app  The application for which to get the image
+     *                     directory. If blank will default to current
+     *                     application.
      *
      * @return string  The image directory uri path.
      */
@@ -935,11 +1050,15 @@ class Registry {
         /* This is the default location for the graphics. */
         $img_dir[$app] = $this->get('themesuri', $app) . '/graphics';
 
+        static $theme_icons;
+
         /* Figure out if this is going to be overridden by any theme
          * settings. */
-        if (isset($GLOBALS['prefs']) && ($theme = $GLOBALS['prefs']->getValue('theme')) &&
-            (@include $this->get('themesfs', 'horde') . '/' . $theme . '/info.php') &&
-            isset($theme_icons) &&
+        if (isset($GLOBALS['prefs']) &&
+            ($theme = $GLOBALS['prefs']->getValue('theme')) &&
+            (isset($theme_icons) ||
+             ((@include $this->get('themesfs', 'horde') . '/' . $theme . '/info.php') &&
+              isset($theme_icons))) &&
             in_array($app, $theme_icons)) {
             $img_dir[$app] = $this->get('themesuri', $app) . '/' . $theme . '/graphics';
         }
@@ -961,13 +1080,10 @@ class Registry {
     }
 
     /**
-     * Query the initial page for an application - the webroot, if
-     * there is no initial_page set, and the initial_page, if it is
-     * set.
+     * Query the initial page for an application - the webroot, if there is no
+     * initial_page set, and the initial_page, if it is set.
      *
-     * @access public
-     *
-     * @param optional string $app  The name of the application.
+     * @param string $app  The name of the application.
      *
      * @return string  URL pointing to the inital page of the application.
      *                 Returns PEAR_Error on error.
@@ -979,47 +1095,51 @@ class Registry {
         }
 
         if (!isset($this->applications[$app])) {
-            return PEAR::raiseError(sprintf(_("'%s' is not configured in the Horde Registry."), $app));
+            return PEAR::raiseError(sprintf(_("\"%s\" is not configured in the Horde Registry."), $app));
         }
         return $this->applications[$app]['webroot'] . '/' . (isset($this->applications[$app]['initial_page']) ? $this->applications[$app]['initial_page'] : '');
     }
 
     /**
-     * Fills the registry's API cache with the available services.
-     *
-     * @access private
+     * @since Horde 3.1
      */
-    function _fillAPICache()
+    function __get($api)
     {
-        if (!empty($this->_apiCache)) {
-            return;
-        }
-
-        $status = array('active', 'notoolbar', 'hidden');
-        if (Auth::isAdmin()) {
-            $status[] = 'admin';
-        }
-        $apps = $this->listApps($status);
-        foreach ($apps as $app) {
-            $_services = $_types = null;
-            $api = $this->get('fileroot', $app) . '/lib/api.php';
-            if (is_readable($api)) {
-                include_once $api;
-            }
-            if (!isset($_services)) {
-                $this->_apiCache[$app] = array();
-            } else {
-                $this->_apiCache[$app] = $_services;
-            }
-            if (isset($_types)) {
-                foreach ($_types as $type => $params) {
-                    /* Prefix non-Horde types with the application
-                     * name. */
-                    $prefix = $app == 'horde' ? '' : "${app}_";
-                    $this->_typeCache[$prefix . $type] = $params;
-                }
-            }
+        if (in_array($api, $this->listAPIs())) {
+            return new RegistryCaller($this, $api);
         }
     }
 
+    /**
+     * Clone should never be called on Registry objects. If it is, die.
+     *
+     * @since Horde 3.1
+     */
+    function __clone()
+    {
+        Horde::fatal('Registry objects should never be cloned.', __FILE__, __LINE__);
+    }
+
+}
+
+/**
+ * @package Horde_Framework
+ * @since Horde 3.1
+ */
+class RegistryCaller {
+
+    var $registry;
+    var $api;
+
+    function __construct($registry, $api)
+    {
+        $this->registry = $registry;
+        $this->api = $api;
+    }
+
+    function __call($method, $args)
+    {
+        return $this->registry->call($this->api . '/' . $method, $args);
+    }
+
 }
