@@ -2,10 +2,10 @@
 /**
  * This is a utility class, every method is static.
  *
- * $Horde: framework/SQL/SQL.php,v 1.30.2.13 2005/07/03 05:52:01 selsky Exp $
+ * $Horde: framework/SQL/SQL.php,v 1.30.2.16 2006/01/18 19:30:58 ben Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2005 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2005-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -63,9 +63,20 @@ class Horde_SQL {
                     return sprintf($query, $lhs, (int)$rhs, (int)$rhs);
                 }
 
-            case 'odbc':
             case 'mssql':
-                // ODBC and MSSQL must have a valid boolean expression
+                // MSSQL must have a valid boolean expression
+                $query = '(CASE WHEN ISNUMERIC(%s) = 1 THEN (%s & %s) ELSE %s END) = %s';
+                if ($bind) {
+                    return array(sprintf(Horde_SQL::escapePrepare($query),
+                                         Horde_SQL::escapePrepare($lhs),
+                                         Horde_SQL::escapePrepare($lhs), '?', '?', '?'),
+                                 array((int)$rhs, (int)$rhs - 1, (int)$rhs));
+                } else {
+                    return sprintf($query, $lhs, $lhs, (int)$rhs, (int)$rhs - 1, (int)$rhs);
+                }
+
+            case 'odbc':
+                // ODBC must have a valid boolean expression
                 $query = '(%s & %s) = %s';
                 if ($bind) {
                     return array(sprintf(Horde_SQL::escapePrepare($query),
