@@ -21,11 +21,11 @@
  * The table structure for the SessionHandler can be found in
  * horde/scripts/sql/horde_sessionhandler.sql.
  *
- * $Horde: framework/SessionHandler/SessionHandler/mysql.php,v 1.16.12.16 2007/04/10 15:55:56 jan Exp $
+ * $Horde: framework/SessionHandler/SessionHandler/mysql.php,v 1.16.12.13 2006/03/25 00:19:42 jan Exp $
  *
- * Copyright 2002-2007 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2006-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2002-2006 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -89,7 +89,7 @@ class SessionHandler_mysql extends SessionHandler {
                          $this->_quote($id),
                          $timeout);
 
-        if (!empty($this->_params['rowlocking'])) {
+        if ($this->_params['rowlocking']) {
             /* Start a transaction. */
             $result = @mysql_query('START TRANSACTION', $this->_db);
             $query .= ' FOR UPDATE';
@@ -106,7 +106,7 @@ class SessionHandler_mysql extends SessionHandler {
 
         $result = @mysql_query($query, $this->_db);
         if (!$result) {
-            Horde::logMessage('Error retrieving session data (id = ' . $id . '): ' . mysql_error($this->_db),
+            Horde::logMessage('Error retrieving session data (id = ' . $id . ')',
                               __FILE__, __LINE__, PEAR_LOG_ERR);
             return '';
         }
@@ -136,15 +136,12 @@ class SessionHandler_mysql extends SessionHandler {
                          time());
 
         $result = @mysql_query($query, $this->_db);
-        if (!$result) {
-            $error = mysql_error($this->_db);
-        }
-        if (empty($this->_params['rowlocking'])) {
+        if (!$this->_params['rowlocking']) {
             @mysql_query('UNLOCK TABLES ' . $this->_params['table'], $this->_db);
         }
         if (!$result) {
             @mysql_query('ROLLBACK', $this->_db);
-            Horde::logMessage('Error writing session data: ' . $error, __FILE__, __LINE__, PEAR_LOG_ERR);
+            Horde::logMessage('Error writing session data', __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
 
@@ -176,15 +173,12 @@ class SessionHandler_mysql extends SessionHandler {
 
         /* Execute the query. */
         $result = @mysql_query($query, $this->_db);
-        if (!$result) {
-            $error = mysql_error($this->_db);
-        }
-        if (empty($this->_params['rowlocking'])) {
+        if (!$this->_params['rowlocking']) {
             @mysql_query('UNLOCK TABLES ' . $this->_params['table'], $this->_db);
         }
         if (!$result) {
             @mysql_query('ROLLBACK', $this->_db);
-            Horde::logMessage('Failed to delete session (id = ' . $id . '): ' . $error, __FILE__, __LINE__, PEAR_LOG_ERR);
+            Horde::logMessage('Failed to delete session (id = ' . $id . ')', __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
 
@@ -216,7 +210,7 @@ class SessionHandler_mysql extends SessionHandler {
         /* Execute the query. */
         $result = @mysql_query($query, $this->_db);
         if (!$result) {
-            Horde::logMessage('Error garbage collecting old sessions: ' . mysql_error($this->_db), __FILE__, __LINE__, PEAR_LOG_ERR);
+            Horde::logMessage('Error garbage collecting old sessions', __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
 
@@ -247,7 +241,7 @@ class SessionHandler_mysql extends SessionHandler {
 
         $result = @mysql_query($query, $this->_db);
         if (!$result) {
-            Horde::logMessage('Error getting session IDs: ' . mysql_error($this->_db),
+            Horde::logMessage('Error getting session IDs',
                               __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
