@@ -3,18 +3,18 @@
 require_once 'Horde/DataTree.php';
 
 /**
- * Horde_Share:: provides an interface to all shares a user might have.
- * Its methods take care of any site-specific restrictions configured in
- * in the application's prefs.php and conf.php files.
+ * Horde_Share:: provides an interface to all shares a user might have.  Its
+ * methods take care of any site-specific restrictions configured in in the
+ * application's prefs.php and conf.php files.
  *
- * $Horde: framework/Share/Share.php,v 1.111.2.7 2005/03/15 02:06:23 chuck Exp $
+ * $Horde: framework/Share/Share.php,v 1.111.2.16 2006/01/01 21:28:35 jan Exp $
  *
- * Copyright 2002-2005 Joel Vandal <jvandal@infoteck.qc.ca>
- * Copyright 2002-2005 Infoteck Internet <webmaster@infoteck.qc.ca>
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Joel Vandal <jvandal@infoteck.qc.ca>
+ * Copyright 2002-2006 Infoteck Internet <webmaster@infoteck.qc.ca>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
- * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you did
+ * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Joel Vandal <jvandal@infoteck.qc.ca>
  * @author  Mike Cochrame <mike@graftonhall.co.nz>
@@ -27,21 +27,21 @@ class Horde_Share {
     /**
      * Pointer to a DataTree instance to manage/store shares
      *
-     * @var DataTree $_datatree
+     * @var DataTree
      */
     var $_datatree;
 
     /**
      * The application we're managing shares for.
      *
-     * @var string $_app
+     * @var string
      */
     var $_app;
 
     /**
      * The subclass of DataTreeObject to instantiate shares as.
      *
-     * @var string $_shareObject
+     * @var string
      */
     var $_shareObject = 'DataTreeObject_Share';
 
@@ -49,14 +49,14 @@ class Horde_Share {
      * A cache of all shares that have been retrieved, so we don't hit the
      * backend again and again for them.
      *
-     * @var array $_cache
+     * @var array
      */
     var $_cache = array();
 
     /**
      * Cache used for listShares().
      *
-     * @var array $_listcache
+     * @var array
      */
     var $_listcache = array();
 
@@ -64,7 +64,7 @@ class Horde_Share {
      * A list of objects that we're currently sorting, for reference during the
      * sorting algorithm.
      *
-     * @var array $_sortList
+     * @var array
      */
     var $_sortList;
 
@@ -74,9 +74,7 @@ class Horde_Share {
      * exists.
      *
      * This method must be invoked as:
-     *   $var = &Horde_Share::singleton($app);
-     *
-     * @access public
+     *   <code>$var = &Horde_Share::singleton($app);</code>
      *
      * @param string $app  The applications that the shares relates to.
      *
@@ -96,8 +94,6 @@ class Horde_Share {
     /**
      * Constructor.
      *
-     * @access public
-     *
      * @param string $app  The application that the shares belong to.
      */
     function Horde_Share($app)
@@ -132,8 +128,6 @@ class Horde_Share {
      * Returns a DataTreeObject_Share object corresponding to the given share
      * name, with the details retrieved appropriately.
      *
-     * @access public
-     *
      * @param string $name  The name of the share to retrieve.
      *
      * @return DataTreeObject_Share  The requested share.
@@ -156,8 +150,6 @@ class Horde_Share {
      * Returns a DataTreeObject_Share object corresponding to the given unique
      * ID, with the details retrieved appropriately.
      *
-     * @access public
-     *
      * @param string $cid  The id of the share to retrieve.
      *
      * @return DataTreeObject_Share  The requested share.
@@ -175,8 +167,6 @@ class Horde_Share {
      * Returns an array of DataTreeObject_Share objects corresponding to the
      * given set of unique IDs, with the details retrieved appropriately.
      *
-     * @access public
-     *
      * @param array $cids  The array of ids to retrieve.
      *
      * @return array  The requested shares.
@@ -204,8 +194,6 @@ class Horde_Share {
     /**
      * Returns a new share object.
      *
-     * @access public
-     *
      * @param string $name  The share's name.
      *
      * @return DataTreeObject_Share  A new share object.
@@ -228,8 +216,6 @@ class Horde_Share {
      * Horde_Share::newShare(), and have any initial details added to it,
      * before this function is called.
      *
-     * @access public
-     *
      * @param DataTreeObject_Share $share  The new share object.
      *
      * @return boolean|PEAR_Error  PEAR_Error on failure.
@@ -241,6 +227,9 @@ class Horde_Share {
         }
 
         $perm = &$GLOBALS['perms']->newPermission($share->getName());
+        if (is_a($perm, 'PEAR_Error')) {
+            return $perm;
+        }
 
         /* Give the owner full access */
         $perm->addUserPermission($share->get('owner'), PERMS_SHOW, false);
@@ -284,8 +273,6 @@ class Horde_Share {
     /**
      * Checks to see if a share has any child shares.
      *
-     * @access public
-     *
      * @param DataTreeObject_Share $share  The share to check for children.
      *
      * @return boolean  True if the specified share has child shares.
@@ -300,9 +287,7 @@ class Horde_Share {
     }
 
     /**
-     * Gets a share's direct parent object.
-     *
-     * @access public
+     * Returns a share's direct parent object.
      *
      * @param string $share  Get the parent of this share.
      *
@@ -316,7 +301,8 @@ class Horde_Share {
         }
 
         if (!$id || ($id == DATATREE_ROOT)) {
-            return PEAR::raiseError('Parent does not exist.');
+            $error = PEAR::raiseError('Parent does not exist.');
+            return $error;
         }
 
         return $this->getShareById($id);
@@ -325,11 +311,9 @@ class Horde_Share {
     /**
      * Returns the ID of a share.
      *
-     * @access public
+     * @param DataTreeObject_Share  The share to return the ID of.
      *
-     * @param DataTreeObjec_Share  The share to return the ID of.
-     *
-     * @return int|PEAR_Error  The share's ID or PEAR_Error on failure.
+     * @return integer  The share's ID or PEAR_Error on failure.
      */
     function getShareId($share)
     {
@@ -339,8 +323,9 @@ class Horde_Share {
     /**
      * Utility function to be used with uasort() for sorting arrays of
      * Horde_Share objects.
-     *
-     * @example uasort($list, array('Horde_Share', '_sortShares'));
+     * Example:<code>
+     * uasort($list, array('Horde_Share', '_sortShares'));
+     * </code>
      *
      * @access private
      */
@@ -373,8 +358,6 @@ class Horde_Share {
     /**
      * Checks if a share exists in the system.
      *
-     * @access public
-     *
      * @param string $share  The share to check.
      *
      * @return boolean  True if the share exists, false otherwise.
@@ -387,21 +370,24 @@ class Horde_Share {
     /**
      * Returns the count of all shares that $userid has access to.
      *
-     * @param string  $userid     The userid of the user to check access for.
-     * @param integer $perm       (optional) The level of permissions required.
-     * @param string  $owner      (optional) Restrict the shares counted to those owned by $owner.
-     * @param string  $parent     (optional) The parent share to start searching at.
-     * @param boolean $allLevels  (optional) Return all levels, or just the direct
-     *                            children of $parent? Defaults to all levels.
+     * @param string  $userid      The userid of the user to check access for.
+     * @param integer $perm        The level of permissions required.
+     * @param mixed   $attributes  Restrict the shares counted to those
+     *                             matching $attributes. An array of
+     *                             attribute/values pairs or a share owner
+     *                             username.
+     * @param string  $parent      The parent share to start searching at.
+     * @param boolean $allLevels   Return all levels, or just the direct
+     *                             children of $parent? Defaults to all levels.
      *
-     * @return integer Number of shares the user has access to.
+     * @return integer  Number of shares the user has access to.
      */
-    function &countShares($userid, $perm = PERMS_SHOW, $owner = null,
-                          $parent = DATATREE_ROOT, $allLevels = true)
+    function countShares($userid, $perm = PERMS_SHOW, $attributes = null,
+                         $parent = DATATREE_ROOT, $allLevels = true)
     {
-        $key = serialize(array($this->_app, $userid, $perm, $owner, $parent, $allLevels, 'count'));
+        $key = serialize(array($this->_app, $userid, $perm, $attributes, $parent, $allLevels, 'count'));
         if (empty($this->_listcache[$key])) {
-            $criteria = $this->getShareCriteria($userid, $perm, $owner);
+            $criteria = $this->getShareCriteria($userid, $perm, $attributes);
             $this->_listcache[$key] = $this->_datatree->countByAttributes($criteria, $parent, $allLevels, 'id');
         }
 
@@ -411,27 +397,42 @@ class Horde_Share {
     /**
      * Returns an array of all shares that $userid has access to.
      *
-     * @param string  $userid     The userid of the user to check access for.
-     * @param integer $perm       The level of permissions required.
-     * @param string  $owner      (optional) Restrict the shares returned to those owned by $owner.
-     * @param string  $parent     The parent share to start searching at.
-     * @param boolean $allLevels  Return all levels, or just the direct
-     *                            children of $parent? Defaults to all levels.
-     * @param integer $from       The share to start listing at.
-     * @param integer $count      The number of shares to return.
+     * @param string  $userid       The userid of the user to check access for.
+     * @param integer $perm         The level of permissions required.
+     * @param mixed   $attributes   Restrict the shares counted to those
+     *                              matching $attributes. An array of
+     *                              attribute/values pairs or a share owner
+     *                              username.
+     * @param string  $parent       The parent share to start searching at.
+     * @param boolean $allLevels    Return all levels, or just the direct
+     *                              children of $parent? Defaults to all
+     *                              levels.
+     * @param integer $from         The share to start listing at.
+     * @param integer $count        The number of shares to return.
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
      *
      * @return array  The shares the user has access to.
      */
-    function &listShares($userid, $perm = PERMS_SHOW, $owner = null,
-                         $parent = DATATREE_ROOT, $allLevels = true, $from = 0, $count = 0)
+    function &listShares($userid, $perm = PERMS_SHOW, $attributes = null,
+                         $parent = DATATREE_ROOT, $allLevels = true, $from = 0,
+                         $count = 0, $sortby_name = null, $sortby_key = null,
+                         $direction = 0)
     {
-        $key = serialize(array($this->_app, $userid, $perm, $owner, $parent, $allLevels, $from, $count));
+        $key = serialize(array($this->_app, $userid, $perm, $attributes,
+                               $parent, $allLevels, $from, $count,
+                               $sortby_name, $sortby_key, $direction));
         if (empty($this->_listcache[$key])) {
-            $criteria = $this->getShareCriteria($userid, $perm, $owner);
-            $sharelist = $this->_datatree->getByAttributes($criteria, $parent, $allLevels, 'id', $from, $count);
+            $criteria = $this->getShareCriteria($userid, $perm, $attributes);
+            $sharelist = $this->_datatree->getByAttributes(
+                $criteria, $parent, $allLevels, 'id', $from, $count,
+                $sortby_name, $sortby_key, $direction);
             if (is_a($sharelist, 'PEAR_Error') || !count($sharelist)) {
-                /* If we got back an error or an empty array, pass it
-                 * back to the caller. */
+                /* If we got back an error or an empty array, pass it back to
+                 * the caller. */
                 return $sharelist;
             }
 
@@ -448,7 +449,7 @@ class Horde_Share {
         }
 
         $result = Horde::callHook('_horde_hook_share_list', array($userid,
-                                  $perm, $owner, $this->_listcache[$key]),
+                                  $perm, $attributes, $this->_listcache[$key]),
                                   'horde', false);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
@@ -459,10 +460,8 @@ class Horde_Share {
 
     /**
      * Lists *all* shares for the current app/share, regardless of
-     * permissions. This is for admin functionality and scripting
-     * tools, and shouldn't be called from user-level code!
-     *
-     * @access public
+     * permissions. This is for admin functionality and scripting tools, and
+     * shouldn't be called from user-level code!
      *
      * @param boolean $parent  Start the listing at a certain point in the
      *                         tree. Defaults to DATATREE_ROOT, the root.
@@ -473,8 +472,7 @@ class Horde_Share {
     {
         $sharelist = $this->_datatree->get(DATATREE_FORMAT_FLAT, $parent, true);
         if (is_a($sharelist, 'PEAR_Error') || !count($sharelist)) {
-            // If we got back an error or an empty array, just
-            // return it.
+            // If we got back an error or an empty array, just return it.
             return $sharelist;
         }
         unset($sharelist[$parent]);
@@ -492,15 +490,16 @@ class Horde_Share {
     }
 
     /**
-     * Get an array of criteria for querying shares.
+     * Returns an array of criteria for querying shares.
      *
-     * @param string  $userid  The userid of the user to check access for.
-     * @param integer $perm    The level of permissions required.
-     * @param string  $owner   (optional) Restrict the shares returned to those owned by $owner.
+     * @param string  $userid      The userid of the user to check access for.
+     * @param integer $perm        The level of permissions required.
+     * @param mixed   $attributes  Restrict the shares returned to those who
+     *                             have these attribute values.
      *
      * @return array  The criteria tree for fetching this user's shares.
      */
-    function getShareCriteria($userid, $perm = PERMS_SHOW, $owner = null)
+    function getShareCriteria($userid, $perm = PERMS_SHOW, $attributes = null)
     {
         if (!empty($userid)) {
             $criteria = array(
@@ -530,8 +529,7 @@ class Horde_Share {
                             array('field' => 'name', 'op' => '=', 'test' => 'perm_default'),
                             array('field' => 'value', 'op' => '&', 'test' => $perm)))));
 
-            // If the user has any group memberships, check for those
-            // also.
+            // If the user has any group memberships, check for those also.
             require_once 'Horde/Group.php';
             $group = &Group::singleton();
             $groups = $group->getGroupMemberships($userid, true);
@@ -550,16 +548,28 @@ class Horde_Share {
                      array('field' => 'value', 'op' => '&', 'test' => $perm)));
         }
 
-        // Restrict to shares owned by $owner if $owner is non-empty
-        // (presumed to be a username).
-        if (!empty($owner)) {
+        if (is_array($attributes)) {
+            // Build attribute/key filter.
+            foreach ($attributes as $key => $value) {
+                $criteria = array(
+                    'AND' => array(
+                        $criteria,
+                        array(
+                            'JOIN' => array(
+                                'AND' => array(
+                                    array('field' => 'name', 'op' => '=', 'test' => $key),
+                                    array('field' => 'value', 'op' => '=', 'test' => $value))))));
+            }
+        } elseif (!empty($attributes)) {
+            // Restrict to shares owned by the user specified in the
+            // $attributes string.
             $criteria = array(
                 'AND' => array(
                     $criteria,
                     array(
                         'JOIN' => array(
                             array('field' => 'name', 'op' => '=', 'test' => 'owner'),
-                            array('field' => 'value', 'op' => '=', 'test' => $owner)))));
+                            array('field' => 'value', 'op' => '=', 'test' => $attributes)))));
         }
 
         return $criteria;
@@ -569,7 +579,6 @@ class Horde_Share {
      * TODO
      *
      * @see Perms::getPermissions
-     * @access public
      *
      * @param TODO
      * @param TODO
@@ -589,8 +598,6 @@ class Horde_Share {
     /**
      * Returns the Identity for a particular share owner.
      *
-     * @access public
-     *
      * @param mixed $share  The share to fetch the Identity for - either the
      *                      string name, or the DataTreeObject_Share object.
      *
@@ -627,16 +634,13 @@ class DataTreeObject_Share extends DataTreeObject {
      * The Horde_Share object which this share came from - needed for
      * updating data in the backend to make changes stick, etc.
      *
-     * @var Horde_Share $_shareOb
+     * @var Horde_Share
      */
     var $_shareOb;
 
     /**
-     * The DataTreeObject_Share constructor. Just makes sure to call
-     * the parent constructor so that the share's name is set
-     * properly.
-     *
-     * @access public
+     * The DataTreeObject_Share constructor. Just makes sure to call the parent
+     * constructor so that the share's name is set properly.
      *
      * @param string $id  The id of the share.
      */
@@ -651,8 +655,6 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Associates a Share object with this share.
      *
-     * @access public
-     *
      * @param Horde_Share $shareOb  The Share object.
      */
     function setShareOb(&$shareOb)
@@ -663,20 +665,17 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Returns this share's parent object.
      *
-     * @access public
-     *
      * @return DataTreeObject_Share  The parent share, if it exists.
      */
     function &getParent()
     {
-        return $this->_shareOb->getParent($this);
+        $parent = $this->_shareOb->getParent($this);
+        return $parent;
     }
 
     /**
      * Gives a user a certain privilege for this share.
      *
-     * @access public
-     *
      * @param string $userid       The userid of the user.
      * @param integer $permission  A PERMS_* constant.
      */
@@ -690,8 +689,6 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Removes a certain privilege for a user from this share.
      *
-     * @access public
-     *
      * @param string $userid       The userid of the user.
      * @param integer $permission  A PERMS_* constant.
      */
@@ -705,8 +702,6 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Gives a group certain privileges for this share.
      *
-     * @access public
-     *
      * @param string $group        The group to add permissions for.
      * @param integer $permission  A PERMS_* constant.
      */
@@ -720,8 +715,6 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Removes a certain privilege from a group.
      *
-     * @access public
-     *
      * @param string $group         The group to remove permissions from.
      * @param constant $permission  A PERMS_* constant.
      */
@@ -735,11 +728,9 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Checks to see if a user has a given permission.
      *
-     * @access public
-     *
-     * @param string $userid   The userid of the user.
-     * @param integer $priv    A PERMS_* constant to test for.
-     * @param string $creator  The creator of the event.
+     * @param string $userid       The userid of the user.
+     * @param integer $permission  A PERMS_* constant to test for.
+     * @param string $creator      The creator of the event.
      *
      * @return boolean  Whether or not $userid has $permission.
      */
@@ -755,8 +746,6 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Removes a user from this share.
      *
-     * @access public
-     *
      * @param string $userid  The userid of the user to remove.
      */
     function removeUser($userid)
@@ -770,10 +759,8 @@ class DataTreeObject_Share extends DataTreeObject {
     }
 
     /**
-     * Returns an array containing all the userids of the users with
-     * access to this share.
-     *
-     * @access public
+     * Returns an array containing all the userids of the users with access to
+     * this share.
      *
      * @param integer $perm_level  List only users with this permission level.
      *                             Defaults to all users.
@@ -787,15 +774,13 @@ class DataTreeObject_Share extends DataTreeObject {
     }
 
     /**
-     * Returns an array containing all the groupids of the groups
-     * with access to this share.
-     *
-     * @access public
+     * Returns an array containing all the groupids of the groups with access
+     * to this share.
      *
      * @param integer $perm_level  List only users with this permission level.
      *                             Defaults to all users.
      *
-     * @return array  The users with access to this share.
+     * @return array  The IDs of the groups with access to this share.
      */
     function listGroups($perm_level = null)
     {
@@ -806,10 +791,8 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * TODO
      *
-     * @access public
-     *
      * @param TODO
-     * @param optional boolean $update  TODO
+     * @param boolean $update  TODO
      *
      * @return TODO
      */
@@ -825,8 +808,6 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * TODO
      *
-     * @access public
-     *
      * @return TODO
      */
     function &getPermission()
@@ -841,8 +822,6 @@ class DataTreeObject_Share extends DataTreeObject {
      * Forces all children of this share to inherit the permissions set on this
      * share.
      *
-     * @access public
-     *
      * @return TODO
      */
     function inheritPermissions()
@@ -861,10 +840,8 @@ class DataTreeObject_Share extends DataTreeObject {
     }
 
     /**
-     * Saves any changes to this object to the backend
-     * permanently. New objects are added instead.
-     *
-     * @access public
+     * Saves any changes to this object to the backend permanently. New objects
+     * are added instead.
      *
      * @return boolean | PEAR_Error  PEAR_Error on failure.
      */
