@@ -1,19 +1,16 @@
 <?php
 /**
- * Horde_Template::
- *
- * Horde Template system. Adapted from bTemplate, by Brian Lozier
+ * Horde Template system. Adapted from bTemplate by Brian Lozier
  * <brian@massassi.net>.
  *
- * $Horde: framework/Template/Template.php,v 1.38.10.1 2005/01/03 12:19:16 jan Exp $
+ * $Horde: framework/Template/Template.php,v 1.38.10.8 2006/01/22 19:12:26 slusarz Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.38.10.1 $
  * @since   Horde 3.0
  * @package Horde_Template
  */
@@ -21,51 +18,72 @@ class Horde_Template {
 
     /**
      * Option values.
-     * @var array $_options
+     *
+     * @var array
      */
     var $_options = array();
 
     /**
      * Directory that templates should be read from.
-     * @var string $_basepath
+     *
+     * @var string
      */
     var $_basepath = '';
 
     /**
      * Reset template variables after parsing?
-     * @var boolean $_resetVars
+     *
+     * @var boolean
      */
     var $_resetVars = true;
 
     /**
      * Tag (scalar) values.
-     * @var array $_scalars
+     *
+     * @var array
      */
     var $_scalars = array();
 
     /**
      * Loop tag values.
-     * @var array $_arrays
+     *
+     * @var array
      */
     var $_arrays = array();
 
     /**
      * Cloop tag values.
-     * @var array $_carrays
+     *
+     * @var array
      */
     var $_carrays = array();
 
     /**
      * If tag values.
-     * @var array $_ifs
+     *
+     * @var array
      */
     var $_ifs = array();
 
+    /**
+     * Name of cached template file.
+     *
+     * @var string
+     */
+    var $_templateFile = null;
+
+    /**
+     * Cached source of template file.
+     *
+     * @var string
+     */
+    var $_template = null;
+
     /**
      * Constructor. Can set the template base path and whether or not
      * to drop template variables after a parsing a template.
      *
-     * @param string $basepath    The directory where templates are read from.
+     * @param string  $basepath   The directory where templates are read from.
      * @param boolean $resetVars  Drop template vars after parsing a template?
      */
     function Horde_Template($basepath = null, $resetVars = true)
@@ -87,6 +105,17 @@ class Horde_Template {
         $this->_options[$option] = $val;
     }
 
+    /**
+     * Set the template contents to a string.
+     *
+     * @param string $template  The template text.
+     */
+    function setTemplate($template)
+    {
+        $this->_template = $template;
+        $this->_templateFile = 'string';
+    }
+
     /**
      * Returns an option's value.
      *
@@ -102,10 +131,10 @@ class Horde_Template {
     /**
      * Sets a tag, loop, or if variable.
      *
-     * @param string|array $tag  Either the tag name or a hash with tag names
-     *                           as keys and tag values as values.
-     * @param mixed $var         The value to replace the tag with.
-     * @param boolean $isIf      Is this for an <if:> tag? (Default: no).
+     * @param string|array $tag   Either the tag name or a hash with tag names
+     *                            as keys and tag values as values.
+     * @param mixed        $var   The value to replace the tag with.
+     * @param boolean      $isIf  Is this for an <if:> tag? (Default: no).
      */
     function set($tag, $var, $isIf = false)
     {
@@ -143,13 +172,13 @@ class Horde_Template {
      */
     function get($tag)
     {
-      if (isset($this->_arrays[$tag])) {
-          return $this->_arrays[$tag];
-      }
-      if (isset($this->_scalars[$tag])) {
-          return $this->_scalars[$tag];
-      }
-      return null;
+        if (isset($this->_arrays[$tag])) {
+            return $this->_arrays[$tag];
+        }
+        if (isset($this->_scalars[$tag])) {
+            return $this->_scalars[$tag];
+        }
+        return null;
     }
 
     /**
@@ -191,6 +220,25 @@ class Horde_Template {
         }
     }
 
+    /**
+     * Retrieve list of blocks present in the template source.
+     *
+     * @param string $filename  The file to fetch the template from.
+     * @return mixed  An array of applications, or PEAR_Error on failure.
+     */
+    function listBlocks()
+    {
+        $blocks = array();
+        if (preg_match_all('<block:([a-zA-Z0-9_]+:[a-zA-Z0-9_]+)>',
+                           $this->_template, $matches, PREG_SET_ORDER)) {
+            foreach ($matches as $match) {
+                $blocks[] = $match[1];
+            }
+        }
+
+        return $blocks;
+    }
+
     /**
      * Fetches a template from the specified file and return the parsed
      * contents.
@@ -201,22 +249,11 @@ class Horde_Template {
      */
     function fetch($filename)
     {
-        // Prepare the path.
-        $file = $this->_basepath . $filename;
-
-        // Open the file.
-        $fp = @fopen($file, 'rb');
-        if (!$fp) {
-            require_once 'PEAR.php';
-            return PEAR::raiseError(sprintf(_("Template '%s' not found."), $file));
+        $contents = $this->_getTemplate($filename);
+        if (is_a($contents, 'PEAR_Error')) {
+            return $contents;
         }
 
-        // Read the file.
-        $contents = fread($fp, filesize($file));
-
-        // Close the file.
-        fclose($fp);
-
         // Parse and return the contents.
         return $this->parse($contents);
     }
@@ -228,8 +265,12 @@ class Horde_Template {
      *
      * @return string  The parsed template.
      */
-    function parse($contents)
+    function parse($contents = null)
     {
+        if (!$contents) {
+            $contents = $this->_template;
+        }
+
         // Process ifs.
         if (!empty($this->_ifs)) {
             foreach ($this->_ifs as $tag => $value) {
@@ -263,6 +304,11 @@ class Horde_Template {
             $contents = $this->_parseGettext($contents);
         }
 
+        // Parse embedded blocks
+        if ($this->getOption('blocks')) {
+            $contents = $this->_parseBlocks($contents);
+        }
+
         // Reset template data unless we're supposed to keep it
         // around.
         if ($this->_resetVars) {
@@ -276,6 +322,8 @@ class Horde_Template {
     /**
      * Returns full start and end tags for a named tag.
      *
+     * @access private
+     *
      * @param string $tag
      * @param string $directive  The kind of tag - tag, if, loop, cloop.
      *
@@ -291,6 +339,8 @@ class Horde_Template {
     /**
      * Formats a scalar tag (default format is <tag:name>).
      *
+     * @access private
+     *
      * @param string $tag  The name of the tag.
      *
      * @return string  The full tag with the current start/end delimiters.
@@ -303,6 +353,8 @@ class Horde_Template {
     /**
      * Extracts a portion of a template.
      *
+     * @access private
+     *
      * @param array $t           The tag to extract. Hash format is:
      *                             $t['b'] - The start tag
      *                             $t['e'] - The end tag
@@ -343,9 +395,89 @@ class Horde_Template {
         return $contents;
     }
 
+    /**
+     * Parses block tags.
+     *
+     * Here we parse block tags in the form:
+     *
+     *  <block:app:name>
+     *   <param:foo>foo parameter's value</param:foo>
+     *   <param:bar>bar parameter's value</param:bar>
+     *   <serial:baz>s:21:"baz parameter's value";</serial:baz>
+     *  </block:app:name>
+     *
+     * ... and replace them with rendered application blocks.
+     *
+     * Block parameters are provided with the nested "param:foo" and
+     * "serial:foo" tags.  The serial-style tags take a serialized PHP
+     * value, so you can use it to provide array, boolean, and complex
+     * type parameters.  The param-style tags provide string parameters.
+     *
+     * @access private
+     *
+     * @param string $contents  The unparsed contents of the file.
+     *
+     * @return string  The parsed contents of the file.
+     */
+    function _parseBlocks($contents)
+    {
+        return preg_replace('!<block:([a-zA-Z_0-9]+):([a-zA-Z_0-9]+)>(.*?)' .
+                            '</block:([a-zA-Z_0-9]+):([a-zA-Z_0-9]+)>!se',
+                            "\$this->_parseBlock('\\1', '\\2', '\\3')",
+                            $contents);
+    }
+
+    /**
+     * Handle the contents of one block tag.
+     *
+     * @access private
+     */
+    function _parseBlock($app, $name, $contents)
+    {
+        require_once 'Horde/Block.php';
+        require_once 'Horde/Block/Collection.php';
+
+        // Undo preg_replace() encoding strangeness.
+        $contents = str_replace('\\"', '"', $contents);
+
+        // Find block's parameters.
+        $params = array();
+        if (preg_match_all('!<param:([a-zA-Z_0-9]+)>(.*?)' .
+                           '</param:([a-zA-Z_0-9]+)>!s', $contents, $matches,
+                           PREG_SET_ORDER)) {
+            foreach ($matches as $match) {
+                $params[$match[1]] = html_entity_decode($match[2]);
+            }
+        }
+        if (preg_match_all('!<serial:([a-zA-Z_0-9]+)>(.*?)' .
+                           '</serial:([a-zA-Z_0-9]+)>!s', $contents, $matches,
+                           PREG_SET_ORDER)) {
+            foreach ($matches as $match) {
+                $params[$match[1]] = unserialize(html_entity_decode($match[2]));
+            }
+        }
+
+        // Construct the block.
+        $block = &Horde_Block_Collection::getBlock($app, $name, $params);
+        if (is_a($block, 'PEAR_Error')) {
+            $result = '<strong>' . sprintf(_("Error creating block: %s"),
+                                      $block->getMessage()) . '</strong>';
+        } else {
+            $result = $block->getContent();
+            if (is_a($result, 'PEAR_Error')) {
+                $result = '<strong>' . sprintf(_("Error getting content: %s"),
+                                          $result->getMessage()) . '</strong>';
+            }
+        }
+
+        return $result;
+    }
+
     /**
      * Parses a given if statement.
      *
+     * @access private
+     *
      * @param string $tag       The name of the if block to parse.
      * @param string $contents  The unparsed contents of the if block.
      *
@@ -392,6 +524,8 @@ class Horde_Template {
     /**
      * Parses the given array for any loops or other uses of the array.
      *
+     * @access private
+     *
      * @param string $tag       The name of the loop to parse.
      * @param array  $array     The values for the loop.
      * @param string $contents  The unparsed contents of the loop.
@@ -420,7 +554,15 @@ class Horde_Template {
                         foreach ($value as $key2 => $value2) {
                             if (!is_array($value2)) {
                                 // Replace associative array tags.
-                                $i = str_replace($this->_getTag($tag . '.' . $key2), $value2, $i);
+                                $aa_tag = $tag . '.' . $key2;
+                                $i = str_replace($this->_getTag($aa_tag), $value2, $i);
+                                $pos = strpos($tag, '.');
+                                if (($pos !== false) &&
+                                    !empty($this->_ifs[substr($tag, 0, $pos)])) {
+                                    $this->_ifs[$aa_tag] = $value2;
+                                    $i = $this->_parseIf($aa_tag, $i);
+                                    unset($this->_ifs[$aa_tag]);
+                                }
                             } else {
                                 // Check to see if it's a nested loop.
                                 $i = $this->_parseLoop($tag . '.' . $key2, $value2, $i);
@@ -467,6 +609,8 @@ class Horde_Template {
     /**
      * Parses the given case loop (cloop).
      *
+     * @access private
+     *
      * @param string $tag       The name of the cloop to parse.
      * @param array  $array     The values for the cloop.
      * @param string $contents  The unparsed contents of the cloop.
@@ -523,4 +667,38 @@ class Horde_Template {
         return $contents;
     }
 
+    /**
+     * Fetch the contents of a template into $this->_template; cache
+     * the filename in $this->_templateFile.
+     *
+     * @access private
+     *
+     * @param string $filename  Location of template file on disk.
+     *
+     * @return string  The loaded template content.
+     */
+    function _getTemplate($filename = null)
+    {
+        if (!is_null($filename) && ($filename != $this->_templateFile)) {
+            $this->_template = null;
+        }
+
+        if (!is_null($this->_template)) {
+            return $this->_template;
+        }
+
+        // Get the contents of the file.
+        $file = $this->_basepath . $filename;
+        $contents = @file_get_contents($file);
+        if ($contents === false) {
+            require_once 'PEAR.php';
+            return PEAR::raiseError(sprintf(_("Template \"%s\" not found."), $file));
+        }
+
+        $this->_template = $contents;
+        $this->_templateFile = $filename;
+
+        return $this->_template;
+    }
+
 }
