@@ -12,10 +12,10 @@ define('TEXT_HTML_NOHTML_NOBREAK', 5);
 /**
  * Text_Filter:: is a parent class for defining stackable text filters.
  *
- * $Horde: framework/Text_Filter/Filter.php,v 1.15.2.4 2005/07/03 05:14:18 selsky Exp $
+ * $Horde: framework/Text_Filter/Filter.php,v 1.15.2.7 2006/01/01 21:28:37 jan Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2004-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -155,12 +155,15 @@ class Text_Filter {
         } else {
             @include_once 'Horde/Text/Filter/' . $driver . '.php';
         }
+
         $class = 'Text_Filter_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $filter = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $filter = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $filter;
     }
 
     /**
