@@ -1,4 +1,7 @@
 <?php
+
+require_once 'Horde.php';
+
 /**
  * The Text_Filter_emoticons:: class finds emoticon strings ( :), etc.) in a
  * block of text and turns them into image links.
@@ -8,15 +11,14 @@
  * entities -- If true the html entity versions of the patterns will be used.
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/emoticons.php,v 1.17.10.1 2005/01/03 12:19:16 jan Exp $
+ * $Horde: framework/Text_Filter/Filter/emoticons.php,v 1.17.10.7 2006/01/01 21:28:38 jan Exp $
  *
- * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2006 Marko Djukic <marko@oblo.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did not
  * receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Marko Djukic <marko@oblo.com>
- * @version $Revision: 1.17.10.1 $
  * @package Horde_Text
  */
 class Text_Filter_emoticons extends Text_Filter {
@@ -24,7 +26,7 @@ class Text_Filter_emoticons extends Text_Filter {
     /**
      * Filter parameters.
      *
-     * @var array $_params
+     * @var array
      */
     var $_params = array('entities' => false);
 
@@ -40,14 +42,16 @@ class Text_Filter_emoticons extends Text_Filter {
 
         /* Loop through possible string emoticons and convert to graphics. */
         foreach ($this->getIcons() as $string => $icon) {
+            $end_pattern = '\s';
             if ($this->_params['entities']) {
                 $string = htmlspecialchars($string);
+                $end_pattern = '(\s|<br />)';
             }
 
             /* Check for a smiley either immediately at the start of a line or
              * following a space. Use {} as the preg delimiters as this is not
              * found in any smiley. */
-            $regexp['{(^|\s)' . preg_quote($string) . '}'] = ' ' . Horde::img($icon . '.png', $string, 'align="middle"', $iconpath);
+            $regexp['{(^|\s)' . preg_quote($string) . $end_pattern . '}'] = ' ' . Horde::img($icon . '.png', $string, 'align="middle" title="' . $string . '"', $iconpath) . (($this->_params['entities']) ? '\\2' : '');
         }
 
         return array('regexp' => $regexp);
@@ -90,7 +94,9 @@ class Text_Filter_emoticons extends Text_Filter {
             ':-(' => 'frown',
             ':D' => 'biggrin',
             '8)' => 'cool',
-            '8p' => 'cooltongue', // '8' => 'cooltongue',
+            // In English, 8PM occurs sufficiently often to specifically
+            // search for and exclude
+            '8p(?<![Mm]\s+)' => 'cooltongue', // '8' => 'cooltongue',
             '8D' => 'coolgrin',
             ':p' => 'tongueout', ':P' => 'tongueout', // ':' => 'tongueout',
             '?:(' => 'confused', '%-(' => 'confused',
