@@ -17,10 +17,10 @@ require_once 'Horde/String.php';
  * 'citeblock'  -- Display cite blocks? Defaults to true,
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/highlightquotes.php,v 1.6.8.2 2005/01/11 20:44:48 chuck Exp $
+ * $Horde: framework/Text_Filter/Filter/highlightquotes.php,v 1.6.8.8 2006/03/14 22:43:39 jan Exp $
  *
- * Copyright 2004-2005 Michael Slusarz <slusarz@curecanti.org>
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2006 Michael Slusarz <slusarz@curecanti.org>
+ * Copyright 2004-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -35,7 +35,7 @@ class Text_Filter_highlightquotes extends Text_Filter {
     /**
      * Filter parameters.
      *
-     * @var array $_params
+     * @var array
      */
     var $_params = array('hideBlocks' => false, 'citeblock' => true);
 
@@ -87,9 +87,8 @@ class Text_Filter_highlightquotes extends Text_Filter {
                                       array($this, '_addQuoteToggles'), $text);
 
         /* Use cite blocks to display the different quoting levels. */
-        $blankline = false;
         $cb = $this->_params['citeblock'];
-        $qlevel = 0;
+        $blankline = $qlevel = 0;
         $text_out = '';
 
         foreach (explode("\n", $text) as $line) {
@@ -111,7 +110,7 @@ class Text_Filter_highlightquotes extends Text_Filter {
                     $line = substr($line, 0, -6);
                 }
             } elseif ($clevel > $qlevel) {
-                if ($blankline) {
+                for ($i = 2; $i <= $blankline; $i++) {
                     $text_out = substr(rtrim($text_out), 0, -6);
                 }
                 for ($i = $qlevel; $i < $clevel; $i++) {
@@ -124,9 +123,12 @@ class Text_Filter_highlightquotes extends Text_Filter {
             }
 
             if ($line == '<br />') {
-                $blankline = true;
+                ++$blankline;
+                if ($clevel) {
+                    ++$blankline;
+                }
             } else {
-                $blankline = false;
+                $blankline = 0;
             }
 
             $text_out .= $line . "\n";
@@ -134,7 +136,7 @@ class Text_Filter_highlightquotes extends Text_Filter {
         }
 
         /* Remove the leading newline we added above, if it's still there. */
-        if ($text_out{0} == "\n") {
+        if ($text_out[0] == "\n") {
             $text_out = substr($text_out, 1);
         }
 
@@ -168,7 +170,7 @@ class Text_Filter_highlightquotes extends Text_Filter {
             $text = '<br />';
         };
         $text .= '<div id="qt_' . $i . '">' .
-            Horde::link('#', '', 'widget', '', 'toggleQuoteBlock(\'' . $i . '\', \'' . $lines . '\'); return false;', '', '', array('style' => 'font-size:70%')) .
+            Horde::link('#', '', 'widget togglequote', '', 'toggleQuoteBlock(\'' . $i . '\', \'' . $lines . '\'); return false;', '', '', array('style' => 'font-size:70%')) .
             ($this->_params['hideBlocks'] ? sprintf(_("[Show Quoted Text - %s lines]"), $lines) : _("[Hide Quoted Text]")) .
             '</a></div><div id="qb_' . $i . '"' . ($this->_params['hideBlocks'] ? ' style="display:none;"' : '') . '>' . $matches[0] . '</div>';
 
