@@ -2,9 +2,9 @@
 /**
  * Highlights simple markup as used in emails or usenet postings.
  *
- * $Horde: framework/Text_Filter/Filter/simplemarkup.php,v 1.1.10.2 2005/01/11 20:44:49 chuck Exp $
+ * $Horde: framework/Text_Filter/Filter/simplemarkup.php,v 1.1.10.4 2006/01/01 21:28:38 jan Exp $
  *
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -24,15 +24,15 @@ class Text_Filter_simplemarkup extends Text_Filter {
     {
         $regexp = array(
             // Bold.
-            '/(\s|\n)(\*[^*\s]+\*)(\s|\r|\n|<br)/i' => '\1<b>\2</b>\3',
+            '/(\s|\n)(\*[^*\s]+\*)(\s|\r|\n|<br)/i' => '\1<strong>\2</strong>\3',
 
             // Underline.
             '/(\s|\n)(_[^_\s]+_)(\s|\r|\n|<br)/i' => '\1<u>\2</u>\3',
 
             // Italic.
-            ';(\s|\n)(/[^/\s]+/)(\s|\r|\n|<br);i' => '\1<i>\2</i>\3');
+            ';(\s|\n)(/[^/\s]+/)(\s|\r|\n|<br);i' => '\1<em>\2</em>\3');
 
         return array('regexp' => $regexp);
     }
 
-}
\ No newline at end of file
+}
