@@ -5,10 +5,10 @@ require_once dirname(__FILE__) . '/Widget.php';
 /**
  * The Horde_UI_Tabs:: class manages and renders a tab-like interface.
  *
- * $Horde: framework/UI/UI/Tabs.php,v 1.27.6.5 2005/03/20 16:13:26 chuck Exp $
+ * $Horde: framework/UI/UI/Tabs.php,v 1.27.6.9 2006/01/01 21:28:40 jan Exp $
  *
  * Copyright 2001-2003 Robert E. Coyle <robertecoyle@hotmail.com>
- * Copyright 2003-2005 Jason M. Felice <jfelice@cronosys.com>
+ * Copyright 2003-2006 Jason M. Felice <jfelice@cronosys.com>
  *
  * See the enclosed file LICENSE for license information (LGPL).
  *
@@ -19,15 +19,14 @@ class Horde_UI_Tabs extends Horde_UI_Widget {
 
     /**
      * The array of tabs.
-     * @var array $_tabs
+     *
+     * @var array
      */
     var $_tabs = array();
 
     /**
      * Add a tab to the interface.
      *
-     * @access public
-     *
      * @param string $title    The text which appears on the tab.
      * @param string $link     The target page.
      * @param mixed  $params   Either a string value to set the tab variable to,
@@ -48,8 +47,6 @@ class Horde_UI_Tabs extends Horde_UI_Widget {
     /**
      * Retrieve the title of the tab with the specified name.
      *
-     * @access public
-     *
      * @param string $tabname  The name of the tab.
      *
      * @return string  The tab's title.
@@ -74,7 +71,7 @@ class Horde_UI_Tabs extends Horde_UI_Widget {
      */
     function render($active_tab = null)
     {
-        $html = "<ul class=\"tabset\">\n";
+        $html = "<div class=\"tabset\"><ul>\n";
 
         $first = true;
         $active = $_SERVER['PHP_SELF'] . $this->_vars->get($this->_name);
@@ -86,10 +83,10 @@ class Horde_UI_Tabs extends Horde_UI_Widget {
                                            $tab['tabname']);
             }
 
-            $class = null;
+            $class = '';
             if ((!is_null($active_tab) && $active_tab == $tab['tabname']) ||
                 ($active == $tab['link'] . $tab['tabname'])) {
-                $class = 'activeTab';
+                $class = ' class="activeTab"';
             }
 
             if (!isset($tab['target'])) {
@@ -102,13 +99,13 @@ class Horde_UI_Tabs extends Horde_UI_Widget {
 
             $accesskey = Horde::getAccessKey($tab['title']);
 
-            $html .= '<li>' .
-                Horde::link($link, $tab['title'], $class, $tab['target'], $tab['onclick'], null, $accesskey) .
-                str_replace(' ', '&nbsp;', Horde::highlightAccessKey($tab['title'], $accesskey)) .
+            $html .= '<li' . $class . '>' .
+                Horde::link($link, '', '', $tab['target'], $tab['onclick'], null, $accesskey) .
+                Horde::highlightAccessKey(str_replace(' ', '&nbsp;', $tab['title']), $accesskey) .
                 "</a> </li>\n";
         }
 
-        return $html . "</ul>\n";
+        return $html . "</ul></div><br class=\"clear\" />\n";
     }
 
 }
