@@ -3,9 +3,9 @@
  * The Horde_UI_VarRenderer:: class provides base functionality for
  * other Horde UI elements.
  *
- * $Horde: framework/UI/UI/VarRenderer.php,v 1.12.10.5 2005/07/03 05:05:55 selsky Exp $
+ * $Horde: framework/UI/UI/VarRenderer.php,v 1.12.10.9 2006/01/01 21:28:40 jan Exp $
  *
- * Copyright 2003-2005 Jason M. Felice <jfelice@cronosys.com>
+ * Copyright 2003-2006 Jason M. Felice <jfelice@cronosys.com>
  *
  * See the enclosed file LICENSE for license information (LGPL).
  *
@@ -21,6 +21,13 @@ class Horde_UI_VarRenderer {
      */
     var $_params;
 
+    /**
+     * Charset to use for output.
+     *
+     * @var string
+     */
+    var $_charset;
+
     /**
      * Constructs a new renderer.
      *
@@ -30,6 +37,7 @@ class Horde_UI_VarRenderer {
     function Horde_UI_VarRenderer($params = array())
     {
         $this->_params = $params;
+        $this->_charset = NLS::getCharset();
     }
 
     /**
@@ -63,10 +71,12 @@ class Horde_UI_VarRenderer {
 
         $class = 'Horde_UI_VarRenderer_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $vr = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $vr = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $vr;
     }
 
     /**
