@@ -13,10 +13,10 @@ define('HORDE_ERROR_DRIVER_CONFIG', 2);
 /**
  * The Util:: class provides generally useful methods of different kinds.
  *
- * $Horde: framework/Util/Util.php,v 1.384.6.3 2005/01/12 15:54:42 chuck Exp $
+ * $Horde: framework/Util/Util.php,v 1.384.6.15 2006/03/28 12:31:12 jan Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -37,12 +37,20 @@ class Util {
      */
     function &cloneObject(&$obj)
     {
+        /* Cloning does *not* currently work if using PHP 5.
+         * This is a known PHP issue: http://bugs.php.net/bug.php?id=27268
+         * Therefore, work around this issue by using the PHP 4
+         * serialize/unserialize clone hack. */
+        /*
         if (version_compare(zend_version(), '2', '>')) {
             return clone($obj);
         } else {
             $newObj = $obj;
             return $newObj;
         }
+        */
+        $ret = unserialize(serialize($obj));
+        return $ret;
     }
 
     /**
@@ -50,8 +58,6 @@ class Util {
      * highlight_string(), that prints the output directly, so that instead it
      * can be returned as a string and used.
      *
-     * @access public
-     *
      * @param string $function  The function to run.
      * @param mixed $arg1       First argument to $function().
      * @param mixed $arg2       Second argument to $function().
@@ -65,7 +71,7 @@ class Util {
         if (func_num_args() == 0) {
             return false;
         }
-        $eval = false;
+        $include = false;
         $args = func_get_args();
         $function = array_shift($args);
         if (is_array($function)) {
@@ -76,17 +82,31 @@ class Util {
                   ($function == 'include_once') ||
                   ($function == 'require') ||
                   ($function == 'require_once')) {
-            $eval = true;
-        } elseif (!function_exists($function) &&
-                  ($function != 'eval')) {
+            $include = true;
+        } elseif (!function_exists($function)) {
             return false;
         }
 
         ob_start();
-        if ($eval) {
-            eval($function . " '" . implode(',', $args) . "';");
-        } elseif ($function == 'eval') {
-            eval($args[0]);
+        if ($include) {
+            $file = implode(',', $args);
+            switch ($function) {
+            case 'include':
+                include $file;
+                break;
+
+            case 'include_once':
+                include_once $file;
+                break;
+
+            case 'require':
+                require $file;
+                break;
+
+            case 'require_once':
+                require_once $file;
+                break;
+            }
         } else {
             call_user_func_array($function, $args);
         }
@@ -101,21 +121,22 @@ class Util {
      * POST, or cookie input. The value being checked MUST be in the global
      * scope.
      *
-     * @access public
-     *
      * @param string $varname  The variable name to check.
+     * @param mixed $default   Default value if the variable isn't present
+     *                         or was specified by the user. Defaults to null.
+     *                         @since Horde 3.1
      *
-     * @return mixed  Null if the var is in user input, the variable value
-     *                otherwise.
+     * @return mixed  $default if the var is in user input or not present,
+     *                the variable value otherwise.
      */
-    function nonInputVar($varname)
+    function nonInputVar($varname, $default = null)
     {
         if (isset($_GET[$varname]) ||
             isset($_POST[$varname]) ||
             isset($_COOKIE[$varname])) {
-            return null;
+            return $default;
         } else {
-            return isset($GLOBALS[$varname]) ? $GLOBALS[$varname] : null;
+            return isset($GLOBALS[$varname]) ? $GLOBALS[$varname] : $default;
         }
     }
 
@@ -124,8 +145,6 @@ class Util {
      * there are existing parameters and whether to use ?, & or &amp; as the
      * glue.  All data will be urlencoded.
      *
-     * @access public
-     *
      * @param string $url       The URL to modify
      * @param mixed $parameter  Either the name=value pair to add
      *                          (DEPRECATED) -or-
@@ -137,8 +156,6 @@ class Util {
      *                          yet, the argument separator gets encoded.
      *
      * @return string  The modified URL.
-     *
-     * @since Horde 2.1
      */
     function addParameter($url, $parameter, $value = null, $encode = true)
     {
@@ -146,11 +163,8 @@ class Util {
             return $url;
         }
 
+        /* Make sure to have an array. */
         if (!is_array($parameter)) {
-            /* This is deprecated should be removed in the future. */
-            if (is_null($value)) {
-                @list($parameter, $value) = explode('=', $parameter, 2);
-            }
             $add = array($parameter => $value);
         } else {
             $add = $parameter;
@@ -201,15 +215,11 @@ class Util {
     /**
      * Removes name=value pairs from a URL.
      *
-     * @access public
-     *
      * @param string $url    The URL to modify.
      * @param mixed $remove  Either a single parameter to remove or an array
      *                       of parameters to remove.
      *
      * @return string  The modified URL.
-     *
-     * @since Horde 2.2
      */
     function removeParameter($url, $remove)
     {
@@ -274,8 +284,6 @@ class Util {
      * Returns a url with the 'nocache' parameter added, if the browser is
      * buggy and caches old URLs.
      *
-     * @access public
-     *
      * @param string $url  The URL to modify.
      *
      * @return string  The requested URI.
@@ -302,8 +310,6 @@ class Util {
     /**
      * Returns a hidden form input containing the session name and id.
      *
-     * @access public
-     *
      * @param boolean $append_session  0 = only if needed, 1 = always.
      *
      * @return string  The hidden form input, if needed/requested.
@@ -321,8 +327,6 @@ class Util {
     /**
      * Prints a hidden form input containing the session name and id.
      *
-     * @access public
-     *
      * @param boolean $append_session  0 = only if needed, 1 = always.
      */
     function pformInput($append_session = 0)
@@ -333,8 +337,6 @@ class Util {
     /**
      * If magic_quotes_gpc is in use, run stripslashes() on $var.
      *
-     * @access public
-     *
      * @param string &$var  The string to un-quote, if necessary.
      *
      * @return string  $var, minus any magic quotes.
@@ -364,8 +366,6 @@ class Util {
      * POST data, the value from the POST data will be returned and the GET
      * value will be ignored.
      *
-     * @access public
-     *
      * @param string $var      The name of the form variable to look for.
      * @param string $default  The value to return if the variable is not
      *                         there.
@@ -382,15 +382,11 @@ class Util {
      * Gets a form variable from GET data, stripped of magic quotes if
      * necessary. This function will NOT return a POST variable.
      *
-     * @access public
-     *
      * @param string $var      The name of the form variable to look for.
      * @param string $default  The value to return if the variable is not
      *                         there.
      *
      * @return string  The cleaned form variable, or $default.
-     *
-     * @since Horde 2.2
      */
     function getGet($var, $default = null)
     {
@@ -403,15 +399,11 @@ class Util {
      * Gets a form variable from POST data, stripped of magic quotes if
      * necessary. This function will NOT return a GET variable.
      *
-     * @access public
-     *
      * @param string $var      The name of the form variable to look for.
      * @param string $default  The value to return if the variable is not
      *                         there.
      *
      * @return string  The cleaned form variable, or $default.
-     *
-     * @since Horde 2.2
      */
     function getPost($var, $default = null)
     {
@@ -423,8 +415,6 @@ class Util {
     /**
      * Determines the location of the system temporary directory.
      *
-     * @access public
-     *
      * @return string  A directory name which can be used for temp files.
      *                 Returns false if one could not be found.
      */
@@ -497,8 +487,6 @@ class Util {
     /**
      * Creates a temporary directory in the system's temporary directory.
      *
-     * @access public
-     *
      * @param boolean $delete   Delete the temporary directory at the end of
      *                          the request?
      * @param string $temp_dir  Use this temporary directory as the directory
@@ -534,6 +522,51 @@ class Util {
         return $temp_dir;
     }
 
+    /**
+     * Returns the canonical path of the string.  Like PHP's built-in
+     * realpath() except the directory need not exist on the local server.
+     *
+     * Algorithim loosely based on code from the Perl File::Spec::Unix module
+     * (version 1.5).
+     *
+     * @since Horde 3.0.5
+     *
+     * @param string $path  A file path.
+     *
+     * @return string  The canonicalized file path.
+     */
+    function realPath($path)
+    {
+        /* Standardize on UNIX directory separators. */
+        if (substr(PHP_OS, 0, 3) == 'WIN') {
+            $path = str_replace('\\', '/', $path);
+        }
+
+        /* xx////xx -> xx/xx
+         * xx/././xx -> xx/xx */
+        $path = preg_replace(array("|/+|", "@(/\.)+(/|\Z(?!\n))@"), array('/', '/'), $path);
+
+        /* ./xx -> xx */
+        if ($path != './') {
+            $path = preg_replace("|^(\./)+|", '', $path);
+        }
+
+        /* /../../xx -> xx */
+        $path = preg_replace("|^/(\.\./?)+|", '/', $path);
+
+        /* xx/ -> xx */
+        if ($path != '/') {
+            $path = preg_replace("|/\Z(?!\n)|", '', $path);
+        }
+
+        /* /xx/.. -> / */
+        while (strpos($path, '/..') !== false) {
+            $path = preg_replace("|/[^/]+/\.\.|", '', $path);
+        }
+
+        return empty($path) ? '/' : $path;
+    }
+
     /**
      * Removes given elements at request shutdown.
      *
@@ -551,8 +584,6 @@ class Util {
      * The second parameter allows the unregistering of previously registered
      * elements.
      *
-     * @access public
-     *
      * @param string $filename   The filename to be deleted at the end of the
      *                           request.
      * @param boolean $register  If true, then register the element for
@@ -566,7 +597,7 @@ class Util {
         static $dirs, $files, $securedel;
 
         /* Initialization of variables and shutdown functions. */
-        if (is_null($dirs)){
+        if (is_null($dirs)) {
             $dirs = array();
             $files = array();
             $securedel = array();
@@ -651,14 +682,12 @@ class Util {
     /**
      * Outputs javascript code to close the current window.
      *
-     * @access public
-     *
      * @param string $code  Any addtional javascript code to run before
      *                      closing the window.
      */
     function closeWindowJS($code = '')
     {
-        echo '<script language="JavaScript" type="text/javascript">' . $code .
+        echo '<script type="text/javascript">' . $code .
             'window.close();</script>';
     }
 
@@ -758,7 +787,7 @@ class Util {
         foreach ($fields as $field) {
             if (!isset($params[$field])) {
                 require_once 'PEAR.php';
-                return PEAR::throwError(sprintf(_("Required '%s' not specified in configuration."), $field, $name),
+                return PEAR::throwError(sprintf(_("Required \"%s\" not specified in configuration."), $field, $name),
                                         HORDE_ERROR_DRIVER_CONFIG,
                                         $info);
             }
