@@ -5,9 +5,9 @@ require_once dirname(__FILE__) . '/rcs.php';
 /**
  * VC_cvs implementation.
  *
- * Copyright 2000-2007 Anil Madhavapeddy, <anil@recoil.org>
+ * Copyright 2000-2006 Anil Madhavapeddy, <anil@recoil.org>
  *
- * $Horde: framework/VC/VC/cvs.php,v 1.32.2.25 2007/01/02 13:54:47 jan Exp $
+ * $Horde: framework/VC/VC/cvs.php,v 1.32.2.18 2006/01/01 21:28:42 jan Exp $
  *
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @package VC
@@ -41,7 +41,7 @@ class VC_cvs extends VC_rcs {
 
     function &queryDir($where)
     {
-        $dir = new VC_Directory_cvs($this, $where);
+        $dir = &new VC_Directory_cvs($this, $where);
         return $dir;
     }
 
@@ -50,7 +50,7 @@ class VC_cvs extends VC_rcs {
         return VC_Checkout_cvs::get($this, $file->queryFullPath(), $rev);
     }
 
-    function getDiff($file, $rev1, $rev2, $type = 'context', $num = 3, $ws = true)
+    function getDiff(&$file, $rev1, $rev2, $type = 'context', $num = 3, $ws = true)
     {
         return VC_Diff_cvs::get($this, $file, $rev1, $rev2, $type, $num, $ws);
     }
@@ -66,7 +66,7 @@ class VC_cvs extends VC_rcs {
 
     function &getAnnotateObject($filename)
     {
-        $blame = new VC_Annotate_cvs($this, $filename, Util::getTempFile('vc', true, $this->_paths['temp']));
+        $blame = &new VC_Annotate_cvs($this, $filename, Util::getTempFile('vc', true, $this->_paths['temp']));
         return $blame;
     }
 
@@ -92,10 +92,10 @@ class VC_Annotate_cvs {
     var $rep;
     var $tmpfile;
 
-    function VC_Annotate_cvs($rep, $file, $tmpfile)
+    function VC_Annotate_cvs(&$rep, $file, $tmpfile)
     {
-        $this->rep = $rep;
-        $this->file = $file;
+        $this->rep = &$rep;
+        $this->file = &$file;
         $this->tmpfile = $tmpfile;
     }
 
@@ -167,7 +167,7 @@ class VC_Annotate_cvs {
             if (preg_match('/^M\s+([\d\.]+)\s+\((.+)\s+(\d+-\w+-\d+)\):.(.*)$/', $line, $regs)) {
                 $entry = array();
                 $entry['rev']    = $regs[1];
-                $entry['author'] = trim($regs[2]);
+                $entry['author'] = $regs[2];
                 $entry['date']   = $regs[3];
                 $entry['line']   = $regs[4];
                 $entry['lineno'] = $lineno++;
@@ -206,21 +206,22 @@ class VC_Checkout_cvs {
      * @return resource|object  Either a PEAR_Error object, or a stream
      *                          pointer to the head of the checkout
      */
-    function get($rep, $fullname, $rev)
+    function get(&$rep, $fullname, $rev)
     {
         if (!VC_Revision::valid($rev)) {
             return PEAR::raiseError('Invalid revision number');
         }
 
         if (VC_WINDOWS) {
+            $Q = '"';
             $mode = 'rb';
-            $q_name = '"' . escapeshellcmd(str_replace('\\', '/', $fullname)) . '"';
+            $fullname = str_replace('\\', '/', $fullname);
         } else {
+            $Q = "'";
             $mode = 'r';
-            $q_name = escapeshellarg($fullname);
         }
 
-        if (!($RCS = popen($rep->getPath('co') . " -p$rev $q_name 2>&1", $mode))) {
+        if (!($RCS = popen($rep->getPath('co') . " -p$rev $Q$fullname$Q 2>&1", $mode))) {
             return PEAR::raiseError('Couldn\'t perform checkout of the requested file');
         }
 
@@ -271,7 +272,7 @@ class VC_Diff_cvs {
      * @return string|boolean  False on failure, or a string containing the
      *                         diff on success.
      */
-    function get($rep, $file, $rev1, $rev2, $type = 'context', $num = 3,
+    function get(&$rep, &$file, $rev1, $rev2, $type = 'context', $num = 3,
                  $ws = true)
     {
         /* Make sure that the file parameter is valid. */
@@ -358,8 +359,8 @@ class VC_Directory_cvs {
      */
     function VC_Directory_cvs($rep, $dn, $pn = '')
     {
-        $this->rep = $rep;
-        $this->parent = $pn;
+        $this->rep = &$rep;
+        $this->parent = &$pn;
         $this->moduleName = $dn;
         $this->dirName = $rep->sourceroot() . "/$dn";
         $this->files = array();
@@ -427,11 +428,11 @@ class VC_Directory_cvs {
                 }
             } elseif (@is_file($path) && substr($name, -2) == ',v') {
                 /* Spawn a new file object to represent this file. */
-                $fl = $this->rep->getFileObject(substr($path, strlen($this->rep->sourceroot()), -2), $cache, $quicklog);
+                $fl = &$this->rep->getFileObject(substr($path, strlen($this->rep->sourceroot()), -2), $cache, $quicklog);
                 if (is_a($fl, 'PEAR_Error')) {
                     return $fl;
                 } else {
-                    $this->files[] = $fl;
+                    $this->files[] = &$fl;
                 }
             }
         }
@@ -442,9 +443,9 @@ class VC_Directory_cvs {
 
         /* If we want to merge the attic, add it in here. */
         if ($showattic) {
-            $atticDir = new VC_Directory_cvs($this->rep, $this->moduleName . '/Attic', $this);
-            if (!is_a($atticDir->browseDir($cache, $quicklog), 'PEAR_Error')) {
-                $this->atticFiles = $atticDir->queryFileList();
+            $atticDir = &new VC_Directory_cvs($this->rep, $this->moduleName . '/Attic', $this);
+            if ($atticDir->browseDir($cache, $quicklog) == 1) {
+                $this->atticFiles = &$atticDir->queryFileList();
                 $this->mergedFiles = array_merge($this->files, $this->atticFiles);
             }
         }
@@ -586,21 +587,22 @@ class VC_File_cvs extends VC_File {
         $this->branches = array();
     }
 
-    function &getFileObject($rep, $filename, $cache = null, $quicklog = false)
+    function &getFileObject(&$rep, $filename, $cache = null, $quicklog = false)
     {
-        /* The version of the cached data. Increment this whenever the
+        /**
+         * The version of the cached data. Increment this whenever the
          * internal storage format changes, such that we must
-         * invalidate prior cached data. */
+         * invalidate prior cached data.
+         */
         $cacheVersion = 2;
         $cacheId = $rep->sourceroot() . '_n' . $filename . '_f' . (int)$quicklog . '_v' . $cacheVersion;
 
-        $ctime = time() - @filemtime($filename . ',v');
         if ($cache &&
-            $cache->exists($cacheId, $ctime)) {
-            $fileOb = unserialize($cache->get($cacheId, $ctime));
+            $cache->exists($cacheId, time() - @filemtime($filename . ',v'))) {
+            $fileOb = unserialize($cache->get($cacheId, time() - @filemtime($filename . ',v')));
             $fileOb->setRepository($rep);
         } else {
-            $fileOb = new VC_File_cvs($filename, $quicklog);
+            $fileOb = &new VC_File_cvs($filename, $quicklog);
             $fileOb->setRepository($rep);
             if (is_a(($result = $fileOb->getBrowseInfo()), 'PEAR_Error')) {
                 return $result;
@@ -751,9 +753,9 @@ class VC_File_cvs extends VC_File {
         /* Call the RCS rlog command to retrieve the file
          * information. */
         $flag = $this->quicklog ? ' -r ' : ' ';
-        $q_file = VC_WINDOWS ? '"' . escapeshellcmd($file) . '"' : escapeshellarg($file);
+        $Q = VC_WINDOWS ? '"' : "'" ;
 
-        $cmd = $this->rep->getPath('rlog') . $flag . $q_file;
+        $cmd = $this->rep->getPath('rlog') . $flag . $Q . $file . $Q;
         exec($cmd, $return_array, $retval);
 
         if ($retval) {
@@ -800,10 +802,10 @@ class VC_File_cvs extends VC_File {
                     $accum[] = $line;
                 } elseif (count($accum)) {
                     // spawn a new VC_log object and add it to the logs hash
-                    $log = new VC_Log_cvs($this);
+                    $log = &new VC_Log_cvs($this);
                     $err = $log->processLog($accum);
                     // TODO: error checks - avsm
-                    $this->logs[$log->queryRevision()] = $log;
+                    $this->logs[$log->queryRevision()] = &$log;
                     $this->revs[] = $log->queryRevision();
                     $accum = array();
                 }
@@ -858,9 +860,9 @@ class VC_Log_cvs {
     /**
      *
      */
-    function VC_Log_cvs($fl)
+    function VC_Log_cvs(&$fl)
     {
-        $this->file = $fl;
+        $this->file = &$fl;
         $this->branches = array();
     }
 
@@ -979,10 +981,12 @@ class VC_Log_cvs {
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @package VC
  */
-class VC_Patchset_cvs extends VC_Patchset {
+class VC_Patchset_cvs {
 
+    var $_rep;
     var $_dir;
     var $_name;
+    var $_patchsets = array();
 
     /**
      * Create a patchset object.
@@ -995,7 +999,7 @@ class VC_Patchset_cvs extends VC_Patchset {
         $this->_dir = dirname($file);
     }
 
-    function &getPatchsetObject($rep, $filename, $cache = null)
+    function &getPatchsetObject(&$rep, $filename, $cache = null)
     {
         /* The version of the cached data. Increment this whenever the
          * internal storage format changes, such that we must
@@ -1003,23 +1007,23 @@ class VC_Patchset_cvs extends VC_Patchset {
         $cacheVersion = 1;
         $cacheId = $rep->sourceroot() . '_n' . $filename . '_f_v' . $cacheVersion;
 
-        $ctime = time() - @filemtime($filename . ',v');
         if ($cache &&
-            $cache->exists($cacheId, $ctime)) {
-            $psOb = unserialize($cache->get($cacheId, $ctime));
-            $psOb->setRepository($rep);
+            $cache->exists($cacheId, time() - @filemtime($filename . ',v'))) {
+            $psOb = unserialize($cache->get($cacheId, time() - @filemtime($filename . ',v')));
         } else {
-            $psOb = new VC_Patchset_cvs($filename);
-            $psOb->setRepository($rep);
-            if (is_a(($result = $psOb->getPatchsets()), 'PEAR_Error')) {
-                return $result;
-            }
+            $psOb = &new VC_Patchset_cvs($filename);
 
             if ($cache) {
                 $cache->set($cacheId, serialize($psOb));
             }
         }
 
+        $psOb->_rep = &$rep;
+
+        if (is_a(($result = $psOb->getPatchsets($rep)), 'PEAR_Error')) {
+            return $result;
+        }
+
         return $psOb;
     }
 
@@ -1027,9 +1031,11 @@ class VC_Patchset_cvs extends VC_Patchset {
      * Populate the object with information about the patchsets that
      * this file is involved in.
      *
+     * @param string $repository  The full repository location.
+     *
      * @return boolean|object  PEAR_Error object on error, or true on success.
      */
-    function getPatchsets()
+    function getPatchsets($repository)
     {
         /* Check that we are actually in the filesystem. */
         if (!is_file($this->getFullPath() . ',v')) {
@@ -1037,15 +1043,14 @@ class VC_Patchset_cvs extends VC_Patchset {
         }
 
         /* Call cvsps to retrieve all patchsets for this file. */
-        $q_root = $this->_rep->sourceroot();
-        $q_root = VC_WINDOWS ? '"' . escapeshellcmd($q_root) . '"' : escapeshellarg($q_root);
+        $Q = VC_WINDOWS ? '"' : "'";
 
-        $cvsps_home = $this->_rep->getPath('cvsps_home');
+        $cvsps_home = $repository->getPath('cvsps_home');
         $HOME = !empty($cvsps_home) ?
             'HOME=' . $cvsps_home . ' ' :
             '';
 
-        $cmd = $HOME . $this->_rep->getPath('cvsps') . ' -u --cvs-direct --root ' . $q_root . ' -f ' . escapeshellarg($this->_name) . ' ' . escapeshellarg($this->_dir);
+        $cmd = $HOME . $repository->getPath('cvsps') . ' -u --cvs-direct --root ' . $Q . $repository->sourceroot() . $Q . ' -f ' . $Q . $this->_name . $Q . ' ' . $Q . $this->_dir . $Q;
         exec($cmd, $return_array, $retval);
         if ($retval) {
             return PEAR::raiseError('Failed to spawn cvsps to retrieve patchset information');
@@ -1081,15 +1086,11 @@ class VC_Patchset_cvs extends VC_Patchset {
                     break;
 
                 case 'Branch':
-                    if (trim($info[1]) != 'HEAD') {
-                        $this->_patchsets[$id]['branch'] = trim($info[1]);
-                    }
+                    $this->_patchsets[$id]['branch'] = trim($info[1]);
                     break;
 
                 case 'Tag':
-                    if (trim($info[1]) != '(none)') {
-                        $this->_patchsets[$id]['tag'] = trim($info[1]);
-                    }
+                    $this->_patchsets[$id]['tag'] = trim($info[1]);
                     break;
 
                 case 'Log':
