@@ -3,12 +3,17 @@
 require_once 'PEAR.php';
 require_once 'Log.php';
 
+define('VFS_QUOTA_METRIC_BYTE', 1);
+define('VFS_QUOTA_METRIC_KB', 2);
+define('VFS_QUOTA_METRIC_MB', 3);
+define('VFS_QUOTA_METRIC_GB', 4);
+
 /**
  * VFS API for abstracted file storage and access.
  *
- * $Horde: framework/VFS/VFS.php,v 1.69.4.2 2005/01/31 09:25:27 jan Exp $
+ * $Horde: framework/VFS/VFS.php,v 1.69.4.13 2006/03/17 17:43:03 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -22,7 +27,7 @@ class VFS {
     /**
      * Hash containing connection parameters.
      *
-     * @var array $_params
+     * @var array
      */
     var $_params = array();
 
@@ -30,14 +35,14 @@ class VFS {
      * List of additional credentials required for this VFS backend (example:
      * For FTP, we need a username and password to log in to the server with).
      *
-     * @var array $_credentials
+     * @var array
      */
     var $_credentials = array();
 
     /**
      * List of permissions and if they can be changed in this VFS backend.
      *
-     * @var array $_permissions
+     * @var array
      */
     var $_permissions = array(
         'owner' => array('read' => false, 'write' => false, 'execute' => false),
@@ -48,7 +53,7 @@ class VFS {
      * A PEAR Log object. If present, will be used to log errors and
      * informational messages about VFS activity.
      *
-     * @var Log $_logger
+     * @var Log
      */
     var $_logger = null;
 
@@ -56,14 +61,19 @@ class VFS {
      * The log level to use - messages with a higher log level than configured
      * here will not be logged. Defaults to only logging errors or higher.
      *
-     * @var integer $_logLevel
+     * @var integer
      */
     var $_logLevel = PEAR_LOG_ERR;
 
     /**
-     * Constructor.
+     * The current size, in bytes, of the VFS item.
      *
-     * @access public
+     * @var integer
+     */
+    var $_vfsSize = null;
+
+    /**
+     * Constructor.
      *
      * @param array $params  A hash containing connection parameters.
      */
@@ -72,6 +82,12 @@ class VFS {
         if (empty($params['user'])) {
             $params['user'] = '';
         }
+        if (empty($params['vfs_quotalimit'])) {
+            $params['vfs_quotalimit'] = -1;
+        }
+        if (empty($params['vfs_quotaroot'])) {
+            $params['vfs_quotaroot'] = '/';
+        }
         $this->_params = $params;
     }
 
@@ -79,8 +95,6 @@ class VFS {
      * Checks the credentials that we have by calling _connect(), to see if
      * there is a valid login.
      *
-     * @access public
-     *
      * @return mixed  True on success, PEAR_Error describing the problem if the
      *                credentials are invalid.
      */
@@ -92,8 +106,6 @@ class VFS {
     /**
      * Sets configuration parameters.
      *
-     * @access public
-     *
      * @param array $params  An associative array with parameter names as keys.
      */
     function setParams($params = array())
@@ -106,8 +118,6 @@ class VFS {
     /**
      * Returns configuration parameters.
      *
-     * @access public
-     *
      * @param string $name  The parameter to return.
      *
      * @return mixed  The parameter value or null if it doesn't exist.
@@ -168,10 +178,24 @@ class VFS {
         }
     }
 
+    /**
+     * Retrieves the size of a file from the VFS.
+     *
+     * @abstract
+     *
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return integer The file size.
+     */
+    function size($path, $name)
+    {
+        return PEAR::raiseError(_("Not supported."));
+    }
+
     /**
      * Retrieves a file from the VFS.
      *
-     * @access public
      * @abstract
      *
      * @param string $path  The pathname to the file.
@@ -184,10 +208,35 @@ class VFS {
         return PEAR::raiseError(_("Not supported."));
     }
 
+    /**
+     * Retrieves a part of a file from the VFS. Particularly useful when
+     * reading large files which would exceed the PHP memory limits if they
+     * were stored in a string.
+     *
+     * @abstract
+     *
+     * @param string  $path       The pathname to the file.
+     * @param string  $name       The filename to retrieve.
+     * @param integer $offset     The offset of the part. (The new offset will
+     *                            be stored in here).
+     * @param integer $length     The length of the part. If the length = -1,
+     *                            the whole part after the offset is retrieved.
+     *                            If more bytes are given as exists after the
+     *                            given offset. Only the available bytes are
+     *                            read.
+     * @param integer $remaining  The bytes that are left, after the part that
+     *                            is retrieved.
+     *
+     * @return string The file data.
+     */
+    function readByteRange($path, $name, &$offset, $length = -1, &$remaining)
+    {
+        return PEAR::raiseError(_("Not supported."));
+    }
+
     /**
      * Stores a file in the VFS.
      *
-     * @access public
      * @abstract
      *
      * @param string $path         The path to store the file in.
@@ -206,7 +255,6 @@ class VFS {
     /**
      * Stores a file in the VFS from raw data.
      *
-     * @access public
      * @abstract
      *
      * @param string $path         The path to store the file in.
@@ -224,7 +272,6 @@ class VFS {
     /**
      * Moves a file through the backend.
      *
-     * @access public
      * @abstract
      *
      * @param string $path  The path of the original file.
@@ -241,7 +288,6 @@ class VFS {
     /**
      * Copies a file through the backend.
      *
-     * @access public
      * @abstract
      *
      * @param string $path  The path of the original file.
@@ -258,7 +304,6 @@ class VFS {
     /**
      * Deletes a file from the VFS.
      *
-     * @access public
      * @abstract
      *
      * @param string $path  The path to delete the file from.
@@ -274,7 +319,6 @@ class VFS {
     /**
      * Renames a file in the VFS.
      *
-     * @access public
      * @abstract
      *
      * @param string $oldpath  The old path to the file.
@@ -292,8 +336,6 @@ class VFS {
     /**
      * Returns if a given file or folder exists in a folder.
      *
-     * @access public
-     *
      * @param string $path  The path to the folder.
      * @param string $name  The file or folder name.
      *
@@ -312,7 +354,6 @@ class VFS {
     /**
      * Creates a folder in the VFS.
      *
-     * @access public
      * @abstract
      *
      * @param string $path  The parent folder.
@@ -329,26 +370,26 @@ class VFS {
      * Automatically creates any necessary parent directories in the specified
      * $path.
      *
-     * @access public
-     *
      * @param string $path  The VFS path to autocreate.
      */
     function autocreatePath($path)
     {
         $dirs = explode('/', $path);
         if (is_array($dirs)) {
-            $cur = '';
+            $cur = '/';
             foreach ($dirs as $dir) {
-                if (!$this->isFolder($cur, $dir)) {
-                    $result = $this->createFolder($cur, $dir);
-                    if (is_a($result, 'PEAR_Error')) {
-                        return $result;
+                if (!empty($dir)) {
+                    if (!$this->isFolder($cur, $dir)) {
+                        $result = $this->createFolder($cur, $dir);
+                        if (is_a($result, 'PEAR_Error')) {
+                            return $result;
+                        }
                     }
+                    if ($cur != '/') {
+                        $cur .= '/';
+                    }
+                    $cur .= $dir;
                 }
-                if (!empty($cur)) {
-                    $cur .= '/';
-                }
-                $cur .= $dir;
             }
         }
 
@@ -358,8 +399,6 @@ class VFS {
     /**
      * Checks if a given item is a folder.
      *
-     * @access public
-     *
      * @param string $path  The parent folder.
      * @param string $name  The item name.
      *
@@ -374,7 +413,6 @@ class VFS {
     /**
      * Deletes a folder from the VFS.
      *
-     * @access public
      * @abstract
      *
      * @param string $path        The parent folder.
@@ -391,8 +429,6 @@ class VFS {
     /**
      * Removes recursively all files and subfolders from the given folder.
      *
-     * @access public
-     *
      * @param string $path  The path of the folder to empty.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
@@ -426,8 +462,6 @@ class VFS {
     /**
      * Returns a file list of the directory passed in.
      *
-     * @access public
-     *
      * @param string $path        The path of the directory.
      * @param mixed $filter       String/hash to filter file/dirname on.
      * @param boolean $dotfiles   Show dotfiles?
@@ -456,7 +490,6 @@ class VFS {
     /**
      * Returns an an unsorted file list of the specified directory.
      *
-     * @access public
      * @abstract
      *
      * @param string $path       The path of the directory.
@@ -475,8 +508,6 @@ class VFS {
     /**
      * Returns the current working directory of the VFS backend.
      *
-     * @access public
-     *
      * @return string  The current working directory.
      */
     function getCurrentDirectory()
@@ -530,7 +561,6 @@ class VFS {
     /**
      * Changes permissions for an item on the VFS.
      *
-     * @access public
      * @abstract
      *
      * @param string $path        The parent folder of the item.
@@ -547,7 +577,6 @@ class VFS {
     /**
      * Returns a sorted list of folders in the specified directory.
      *
-     * @access public
      * @abstract
      *
      * @param string $path         The path of the directory to get the
@@ -565,8 +594,6 @@ class VFS {
     /**
      * Returns the list of additional credentials required, if any.
      *
-     * @access public
-     *
      * @return array  Credential list.
      */
     function getRequiredCredentials()
@@ -578,8 +605,6 @@ class VFS {
      * Returns an array specifying what permissions are changeable for this
      * VFS implementation.
      *
-     * @access public
-     *
      * @return array  Changeable permisions.
      */
     function getModifiablePermissions()
@@ -591,8 +616,6 @@ class VFS {
      * Converts a string to all lowercase characters ignoring the current
      * locale.
      *
-     * @access public
-     *
      * @param string $string  The string to be lowercased
      *
      * @return string  The string with lowercase characters
@@ -609,8 +632,6 @@ class VFS {
     /**
      * Returns the character (not byte) length of a string.
      *
-     * @access public
-     *
      * @param string $string   The string to return the length of.
      * @param string $charset  The charset to use when calculating the
      *                         string's length.
@@ -632,9 +653,247 @@ class VFS {
     }
 
     /**
-     * Attempts to return a concrete VFS instance based on $driver.
+     * Returns the size of a file.
      *
-     * @access public
+     * @abstract
+     * @since Horde 3.1
+     *
+     * @param string $path  The path to the file.
+     * @param string $name  The name of the file.
+     *
+     * @return integer  The size of the file, in bytes, or PEAR_Error on
+     *                  failure.
+     */
+    function getFileSize($path, $name)
+    {
+        return PEAR::raiseError(_("Not supported."));
+    }
+
+    /**
+     * Returns the size of a folder
+     *
+     * @since Horde 3.1
+     *
+     * @param string $path  The path to the folder.
+     * @param string $name  The name of the folder.
+     *
+     * @return integer  The size of the folder, in bytes, or PEAR_Error on
+     *                  failure.
+     */
+    function getFolderSize($path = null, $name = null)
+    {
+        $size = 0;
+        $root = ((!is_null($path)) ? $path . '/' : '') . $name;
+        $object_list = $this->listFolder($root, null, true, false, true);
+        foreach ($object_list as $key => $val) {
+            if (isset($val['subdirs'])) {
+                $size += $this->getFolderSize($root, $key);
+            } else {
+                $size += @$this->getFileSize($root, $key);
+            }
+        }
+
+        return $size;
+    }
+
+    /**
+     * Returns the size of the VFS item.
+     *
+     * @since Horde 3.1
+     *
+     * @return integer  The size, in bytes, of the VFS item.
+     */
+    function getVFSSize()
+    {
+        if (is_null($this->_vfsSize)) {
+            $this->_vfsSize = $this->getFolderSize($this->_params['vfs_quotaroot']);
+        }
+        return $this->_vfsSize;
+    }
+
+    /**
+     * Sets the VFS quota limit.
+     *
+     * @since Horde 3.1
+     *
+     * @param integer $quota   The limit to apply.
+     * @param integer $metric  The metric to multiply the quota into.
+     */
+    function setQuota($quota, $metric = VFS_QUOTA_METRIC_BYTE)
+    {
+        switch ($metric) {
+        case VFS_QUOTA_METRIC_KB:
+            $quota *= pow(2, 10);
+            break;
+
+        case VFS_QUOTA_METRIC_MB:
+            $quota *= pow(2, 20);
+            break;
+
+        case VFS_QUOTA_METRIC_GB:
+            $quota *= pow(2, 30);
+            break;
+        }
+
+        $this->_params['vfs_quotalimit'] = $quota;
+    }
+
+    /**
+     * Sets the VFS quota root.
+     *
+     * @since Horde 3.1
+     *
+     * @param string $dir  The root directory for the quota determination.
+     */
+    function setQuotaRoot($dir)
+    {
+        $this->_params['vfs_quotaroot'] = $dir;
+    }
+
+    /**
+     * Get quota information (used/allocated), in bytes.
+     *
+     * @since Horde 3.1
+     *
+     * @return mixed  An associative array.
+     *                'limit' = Maximum quota allowed
+     *                'usage' = Currently used portion of quota (in bytes)
+     *                Returns PEAR_Error on failure.
+     */
+    function getQuota()
+    {
+        if (empty($this->_params['vfs_quotalimit'])) {
+            return PEAR::raiseError(_("No quota set."));
+        } else {
+            $usage = $this->getVFSSize();
+            if (is_a($usage, 'PEAR_Error')) {
+                 return $usage;
+            } else {
+                return array('usage' => $this->getVFSSize(), 'limit' => $this->_params['vfs_quotalimit']);
+            }
+        }
+    }
+
+    /**
+     * Determines the location of the system temporary directory.
+     *
+     * @access protected
+     *
+     * @return string  A directory name which can be used for temp files.
+     *                 Returns false if one could not be found.
+     */
+    function _getTempDir()
+    {
+        $tmp_locations = array('/tmp', '/var/tmp', 'c:\WUTemp', 'c:\temp',
+                               'c:\windows\temp', 'c:\winnt\temp');
+
+        /* Try PHP's upload_tmp_dir directive. */
+        $tmp = ini_get('upload_tmp_dir');
+
+        /* Otherwise, try to determine the TMPDIR environment variable. */
+        if (empty($tmp)) {
+            $tmp = getenv('TMPDIR');
+        }
+
+        /* If we still cannot determine a value, then cycle through a list of
+         * preset possibilities. */
+        while (empty($tmp) && sizeof($tmp_locations)) {
+            $tmp_check = array_shift($tmp_locations);
+            if (@is_dir($tmp_check)) {
+                $tmp = $tmp_check;
+            }
+        }
+
+        /* If it is still empty, we have failed, so return false; otherwise
+         * return the directory determined. */
+        return empty($tmp) ? false : $tmp;
+    }
+
+    /**
+     * Creates a temporary file.
+     *
+     * @access protected
+     *
+     * @return string   Returns the full path-name to the temporary file or
+     *                  false if a temporary file could not be created.
+     */
+    function _getTempFile()
+    {
+        $tmp_dir = $this->_getTempDir();
+        if (empty($tmp_dir)) {
+            return false;
+        }
+
+        $tmp_file = tempnam($tmp_dir, 'vfs');
+
+        /* If the file was created, then register it for deletion and
+         * return. */
+        if (empty($tmp_file)) {
+            return false;
+        } else {
+            return $tmp_file;
+        }
+    }
+
+    /**
+     * Checks the quota when preparing to write data.
+     *
+     * @access private
+     *
+     * @param string $mode   Either 'string' or 'file'.  If 'string', $data is
+     *                       the data to be written.  If 'file', $data is the
+     *                       filename containing the data to be written.
+     * @param string &$data  Either the data or the filename to the data.
+     *
+     * @return mixed  PEAR_Error on error, true on success.
+     */
+    function _checkQuotaWrite($mode, &$data)
+    {
+        if ($this->_params['vfs_quotalimit'] != -1) {
+            if ($mode == 'file') {
+                $filesize = @filesize($data);
+                if ($filesize === false) {
+                    return PEAR::raiseError(_("Unable to read VFS file (filesize() failed)."));
+               }
+            } else {
+                $filesize = strlen($data);
+            }
+            if (($this->getVFSSize() + $filesize) > $this->_params['vfs_quotalimit']) {
+                return PEAR::raiseError(_("Unable to write VFS file, quota will be exceeded."));
+            } elseif ($this->_vfsSize !== 0) {
+                $this->_vfsSize += $filesize;
+            }
+        }
+
+        return true;
+    }
+
+    /**
+     * Checks the quota when preparing to delete data.
+     *
+     * @access private
+     *
+     * @param string $path  The path the file is located in.
+     * @param string $name  The filename.
+     *
+     * @return mixed  PEAR_Error on error, true on success.
+     */
+    function _checkQuotaDelete($path, $name)
+    {
+        if (($this->_params['vfs_quotalimit'] != -1) &&
+            !empty($this->_vfsSize)) {
+            $filesize = $this->getFileSize($path, $name);
+            if (is_a($filesize, 'PEAR_Error')) {
+                return PEAR::raiseError(_("Unable to read VFS file (getFileSize() failed)."));
+            }
+            $this->_vfsSize -= $filesize;
+        }
+
+        return true;
+    }
+
+    /**
+     * Attempts to return a concrete VFS instance based on $driver.
      *
      * @param mixed $driver  The type of concrete VFS subclass to return. This
      *                       is based on the storage driver ($driver). The
@@ -650,10 +909,12 @@ class VFS {
         include_once 'VFS/' . $driver . '.php';
         $class = 'VFS_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $vfs = &new $class($params);
         } else {
-            return PEAR::raiseError(sprintf(_("Class definition of %s not found."), $class));
+            $vfs = PEAR::raiseError(sprintf(_("Class definition of %s not found."), $class));
         }
+
+        return $vfs;
     }
 
     /**
@@ -666,8 +927,6 @@ class VFS {
      *
      * This method must be invoked as: $var = &VFS::singleton()
      *
-     * @access public
-     *
      * @param mixed $driver  The type of concrete VFS subclass to return. This
      *                       is based on the storage driver ($driver). The
      *                       code is dynamically included.
