@@ -2,18 +2,16 @@
 /**
  * VFS implementation for a standard filesystem.
  *
- * <pre>
- * Required values for $params:
- *      'vfsroot'       The root path
- * </pre>
+ * Required parameters:<pre>
+ *   'vfsroot'  The root path</pre>
  *
  * Note: The user that your webserver runs as (commonly 'nobody',
  * 'apache', or 'www-data') MUST have read/write permission to the
- * directory you specific as the 'vfsroot'.
+ * directory you specify as the 'vfsroot'.
  *
- * $Horde: framework/VFS/VFS/file.php,v 1.63.4.3 2005/03/28 17:24:29 chuck Exp $
+ * $Horde: framework/VFS/VFS/file.php,v 1.63.4.18 2006/03/24 16:41:48 slusarz Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -28,7 +26,7 @@ class VFS_file extends VFS {
      * List of permissions and if they can be changed in this VFS
      * backend.
      *
-     * @var array $_permissions
+     * @var array
      */
     var $_permissions = array(
         'owner' => array('read' => true, 'write' => true, 'execute' => true),
@@ -39,9 +37,7 @@ class VFS_file extends VFS {
     /**
      * Constructs a new Filesystem based VFS object.
      *
-     * @access public
-     *
-     * @param optional array $params  A hash containing connection parameters.
+     * @param array $params  A hash containing connection parameters.
      */
     function VFS_file($params = array())
     {
@@ -56,9 +52,20 @@ class VFS_file extends VFS {
     }
 
     /**
-     * Retrieve a file from the VFS.
+     * Retrieves the filesize from the VFS.
      *
-     * @access public
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return integer The file size.
+     */
+    function size($path, $name)
+    {
+        return filesize($this->_getNativePath($path, $name));
+    }
+
+    /**
+     * Retrieve a file from the VFS.
      *
      * @param string $path  The pathname to the file.
      * @param string $name  The filename to retrieve.
@@ -68,16 +75,72 @@ class VFS_file extends VFS {
     function read($path, $name)
     {
         $file = $this->_getNativePath($path, $name);
+        if (function_exists('file_get_contents')) {
+            $data = file_get_contents($file);
+        } else {
+            $fp = @fopen($file, 'rb');
+            if (!$fp) {
+                return PEAR::raiseError(_("Unable to open VFS file."));
+            }
+            $filesize = filesize($file);
+            if ($filesize == 0) {
+                $data = '';
+            } else {
+                $data = fread($fp, $filesize);
+            }
+            fclose($fp);
+        }
+
+        return $data;
+    }
+
+    /**
+     * Retrieves a part of a file from the VFS. Particularly useful when
+     * reading large files which would exceed the PHP memory limits if they
+     * were stored in a string.
+     *
+     * @abstract
+     *
+     * @param string  $path       The pathname to the file.
+     * @param string  $name       The filename to retrieve.
+     * @param integer $offset     The offset of the part. (The new offset will
+     *                            be stored in here).
+     * @param integer $length     The length of the part. If the length = -1,
+     *                            the whole part after the offset is retrieved.
+     *                            If more bytes are given as exists after the
+     *                            given offset. Only the available bytes are
+     *                            read.
+     * @param integer $remaining  The bytes that are left, after the part that
+     *                            is retrieved.
+     *
+     * @return string The file data.
+     */
+    function readByteRange($path, $name, &$offset, $length = -1, &$remaining)
+    {
+        if ($offset < 0) {
+            return PEAR::raiseError(sprintf(_("Wrong offset %d while reading a VFS file."), $offset));
+        }
+
+        // Calculate how many bytes MUST be read, so the remainging
+        // bytes and the new offset can be calculated correctly.
+        $file = $this->_getNativePath($path, $name);
+        $size = filesize ($file);
+        if ($length == -1 || (($length + $offset) > $size)) {
+            $length = $size - $offset;
+        }
+        if ($remaining < 0) {
+            $remaining = 0;
+        }
+
         $fp = @fopen($file, 'rb');
         if (!$fp) {
             return PEAR::raiseError(_("Unable to open VFS file."));
         }
-        $filesize = filesize($file);
-        if ($filesize == 0) {
-            $data = '';
-        } else {
-            $data = fread($fp, $filesize);
-        }
+        fseek($fp, $offset);
+        $data = fread($fp, $length);
+        $offset = ftell($fp);
+        $remaining = $size - $offset;
+
         fclose($fp);
 
         return $data;
@@ -87,13 +150,11 @@ class VFS_file extends VFS {
      * Store a file in the VFS, with the data copied from a temporary
      * file.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $name                  The filename to use.
-     * @param string $tmpFile               The temporary file containing the
-     *                                      data to be stored.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $tmpFile      The temporary file containing the data to be
+     *                             stored.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -110,6 +171,11 @@ class VFS_file extends VFS {
             }
         }
 
+        $res = $this->_checkQuotaWrite('file', $tmpFile);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         // Since we already have the data in a file, don't read it
         // into PHP's memory at all - just copy() it to the new
         // location. We leave it to the caller to clean up the
@@ -124,8 +190,6 @@ class VFS_file extends VFS {
     /**
      * Moves a file in the database and the file system.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      *
@@ -133,6 +197,11 @@ class VFS_file extends VFS {
      */
     function move($path, $name, $dest)
     {
+        $orig = $this->_getNativePath($path, $name);
+        if (preg_match('|^' . preg_quote($orig) . '[$/]|', $dest)) {
+            return PEAR::raiseError(_("Can not move file(s) - destination is within source."));
+        }
+
         $fileCheck = $this->listFolder($dest, false);
         foreach ($fileCheck as $file) {
             if ($file['name'] == $name) {
@@ -140,7 +209,7 @@ class VFS_file extends VFS {
             }
         }
 
-        if (!@rename($this->_getNativePath($path, $name), $this->_getNativePath($dest, $name))) {
+        if (!@rename($orig, $this->_getNativePath($dest, $name))) {
             return PEAR::raiseError(_("Unable to move VFS file."));
         }
 
@@ -150,8 +219,6 @@ class VFS_file extends VFS {
     /**
      * Copies a file through the backend.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      * @param string $dest  The destination of the file.
@@ -160,8 +227,9 @@ class VFS_file extends VFS {
      */
     function copy($path, $name, $dest)
     {
-        if ($path == $dest) {
-            return PEAR::raiseError(_("The file can not be copied onto itself."));
+        $orig = $this->_getNativePath($path, $name);
+        if (preg_match('|^' . preg_quote($orig) . '[$/]|', $dest)) {
+            return PEAR::raiseError(_("Can not copy file(s) - source and destination are the same."));
         }
 
         $fileCheck = $this->listFolder($dest, false);
@@ -171,7 +239,7 @@ class VFS_file extends VFS {
             }
         }
 
-        if (!@copy($this->_getNativePath($path, $name), $this->_getNativePath($dest, $name))) {
+        if (!@copy($orig, $this->_getNativePath($dest, $name))) {
             return PEAR::raiseError(_("Unable to copy VFS file."));
         }
 
@@ -181,12 +249,10 @@ class VFS_file extends VFS {
     /**
      * Store a file in the VFS from raw data.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $name                  The filename to use.
-     * @param string $data                  The file data.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $data         The file data.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -213,6 +279,11 @@ class VFS_file extends VFS {
             }
         }
 
+        $res = $this->_checkQuotaWrite('string', $data);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         // Otherwise we go ahead and try to write out the file.
         $fp = @fopen($this->_getNativePath($path, $name), 'w');
         if (!$fp) {
@@ -229,8 +300,6 @@ class VFS_file extends VFS {
     /**
      * Delete a file from the VFS.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      *
@@ -238,19 +307,24 @@ class VFS_file extends VFS {
      */
     function deleteFile($path, $name)
     {
+        $res = $this->_checkQuotaDelete($path, $name);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         if (!@unlink($this->_getNativePath($path, $name))) {
             return PEAR::raiseError(_("Unable to delete VFS file."));
         }
+
+        return true;
     }
 
     /**
      * Delete a folder from the VFS.
      *
-     * @access public
-     *
-     * @param string $path                 The path to delete the folder from.
-     * @param string $name                 The foldername to use.
-     * @param optional boolean $recursive  Force a recursive delete?
+     * @param string $path        The path to delete the folder from.
+     * @param string $name        The foldername to use.
+     * @param boolean $recursive  Force a recursive delete?
      *
      * @return mixed True on success or a PEAR_Error object on failure.
      */
@@ -282,8 +356,6 @@ class VFS_file extends VFS {
     /**
      * Creates a folder on the VFS.
      *
-     * @access public
-     *
      * @param string $path  The path to delete the folder from.
      * @param string $name  The foldername to use.
      *
@@ -301,8 +373,6 @@ class VFS_file extends VFS {
     /**
      * Check if a given pathname is a folder.
      *
-     * @access public
-     *
      * @param string $path  The path to the folder.
      * @param string $name  The file/folder name.
      *
@@ -316,8 +386,6 @@ class VFS_file extends VFS {
     /**
      * Changes permissions for an item in the VFS.
      *
-     * @access public
-     *
      * @param string $path         The path of directory of the item.
      * @param string $name         The name of the item.
      * @param integer $permission  The octal value of the new permission.
@@ -336,13 +404,10 @@ class VFS_file extends VFS {
     /**
      * Return a list of the contents of a folder.
      *
-     * @access public
-     *
-     * @param string $path                The path of the directory.
-     * @param optional mixed $filter      String/hash to filter file/dirname
-     *                                    on.
-     * @param optional boolean $dotfiles  Show dotfiles?
-     * @param optional boolean $dironly   Show only directories?
+     * @param string $path       The path of the directory.
+     * @param mixed $filter      String/hash to filter file/dirname on.
+     * @param boolean $dotfiles  Show dotfiles?
+     * @param boolean $dironly   Show only directories?
      *
      * @return array  File list on success, PEAR_Error on error.
      */
@@ -466,13 +531,10 @@ class VFS_file extends VFS {
     /**
      * Returns a sorted list of folders in specified directory.
      *
-     * @access public
-     *
-     * @param optional string $path         The path of the directory to get
-     *                                      the directory list for.
-     * @param optional mixed $filter        Hash of items to filter based on
-     *                                      folderlist.
-     * @param optional boolean $dotfolders  Include dotfolders?
+     * @param string $path         The path of the directory to get the
+     *                             directory list for.
+     * @param mixed $filter        Hash of items to filter based on folderlist.
+     * @param boolean $dotfolders  Include dotfolders?
      *
      * @return mixed  Folder list on success or a PEAR_Error object on failure.
      */
@@ -493,10 +555,10 @@ class VFS_file extends VFS {
             return $folderList;
         }
 
-        foreach ($folderList as $files) {
-            $folders[$folder['val']] = array('val' => $path . '/' . $files['name'],
-                                             'abbrev' => $files['name'],
-                                             'label' => $path . '/' . $files['name']);
+        foreach ($folderList as $name => $files) {
+            $folders[$name] = array('val' => $path . '/' . $files['name'],
+                                    'abbrev' => $files['name'],
+                                    'label' => $path . '/' . $files['name']);
         }
 
         ksort($folders);
@@ -547,8 +609,6 @@ class VFS_file extends VFS {
     /**
      * Rename a file or folder in the VFS.
      *
-     * @access public
-     *
      * @param string $oldpath  The old path to the file.
      * @param string $oldname  The old filename.
      * @param string $newpath  The new path of the file.
@@ -558,6 +618,12 @@ class VFS_file extends VFS {
      */
     function rename($oldpath, $oldname, $newpath, $newname)
     {
+        if (!@is_dir($this->_getNativePath($newpath))) {
+            if (is_a($res = $this->autocreatePath($newpath), 'PEAR_Error')) {
+                return $res;
+            }
+        }
+
         if (!@rename($this->_getNativePath($oldpath, $oldname),
                      $this->_getNativePath($newpath, $newname))) {
             return PEAR::raiseError(sprintf(_("Unable to rename VFS file %s/%s."), $oldpath, $oldname));
@@ -572,8 +638,8 @@ class VFS_file extends VFS {
      *
      * @access private
      *
-     * @param optional string $path  The VFS file path.
-     * @param optional string $name  The VFS filename.
+     * @param string $path  The VFS file path.
+     * @param string $name  The VFS filename.
      *
      * @return string  The full native filename.
      */
@@ -623,4 +689,24 @@ class VFS_file extends VFS {
         }
     }
 
+    /**
+     * Returns the size of a file.
+     *
+     * @since Horde 3.1
+     *
+     * @param string $path  The path to the file.
+     * @param string $name  The name of the file.
+     *
+     * @return integer  The size of the file, in bytes, or PEAR_Error on
+     *                  failure.
+     */
+    function getFileSize($path, $name)
+    {
+        $size = @filesize($this->_getNativePath($path, $name));
+        if ($size === false) {
+            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s/%s\"."), $path, $name));
+        }
+        return $size;
+    }
+
 }
