@@ -8,8 +8,6 @@
  *      'hostspec'       The ftp server to connect to.</pre>
  *
  * Optional values for $params:<pre>
- *      'lsformat'       The return formatting from the 'ls' command).
- *                       Values: 'aix', 'standard' (default)
  *      'maplocalids'    If true and the POSIX extension is available, the
  *                       driver will map the user and group IDs returned from
  *                       the FTP server with the local IDs from the local
@@ -23,21 +21,19 @@
  *                       support, TLS transport-level encryption will be
  *                       negotiated with the server.
  *      'timeout'        If defined, use this value as the timeout for the
- *                       server.
- *      'type'           The type of the remote FTP server.
- *                       Possible values: 'unix', 'win', 'netware'
- *                       By default, we attempt to auto-detect type.</pre>
+ *                       server.</pre>
  *
- * $Horde: framework/VFS/lib/VFS/ftp.php,v 1.1.2.5 2008/06/16 12:48:24 jan Exp $
+ * $Horde: framework/VFS/VFS/ftp.php,v 1.75.4.18 2006/03/24 16:41:49 slusarz Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
- * Copyright 2002-2007 Michael Varghese <mike.varghese@ascellatech.com>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Michael Varghese <mike.varghese@ascellatech.com>
  *
- * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you did
+ * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Michael Varghese <mike.varghese@ascellatech.com>
+ * @since   Horde 2.2
  * @package VFS
  */
 class VFS_ftp extends VFS {
@@ -80,35 +76,6 @@ class VFS_ftp extends VFS {
      */
     var $_gids = array();
 
-    /**
-     */
-    var $_type;
-
-    /**
-     * Returns the size of a file.
-     *
-     * @access public
-     *
-     * @param string $path  The path of the file.
-     * @param string $name  The filename.
-     *
-     * @return integer  The size of the file in bytes or PEAR_Error on
-     *                  failure.
-     */
-    function size($path, $name)
-    {
-        $conn = $this->_connect();
-        if (is_a($conn, 'PEAR_Error')) {
-            return $conn;
-        }
-
-        if (($size = @ftp_size($this->_stream, $this->_getPath($path, $name))) === false) {
-            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s\"."), $this->_getPath($path, $name)));
-        }
-
-        return $size;
-    }
-
     /**
      * Retrieves a file from the VFS.
      *
@@ -118,76 +85,40 @@ class VFS_ftp extends VFS {
      * @return string  The file data.
      */
     function read($path, $name)
-    {
-        $file = $this->readFile($path, $name);
-        if (is_a($file, 'PEAR_Error')) {
-            return $file;
-        }
-
-        $size = filesize($file);
-        if ($size === 0) {
-            return '';
-        }
-
-        return file_get_contents($file);
-    }
-
-    /**
-     * Retrieves a file from the VFS as an on-disk local file.
-     *
-     * This function provides a file on local disk with the data of a VFS file
-     * in it. This file <em>cannot</em> be modified! The behavior if you do
-     * modify it is undefined. It will be removed at the end of the request.
-     *
-     * @param string $path  The pathname to the file.
-     * @param string $name  The filename to retrieve.
-     *
-     * @return string A local filename.
-     */
-    function readFile($path, $name)
     {
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
-        // Create a temporary file and register it for deletion at the
-        // end of this request.
-        $localFile = $this->_getTempFile();
-        if (!$localFile) {
-            return PEAR::raiseError(_("Unable to create temporary file."));
-        }
-        register_shutdown_function(create_function('', '@unlink(\'' . addslashes($localFile) . '\');'));
-
-        $result = @ftp_get(
-            $this->_stream,
-            $localFile,
-            $this->_getPath($path, $name),
-            FTP_BINARY);
-        if ($result === false) {
+        $tmpFile = $this->_getTempFile();
+        $fetch = @ftp_get($this->_stream, $tmpFile,
+                          $this->_getPath($path, $name), FTP_BINARY);
+        if ($fetch === false) {
             return PEAR::raiseError(sprintf(_("Unable to open VFS file \"%s\"."), $this->_getPath($path, $name)));
         }
 
-        return $localFile;
-    }
-
-    /**
-     * Open a stream to a file in the VFS.
-     *
-     * @param string $path  The pathname to the file.
-     * @param string $name  The filename to retrieve.
-     *
-     * @return resource  The stream.
-     */
-    function readStream($path, $name)
-    {
-        $file = $this->readFile($path, $name);
-        if (is_a($file, 'PEAR_Error')) {
-            return $file;
+        $size = filesize($tmpFile);
+        if ($size === 0) {
+            return '';
         }
 
-        $mode = OS_WINDOWS ? 'rb' : 'r';
-        return fopen($file, $mode);
+        if (OS_WINDOWS) {
+            $mode = 'rb';
+        } else {
+            $mode = 'r';
+        }
+
+        if (function_exists('file_get_contents')) {
+            $data = file_get_contents($tmpFile);
+        } else {
+            $fp = fopen($tmpFile, $mode);
+            $data = fread($fp, $size);
+            fclose($fp);
+        }
+        unlink($tmpFile);
+
+        return $data;
     }
 
     /**
@@ -284,6 +215,31 @@ class VFS_ftp extends VFS {
         return true;
     }
 
+    /**
+     * Returns the size of a file.
+     *
+     * @access public
+     *
+     * @param string $path  The path of the file.
+     * @param string $name  The filename.
+     *
+     * @return integer  The size of the file in bytes or PEAR_Error on
+     *                  failure.
+     */
+    function getFileSize($path, $name)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        if (($size = @ftp_size($this->_stream, $this->_getPath($path, $name))) === false) {
+            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s\"."), $this->_getPath($path, $name)));
+        }
+
+        return $size;
+    }
+
     /**
      * Checks if a given item is a folder.
      *
@@ -462,34 +418,25 @@ class VFS_ftp extends VFS {
         }
 
         $files = array();
-
-        if (empty($this->_type)) {
-            if (!empty($this->_params['type'])) {
-                $this->_type = $this->_params['type'];
-            } else {
-                $type = VFS::strtolower(@ftp_systype($this->_stream));
-                if ($type == 'unknown') {
-                    // Go with unix-style listings by default.
-                    $type = 'unix';
-                } elseif (strpos($type, 'win') !== false) {
-                    $type = 'win';
-                } elseif (strpos($type, 'netware') !== false) {
-                    $type = 'netware';
-                }
-
-                $this->_type = $type;
-            }
+        $type = VFS::strtolower(@ftp_systype($this->_stream));
+        if ($type == 'unknown') {
+            // Go with unix-style listings by default.
+            $type = 'unix';
+        } elseif (strpos($type, 'win') !== false) {
+            $type = 'win';
+        } elseif (strpos($type, 'netware') !== false) {
+            $type = 'netware';
         }
 
         $olddir = $this->getCurrentDirectory();
-        if (strlen($path)) {
+        if (!empty($path)) {
             $res = $this->_setPath($path);
             if (is_a($res, 'PEAR_Error')) {
                 return $res;
             }
         }
 
-        if ($this->_type == 'unix') {
+        if ($type == 'unix') {
             // If we don't want dotfiles, We can save work here by not
             // doing an ls -a and then not doing the check later (by
             // setting $dotfiles to true, the if is short-circuited).
@@ -525,8 +472,7 @@ class VFS_ftp extends VFS {
         foreach ($list as $line) {
             $file = array();
             $item = preg_split('/\s+/', $line);
-            if (($this->_type == 'unix') ||
-                (($this->_type == 'win') && !preg_match('|\d\d-\d\d-\d\d|', $item[0]))) {
+            if ($type == 'unix' || ($type == 'win' && !preg_match('|\d\d-\d\d-\d\d|', $item[0]))) {
                 if (count($item) < 8 || substr($line, 0, 5) == 'total') {
                     continue;
                 }
@@ -547,13 +493,7 @@ class VFS_ftp extends VFS {
                     $file['owner'] = $item[2];
                     $file['group'] = $item[3];
                 }
-
-                if (!empty($this->_params['lsformat']) &&
-                    ($this->_params['lsformat'] == 'aix')) {
-                    $file['name'] = substr($line, strpos($line, sprintf("%s %2s %-5s", $item[5], $item[6], $item[7])) + 13);
-                } else {
-                    $file['name'] = substr($line, strpos($line, sprintf("%s %2s %5s", $item[5], $item[6], $item[7])) + 13);
-                }
+                $file['name'] = substr($line, strpos($line, sprintf("%s %2s %5s", $item[5], $item[6], $item[7])) + 13);
 
                 // Filter out '.' and '..' entries.
                 if (preg_match('/^\.\.?\/?$/', $file['name'])) {
@@ -611,12 +551,8 @@ class VFS_ftp extends VFS {
                 } else {
                     $file['date'] = strtotime('00:00:00' . $item[5] . ' ' . $item[6] . ' ' . $item[7]);
                 }
-            } elseif ($this->_type == 'netware') {
-                if (count($item) < 8 || substr($line, 0, 5) == 'total') {
-                    continue;
-                }
-
-                $file = array();
+            } elseif ($type == 'netware') {
+                $file = Array();
                 $file['perms'] = $item[1];
                 $file['owner'] = $item[2];
                 if ($item[0] == 'd') {
@@ -625,14 +561,6 @@ class VFS_ftp extends VFS {
                     $file['type'] = '**none';
                 }
                 $file['size'] = $item[3];
-
-                // We don't know the timezone here. Just report what the FTP server says.
-                if (strpos($item[6], ':') !== false) {
-                    $file['date'] = strtotime($item[6] . ':00 ' . $item[5] . ' ' . $item[4] . ' ' . date('Y'));
-                } else {
-                    $file['date'] = strtotime('00:00:00 ' . $item[5] . ' ' . $item[4] . ' ' . $item[6]);
-                }
-
                 $file['name'] = $item[7];
                 $index = 8;
                 while ($index < count($item)) {
@@ -735,18 +663,17 @@ class VFS_ftp extends VFS {
     /**
      * Copies a file through the backend.
      *
-     * @param string $path         The path of the original file.
-     * @param string $name         The name of the original file.
-     * @param string $dest         The name of the destination directory.
-     * @param boolean $autocreate  Automatically create directories?
+     * @param string $path  The path of the original file.
+     * @param string $name  The name of the original file.
+     * @param string $dest  The name of the destination directory.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function copy($path, $name, $dest, $autocreate = false)
+    function copy($path, $name, $dest)
     {
         $orig = $this->_getPath($path, $name);
-        if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
-            return PEAR::raiseError(_("Cannot copy file(s) - source and destination are the same."));
+        if (preg_match('|^' . preg_quote($orig) . '[$/]|', $dest)) {
+            return PEAR::raiseError(_("Can not copy file(s) - source and destination are the same."));
         }
 
         $conn = $this->_connect();
@@ -754,13 +681,6 @@ class VFS_ftp extends VFS {
             return $conn;
         }
 
-        if ($autocreate) {
-            $result = $this->autocreatePath($dest);
-            if (is_a($result, 'PEAR_Error')) {
-                return $result;
-            }
-        }
-
         $fileCheck = $this->listFolder($dest, null, true);
         if (is_a($fileCheck, 'PEAR_Error')) {
             return $fileCheck;
@@ -771,10 +691,32 @@ class VFS_ftp extends VFS {
             }
         }
 
-        if ($this->isFolder($path, $name)) {
-            if (is_a($result = $this->_copyRecursive($path, $name, $dest), 'PEAR_Error')) {
+        $isDir = false;
+        $dirCheck = $this->listFolder($path, null, false);
+        if (is_a($dirCheck, 'PEAR_Error')) {
+            return $dirCheck;
+        }
+        foreach ($dirCheck as $file) {
+            if ($file['name'] == $name && $file['type'] == '**dir') {
+                $isDir = true;
+                break;
+            }
+        }
+
+        if ($isDir) {
+            $result = $this->createFolder($dest, $name);
+
+            if (is_a($result, 'PEAR_Error')) {
                 return $result;
             }
+
+            $file_list = $this->listFolder($this->_getPath($path, $name));
+            foreach ($file_list as $file) {
+                $result = $this->copy($this->_getPath($path, $name), $file['name'], $this->_getPath($dest, $name));
+                if (is_a($result, 'PEAR_Error')) {
+                    return $result;
+                }
+            }
         } else {
             $tmpFile = $this->_getTempFile();
             $fetch = @ftp_get($this->_stream, $tmpFile, $orig, FTP_BINARY);
@@ -797,18 +739,17 @@ class VFS_ftp extends VFS {
     /**
      * Moves a file through the backend.
      *
-     * @param string $path         The path of the original file.
-     * @param string $name         The name of the original file.
-     * @param string $dest         The destination file name.
-     * @param boolean $autocreate  Automatically create directories?
+     * @param string $path  The path of the original file.
+     * @param string $name  The name of the original file.
+     * @param string $dest  The destination file name.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function move($path, $name, $dest, $autocreate = false)
+    function move($path, $name, $dest)
     {
         $orig = $this->_getPath($path, $name);
-        if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
-            return PEAR::raiseError(_("Cannot move file(s) - destination is within source."));
+        if (preg_match('|^' . preg_quote($orig) . '[$/]|', $dest)) {
+            return PEAR::raiseError(_("Can not move file(s) - destination is within source."));
         }
 
         $conn = $this->_connect();
@@ -816,17 +757,7 @@ class VFS_ftp extends VFS {
             return $conn;
         }
 
-        if ($autocreate) {
-            $result = $this->autocreatePath($dest);
-            if (is_a($result, 'PEAR_Error')) {
-                return $result;
-            }
-        }
-
         $fileCheck = $this->listFolder($dest, null, true);
-        if (is_a($fileCheck, 'PEAR_Error')) {
-            return $fileCheck;
-        }
         foreach ($fileCheck as $file) {
             if ($file['name'] == $name) {
                 return PEAR::raiseError(sprintf(_("%s already exists."), $this->_getPath($dest, $name)));
@@ -850,7 +781,7 @@ class VFS_ftp extends VFS {
         if (is_a($connected = $this->_connect(), 'PEAR_Error')) {
             return $connected;
         }
-        return @ftp_pwd($this->_stream);
+        return ftp_pwd($this->_stream);
     }
 
     /**
@@ -870,6 +801,25 @@ class VFS_ftp extends VFS {
         return true;
     }
 
+    /**
+     * Returns the full path of an item.
+     *
+     * @access private
+     *
+     * @param string $path  The directory of the item.
+     * @param string $name  The name of the item.
+     *
+     * @return mixed  Full path to the file when $path is not empty and just
+     *                $name when not set.
+     */
+    function _getPath($path, $name)
+    {
+        if ($path !== '') {
+            return ($path . '/' . $name);
+        }
+        return ($name);
+    }
+
     /**
      * Returns the parent directory of the specified path.
      *
@@ -941,8 +891,7 @@ class VFS_ftp extends VFS {
 
             $connected = @ftp_login($this->_stream, $this->_params['username'], $this->_params['password']);
             if (!$connected) {
-                @ftp_quit($this->_stream);
-                $this->_stream = false;
+                $this->_disconnect();
                 return PEAR::raiseError(_("Authentication to FTP server failed."));
             }
 
@@ -958,4 +907,15 @@ class VFS_ftp extends VFS {
         return true;
     }
 
+    /**
+     * Disconnects from the FTP server and cleans up the connection.
+     *
+     * @access private
+     */
+    function _disconnect()
+    {
+        @ftp_quit($this->_stream);
+        $this->_stream = false;
+    }
+
 }
