@@ -55,7 +55,7 @@ define('VFS_FLAG_WRITE', 2);
  *    mssql.textsize = 0 ; zero to pass through
  * </code>
  *
- * $Horde: framework/VFS/VFS/musql.php,v 1.30.4.16 2006/05/31 04:50:02 slusarz Exp $
+ * $Horde: framework/VFS/VFS/musql.php,v 1.30.4.14 2006/03/12 18:48:00 jan Exp $
  *
  * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
@@ -64,6 +64,7 @@ define('VFS_FLAG_WRITE', 2);
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
+ * @since   Horde 2.2
  * @package VFS
  */
 class VFS_musql extends VFS_sql {
@@ -372,17 +373,17 @@ class VFS_musql extends VFS_sql {
             return $conn;
         }
 
-        $length_op = $this->_getFileSizeOp();
-        $fileList = $this->_db->getAll(sprintf('SELECT vfs_name, vfs_type, vfs_modified, vfs_owner, vfs_perms, %s(vfs_data) FROM %s
+        $files = array();
+        $fileList = array();
+
+        $fileList = $this->_db->getAll(sprintf('SELECT vfs_name, vfs_type, vfs_modified, vfs_owner, vfs_perms, vfs_data FROM %s
                                                WHERE vfs_path = ? AND (vfs_owner = ? or vfs_perms && ?)',
-                                               $length_op,
                                                $this->_params['table']),
                                        array($path, $this->_params['user'], VFS_FLAG_READ));
         if (is_a($fileList, 'PEAR_Error')) {
             return $fileList;
         }
 
-        $files = array();
         foreach ($fileList as $line) {
             // Filter out dotfiles if they aren't wanted.
             if (!$dotfiles && substr($line[0], 0, 1) == '.') {
@@ -400,7 +401,7 @@ class VFS_musql extends VFS_sql {
                     $file['type'] = VFS::strtolower($name[count($name) - 1]);
                 }
 
-                $file['size'] = $line[5];
+                $file['size'] = VFS::strlen($line[5]);
             } elseif ($line[1] == VFS_FOLDER) {
                 $file['type'] = '**dir';
                 $file['size'] = -1;
