@@ -1,35 +1,37 @@
 <?php
 
-/** @constant integer VFS_FILE  File value for vfs_type column. */
+/**
+ * File value for vfs_type column.
+ */
 define('VFS_FILE', 1);
 
-/** @constant integer VFS_FOLDER  Folder value for vfs_type column. */
+/**
+ * Folder value for vfs_type column.
+ */
 define('VFS_FOLDER', 2);
 
 /**
  * VFS implementation for PHP's PEAR database abstraction layer.
  *
- * <pre>
- * Required values for $params:
- *      'phptype'       The database type (ie. 'pgsql', 'mysql, etc.).
+ * Required values for $params:<pre>
+ *      'phptype'       The database type (ie. 'pgsql', 'mysql', etc.).</pre>
+ *
+ * Optional values:<pre>
+ *      'table'         The name of the vfs table in 'database'. Defaults to
+ *                      'horde_vfs'.</pre>
+ *
+ * Required by some database implementations:<pre>
  *      'hostspec'      The hostname of the database server.
  *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
+ *      'database'      The name of the database.
  *      'username'      The username with which to connect to the database.
  *      'password'      The password associated with 'username'.
- *      'database'      The name of the database.
- *
- * Optional values:
- *      'table'         The name of the vfs table in 'database'. Defaults to
- *                      'horde_vfs'.
- *
- * Required by some database implementations:
  *      'options'       Additional options to pass to the database.
  *      'tty'           The TTY on which to connect to the database.
- *      'port'          The port on which to connect to the database.
- * </pre>
+ *      'port'          The port on which to connect to the database.</pre>
  *
  * The table structure for the VFS can be found in
- * horde/scripts/db/vfs.sql.
+ * data/vfs.sql.
  *
  * Database specific notes:
  *
@@ -44,15 +46,14 @@ define('VFS_FOLDER', 2);
  *    mssql.textsize = 0 ; zero to pass through
  * </pre>
  *
- * $Horde: framework/VFS/VFS/sql.php,v 1.85.4.3 2005/02/17 22:09:10 jan Exp $
+ * $Horde: framework/VFS/VFS/sql.php,v 1.85.4.24 2006/03/12 18:48:26 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.85.4.3 $
  * @since   Horde 2.2
  * @package VFS
  */
@@ -61,14 +62,32 @@ class VFS_sql extends VFS {
     /**
      * Handle for the current database connection.
      *
-     * @var object DB $_db
+     * @var DB
      */
     var $_db = false;
 
     /**
-     * Retrieve a file from the VFS.
+     * Retrieves the filesize from the VFS.
      *
-     * @access public
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return int The file size.
+     */
+    function size($path, $name)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        return strlen($this->_readBlob($this->_params['table'], 'vfs_data',
+                                       array('vfs_path' => $this->_convertPath($path),
+                                             'vfs_name' => $name)));
+    }
+
+    /**
+     * Retrieve a file from the VFS.
      *
      * @param string $path  The pathname to the file.
      * @param string $name  The filename to retrieve.
@@ -83,14 +102,58 @@ class VFS_sql extends VFS {
         }
 
         return $this->_readBlob($this->_params['table'], 'vfs_data',
-                                array('vfs_path' => $path,
+                                array('vfs_path' => $this->_convertPath($path),
                                       'vfs_name' => $name));
     }
 
     /**
-     * Stores a file in the VFS.
+     * Retrieves a part of a file from the VFS. Particularly useful
+     * when reading large files which would exceed the PHP memory
+     * limits if they were stored in a string.
      *
-     * @access public
+     * @param string  $path       The pathname to the file.
+     * @param string  $name       The filename to retrieve.
+     * @param integer $offset     The offset of the part. (The new offset will be
+     *                            stored in here).
+     * @param integer $length     The length of the part. If the length = -1, the
+     *                            whole part after the offset is retrieved. If
+     *                            more bytes are given as exists after the given
+     *                            offset. Only the available bytes are read.
+     * @param integer $remaining  The bytes that are left, after the part that is
+     *                            retrieved.
+     *
+     * @return string The file data.
+     */
+    function readPart($path, $name, &$offset, $length = -1, &$remaining)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        $data = $this->_readBlob($this->_params['table'], 'vfs_data',
+                                 array('vfs_path' => $this->_convertPath($path),
+                                       'vfs_name' => $name));
+
+        // Calculate how many bytes MUST be read, so the remainging
+        // bytes and the new offset can be calculated correctly.
+        $size = strlen ($data);
+        if ($length == -1 || (($length + $offset) > $size)) {
+            $length = $size - $offset;
+        }
+        if ($remaining < 0) {
+            $remaining = 0;
+        }
+
+        $data = substr($data, $offset, $length);
+        $offset = $offset + $length;
+        $remaining = $size - $offset;
+
+        return $data;
+    }
+
+    /**
+     * Stores a file in the VFS.
      *
      * @param string $path         The path to store the file in.
      * @param string $name         The filename to use.
@@ -102,16 +165,22 @@ class VFS_sql extends VFS {
      */
     function write($path, $name, $tmpFile, $autocreate = false)
     {
-        $size = filesize($tmpFile);
-        if ($size === 0) {
-            $data = '';
+        /* Don't need to check quota here since it will be checked when
+         * writeData() is called. */
+        if (function_exists('file_get_contents')) {
+            $data = file_get_contents($tmpFile);
         } else {
-            $dataFP = @fopen($tmpFile, 'rb');
-            if (!$dataFP) {
-                return PEAR::raiseError(sprintf(_("The temporary file \"%s\" does not exist."), $tmpFile));
+            $size = filesize($tmpFile);
+            if ($size === 0) {
+                $data = '';
+                } else {
+                $dataFP = @fopen($tmpFile, 'rb');
+                if (!$dataFP) {
+                    return PEAR::raiseError(sprintf(_("The temporary file \"%s\" does not exist."), $tmpFile));
+                }
+                $data = @fread($dataFP, $size);
+                @fclose($dataFP);
             }
-            $data = @fread($dataFP, $size);
-            @fclose($dataFP);
         }
 
         return $this->writeData($path, $name, $data, $autocreate);
@@ -120,29 +189,34 @@ class VFS_sql extends VFS {
     /**
      * Store a file in the VFS from raw data.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $name                  The filename to use.
-     * @param string $data                  The file data.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $data         The file data.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
     function writeData($path, $name, $data, $autocreate = false)
     {
+        $res = $this->_checkQuotaWrite('string', $data);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
+        $path = $this->_convertPath($path);
+
         /* Check to see if the data already exists. */
-        $sql = sprintf('SELECT vfs_id FROM %s WHERE vfs_path %s AND vfs_name = %s',
+        $sql = sprintf('SELECT vfs_id FROM %s WHERE vfs_path %s AND vfs_name = ?',
                        $this->_params['table'],
-                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path),
-                       $this->_db->quote($name));
+                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path));
+        $values = array($name);
         $this->log($sql, PEAR_LOG_DEBUG);
-        $id = $this->_db->getOne($sql);
+        $id = $this->_db->getOne($sql, $values);
 
         if (is_a($id, 'PEAR_Error')) {
             $this->log($id, PEAR_LOG_ERR);
@@ -187,8 +261,6 @@ class VFS_sql extends VFS {
     /**
      * Delete a file from the VFS.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      *
@@ -196,18 +268,24 @@ class VFS_sql extends VFS {
      */
     function deleteFile($path, $name)
     {
+        $res = $this->_checkQuotaDelete($path, $name);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
-        $sql = sprintf('DELETE FROM %s WHERE vfs_type = %s AND vfs_path %s AND vfs_name = %s',
+        $path = $this->_convertPath($path);
+
+        $sql = sprintf('DELETE FROM %s WHERE vfs_type = ? AND vfs_path %s AND vfs_name = ?',
                        $this->_params['table'],
-                       $this->_db->quote(VFS_FILE),
-                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path),
-                       $this->_db->quote($name));
+                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path));
+        $values = array(VFS_FILE, $name);
         $this->log($sql, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($sql);
+        $result = $this->_db->query($sql, $values);
 
         if ($this->_db->affectedRows() == 0) {
             return PEAR::raiseError(_("Unable to delete VFS file."));
@@ -219,8 +297,6 @@ class VFS_sql extends VFS {
     /**
      * Rename a file or folder in the VFS.
      *
-     * @access public
-     *
      * @param string $oldpath  The old path to the file.
      * @param string $oldname  The old filename.
      * @param string $newpath  The new path of the file.
@@ -235,15 +311,28 @@ class VFS_sql extends VFS {
             return $conn;
         }
 
-        $sql = sprintf('UPDATE %s SET vfs_path = %s, vfs_name = %s, vfs_modified = %s WHERE vfs_path = %s AND vfs_name = %s',
-                       $this->_params['table'],
-                       $this->_db->quote($newpath),
-                       $this->_db->quote($newname),
-                       $this->_db->quote(time()),
-                       $this->_db->quote($oldpath),
-                       $this->_db->quote($oldname));
+        if (strpos($newpath, '/') === false) {
+            $parent = '';
+            $path = $newpath;
+        } else {
+            list($parent, $path) = explode('/', $newpath, 2);
+        }
+        if (!$this->isFolder($parent, $path)) {
+            if (is_a($result = $this->autocreatePath($newpath), 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
+        $oldpath = $this->_convertPath($oldpath);
+        $newpath = $this->_convertPath($newpath);
+
+        $sql  = 'UPDATE ' . $this->_params['table'];
+        $sql .= ' SET vfs_path = ?, vfs_name = ?, vfs_modified = ? WHERE vfs_path = ? AND vfs_name = ?';
         $this->log($sql, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($sql);
+
+        $values = array($newpath, $newname, time(), $oldpath, $oldname);
+
+        $result = $this->_db->query($sql, $values);
 
         if ($this->_db->affectedRows() == 0) {
             return PEAR::raiseError(_("Unable to rename VFS file."));
@@ -261,8 +350,6 @@ class VFS_sql extends VFS {
     /**
      * Creates a folder on the VFS.
      *
-     * @access public
-     *
      * @param string $path  Holds the path of directory to create folder.
      * @param string $name  Holds the name of the new folder.
      *
@@ -281,26 +368,21 @@ class VFS_sql extends VFS {
             return $id;
         }
 
-        $sql = sprintf('INSERT INTO %s (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified, vfs_owner) VALUES (%s, %s, %s, %s, %s, %s)',
-                       $this->_params['table'],
-                       $this->_db->quote($id),
-                       $this->_db->quote(VFS_FOLDER),
-                       $this->_db->quote($path),
-                       $this->_db->quote($name),
-                       $this->_db->quote(time()),
-                       $this->_db->quote($this->_params['user']));
+        $sql  = 'INSERT INTO ' . $this->_params['table'];
+        $sql .= ' (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified, vfs_owner) VALUES (?, ?, ?, ?, ?, ?)';
         $this->log($sql, PEAR_LOG_DEBUG);
-        return $this->_db->query($sql);
+
+        $values = array($id, VFS_FOLDER, $this->_convertPath($path), $name, time(), $this->_params['user']);
+
+        return $this->_db->query($sql, $values);
     }
 
     /**
      * Delete a folder from the VFS.
      *
-     * @access public
-     *
-     * @param string $path                 The path of the folder.
-     * @param string $name                 The folder name to use.
-     * @param optional boolean $recursive  Force a recursive delete?
+     * @param string $path        The path of the folder.
+     * @param string $name        The folder name to use.
+     * @param boolean $recursive  Force a recursive delete?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -311,6 +393,8 @@ class VFS_sql extends VFS {
             return $conn;
         }
 
+        $path = $this->_convertPath($path);
+
         $folderPath = $this->_getNativePath($path, $name);
 
         /* Check if not recursive and fail if directory not empty */
@@ -349,12 +433,12 @@ class VFS_sql extends VFS {
         }
 
         /* All ok now delete the actual folder */
-        $sql = sprintf('DELETE FROM %s WHERE vfs_path %s AND vfs_name = %s',
+        $sql = sprintf('DELETE FROM %s WHERE vfs_path %s AND vfs_name = ?',
                        $this->_params['table'],
-                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path),
-                       $this->_db->quote($name));
+                       (empty($path) && $this->_db->dbsyntax == 'oci8') ? ' IS NULL' : ' = ' . $this->_db->quote($path));
+        $values = array($name);
         $this->log($sql, PEAR_LOG_DEBUG);
-        $delete = $this->_db->query($sql);
+        $delete = $this->_db->query($sql, $values);
         if (is_a($delete, 'PEAR_Error')) {
             $this->log($delete, PEAR_LOG_ERR);
             return PEAR::raiseError(sprintf(_("Unable to delete VFS directory: %s."), $delete->getMessage()));
@@ -366,24 +450,24 @@ class VFS_sql extends VFS {
     /**
      * Return a list of the contents of a folder.
      *
-     * @access public
-     *
-     * @param string $path                The directory path.
-     * @param optional mixed $filter      String/hash of items to filter based
-     *                                    on filename.
-     * @param optional boolean $dotfiles  Show dotfiles?
-     * @param optional boolean $dironly   Show directories only?
+     * @param string $path       The directory path.
+     * @param mixed $filter      String/hash of items to filter based on
+     *                           filename.
+     * @param boolean $dotfiles  Show dotfiles?
+     * @param boolean $dironly   Show directories only?
      *
      * @return mixed  File list on success or false on failure.
      */
     function _listFolder($path, $filter = null, $dotfiles = true,
-                        $dironly = false)
+                         $dironly = false)
     {
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
+        $path = $this->_convertPath($path);
+
         $files = array();
         $fileList = array();
 
@@ -428,8 +512,8 @@ class VFS_sql extends VFS {
 
             $file['date'] = $line[3];
             $file['owner'] = $line[4];
-            $file['perms'] = '-';
-            $file['group'] = '-';
+            $file['perms'] = '';
+            $file['group'] = '';
 
             // filtering
             if ($this->_filterMatch($filter, $file['name'])) {
@@ -451,13 +535,11 @@ class VFS_sql extends VFS {
     /**
      * Returns a sorted list of folders in specified directory.
      *
-     * @access public
-     *
-     * @param optional string $path         The path of the directory to get
-     *                                      the directory list for.
-     * @param optional mixed $filter        String/hash of items to filter
-     *                                      based on folderlist.
-     * @param optional boolean $dotfolders  Include dotfolders?
+     * @param string $path         The path of the directory to get the
+     *                             directory list for.
+     * @param mixed $filter        String/hash of items to filter based on
+     *                             folderlist.
+     * @param boolean $dotfolders  Include dotfolders?
      *
      * @return mixed  Folder list on success or PEAR_Error object on failure.
      */
@@ -468,12 +550,15 @@ class VFS_sql extends VFS {
             return $conn;
         }
 
-        $sql = sprintf('SELECT vfs_name, vfs_path FROM %s WHERE vfs_path = %s AND vfs_type = %s',
-                       $this->_params['table'],
-                       $this->_db->quote($path),
-                       VFS_FOLDER);
+        $path = $this->_convertPath($path);
+
+        $sql  = 'SELECT vfs_name, vfs_path FROM ' . $this->_params['table'];
+        $sql .= ' WHERE vfs_path = ? AND vfs_type = ?';
         $this->log($sql, PEAR_LOG_DEBUG);
-        $folderList = $this->_db->getAll($sql);
+
+        $values = array($path, VFS_FOLDER);
+
+        $folderList = $this->_db->getAll($sql, $values);
         if (is_a($folderList, 'PEAR_Error')) {
             return $folderList;
         }
@@ -533,23 +618,27 @@ class VFS_sql extends VFS {
      */
     function _recursiveRename($oldpath, $oldname, $newpath, $newname)
     {
-        $sql = sprintf('SELECT vfs_name FROM %s WHERE vfs_type = %s AND vfs_path = %s',
-                       $this->_params['table'],
-                       $this->_db->quote(VFS_FOLDER),
-                       $this->_db->quote($this->_getNativePath($oldpath, $oldname)));
+        $oldpath = $this->_convertPath($oldpath);
+        $newpath = $this->_convertPath($newpath);
+
+        $sql  = 'SELECT vfs_name FROM ' . $this->_params['table'];
+        $sql .= ' WHERE vfs_type = ? AND vfs_path = ?';
         $this->log($sql, PEAR_LOG_DEBUG);
-        $folderList = $this->_db->getCol($sql);
+
+        $values = array(VFS_FOLDER, $this->_getNativePath($oldpath, $oldname));
+
+        $folderList = $this->_db->getCol($sql, 0, $values);
 
         foreach ($folderList as $folder) {
             $this->_recursiveRename($this->_getNativePath($oldpath, $oldname), $folder, $this->_getNativePath($newpath, $newname), $folder);
         }
 
-        $sql = sprintf('UPDATE %s SET vfs_path = %s WHERE vfs_path = %s',
-                       $this->_params['table'],
-                       $this->_db->quote($this->_getNativePath($newpath, $newname)),
-                       $this->_db->quote($this->_getNativePath($oldpath, $oldname)));
+        $sql = 'UPDATE ' . $this->_params['table'] . ' SET vfs_path = ? WHERE vfs_path = ?';
         $this->log($sql, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($sql);
+
+        $values = array($this->_getNativePath($newpath, $newname), $this->_getNativePath($oldpath, $oldname));
+
+        $result = $this->_db->query($sql, $values);
 
         return $result;
     }
@@ -595,13 +684,22 @@ class VFS_sql extends VFS {
                 return PEAR::raiseError(_("No configuration information specified for SQL VFS."));
             }
 
-            $required = array('phptype', 'hostspec', 'username', 'password', 'database');
+            $required = array('phptype');
             foreach ($required as $val) {
                 if (!isset($this->_params[$val])) {
-                    return PEAR::raiseError(sprintf(_("Required '%s' not specified in VFS configuration."), $val));
+                    return PEAR::raiseError(sprintf(_("Required \"%s\" not specified in VFS configuration."), $val));
                 }
             }
 
+            if (!isset($this->_params['database'])) {
+                $this->_params['database'] = '';
+            }
+            if (!isset($this->_params['username'])) {
+                $this->_params['username'] = '';
+            }
+            if (!isset($this->_params['hostspec'])) {
+                $this->_params['hostspec'] = '';
+            }
             if (!isset($this->_params['table'])) {
                 $this->_params['table'] = 'horde_vfs';
             }
@@ -617,7 +715,16 @@ class VFS_sql extends VFS {
                 return $error;
             }
 
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            
+            // Set DB portability options.
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
+
         }
 
         return true;
@@ -731,7 +838,7 @@ class VFS_sql extends VFS {
         case 'oci8':
             foreach ($attributes as $key => $value) {
                 $fields[] = $key;
-                $values[] = $this->_db->quote($value);
+                $values[] = $this->_db->quoteSmart($value);
             }
 
             $statement = OCIParse($this->_db->connection,
@@ -778,8 +885,7 @@ class VFS_sql extends VFS {
 
         /* Execute the query. */
         $this->log($query, PEAR_LOG_DEBUG);
-        $stmt = $this->_db->prepare($query);
-        return $this->_db->execute($stmt, $values);
+        return $this->_db->query($query, $values);
     }
 
     /**
@@ -811,7 +917,7 @@ class VFS_sql extends VFS {
             }
 
             $statement = OCIParse($this->_db->connection,
-                                  sprintf('SELECT %s FROM %s FOR UPDATE WHERE %s',
+                                  sprintf('SELECT %s FROM %s WHERE %s FOR UPDATE',
                                           $field,
                                           $table,
                                           $wherestring));
@@ -860,8 +966,133 @@ class VFS_sql extends VFS {
 
         /* Execute the query. */
         $this->log($query, PEAR_LOG_DEBUG);
-        $stmt = $this->_db->prepare($query);
-        return $this->_db->execute($stmt, $values);
+        return $this->_db->query($query, $values);
+    }
+
+    /**
+     * Convert the pathname from regular filesystem form to the internal
+     * format needed to access the file in the database.
+     * Namely, we will treat '/' as a base directory as this is pretty much
+     * the standard way to access base directories over most filesystems.
+}    *
+     * @access private
+     *
+     * @param string $path  A VFS path.
+     *
+     * @return string  The path with any beginning slash stripped off.
+     */
+    function _convertPath($path)
+    {
+        if (!empty($path) && (strpos($path, '/') === 0)) {
+            $path = strval(substr($path, 1));
+        }
+
+        return $path;
+    }
+
+    /**
+     * Returns the size of a file.
+     *
+     * @access public
+     *
+     * @param string $path  The path of the file.
+     * @param string $name  The filename.
+     *
+     * @return integer  The size of the file in bytes or PEAR_Error on
+     *                  failure.
+     */
+    function getFileSize($path, $name)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        switch ($this->_db->dbsyntax) {
+        case 'mysql':
+            $length = 'LENGTH';
+            break;
+
+        case 'oci8':
+            $length = 'LENGTHB';
+            break;
+
+        case 'mssql':
+        case 'sybase':
+            $length = 'DATALENGTH';
+            break;
+
+        case 'pgsql':
+        default:
+            $length = 'OCTET_LENGTH';
+        }
+	
+        $sql = sprintf(
+            'SELECT %s(vfs_data) FROM %s WHERE vfs_path = ? AND vfs_name = ?',
+            $length,
+            $this->_params['table']
+        );
+        $values = array($this->_convertPath($path), $name);
+        $this->log($sql, PEAR_LOG_DEBUG);
+        $size = $this->_db->getOne($sql, $values);
+
+        if (is_null($size)) {
+            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s/%s\"."), $path, $name));
+        }
+
+        return $size;
+    }
+
+    /**
+     * Returns the size of a file.
+     *
+     * @access public
+     *
+     * @param string $path  The path of the file.
+     * @param string $name  The filename.
+     *
+     * @return integer  The size of the folder in bytes or PEAR_Error on
+     *                  failure.
+     */
+    function getFolderSize($path = null, $name = null)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        $where = (is_null($path)) ? null : sprintf('WHERE vfs_path = %s', ((empty($path)) ? '""' : $this->_db->quote($this->_convertPath($path))));
+
+        switch ($this->_db->dbsyntax) {
+        case 'mysql':
+            $length = 'LENGTH';
+            break;
+
+        case 'oci8':
+            $length = 'LENGTHB';
+            break;
+
+        case 'mssql':
+        case 'sybase':
+            $length = 'DATALENGTH';
+            break;
+
+        case 'pgsql':
+        default:
+            $length = 'OCTET_LENGTH';
+        }
+
+
+        $sql = sprintf(
+            'SELECT SUM(%s(vfs_data)) FROM %s %s',
+            $length,
+            $this->_params['table'],
+            $where
+        );
+        $this->log($sql, PEAR_LOG_DEBUG);
+        $size = $this->_db->getOne($sql);
+
+        return ((!is_null($size)) ? $size : 0);
     }
 
 }
