@@ -1,43 +1,44 @@
 <?php
 
-/** @constant integer VFS_FILE  File value for vfs_type column. */
+/**
+ * File value for vfs_type column.
+ */
 define('VFS_FILE', 1);
 
-/** @constant integer VFS_FOLDER  Folder value for vfs_type column. */
+/**
+ * Folder value for vfs_type column.
+ */
 define('VFS_FOLDER', 2);
 
 /**
  * VFS:: implementation using PHP's PEAR database abstraction
  * layer and local file system for file storage.
  *
- * <pre>
- * Required values for $params:
- *      'phptype'       The database type (ie. 'pgsql', 'mysql, etc.).
+ * Required values for $params:<pre>
+ *      'phptype'       The database type (ie. 'pgsql', 'mysql', etc.).
+ *      'vfsroot'       The root directory of where the files should be
+ *                      actually stored.</pre>
+ *
+ * Optional values:<pre>
+ *      'table'         The name of the vfs table in 'database'. Defaults to
+ *                      'horde_vfs'.</pre>
+ *
+ * Required by some database implementations:<pre>
  *      'hostspec'      The hostname of the database server.
  *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
+ *      'database'      The name of the database.
  *      'username'      The username with which to connect to the database.
  *      'password'      The password associated with 'username'.
- *      'database'      The name of the database.
- *      'vfsroot'       The root directory of where the files should be
- *                      actually stored.
- *
- * Optional values:
- *      'table'         The name of the vfs table in 'database'. Defaults to
- *                      'horde_vfs'.
- *
- * Required by some database implementations:
  *      'options'       Additional options to pass to the database.
  *      'tty'           The TTY on which to connect to the database.
- *      'port'          The port on which to connect to the database.
- * </pre>
+ *      'port'          The port on which to connect to the database.</pre>
  *
  * The table structure for the VFS can be found in
- * horde/scripts/db/vfs.sql.
+ * data/vfs.sql.
  *
- * $Horde: framework/VFS/VFS/sql_file.php,v 1.44.4.1 2005/03/16 13:27:03 jan Exp $
+ * $Horde: framework/VFS/VFS/sql_file.php,v 1.44.4.19 2006/03/24 16:41:49 slusarz Exp $
  *
  * @author  Michael Varghese <mike.varghese@ascellatech.com>
- * @version $Revision: 1.44.4.1 $
  * @since   Horde 2.2
  * @package VFS
  */
@@ -46,15 +47,13 @@ class VFS_sql_file extends VFS {
     /**
      * Handle for the current database connection.
      *
-     * @var object DB $_db
+     * @var DB
      */
     var $_db = false;
 
     /**
      * Retrieve a file from the VFS.
      *
-     * @access public
-     *
      * @param string $path  The pathname to the file.
      * @param string $name  The filename to retrieve.
      *
@@ -68,13 +67,17 @@ class VFS_sql_file extends VFS {
         }
 
         $file = $this->_getNativePath($path, $name);
-        $fp = @fopen($file, 'rb');
-        if (!$fp) {
-            return PEAR::raiseError(_("Unable to open VFS file."));
-        }
 
-        $data = fread($fp, filesize($file));
-        fclose($fp);
+        if (function_exists('file_get_contents')) {
+            $data = file_get_contents($file);
+        } else {
+            $fp = @fopen($file, 'rb');
+            if (!$fp) {
+                return PEAR::raiseError(_("Unable to open VFS file."));
+            }
+            $data = fread($fp, filesize($file));
+            fclose($fp);
+        }
 
         return $data;
     }
@@ -83,38 +86,46 @@ class VFS_sql_file extends VFS {
      * Store a file in the VFS, with the data copied from a temporary
      * file.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $name                  The filename to use.
-     * @param string $tmpFile               The temporary file containing the
-     *                                      data to be stored.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $tmpFile      The temporary file containing the data to be
+     *                             stored.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
     function write($path, $name, $tmpFile, $autocreate = false)
     {
-        $dataFP = @fopen($tmpFile, 'rb');
-        $data = @fread($dataFP, filesize($tmpFile));
-        fclose($dataFP);
+        /* No need to check quota here as we will check it when we call
+         * writeData(). */
+
+        if (function_exists('file_get_contents')) {
+            $data = file_get_contents($tmpFile);
+        } else {
+            $dataFP = @fopen($tmpFile, 'rb');
+            $data = @fread($dataFP, filesize($tmpFile));
+            fclose($dataFP);
+        }
         return $this->writeData($path, $name, $data, $autocreate);
     }
 
     /**
      * Store a file in the VFS from raw data.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $name                  The filename to use.
-     * @param string $data                  The file data.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $data         The file data.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
     function writeData($path, $name, $data, $autocreate = false)
     {
+        $res = $this->_checkQuotaWrite('string', $data);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $fp = @fopen($this->_getNativePath($path, $name), 'w');
         if (!$fp) {
             if ($autocreate) {
@@ -128,7 +139,7 @@ class VFS_sql_file extends VFS {
                 }
             } else {
                 return PEAR::raiseError(_("Unable to open VFS file for writing."));
-	    }
+            }
         }
 
         if (!@fwrite($fp, $data)) {
@@ -144,8 +155,6 @@ class VFS_sql_file extends VFS {
     /**
      * Moves a file in the database and the file system.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The old filename.
      * @param string $dest  The new filename.
@@ -154,6 +163,11 @@ class VFS_sql_file extends VFS {
      */
     function move($path, $name, $dest)
     {
+        $orig = $this->_getNativePath($path, $name);
+        if (preg_match('|^' . preg_quote($orig) . '[$/]|', $dest)) {
+            return PEAR::raiseError(_("Can not move file(s) - destination is within source."));
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
@@ -176,8 +190,6 @@ class VFS_sql_file extends VFS {
     /**
      * Copies a file through the backend.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      * @param string $dest  The destination of the file.
@@ -186,6 +198,11 @@ class VFS_sql_file extends VFS {
      */
     function copy($path, $name, $dest)
     {
+        $orig = $this->_getNativePath($path, $name);
+        if (preg_match('|^' . preg_quote($orig) . '[$/]|', $dest)) {
+            return PEAR::raiseError(_("Can not copy file(s) - source and destination are the same."));
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
@@ -202,27 +219,21 @@ class VFS_sql_file extends VFS {
             return PEAR::raiseError(_("Unable to copy VFS file."));
         }
 
-        if (is_dir($this->_getNativePath($path, $name))) {
+        if (is_dir($orig)) {
             return $this->_recursiveCopy($path, $name, $dest);
         }
 
-        if (!@copy($this->_getNativePath($path, $name), $this->_getNativePath($dest, $name))) {
+        if (!@copy($orig, $this->_getNativePath($dest, $name))) {
             return PEAR::raiseError(_("Unable to copy VFS file."));
         }
 
         $id = $this->_db->nextId($this->_params['table']);
 
-        $query = sprintf('INSERT INTO %s (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified, vfs_owner) VALUES (%s, %s, %s, %s, %s, %s)',
-                         $this->_params['table'],
-                         $this->_db->quote($id),
-                         $this->_db->quote(VFS_FILE),
-                         $this->_db->quote($dest),
-                         $this->_db->quote($name),
-                         $this->_db->quote(time()),
-                         $this->_db->quote($this->_params['user'])
-                         );
+        $query = sprintf('INSERT INTO %s (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified, vfs_owner) VALUES (?, ?, ?, ?, ?, ?)',
+                         $this->_params['table']);
+        $values = array($id, VFS_FILE, $dest, $name, time(), $this->_params['user']);
 
-        $result = $this->_db->query($query);
+        $result = $this->_db->query($query, $values);
 
         if (is_a($result, 'PEAR_Error')) {
             unlink($this->_getNativePath($dest, $name));
@@ -235,8 +246,6 @@ class VFS_sql_file extends VFS {
     /**
      * Creates a folder on the VFS.
      *
-     * @access public
-     *
      * @param string $path  Holds the path of directory to create folder.
      * @param string $name  Holds the name of the new folder.
      *
@@ -251,22 +260,17 @@ class VFS_sql_file extends VFS {
 
         $id = $this->_db->nextId($this->_params['table']);
         $result = $this->_db->query(sprintf('INSERT INTO %s (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified, vfs_owner)
-                                         VALUES (%s, %s, %s, %s, %s, %s)',
-                                         $this->_params['table'],
-                                         $this->_db->quote($id),
-                                         $this->_db->quote(VFS_FOLDER),
-                                         $this->_db->quote($path),
-                                         $this->_db->quote($name),
-                                         $this->_db->quote(time()),
-                                         $this->_db->quote($this->_params['user'])));
+                                            VALUES (?, ?, ?, ?, ?, ?)',
+                                            $this->_params['table']),
+                                    array($id, VFS_FOLDER, $path, $name, time(), $this->_params['user']));
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
 
         if (!@mkdir($this->_getNativePath($path, $name))) {
-            $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = %s',
-                                                $this->_params['table'],
-                                                $this->_db->quote($id)));
+            $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_id = ?',
+                                                $this->_params['table']),
+                                        array($id));
             return PEAR::raiseError(_("Unable to create VFS directory."));
         }
 
@@ -276,8 +280,6 @@ class VFS_sql_file extends VFS {
     /**
      * Rename a file or folder in the VFS.
      *
-     * @access public
-     *
      * @param string $oldpath  The old path to the file.
      * @param string $oldname  The old filename.
      * @param string $newpath  The new path of the file.
@@ -292,38 +294,46 @@ class VFS_sql_file extends VFS {
             return $conn;
         }
 
-        $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = %s, vfs_name = %s, vfs_modified = %s
-                                            WHERE vfs_path = %s AND vfs_name = %s',
-                                            $this->_params['table'],
-                                            $this->_db->quote($newpath),
-                                            $this->_db->quote($newname),
-                                            $this->_db->quote(time()),
-                                            $this->_db->quote($oldpath),
-                                            $this->_db->quote($oldname)));
+        if (strpos($newpath, '/') === false) {
+            $parent = '';
+            $path = $newpath;
+        } else {
+            list($parent, $path) = explode('/', $newpath, 2);
+        }
+        if (!$this->isFolder($parent, $path)) {
+            if (is_a($result = $this->autocreatePath($newpath), 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
+        $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = ?, vfs_name = ?, vfs_modified = ?
+                                            WHERE vfs_path = ? AND vfs_name = ?',
+                                            $this->_params['table']),
+                                    array($newpath, $newname, time(), $oldpath, $oldname));
 
         if ($this->_db->affectedRows() == 0) {
             return PEAR::raiseError(_("Unable to rename VFS file."));
         }
 
         if (is_a($this->_recursiveSQLRename($oldpath, $oldname, $newpath, $newname), 'PEAR_Error')) {
-            $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = %s, vfs_name = %s
-                                                WHERE vfs_path = %s AND vfs_name = %s',
-                                                $this->_params['table'],
-                                                $this->_db->quote($oldpath),
-                                                $this->_db->quote($oldname),
-                                                $this->_db->quote($newpath),
-                                                $this->_db->quote($newname)));
+            $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = ?, vfs_name = ?
+                                                WHERE vfs_path = ? AND vfs_name = ?',
+                                                $this->_params['table']),
+                                        array($oldpath, $oldname, $newpath, $newname));
             return PEAR::raiseError(_("Unable to rename VFS directory."));
         }
 
+        if (!@is_dir($this->_getNativePath($newpath))) {
+            if (is_a($res = $this->autocreatePath($newpath), 'PEAR_Error')) {
+                return $res;
+            }
+        }
+
         if (!@rename($this->_getNativePath($oldpath, $oldname), $this->_getNativePath($newpath, $newname))) {
-            $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = %s, vfs_name = %s
-                                                WHERE vfs_path = %s AND vfs_name = %s',
-                                                $this->_params['table'],
-                                                $this->_db->quote($oldpath),
-                                                $this->_db->quote($oldname),
-                                                $this->_db->quote($newpath),
-                                                $this->_db->quote($newname)));
+            $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = ?, vfs_name = ?
+                                                WHERE vfs_path = ? AND vfs_name = ?',
+                                                $this->_params['table']),
+                                        array($oldpath, $oldname, $newpath, $newname));
             return PEAR::raiseError(_("Unable to rename VFS file."));
         }
 
@@ -333,11 +343,9 @@ class VFS_sql_file extends VFS {
     /**
      * Delete a folder from the VFS.
      *
-     * @access public
-     *
-     * @param string $path                 The path to delete the folder from.
-     * @param string $name                 The foldername to use.
-     * @param optional boolean $recursive  Force a recursive delete?
+     * @param string $path        The path to delete the folder from.
+     * @param string $name        The foldername to use.
+     * @param boolean $recursive  Force a recursive delete?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -364,11 +372,9 @@ class VFS_sql_file extends VFS {
             }
         }
 
-        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = %s AND vfs_path = %s AND vfs_name = %s',
-                                            $this->_params['table'],
-                                            $this->_db->quote(VFS_FOLDER),
-                                            $this->_db->quote($path),
-                                            $this->_db->quote($name)));
+        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = ? AND vfs_path = ? AND vfs_name = ?',
+                                            $this->_params['table']),
+                                    array(VFS_FOLDER, $path, $name));
 
         if ($this->_db->affectedRows() == 0 || is_a($result, 'PEAR_Error')) {
             return PEAR::raiseError(_("Unable to delete VFS directory."));
@@ -388,8 +394,6 @@ class VFS_sql_file extends VFS {
     /**
      * Delete a file from the VFS.
      *
-     * @access public
-     *
      * @param string $path  The path to store the file in.
      * @param string $name  The filename to use.
      *
@@ -397,16 +401,19 @@ class VFS_sql_file extends VFS {
      */
     function deleteFile($path, $name)
     {
+        $res = $this->_checkQuotaDelete($path, $name);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
-        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = %s AND vfs_path = %s AND vfs_name = %s',
-                                            $this->_params['table'],
-                                            $this->_db->quote(VFS_FILE),
-                                            $this->_db->quote($path),
-                                            $this->_db->quote($name)));
+        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = ? AND vfs_path = ? AND vfs_name = ?',
+                                            $this->_params['table']),
+                                    array(VFS_FILE, $path, $name));
 
         if ($this->_db->affectedRows() == 0) {
             return PEAR::raiseError(_("Unable to delete VFS file."));
@@ -419,18 +426,18 @@ class VFS_sql_file extends VFS {
         if (!@unlink($this->_getNativePath($path, $name))) {
             return PEAR::raiseError(_("Unable to delete VFS file."));
         }
+
+        return true;
     }
 
     /**
      * Return a list of the contents of a folder.
      *
-     * @access public
-     *
-     * @param string $path                The directory path.
-     * @param optional mixed $filter      String/hash of items to filter based
-     *                                    on filename.
-     * @param optional boolean $dotfiles  Show dotfiles?
-     * @param optional boolean $dironly   Show directories only?
+     * @param string $path       The directory path.
+     * @param mixed $filter      String/hash of items to filter based on
+     *                           filename.
+     * @param boolean $dotfiles  Show dotfiles?
+     * @param boolean $dironly   Show directories only?
      *
      * @return mixed  File list on success or false on failure.
      */
@@ -446,9 +453,9 @@ class VFS_sql_file extends VFS {
         $fileList = array();
 
         $fileList = $this->_db->getAll(sprintf('SELECT vfs_name, vfs_type, vfs_modified, vfs_owner FROM %s
-                                               WHERE vfs_path = %s',
-                                               $this->_params['table'],
-                                               $this->_db->quote($path)));
+                                               WHERE vfs_path = ?',
+                                               $this->_params['table']),
+                                       array($path));
         if (is_a($fileList, 'PEAR_Error')) {
             return $fileList;
         }
@@ -478,8 +485,8 @@ class VFS_sql_file extends VFS {
 
             $file['date'] = $line[2];
             $file['owner'] = $line[3];
-            $file['perms'] = '-';
-            $file['group'] = '-';
+            $file['perms'] = '';
+            $file['group'] = '';
 
             // Filtering.
             if ($this->_filterMatch($filter, $file['name'])) {
@@ -501,13 +508,11 @@ class VFS_sql_file extends VFS {
     /**
      * Returns a sorted list of folders in specified directory.
      *
-     * @access public
-     *
-     * @param optional string $path         The path of the directory to get
-     *                                      the directory list for.
-     * @param optional mixed $filter        String/hash of items to filter
-     *                                      based on folderlist.
-     * @param optional boolean $dotfolders  Include dotfolders?
+     * @param string $path         The path of the directory to get the
+     *                             directory list for.
+     * @param mixed $filter        String/hash of items to filter based on
+     *                             folderlist.
+     * @param boolean $dotfolders  Include dotfolders?
      *
      * @return mixed  Folder list on success or a PEAR_Error object on failure.
      */
@@ -518,12 +523,10 @@ class VFS_sql_file extends VFS {
             return $conn;
         }
 
-        $sql = sprintf('SELECT vfs_name, vfs_path FROM %s WHERE vfs_path = %s AND vfs_type = %s',
-                       $this->_params['table'],
-                       $this->_db->quote($path),
-                       VFS_FOLDER);
+        $sql = sprintf('SELECT vfs_name, vfs_path FROM %s WHERE vfs_path = ? AND vfs_type = ?',
+                       $this->_params['table']);
 
-        $folderList = $this->_db->getAll($sql);
+        $folderList = $this->_db->getAll($sql, array($path, $VFS_FOLDER));
         if (is_a($folderList, 'PEAR_Error')) {
             return $folderList;
         }
@@ -607,9 +610,9 @@ class VFS_sql_file extends VFS {
      *
      * @access private
      *
-     * @param string $path                  The path to store the file in.
-     * @param string $name                  The filename to use.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -619,27 +622,23 @@ class VFS_sql_file extends VFS {
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
-	// File already exists in database
-	if ($this->exists($path, $name)) {
-	    $query = sprintf('UPDATE %s SET vfs_modified = %s WHERE vfs_path = %s AND vfs_name = %s',
-			$this->_params['table'],
-			$this->_db->quote(time()),
-			$this->_db->quote($path),
-			$this->_db->quote($name));
-	} else {
+
+        // File already exists in database
+        if ($this->exists($path, $name)) {
+            $query = 'UPDATE ' . $this->_params['table'] .
+                     ' SET vfs_modified = ?' .
+                     ' WHERE vfs_path = ? AND vfs_name = ?';
+            $values = array(time(), $path, $name);
+        } else {
             $id = $this->_db->nextId($this->_params['table']);
 
-            $query = sprintf('INSERT INTO %s (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified,' .
-                             ' vfs_owner) VALUES (%s, %s, %s, %s, %s, %s)',
-                             $this->_params['table'],
-                             $this->_db->quote($id),
-                             $this->_db->quote(VFS_FILE),
-                             $this->_db->quote($path),
-                             $this->_db->quote($name),
-                             $this->_db->quote(time()),
-                             $this->_db->quote($this->_params['user']));
-	}
-        return $this->_db->query($query);
+            $query = 'INSERT INTO ' . $this->_params['table'] .
+                     ' (vfs_id, vfs_type, vfs_path, vfs_name, vfs_modified,' .
+                     ' vfs_owner) VALUES (?, ?, ?, ?, ?, ?)';
+            $values = array($id, VFS_FILE, $path, $name, time(),
+                            $this->_params['user']);
+        }
+        return $this->_db->query($query, $values);
     }
 
     /**
@@ -656,19 +655,19 @@ class VFS_sql_file extends VFS {
      */
     function _recursiveSQLRename($oldpath, $oldname, $newpath, $newname)
     {
-        $folderList = $this->_db->getCol(sprintf('SELECT vfs_name FROM %s WHERE vfs_type = %s AND vfs_path = %s',
-                                                 $this->_params['table'],
-                                                 $this->_db->quote(VFS_FOLDER),
-                                                 $this->_db->quote($this->_getSQLNativePath($oldpath, $oldname))));
+        $folderList = $this->_db->getCol(sprintf('SELECT vfs_name FROM %s WHERE vfs_type = ? AND vfs_path = ?',
+                                                 $this->_params['table']),
+                                         0,
+                                         array(VFS_FOLDER, $this->_getSQLNativePath($oldpath, $oldname)));
 
         foreach ($folderList as $folder) {
             $this->_recursiveSQLRename($this->_getSQLNativePath($oldpath, $oldname), $folder, $this->_getSQLNativePath($newpath, $newname), $folder);
         }
 
-        $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = %s WHERE vfs_path = %s',
-                                            $this->_params['table'],
-                                            $this->_db->quote($this->_getSQLNativePath($newpath, $newname)),
-                                            $this->_db->quote($this->_getSQLNativePath($oldpath, $oldname))));
+        $result = $this->_db->query(sprintf('UPDATE %s SET vfs_path = ? WHERE vfs_path = ?',
+                                            $this->_params['table']),
+                                    array($this->_getSQLNativePath($newpath, $newname),
+                                          $this->_getSQLNativePath($oldpath, $oldname)));
 
         if (is_a($result, 'PEAR_Error')) {
             return $result;
@@ -688,28 +687,25 @@ class VFS_sql_file extends VFS {
      */
     function _recursiveSQLDelete($path, $name)
     {
-        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = %s AND vfs_path = %s',
-                                            $this->_params['table'],
-                                            $this->_db->quote(VFS_FILE),
-                                            $this->_db->quote($this->_getSQLNativePath($path, $name))));
+        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = ? AND vfs_path = ?',
+                                            $this->_params['table']),
+                                    array(VFS_FILE, $this->_getSQLNativePath($path, $name)));
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
 
-        $folderList = $this->_db->getCol(sprintf('SELECT vfs_name FROM %s WHERE vfs_type = %s AND vfs_path = %s',
-                                                 $this->_params['table'],
-                                                 $this->_db->quote(VFS_FOLDER),
-                                                 $this->_db->quote($this->_getSQLNativePath($path, $name))));
+        $folderList = $this->_db->getCol(sprintf('SELECT vfs_name FROM %s WHERE vfs_type = ? AND vfs_path = ?',
+                                                 $this->_params['table']),
+                                         0,
+                                         array(VFS_FOLDER, $this->_getSQLNativePath($path, $name)));
 
         foreach ($folderList as $folder) {
             $this->_recursiveSQLDelete($this->_getSQLNativePath($path, $name), $folder);
         }
 
-        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = %s AND vfs_name = %s AND vfs_path = %s',
-                                            $this->_params['table'],
-                                            $this->_db->quote(VFS_FOLDER),
-                                            $this->_db->quote($name),
-                                            $this->_db->quote($path)));
+        $result = $this->_db->query(sprintf('DELETE FROM %s WHERE vfs_type = ? AND vfs_name = ? AND vfs_path = ?',
+                                            $this->_params['table']),
+                                    array(VFS_FOLDER, $name, $path));
 
         return $result;
     }
@@ -757,13 +753,25 @@ class VFS_sql_file extends VFS {
                 return PEAR::raiseError(_("No configuration information specified for SQL-File VFS."));
             }
 
-            $required = array('phptype', 'hostspec', 'username', 'password', 'database', 'vfsroot');
+            $required = array('phptype', 'vfsroot');
             foreach ($required as $val) {
                 if (!isset($this->_params[$val])) {
-                    return PEAR::raiseError(sprintf(_("Required '%s' not specified in VFS configuration."), $val));
+                    return PEAR::raiseError(sprintf(_("Required \"%s\" not specified in VFS configuration."), $val));
                 }
             }
 
+            if (!isset($this->_params['database'])) {
+                $this->_params['database'] = '';
+            }
+
+            if (!isset($this->_params['username'])) {
+                $this->_params['username'] = '';
+            }
+
+            if (!isset($this->_params['hostspec'])) {
+                $this->_params['hostspec'] = '';
+            }
+
             if (!isset($this->_params['table'])) {
                 $this->_params['table'] = 'horde_vfs';
             }
@@ -778,7 +786,15 @@ class VFS_sql_file extends VFS {
                 return $error;
             }
 
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            // Set DB portability options.
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
+
         }
 
         return true;
@@ -845,4 +861,24 @@ class VFS_sql_file extends VFS {
         return $path . '/' . $name;
     }
 
+    /**
+     * Returns the size of a file.
+     *
+     * @access public
+     *
+     * @param string $path  The path of the file.
+     * @param string $name  The filename.
+     *
+     * @return integer  The size of the file in bytes or PEAR_Error on
+     *                  failure.
+     */
+    function getFileSize($path, $name)
+    {
+        if (($size = @filesize($this->_getNativePath($path, $name))) === false){
+            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s/%s\".", $path, $name)));
+        }
+
+        return $size;
+    }
+
 }
