@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: framework/XML_WBXML/WBXML/ContentHandler.php,v 1.9.10.2 2005/01/20 10:35:09 jan Exp $
+ * $Horde: framework/XML_WBXML/WBXML/ContentHandler.php,v 1.9.10.6 2006/01/01 21:28:45 jan Exp $
  *
- * Copyright 2003-2005 Anthony Mills <amills@pyramid6.com>
+ * Copyright 2003-2006 Anthony Mills <amills@pyramid6.com>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -66,13 +66,18 @@ class XML_WBXML_ContentHandler {
         return $this->_output;
     }
 
+    function getOutputSize()
+    {
+        return strlen($this->_output);
+    }
+
     function startElement($uri, $element, $attrs)
     {
         $this->_output .= '<' . $element;
 
         $currentUri = $this->_currentUri->top();
 
-        if ((!$currentUri) || ($currentUri != $uri)) {
+        if (((!$currentUri) || ($currentUri != $uri)) && $uri) {
             $this->_output .= ' xmlns="' . $uri . '"';
         }
 
@@ -129,7 +134,7 @@ class XML_WBXML_LifoQueue {
 
     function push($obj)
     {
-        array_push($this->_queue, $obj);
+        $this->_queue[] = $obj;
     }
 
     function pop()
