@@ -5,15 +5,21 @@
  * This file defines Horde's external API interface. Other
  * applications can interact with Horde through this API.
  *
- * $Horde: horde/lib/api.php,v 1.43 2004/12/11 12:11:53 jan Exp $
+ * $Horde: horde/lib/api.php,v 1.43.2.3 2005/10/18 11:33:39 jan Exp $
  *
  * @package Horde
  */
 
+$_services['perms'] = array(
+    'args' => array(),
+    'type' => '{urn:horde}stringArray');
+
 $_types = array(
-    'stringArray' => array(array('item' => 'string')),
+    '{urn:horde}stringArray' => array(array('item' => 'string')),
     'hashItem' => array('key' => 'string', 'value' => 'string'),
-    'hash' => array(array('item' => '{urn:horde}hashItem')));
+    'hash' => array(array('item' => '{urn:horde}hashItem')),
+    'hashHashItem' => array('key' => 'string', 'value' => '{urn:horde}hash'),
+    'hashHash' => array(array('item' => '{urn:horde}hashHashItem')));
 
 $_services['admin_list'] = array(true);
 
@@ -21,14 +27,6 @@ $_services['removeUserData'] = array(
     'args' => array('user' => 'string')
 );
 
-/**
- * General API for deleting Horde_Links
- */
-$_services['deleteLink'] = array(
-    'link' => '%application%/services/links/delete.php?' .
-              'link_data=|link_data|&return_url=|url|'
-);
-
 $_services['listApps'] = array(
     'args' => array('filter' => '{urn:horde}stringArray'),
     'type' => '{urn:horde}stringArray'
@@ -49,6 +47,16 @@ $_services['blockContent'] = array(
     'type' => 'string'
 );
 
+$_services['getPreference'] = array(
+    'args' => array('app' => 'string', 'pref' => 'string'),
+    'type' => 'string'
+);
+
+$_services['setPreference'] = array(
+    'args' => array('app' => 'string', 'pref' => 'string', 'value' => 'string'),
+    'type' => 'boolean'
+);
+
 $_services['blocks'] = array(
     'args' => array(),
     'type' => '{urn:horde}hash'
@@ -70,6 +78,17 @@ function &_horde_block($app, $name, $params = array())
     return $result;
 }
 
+function _horde_perms()
+{
+    $perms = array();
+
+    $perms['tree']['horde']['max_blocks'] = false;
+    $perms['title']['horde:max_blocks'] = _("Maximum Number of Portal Blocks");
+    $perms['type']['horde:max_blocks'] = 'int';
+
+    return $perms;
+}
+
 function _horde_blockTitle($app, $name, $params = array())
 {
     $block = &_horde_block($app, $name, $params);
@@ -120,11 +139,15 @@ function _horde_admin_list()
                  'datatree' => array(
                      'link' => '%application%/admin/datatree.php',
                      'name' => _("_DataTree"),
-                     'icon' => 'perms.png'),
+                     'icon' => 'datatree.png'),
+                 'sessions' => array(
+                     'link' => '%application%/admin/sessions.php',
+                     'name' => _("Sessions"),
+                     'icon' => 'user.png'),
                  'phpshell' => array(
                      'link' => '%application%/admin/phpshell.php',
                      'name' => _("P_HP Shell"),
-                     'icon' => 'shell.png'),
+                     'icon' => 'mime/php.png'),
                  'sqlshell' => array(
                      'link' => '%application%/admin/sqlshell.php',
                      'name' => _("S_QL Shell"),
@@ -156,3 +179,15 @@ function _horde_listAPIs()
 {
     return $GLOBALS['registry']->listAPIs();
 }
+
+function _horde_getPreference($app, $pref)
+{
+    $GLOBALS['registry']->loadPrefs($app);
+    return $GLOBALS['prefs']->getValue($pref);
+}
+
+function _horde_setPreference($app, $pref, $value)
+{
+    $GLOBALS['registry']->loadPrefs($app);
+    return $GLOBALS['prefs']->setValue($pref, $value);
+}
