@@ -5,9 +5,9 @@
  * This file sets up any necessary include path variables and includes
  * the minimum required Horde libraries.
  *
- * $Horde: horde/lib/core.php,v 1.26.6.1 2005/01/03 12:25:41 jan Exp $
+ * $Horde: horde/lib/core.php,v 1.26.6.7 2006/01/07 16:40:28 chuck Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -17,6 +17,25 @@
 ini_set('magic_quotes_sybase', 0);
 ini_set('magic_quotes_runtime', 0);
 
+/* Unset all variables populated through register_globals. */
+if (ini_get('register_globals')) {
+    foreach (array_keys($_GET) as $key) {
+        unset($$key);
+    }
+    foreach (array_keys($_POST) as $key) {
+        unset($$key);
+    }
+    foreach (array_keys($_COOKIE) as $key) {
+        unset($$key);
+    }
+    foreach (array_keys($_ENV) as $key) {
+        unset($$key);
+    }
+    foreach (array_keys($_SERVER) as $key) {
+        unset($$key);
+    }
+}
+
 /* If the Horde Framework packages are not installed in PHP's global
  * include_path, you must add an ini_set() call here to add their location to
  * the include_path. */
