@@ -1,7 +1,7 @@
 #!/usr/local/bin/php
 <?php
 /**
- * $Horde: horde/scripts/remove_prefs.php,v 1.3.10.4 2005/10/13 08:33:20 selsky Exp $
+ * $Horde: horde/scripts/remove_prefs.php,v 1.3.10.6 2006/01/18 19:57:06 ben Exp $
  *
  * This script removes a pref from users' settings. Helps when a setting is
  * to be moved from locked = false, to locked = true and there have already
@@ -50,7 +50,15 @@ $db = &DB::connect($conf['sql']);
 if (is_a($db, 'PEAR_Error')) {
    var_dump($db); exit;
 }
-$db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+
+// Set DB portability options.
+switch ($db->phptype) {
+case 'mssql':
+    $db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+    break;
+default:
+    $db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+}
 
 if ($live) {
     $sql = 'DELETE FROM horde_prefs WHERE pref_scope = ? AND pref_name = ?';
