@@ -40,8 +40,10 @@ go
 CREATE TABLE horde_users (
   user_uid varchar(256) NOT NULL,
   user_pass varchar(256) NOT NULL,
+  user_soft_expiration_date numeric(10,0),
+  user_hard_expiration_date numeric(10,0),
   PRIMARY KEY  (user_uid)
-) 
+)
 go
 
 CREATE TABLE horde_datatree (
@@ -56,7 +58,7 @@ CREATE TABLE horde_datatree (
   PRIMARY KEY  (datatree_id),
   FOREIGN KEY (user_uid)
     REFERENCES horde_users(user_uid)
- ) 
+ )
 go
 
 CREATE TABLE horde_prefs (
@@ -66,21 +68,21 @@ CREATE TABLE horde_prefs (
   pref_value text NULL,
   PRIMARY KEY  (pref_uid,pref_scope,pref_name)
 )
-go 
+go
 
 CREATE TABLE horde_sessionhandler (
   session_id varchar(32) NOT NULL,
   session_lastmodified numeric(11,0) NOT NULL,
   session_data image NULL,
   PRIMARY KEY  (session_id)
-) 
+)
 go
 
 
 -- CREATE TABLE horde_datatree_seq (
 --   id numeric(10,0) IDENTITY NOT NULL,
 --   PRIMARY KEY  (id)
--- ) 
+-- )
 -- go
 
 -- CREATE TABLE horde_tokens (
@@ -88,7 +90,7 @@ go
 --   token_id varchar(32) NOT NULL,
 --   token_timestamp numeric(20,0) NOT NULL,
 --   PRIMARY KEY  (token_address,token_id)
--- ) 
+-- )
 -- go
 
 -- CREATE TABLE horde_vfs (
@@ -100,7 +102,7 @@ go
 --   vfs_owner varchar(256) NOT NULL,
 --   vfs_data image NULL,
 --   PRIMARY KEY  (vfs_id)
--- ) 
+-- )
 -- go
 
 -- CREATE TABLE horde_muvfs (
