@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: horde/services/help/index.php,v 2.80.10.1 2005/01/03 12:25:45 jan Exp $
+ * $Horde: horde/services/help/index.php,v 2.80.10.6 2006/03/28 12:31:15 jan Exp $
  *
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -39,6 +39,7 @@ if ($show == 'index') {
     $menu_url = Util::addParameter($menu_url, array('module' => $module,
                                                     'show' => 'menu'));
     require HORDE_TEMPLATES . '/help/index.inc';
+    exit;
 } else {
     $bodyClass = 'help';
     require HORDE_TEMPLATES . '/common-header.inc';
@@ -47,13 +48,13 @@ if ($show == 'index') {
         $url = Horde::url($registry->get('webroot', 'horde') . '/services/help/');
         $url = Util::addParameter($url, 'module', $module);
         $topics_link = Util::addParameter($url, 'show', 'topics');
-        $topics_link = Horde::link($topics_link, _("List Help Topics"), 'header', 'help_main') . _("List Help Topics") . '</a>';
+        $topics_link = Horde::link($topics_link, null, 'header', 'help_main') . _("List Help Topics") . '</a>';
         $about_link = Util::addParameter($url, 'show', 'about');
-        $about_link = Horde::link($about_link, _("About..."), 'header', 'help_main') . _("About...") . '</a>';
+        $about_link = Horde::link($about_link, null, 'header', 'help_main') . _("About...") . '</a>';
         require HORDE_TEMPLATES . '/help/menu.inc';
     } elseif ($show == 'about') {
         require $fileroot . '/lib/version.php';
-        eval('$version = "' . ucfirst($module) . ' " . ' . String::upper($module) . '_VERSION;');
+        $version = String::ucfirst($module) . ' ' . constant(String::upper($module) . '_VERSION');
         $credits = Util::bufferOutput('include', $fileroot . '/docs/CREDITS');
         $credits = String::convertCharset($credits, 'iso-8859-1', NLS::getCharset());
         require HORDE_TEMPLATES . '/help/about.inc';
