@@ -1,10 +1,10 @@
 <?php
 /**
- * $Horde: horde/services/portal/sidebar.php,v 1.4.2.3 2005/03/07 10:12:06 jan Exp $
+ * $Horde: horde/services/portal/sidebar.php,v 1.4.2.13 2006/03/20 15:00:06 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
- * Copyright 2003-2005 Michael Pawlowsky <mjpawlowsky@yahoo.com>
+ * Copyright 1999-2006 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
+ * Copyright 2003-2006 Michael Pawlowsky <mjpawlowsky@yahoo.com>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -19,7 +19,7 @@
  */
 function canSee($app, $params, &$hasChildren)
 {
-    global $perms, $registry;
+    global $registry;
 
     static $cache = array();
     static $isAdmin;
@@ -37,23 +37,27 @@ function canSee($app, $params, &$hasChildren)
         $user = Auth::getAuth();
     }
 
-    // Check if the current user has permisson to see this
-    // application, and if the application is active. Headings are
-    // visible to everyone (but get filtered out later if they have no
-    // children). Administrators always see all applications except
-    // those marked 'inactive'. Anyone with SHOW permissions can see
-    // an application, but READ is needed to actually use the
-    // application. You can use this distinction to show applications
-    // to guests that they need to log in to use. If you don't want
-    // them to see apps they can't use, then don't give guests SHOW
-    // permissions to anything.
-    if (// Don't show applications that aren't installed, even if
-        // they're configured.
+    // Check if the current user has permisson to see this application, and if
+    // the application is active. Headings are visible to everyone (but get
+    // filtered out later if they have no children). Administrators always see
+    // all applications except those marked 'inactive'. Anyone with SHOW
+    // permissions can see an application, but READ is needed to actually use
+    // the application. You can use this distinction to show applications to
+    // guests that they need to log in to use. If you don't want them to see
+    // apps they can't use, then don't give guests SHOW permissions to
+    // anything.
+    if (// Don't show applications that aren't installed, even if they're
+        // configured.
         (isset($params['fileroot']) && !is_dir($params['fileroot'])) ||
 
-        // Filter out entries that are disabled, hidden or shouldn't
-        // show up in the menu.
-        $params['status'] == 'notoolbar' || $params['status'] == 'hidden' || $params['status'] == 'inactive') {
+        // Don't show blocks of applications that aren't installed.
+        ($params['status'] == 'block' &&
+         !is_dir($registry->get('fileroot', $params['app']))) ||
+
+        // Filter out entries that are disabled, hidden or shouldn't show up
+        // in the menu.
+        $params['status'] == 'notoolbar' || $params['status'] == 'hidden' ||
+        $params['status'] == 'inactive') {
 
         $cache[$app] = false;
 
@@ -63,8 +67,8 @@ function canSee($app, $params, &$hasChildren)
               // Admins see everything that makes it to this point.
               ($isAdmin ||
 
-               // Users who have SHOW permissions to active or block
-               // entries see them.
+               // Users who have SHOW permissions to active or block entries
+               // see them.
                ($registry->hasPermission($app, PERMS_SHOW) &&
                 ($params['status'] == 'active' ||
                  $params['status'] == 'block')))) {
@@ -88,7 +92,7 @@ function canSee($app, $params, &$hasChildren)
  */
 function buildMenu()
 {
-    global $conf, $perms, $registry;
+    global $conf, $registry;
 
     $apps = array();
     $children = array();
@@ -150,8 +154,44 @@ function buildMenu()
         $conf['prefs']['driver'] != '' && $conf['prefs']['driver'] != 'none') {
         $menu['options'] = array('name' => _("Options"),
                                  'status' => 'active',
-                                 'icon' => $registry->getImageDir() . '/prefs.png',
-                                 'url' => Horde::applicationUrl('services/prefs.php'));
+                                 'icon' => $registry->getImageDir() . '/prefs.png');
+
+        /* Get a list of configurable applications. */
+        $prefs_apps = array();
+        foreach ($registry->applications as $application => $params) {
+            if ($params['status'] == 'heading' ||
+                $params['status'] == 'block' ||
+                !file_exists($registry->get('fileroot', $application) . '/config/prefs.php')) {
+                continue;
+            }
+
+            /* Check if the current user has permission to see this
+             * application, and if the application is active.
+             * Administrators always see all applications. */
+            if ((Auth::isAdmin() && $params['status'] != 'inactive') ||
+                ($registry->hasPermission($application) &&
+                 ($params['status'] == 'active'))) {
+                $prefs_apps[$application] = _($params['name']);
+            }
+        }
+
+        if (!empty($prefs_apps['horde'])) {
+            $menu['options_' . 'horde'] = array('name' => _("Global Options"),
+                                                'status' => 'active',
+                                                'menu_parent' => 'options',
+                                                'icon' => $registry->get('icon', 'horde'),
+                                                'url' => Horde::applicationUrl('services/prefs.php?app=horde'));
+            unset($prefs_apps['horde']);
+        }
+
+        asort($prefs_apps);
+        foreach ($prefs_apps as $app => $name) {
+            $menu['options_' . $app] = array('name' => $name,
+                                             'status' => 'active',
+                                             'menu_parent' => 'options',
+                                             'icon' => $registry->get('icon', $app),
+                                             'url' => Horde::applicationUrl('services/prefs.php?app=' . $app));
+        }
     }
 
     if (Auth::isAuthenticated()) {
@@ -190,15 +230,12 @@ $tree->setOption(array('target' => $is_mozbar ? '_content' : 'horde_main'));
 $menu = buildMenu();
 foreach ($menu as $app => $params) {
     if ($params['status'] == 'block') {
-        if (isset($menu[$params['app']])) {
-            $block = &Horde_Block_Collection::getBlock($params['app'], $params['blockname']);
-            if (is_a($block, 'PEAR_Error')) {
-                Horde::fatal($block, __FILE__, __LINE__);
-            }
-            $block->buildTree($tree,
-                              isset($params['menu_parent']) ? 1 : 0,
-                              isset($params['menu_parent']) ? $params['menu_parent'] : null);
+        $block = &Horde_Block_Collection::getBlock($params['app'], $params['blockname']);
+        if (is_a($block, 'PEAR_Error')) {
+            Horde::fatal($block, __FILE__, __LINE__);
         }
+        $block->buildTree($tree, 0,
+                          isset($params['menu_parent']) ? $params['menu_parent'] : null);
     } else {
         // Need to run the name through gettext since the user's
         // locale may not have been loaded when registry.php was
@@ -220,12 +257,7 @@ foreach ($menu as $app => $params) {
                              'icon' => isset($params['icon']) ? $params['icon'] : $registry->get('icon', $app),
                              'icondir' => '',
                              );
-        if (empty($params['menu_parent'])) {
-            // Standalone link or container heading.
-            $tree->addNode($app, null, $name, 0, false, $node_params);
-        } else {
-            $tree->addNode($app, $params['menu_parent'], $name, 1, false, $node_params);
-        }
+        $tree->addNode($app, !empty($params['menu_parent']) ? $params['menu_parent'] : null, $name, 0, false, $node_params);
     }
 }
 
@@ -247,4 +279,3 @@ Horde::addScriptFile('httpclient.js', 'horde', true);
 Horde::addScriptFile('hideable.js', 'horde', true);
 require HORDE_TEMPLATES . '/common-header.inc';
 require HORDE_TEMPLATES . '/portal/sidebar.inc';
-require HORDE_TEMPLATES . '/common-footer.inc';
