@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: horde/services/resetpassword.php,v 1.5.10.1 2005/01/03 12:25:45 jan Exp $
+ * $Horde: horde/services/resetpassword.php,v 1.5.10.4 2006/01/01 21:29:10 jan Exp $
  *
- * Copyright 2004-2005 Marko Djukic <marko@oblo.com>
+ * Copyright 2004-2006 Marko Djukic <marko@oblo.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -66,7 +66,7 @@ if ($can_validate && $form->validate($vars)) {
         $password = $auth->resetPassword($info['username']);
 
         require_once 'Mail.php';
-        $mailer = &Mail::factory($conf['mailer']['type'], $conf['mailer']['params']);
+        $mailer = Mail::factory($conf['mailer']['type'], $conf['mailer']['params']);
 
         /* Set up the email headers and body. */
         $headers['From'] = $email;
