@@ -1,14 +1,13 @@
 <?php
 /**
- * $Horde: horde/test.php,v 1.122.6.26 2008/07/22 21:49:32 slusarz Exp $
+ * $Horde: horde/test.php,v 1.122.6.16 2006/01/01 21:28:04 jan Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2006 Brent J. Nordquist <bjn@horde.org>
+ * Copyright 1999-2006 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
- * See the enclosed file COPYING for license information (LGPL). If you
+ * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
- *
- * @author Brent J. Nordquist <bjn@horde.org>
- * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 /* Register a session. */
@@ -24,20 +23,20 @@ include_once 'lib/core.php';
  * core.php. If String:: isn't defined, then we're not finding some critical
  * libraries. */
 if (!class_exists('String')) {
-    echo '<br /><h2 style="color:red">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough and display_errors is on, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://www.horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</h2>';
+    echo '<br /><span style="color: red; font-size: 18px; font-weight: bold;">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files AND are installing a CVS checkout, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</span>';
     exit;
 }
 
 /* Initialize the Horde_Test:: class. */
 if (!include_once 'lib/Test.php') {
     /* Try and provide enough information to debug the missing file. */
-    echo '<br /><h2 style="color:red">Unable to find horde/lib/Test.php. Your Horde installation may be missing critical files, or PHP may not have sufficient permissions to include files. There may be error messages printed above this message that will help you in debugging the problem.</h2>';
+    echo '<br /><span style="color: red; font-size: 18px; font-weight: bold;">Unable to find horde/lib/Test.php. Your Horde installation may be missing critical files, or PHP may not have sufficient permissions to include files. There may be error messages printed above this message that will help you in debugging the problem.</span>';
     exit;
 }
 
 /* If we've gotten this far, we should have found enough of Horde to run
  * tests. Create the testing object. */
-$horde_test = new Horde_Test();
+$horde_test = &new Horde_Test();
 
 /* Horde definitions. */
 $module = 'Horde';
@@ -46,103 +45,55 @@ $module_version = HORDE_VERSION;
 
 /* PHP module capabilities. */
 $module_list = array(
-    'ctype' => array(
-        'descrip' => 'Ctype Support',
-        'error' => 'The ctype functions are required by the help system, the weather portal blocks, and a few Horde applications.'),
-    (version_compare(PHP_VERSION, '5') < 0 ? 'domxml' : 'dom') => array(
-        'descrip' => 'DOM XML Support',
-        'error' => 'DOM support is required for the configuration frontend and Kolab support.'),
-    'ftp' => array(
-        'descrip' => 'FTP Support',
-        'error' => 'FTP support is only required if you want to authenticate against an FTP server, upload your configuration files with FTP, or use an FTP server for file storage.'),
-    'gd' => array(
-        'descrip' => 'GD Support',
-        'error' => 'Horde will use the GD extension to perform manipulations on image data. You can also use the ImageMagick software to do these manipulations instead.'),
+    'ctype' => 'Ctype Support',
+    (version_compare(phpversion(), '5') < 0 ? 'domxml' : 'dom') => 'DOM XML Support',
+    'ftp' => 'FTP Support',
+    'gd' => 'GD Support',
     'gettext' => array(
         'descrip' => 'Gettext Support',
         'error' => 'Horde will not run without gettext support. Compile PHP with <code>--with-gettext</code> before continuing.',
-        'fatal' => true),
-    'geoip' => array(
-        'descrip' => 'GeoIP Support (via PECL extension)',
-        'error' => 'Horde can optionally use the GeoIP extension to provide faster country name lookups.'),
+        'fatal' => true
+    ),
     'iconv' => array(
         'descrip' => 'Iconv Support',
         'error' => 'If you want to take full advantage of Horde\'s localization features and character set support, you will need the iconv extension.'
     ),
-    'iconv_libiconv' => array(
-        'descrip' => 'GNU Iconv Support',
-        'error' => 'For best results make sure the iconv extension is linked against GNU libiconv.',
-        'function' => '_check_iconv_implementation'),
-    'imap' => array(
-        'descrip' => 'IMAP Support',
-        'error' => 'The IMAP extension is required if you want to authenticate against an IMAP server, or use any webmail application.'),
-    'json' => array(
-        'descrip' => 'JSON Support',
-        'error' => 'Horde can optionally use the JSON extension to provide faster Javascript generation for rich GUI applications.'),
-    'ldap' => array(
-        'descrip' => 'LDAP Support',
-        'error' => 'LDAP support is only required if you want to use an LDAP server for anything like authentication, address books, or preference storage.'),
-    'lzf' => array(
-        'descrip' => 'LZF Compression Support',
-        'error' => 'If the lzf PECL module is available, Horde can compress some cached data in your session to make your session size smaller.'),
+    'imap' => 'IMAP Support',
+    'ldap' => 'LDAP Support',
     'mbstring' => array(
         'descrip' => 'Mbstring Support',
-        'error' => 'If you want to take full advantage of Horde\'s localization features and character set support, you will need the mbstring extension.'),
-    'pcre' => array(
-        'descrip' => 'PCRE Support',
-        'error' => 'Horde will not run without PCRE support. Compile PHP with <code>--with-pcre</code> before continuing.',
-        'fatal' => true),
-    'pcre_utf8' => array(
-        'descrip' => 'PCRE UTF-8 Support',
-        'error' => 'If you want to take full advantage of Horde\'s localization features and character set support, you will need PCRE with UTF-8 support.',
-        'function' => '_check_pcre_utf8'),
+        'error' => 'If you want to take full advantage of Horde\'s localization features and character set support, you will need the mbstring extension.'
+    ),
     'mcrypt' => array(
         'descrip' => 'Mcrypt Support',
-        'error' => 'Mcrypt is a general-purpose cryptography library which is broader and significantly more efficient (FASTER!) than PHP\'s own cryptographic code and will provider faster logins.'),
-    'memcache' => array(
-        'descrip' => 'memcached Support (memcache)',
-        'error' => 'The memcache PECL module is needed only if you are using the memcached SessionHandler. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'),
+        'error' => 'Mcrypt is a general-purpose cryptography library which is broader and significantly more efficient (FASTER!) than PHP\'s own cryptographic code and will provider faster logins.'
+    ),
     'fileinfo' => array(
         'descrip' => 'MIME Magic Support (fileinfo)',
-        'error' => 'The fileinfo PECL module or the mime_magic PHP extension (see below) will most likely provide faster MIME Magic lookups than the built-in Horde PHP magic code. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'),
+        'error' => 'The fileinfo PECL module or the mime_magic PHP extension (see below) will most likely provide faster MIME Magic lookups than the built-in Horde PHP magic code. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'
+    ),
+    'memcache' => array(
+        'descrip' => 'memcached Support (memcache)',
+        'error' => 'The memcache PECL module is needed only if you are using the memcached SessionHandler. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'
+    ),
     'mime_magic' => array(
         'descrip' => 'MIME Magic Support (mime_magic)',
-        'error' => 'The fileinfo PECL module (see above) or the mime_magic PHP extension will most likely provide faster MIME Magic lookups than the built-in Horde PHP magic code. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'),
-    'mysql' => array(
-        'descrip' => 'MySQL Support',
-        'error' => 'The MySQL extension is only required if you want to use a MySQL database server for data storage.'),
-    'openssl' => array(
-        'descrip' => 'OpenSSL Support',
-        'error' => 'The OpenSSL extension is required for any kind of S/MIME support.'),
-    'pgsql' => array(
-        'descrip' => 'PostgreSQL Support',
-        'error' => 'The PostgreSQL extension is only required if you want to use a PostgreSQL database server for data storage.'),
+        'error' => 'The fileinfo PECL module (see above) or the mime_magic PHP extension will most likely provide faster MIME Magic lookups than the built-in Horde PHP magic code. See horde/docs/INSTALL for information on how to install PECL/PHP extensions.'
+    ),
+    'mysql' => 'MySQL Support',
+    'openssl' => 'OpenSSL Support',
+    'pgsql' => 'PostgreSQL Support',
     'session' => array(
         'descrip' => 'Session Support',
         'fatal' => true),
-    'xml' => array(
-        'descrip' => 'XML Support',
-        'error' => 'XML support is required for the help system.'),
+    'xml' => 'XML Support',
     'zlib' => array(
         'descrip' => 'Zlib Support',
-        'error' => 'The zlib module is highly recommended for use with Horde.  It allows page compression and handling of ZIP and GZ data. Compile PHP with <code>--with-zlib</code> to activate.'),
+        'error' => 'The zlib module is highly recommended for use with Horde.  It allows page compression and handling of ZIP and GZ data. Compile PHP with <code>--with-zlib</code> to activate.',
+        'fatal' => false
+    ),
 );
 
-/**
- * Additional check for PCRE UTF-8 support.
- */
-function _check_pcre_utf8()
-{
-    return (extension_loaded('pcre') && @preg_match('/.*/u', 'test') !== false);
-}
-
-/**
- * Additional check for iconv module implementation.
- */
-function _check_iconv_implementation()
-{
-    return extension_loaded('iconv') && in_array(ICONV_IMPL, array('libiconv', 'glibc'));
-}
 
 /* PHP Settings. */
 $setting_list = array(
@@ -151,8 +102,8 @@ $setting_list = array(
         'error' => 'magic_quotes_runtime may cause problems with database inserts, etc. Turn it off.'
     ),
     'memory_limit' => array(
-        'setting' => 'value',
-        'error' => 'If PHP\'s internal memory limit is not set high enough Horde will not be able to handle large data items (e.g. large mail attachments in IMP). You should set the value of memory_limit in php.ini to a sufficiently high value - at least 64M is recommended.'
+        'setting' => false,
+        'error' => 'If PHP\'s internal memory limit is turned on and if not set high enough Horde will not be able to handle large data items (e.g. large mail attachments in IMP). If possible, you should disable the PHP memory limit by recompiling PHP <em>without</em> the "--enable-memory-limit" flag. If this is not possible, then you should set the value of memory_limit in php.ini to a sufficiently high value (Current value of memory_limit: ' . ini_get('memory_limit') . ').'
     ),
     'safe_mode' => array(
         'setting' => false,
@@ -165,17 +116,10 @@ $setting_list = array(
     'session.auto_start' => array(
         'setting' => false,
         'error' => 'Horde won\'t work with automatically started sessions, because it explicitly creates new session when necessary to protect against session fixations.'
-    ),
-    'zlib.output_compression' => array(
-        'setting' => false,
-        'error' => 'You should not enable output compression unconditionally because some browsers and scripts don\'t work well with output compression. Enable compression in Horde\'s configuration instead, so that we have full control over the conditions where to enable and disable it.'
-    ),
-    'zend_accelerator.compress_all' => array(
-        'setting' => false,
-        'error' => 'You should not enable output compression unconditionally because some browsers and scripts don\'t work well with output compression. Enable compression in Horde\'s configuration instead, so that we have full control over the conditions where to enable and disable it.'
-    ),
+    )
 );
 
+
 /* PEAR */
 $pear_list = array(
     'Mail' => array(
@@ -183,13 +127,8 @@ $pear_list = array(
         'error' => 'You do not have the Mail package installed on your system. See the INSTALL file for instructions on how to install the package.'
     ),
     'Mail_Mime' => array(
-        'path' => 'Mail/mime.php',
-        'error' => 'You do not have the Mail_Mime package installed on your system. See the INSTALL file for instructions on how to install the package.',
-        'required' => true
-    ),
-    'Mail_mimeDecode' => array(
         'path' => 'Mail/mimeDecode.php',
-        'error' => 'You do not have the Mail_mimeDecode package installed on your system. See the INSTALL file for instructions on how to install the package.',
+        'error' => 'You do not have the Mail_Mime package installed on your system. See the INSTALL file for instructions on how to install the package.',
         'required' => true
     ),
     'Log' => array(
@@ -203,10 +142,6 @@ $pear_list = array(
         'error' => 'You will need DB if you are using SQL drivers for preferences, contacts (Turba), etc.',
         'function' => '_check_pear_db_version'
     ),
-    'MDB2' => array(
-        'path' => 'MDB2.php',
-        'error' => 'You will need MDB2 if you are using the SQL driver for Shares.',
-    ),
     'Net_Socket' => array(
         'path' => 'Net/Socket.php',
         'error' => 'Make sure you are using a version of PEAR which includes the Net_Socket class, or that you have installed the Net_Socket package seperately. See the INSTALL file for instructions on installing Net_Socket.'
@@ -223,9 +158,9 @@ $pear_list = array(
         'path' => 'HTTP/Request.php',
         'error' => 'Parts of Horde (HTML composition in IMP, Jonah, the XML-RPC client/server) use the HTTP_Request library to retrieve URLs and do other HTTP requests.'
     ),
-    'HTTP_WebDAV_Server' => array(
-        'path' => 'HTTP/WebDAV/Server.php',
-        'error' => 'The HTTP_WebDAV_Server is required, if you want to use the WebDAV interface of Horde, e.g. to access calendars or tasklists with external clients.'
+    'File' => array(
+        'path' => 'File.php',
+        'error' => 'Horde will require the File class when importing CSV files.'
     ),
     'Net_SMTP' => array(
         'path' => 'Net/SMTP.php',
@@ -261,6 +196,33 @@ function _check_pear_db_version()
     }
 }
 
+/* There is a version of the File.php that wants a PEAR::registerShutdownFunc
+ * which is what newpear tests for.  This function seems to only have existed
+ * for a little while, but the PEAR group hasn't fixed this yet, so some
+ * people get an error if they have a certain (release) version of PEAR.  We
+ * test for it here.  If it's a safe version, test as normal, otherwise try a
+ * less effective but non-erroring test for File::CSV. */
+if (!$horde_test->isRecentPEAR()) {
+    $path = ini_get('include_path');
+    if (empty($path)) {
+        $path = '.';
+    }
+    $pear_list['File_CSV']['path'] = 'XXXXXX.php';
+    foreach (explode(':', $path) as $dir) {
+        if (substr($dir, -1) != '/') {
+            $dir .= '/';
+        }
+        $file1 = $dir . 'File/CSV.php';
+        $file2 = $dir . 'File.php';
+        $file3 = $dir . 'PEAR.php';
+        if (file_exists($file1) && file_exists($file2) && file_exists($file3)) {
+            /* Change the filename to something we know will be found. */
+            $pear_list['File_CSV']['path'] = 'PEAR.php';
+            break;
+        }
+    }
+}
+
 /* Required configuration files. */
 $file_list = array(
     'config/conf.php' => null,
@@ -288,25 +250,41 @@ if (!empty($_GET['mode'])) {
         exit;
 
     case 'phpinfo':
-        echo '<a href="' . htmlspecialchars($url) . '?mode=test">&lt;&lt; Back to test.php</a>';
+        echo '<a href="' . $url . '?mode=test">&lt;&lt; Back to test.php</a>';
         phpinfo();
         exit;
 
+    case 'filetest':
+        echo '<a href="' . $url . '?mode=test">&lt;&lt; Back to test.php</a>';
+        ?>
+        <html>
+        <body bgcolor="white" text="black">
+        <p><font face="Helvetica, Arial, sans-serif" size="2">
+        There are some versions of PEAR that were released at times that
+        did not contain a function that File.php expects.  This does not
+        seem to have caused problems for anyone except when running our
+        test script, but you need to be aware of it.  If you see a fatal
+        error in File.php listed below this text, you have one of the
+        combinations of problem files.  Make sure you have the latest
+        PEAR and File packages available (<a href="http://pear.php.net/">
+        see http://pear.php.net</a>).  If you do not get the error, you are
+        in fine shape.</p>
+        <?php include_once 'File/CSV.php';
+        exit;
+
     case 'unregister':
         unset($_SESSION['horde_test_count']);
         ?>
         <html>
-        <body>
-        The test session has been unregistered.<br />
-        <a href="test.php">Go back</a> to the test.php page.<br />
-        </body>
-        </html>
+        <body bgcolor="white" text="black">
+        <font face="Helvetica, Arial, sans-serif" size="2">
+        The test session has been unregistered.<br>
+        <a href="test.php">Go back</a> to the test.php page.<br>
         <?php
         exit;
     }
 }
 
-$test_app = 'horde';
 require TEST_TEMPLATES . 'header.inc';
 require TEST_TEMPLATES . 'version.inc';
 
@@ -352,11 +330,10 @@ require TEST_TEMPLATES . 'php_version.inc';
 <ul>
     <?php echo $horde_test->phpSettingCheck(array('file_uploads' => array('setting' => true, 'error' => 'file_uploads must be enabled for some features like sending emails with IMP.' ))) ?>
 <?php if ($dir = ini_get('upload_tmp_dir')): ?>
-    <li>upload_tmp_dir: <strong style="color:"<?php echo is_writable($dir) ? 'green' : 'red' ?>"><?php echo $dir ?></strong></li>
+    <li>upload_tmp_dir: <font color="<?php echo is_writable($dir) ? 'green' : 'red' ?>"><strong><?php echo $dir ?></strong></font></li>
 <?php endif; ?>
     <li>upload_max_filesize: <?php echo ini_get('upload_max_filesize') ?></li>
-    <li>post_max_size: <?php echo ini_get('post_max_size') ?><br />
-    This value should be several times the expect largest upload size (notwithstanding any upload limits present in an application). Any upload that exceeds this size will cause any state information sent along with the uploaded data to be lost. This is a PHP limitation and can not be worked around.</li>
+    <li>post_max_size: <?php echo ini_get('post_max_size') ?></li>
 </ul>
 
 <h1>Required Horde Configuration Files</h1>
